@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
@@ -86,20 +86,17 @@ bool FFPTexturing::resolveUniformParams(TextureUnitParams* textureUnitParams, Pr
 {
 	Program* vsProgram = programSet->getCpuVertexProgram();
 	Program* psProgram = programSet->getCpuFragmentProgram();
-	
+	bool hasError = false;
 	
 	// Resolve texture sampler parameter.		
 	textureUnitParams->mTextureSampler = psProgram->resolveParameter(textureUnitParams->mTextureSamplerType, textureUnitParams->mTextureSamplerIndex, (uint16)GPV_GLOBAL, "gTextureSampler");
-	if (textureUnitParams->mTextureSampler.get() == NULL)
-		return false;
+	hasError |= !(textureUnitParams->mTextureSampler.get());
 	
-		
 	// Resolve texture matrix parameter.
 	if (needsTextureMatrix(textureUnitParams->mTextureUnitState))
 	{				
 		textureUnitParams->mTextureMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_TEXTURE_MATRIX, textureUnitParams->mTextureSamplerIndex);
-		if (textureUnitParams->mTextureMatrix.get() == NULL)
-			return false;
+		hasError |= !(textureUnitParams->mTextureMatrix.get());
 	}
 
 	switch (textureUnitParams->mTexCoordCalcMethod)
@@ -111,43 +108,29 @@ bool FFPTexturing::resolveUniformParams(TextureUnitParams* textureUnitParams, Pr
 	case TEXCALC_ENVIRONMENT_MAP:
 	case TEXCALC_ENVIRONMENT_MAP_PLANAR:	
 	case TEXCALC_ENVIRONMENT_MAP_NORMAL:
-		
-		mWorldITMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_INVERSE_TRANSPOSE_WORLD_MATRIX, 0);
-		if (mWorldITMatrix.get() == NULL)		
-			return false;	
-		
+		//TODO: change the following 'mWorldITMatrix' member to 'mWorldViewITMatrix'
+		mWorldITMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_INVERSE_TRANSPOSE_WORLDVIEW_MATRIX, 0);
 		mViewMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_VIEW_MATRIX, 0);
-		if (mViewMatrix.get() == NULL)		
-			return false;				
+		mWorldMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_WORLD_MATRIX, 0);
 		
+		hasError |= !(mWorldITMatrix.get())  || !(mViewMatrix.get()) || !(mWorldITMatrix.get());
 		break;
 
 	case TEXCALC_ENVIRONMENT_MAP_REFLECTION:
 		mWorldMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_WORLD_MATRIX, 0);
-		if (mWorldMatrix.get() == NULL)		
-			return false;	
-
 		mWorldITMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_INVERSE_TRANSPOSE_WORLD_MATRIX, 0);
-		if (mWorldITMatrix.get() == NULL)		
-			return false;	
-
 		mViewMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_VIEW_MATRIX, 0);
-		if (mViewMatrix.get() == NULL)		
-			return false;	
-
+		
+		hasError |= !(mWorldMatrix.get()) || !(mWorldITMatrix.get()) || !(mViewMatrix.get());
 		break;
 
-
 	case TEXCALC_PROJECTIVE_TEXTURE:
 
 		mWorldMatrix = vsProgram->resolveAutoParameterInt(GpuProgramParameters::ACT_WORLD_MATRIX, 0);
-		if (mWorldMatrix.get() == NULL)		
-			return false;	
-
 		textureUnitParams->mTextureViewProjImageMatrix = vsProgram->resolveParameter(GCT_MATRIX_4X4, -1, (uint16)GPV_LIGHTS, "gTexViewProjImageMatrix");
-		if (textureUnitParams->mTextureViewProjImageMatrix.get() == NULL)		
-			return false;	
-
+		
+		hasError |= !(mWorldMatrix.get()) || !(textureUnitParams->mTextureViewProjImageMatrix.get());
+		
 		const TextureUnitState::EffectMap&		effectMap = textureUnitParams->mTextureUnitState->getEffects();	
 		TextureUnitState::EffectMap::const_iterator	effi;
 
@@ -160,14 +143,17 @@ bool FFPTexturing::resolveUniformParams(TextureUnitParams* textureUnitParams, Pr
 			}
 		}
 
-		
-
-		if (textureUnitParams->mTextureProjector == NULL)		
-			return false;	
-
+		hasError |= !(textureUnitParams->mTextureProjector);
 		break;
 	}
 
+	
+	if (hasError)
+	{
+		OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, 
+				"Not all parameters could be constructed for the sub-render state.",
+				"FFPTexturing::resolveUniformParams" );
+	}
 	return true;
 }
 
@@ -181,7 +167,8 @@ bool FFPTexturing::resolveFunctionsParams(TextureUnitParams* textureUnitParams,
 	Function* vsMain   = vsProgram->getEntryPointFunction();
 	Function* psMain   = psProgram->getEntryPointFunction();
 	Parameter::Content texCoordContent = Parameter::SPC_UNKNOWN;
-	
+	bool hasError = false;
+
 	switch (textureUnitParams->mTexCoordCalcMethod)
 	{
 		case TEXCALC_NONE:					
@@ -194,37 +181,32 @@ bool FFPTexturing::resolveFunctionsParams(TextureUnitParams* textureUnitParams,
 				textureUnitParams->mTextureUnitState->getTextureCoordSet(), 
 				Parameter::Content(Parameter::SPC_TEXTURE_COORDINATE0 + textureUnitParams->mTextureUnitState->getTextureCoordSet()),
 				textureUnitParams->mVSInTextureCoordinateType);	
-			if (textureUnitParams->mVSInputTexCoord.get() == NULL)			
-				return false;		
+			hasError |= !(textureUnitParams->mVSInputTexCoord.get());
 			break;
 
 		case TEXCALC_ENVIRONMENT_MAP:
 		case TEXCALC_ENVIRONMENT_MAP_PLANAR:		
 		case TEXCALC_ENVIRONMENT_MAP_NORMAL:
 			// Resolve vertex normal.
+			mVSInputPos = vsMain->resolveInputParameter(Parameter::SPS_POSITION, 0, Parameter::SPC_POSITION_OBJECT_SPACE, GCT_FLOAT4);
 			mVSInputNormal = vsMain->resolveInputParameter(Parameter::SPS_NORMAL, 0, Parameter::SPC_NORMAL_OBJECT_SPACE, GCT_FLOAT3);
-			if (mVSInputNormal.get() == NULL)			
-				return false;									
+			hasError |= !(mVSInputNormal.get()) || !(mVSInputPos.get());
 			break;	
 
 		case TEXCALC_ENVIRONMENT_MAP_REFLECTION:
 
 			// Resolve vertex normal.
 			mVSInputNormal = vsMain->resolveInputParameter(Parameter::SPS_NORMAL, 0, Parameter::SPC_NORMAL_OBJECT_SPACE, GCT_FLOAT3);
-			if (mVSInputNormal.get() == NULL)			
-				return false;		
-
 			// Resolve vertex position.
 			mVSInputPos = vsMain->resolveInputParameter(Parameter::SPS_POSITION, 0, Parameter::SPC_POSITION_OBJECT_SPACE, GCT_FLOAT4);
-			if (mVSInputPos.get() == NULL)			
-				return false;		
+			
+			hasError |= !(mVSInputNormal.get()) || !(mVSInputPos.get());
 			break;
 
 		case TEXCALC_PROJECTIVE_TEXTURE:
 			// Resolve vertex position.
 			mVSInputPos = vsMain->resolveInputParameter(Parameter::SPS_POSITION, 0, Parameter::SPC_POSITION_OBJECT_SPACE, GCT_FLOAT4);
-			if (mVSInputPos.get() == NULL)			
-				return false;		
+			hasError |= !(mVSInputPos.get());
 			break;
 	}
 
@@ -234,19 +216,12 @@ bool FFPTexturing::resolveFunctionsParams(TextureUnitParams* textureUnitParams,
 		texCoordContent,
 		textureUnitParams->mVSOutTextureCoordinateType);
 
-	if (textureUnitParams->mVSOutputTexCoord.get() == NULL)
-		return false;
-		
-
 	// Resolve ps input texture coordinates.
 	textureUnitParams->mPSInputTexCoord = psMain->resolveInputParameter(Parameter::SPS_TEXTURE_COORDINATES, 
 		textureUnitParams->mVSOutputTexCoord->getIndex(),
 		textureUnitParams->mVSOutputTexCoord->getContent(),
 		textureUnitParams->mVSOutTextureCoordinateType);
 
-	if (textureUnitParams->mPSInputTexCoord.get() == NULL)
-		return false;
-
 	const ShaderParameterList& inputParams = psMain->getInputParameters();
 	const ShaderParameterList& localParams = psMain->getLocalParameters();
 
@@ -254,23 +229,25 @@ bool FFPTexturing::resolveFunctionsParams(TextureUnitParams* textureUnitParams,
 	if (mPSDiffuse.get() == NULL)
 	{
 		mPSDiffuse = psMain->getParameterByContent(localParams, Parameter::SPC_COLOR_DIFFUSE, GCT_FLOAT4);
-		if (mPSDiffuse.get() == NULL)
-			return false;
 	}
 
 	mPSSpecular = psMain->getParameterByContent(inputParams, Parameter::SPC_COLOR_SPECULAR, GCT_FLOAT4);
 	if (mPSSpecular.get() == NULL)
 	{
 		mPSSpecular = psMain->getParameterByContent(localParams, Parameter::SPC_COLOR_SPECULAR, GCT_FLOAT4);
-		if (mPSSpecular.get() == NULL)
-			return false;
 	}
 
 	mPSOutDiffuse = psMain->resolveOutputParameter(Parameter::SPS_COLOR, 0, Parameter::SPC_COLOR_DIFFUSE, GCT_FLOAT4);
-	if (mPSOutDiffuse.get() == NULL)	
-		return false;
-
 	
+	hasError |= !(textureUnitParams->mVSOutputTexCoord.get()) || !(textureUnitParams->mPSInputTexCoord.get()) || 
+		!(mPSDiffuse.get()) || !(mPSSpecular.get()) || !(mPSOutDiffuse.get());
+	
+	if (hasError)
+	{
+		OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, 
+				"Not all parameters could be constructed for the sub-render state.",
+				"FFPTexturing::resolveFunctionsParams" );
+	}
 	return true;
 }
 
@@ -343,8 +320,11 @@ bool FFPTexturing::addVSFunctionInvocations(TextureUnitParams* textureUnitParams
 		{
 			texCoordCalcFunc = OGRE_NEW FunctionInvocation(FFP_FUNC_GENERATE_TEXCOORD_ENV_SPHERE,  FFP_VS_TEXTURING, textureUnitParams->mTextureSamplerIndex); 
 
-			texCoordCalcFunc->pushOperand(mWorldITMatrix, Operand::OPS_IN);
+			//TODO: Add field member mWorldViewITMatrix 
+			texCoordCalcFunc->pushOperand(mWorldMatrix, Operand::OPS_IN);	
 			texCoordCalcFunc->pushOperand(mViewMatrix, Operand::OPS_IN);	
+			texCoordCalcFunc->pushOperand(mWorldITMatrix, Operand::OPS_IN);
+			texCoordCalcFunc->pushOperand(mVSInputPos, Operand::OPS_IN);
 			texCoordCalcFunc->pushOperand(mVSInputNormal, Operand::OPS_IN);	
 			texCoordCalcFunc->pushOperand(textureUnitParams->mVSOutputTexCoord, Operand::OPS_OUT);
 		}
@@ -352,10 +332,12 @@ bool FFPTexturing::addVSFunctionInvocations(TextureUnitParams* textureUnitParams
 		{
 			texCoordCalcFunc = OGRE_NEW FunctionInvocation(FFP_FUNC_GENERATE_TEXCOORD_ENV_SPHERE,  FFP_VS_TEXTURING, textureUnitParams->mTextureSamplerIndex); 
 
-			texCoordCalcFunc->pushOperand(mWorldITMatrix, Operand::OPS_IN);
+			texCoordCalcFunc->pushOperand(mWorldMatrix, Operand::OPS_IN);	
 			texCoordCalcFunc->pushOperand(mViewMatrix, Operand::OPS_IN);	
-			texCoordCalcFunc->pushOperand(textureUnitParams->mTextureMatrix, Operand::OPS_IN);
+			texCoordCalcFunc->pushOperand(mWorldITMatrix, Operand::OPS_IN);
+			texCoordCalcFunc->pushOperand(mVSInputPos, Operand::OPS_IN);
 			texCoordCalcFunc->pushOperand(mVSInputNormal, Operand::OPS_IN);	
+			texCoordCalcFunc->pushOperand(textureUnitParams->mTextureMatrix, Operand::OPS_IN);
 			texCoordCalcFunc->pushOperand(textureUnitParams->mVSOutputTexCoord, Operand::OPS_OUT);
 		}			
 		break;
@@ -470,7 +452,7 @@ bool FFPTexturing::addPSFunctionInvocations(TextureUnitParams* textureUnitParams
 	addPSBlendInvocations(psMain, source1, source2, texel, 
 		textureUnitParams->mTextureSamplerIndex,
 		colourBlend, groupOrder, internalCounter, 
-		needDifferentAlphaBlend ? (Operand::OPM_X | Operand::OPM_Y | Operand::OPM_Z )  : Operand::OPM_ALL);
+		needDifferentAlphaBlend ? Operand::OPM_XYZ : Operand::OPM_ALL);
 
 	// Case we need different alpha channel code.
 	if (needDifferentAlphaBlend)
@@ -576,7 +558,7 @@ void FFPTexturing::addPSArgumentInvocations(Function* psMain,
 			curFuncInvocation->pushOperand(ParameterFactory::createConstParamFloat(colourValue.a), Operand::OPS_IN);		
 		}
 		
-		curFuncInvocation->pushOperand(arg, Operand::OPS_IN);	
+		curFuncInvocation->pushOperand(arg, Operand::OPS_OUT);
 		psMain->addAtomInstance(curFuncInvocation);	
 		break;
 	}
@@ -904,6 +886,10 @@ void FFPTexturing::setTextureUnit(unsigned short index, TextureUnitState* textur
 		curParams.mTextureSamplerType = GCT_SAMPLER2D;
 		curParams.mVSInTextureCoordinateType = GCT_FLOAT2;
 		break;
+    case TEX_TYPE_2D_RECT:
+        curParams.mTextureSamplerType = GCT_SAMPLERRECT;
+        curParams.mVSInTextureCoordinateType = GCT_FLOAT2;
+        break;
     case TEX_TYPE_2D_ARRAY:
         curParams.mTextureSamplerType = GCT_SAMPLER2DARRAY;
         curParams.mVSInTextureCoordinateType = GCT_FLOAT3;
