@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
@@ -34,12 +34,7 @@ THE SOFTWARE.
 
 #include "OgreShaderFunctionAtom.h"
 #include "OgreResourceGroupManager.h"
-
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-#	define ENDL "\n"
-#else
-#	define ENDL std::endl
-#endif
+#include "OgreRoot.h"
 
 namespace Ogre {
     namespace RTShader {
@@ -58,7 +53,7 @@ namespace Ogre {
         //-----------------------------------------------------------------------
         GLSLESProgramWriter::GLSLESProgramWriter()
         {
-            mGLSLVersion = 100;
+            mGLSLVersion = Root::getSingleton().getRenderSystem()->getNativeShadingLanguageVersion();
             initializeStringMaps();
             mFunctionCacheMap.clear();
         }
@@ -110,6 +105,10 @@ namespace Ogre {
                 // 3. The name
                 if(paramTokens.size() == 3)
                 {
+					StringUtil::trim(paramTokens[0]);
+					StringUtil::trim(paramTokens[1]);
+					StringUtil::trim(paramTokens[2]);
+
                     Operand::OpSemantic semantic = Operand::OPS_IN;
                     GpuConstantType gpuType = GCT_UNKNOWN;
 
@@ -186,18 +185,26 @@ namespace Ogre {
                 {
                     StringVector moreTokens = StringUtil::split(*it, " ");
 
+					if (!moreTokens.empty())
+					{
                     FunctionMap::const_iterator itFuncCache = mFunctionCacheMap.begin();
-                    for (; itFuncCache != mFunctionCacheMap.end(); ++itFuncCache)
-                    {
-                        if(itFuncCache->first.getFunctionName() == moreTokens.back())
-                        {
-                            // Add the function declaration
-                            depVector.push_back(FunctionInvocation((*itFuncCache).first));
 
-                            discoverFunctionDependencies(itFuncCache->first, depVector);
-                        }
-                    }
-                }
+						for (; itFuncCache != mFunctionCacheMap.end(); ++itFuncCache)
+						{
+
+							FunctionInvocation fi = itFuncCache->first;
+						
+							if(fi.getFunctionName() == moreTokens.back())
+							{
+								// Add the function declaration
+								depVector.push_back(FunctionInvocation((*itFuncCache).first));
+
+								discoverFunctionDependencies(itFuncCache->first, depVector);
+							}
+						}
+					}
+				}
+                
             }
             else
             {
@@ -215,9 +222,19 @@ namespace Ogre {
             mGpuConstTypeMap[GCT_FLOAT4] = "vec4";
             mGpuConstTypeMap[GCT_SAMPLER1D] = "sampler2D";
             mGpuConstTypeMap[GCT_SAMPLER2D] = "sampler2D";
+            mGpuConstTypeMap[GCT_SAMPLER3D] = "sampler3D";
+        	mGpuConstTypeMap[GCT_SAMPLER2DARRAY] = "sampler2DArray";
             mGpuConstTypeMap[GCT_SAMPLERCUBE] = "samplerCube";
+            mGpuConstTypeMap[GCT_SAMPLER1DSHADOW] = "sampler1DShadow";
+            mGpuConstTypeMap[GCT_SAMPLER2DSHADOW] = "sampler2DShadow";
             mGpuConstTypeMap[GCT_MATRIX_2X2] = "mat2";
+            mGpuConstTypeMap[GCT_MATRIX_2X3] = "mat2x3";
+            mGpuConstTypeMap[GCT_MATRIX_2X4] = "mat2x4";
+            mGpuConstTypeMap[GCT_MATRIX_3X2] = "mat3x2";
             mGpuConstTypeMap[GCT_MATRIX_3X3] = "mat3";
+            mGpuConstTypeMap[GCT_MATRIX_3X4] = "mat3x4";
+            mGpuConstTypeMap[GCT_MATRIX_4X2] = "mat4x2";
+            mGpuConstTypeMap[GCT_MATRIX_4X3] = "mat4x3";
             mGpuConstTypeMap[GCT_MATRIX_4X4] = "mat4";
             mGpuConstTypeMap[GCT_INT1] = "int";
             mGpuConstTypeMap[GCT_INT2] = "int2";
@@ -244,11 +261,7 @@ namespace Ogre {
 
         //-----------------------------------------------------------------------
         void GLSLESProgramWriter::writeSourceCode(
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-			StringSerialiser& os,
-#else
 			std::ostream& os, 
-#endif
 			Program* program)
         {
             GpuProgramType gpuType = program->getType();
@@ -269,23 +282,37 @@ namespace Ogre {
             UniformParameterConstIterator itUniformParam = parameterList.begin();
             
             // Write the current version (this forces the driver to fulfill the glsl es standard)
-            os << "#version "<< mGLSLVersion << ENDL;
+            os << "#version "<< mGLSLVersion;
+
+            // Starting with ES 3.0 the version must contain the string "es" after the version number with a space separating them
+            if(mGLSLVersion > 100)
+                os << " es";
+
+            os << std::endl;
 
             // Default precision declaration is required in fragment and vertex shaders.
-            os << "precision highp float;" << ENDL;
-            os << "precision highp int;" << ENDL;
+            os << "precision highp float;" << std::endl;
+            os << "precision highp int;" << std::endl;
+
+            // Redefine texture functions to maintain reusability
+            if(mGLSLVersion > 100)
+            {
+                os << "#define texture2D texture" << std::endl;
+                os << "#define texture3D texture" << std::endl;
+                os << "#define textureCube texture" << std::endl;
+            }
 
             // Generate source code header.
             writeProgramTitle(os, program);
-            os<< ENDL;
+            os<< std::endl;
 
             // Embed dependencies.
             writeProgramDependencies(os, program);
-            os << ENDL;
+            os << std::endl;
 
             // Generate global variable code.
             writeUniformParametersTitle(os, program);
-            os << ENDL;
+            os << std::endl;
 
             // Write the uniforms 
             for (itUniformParam = parameterList.begin();  itUniformParam != parameterList.end(); ++itUniformParam)
@@ -300,9 +327,9 @@ namespace Ogre {
 				{
 					os << "[" << pUniformParam->getSize() << "]";	
 				}
-                os << ";" << ENDL;
+                os << ";" << std::endl;
             }
-            os << ENDL;			
+            os << std::endl;			
 
             // Write program function(s).
             for (itFunction = functionList.begin(); itFunction != functionList.end(); ++itFunction)
@@ -319,16 +346,16 @@ namespace Ogre {
                 writeOutParameters(os, curFunction, gpuType);
                             
                 // The function name must always main.
-                os << "void main() {" << ENDL;
+                os << "void main() {" << std::endl;
 
-                if (gpuType == GPT_FRAGMENT_PROGRAM)
-                {
-                    os << "\tvec4 outputColor;" << ENDL;
-                }
-                else if (gpuType == GPT_VERTEX_PROGRAM)
-                {
-                    os << "\tvec4 outputPosition;" << ENDL;
-                }
+				if (gpuType == GPT_FRAGMENT_PROGRAM)
+				{
+					os << "\tvec4 outputColor;" << std::endl;
+				}
+				else if (gpuType == GPT_VERTEX_PROGRAM)
+				{
+					os << "\tvec4 outputPosition;" << std::endl;
+				}
 
                 // Write local parameters.
                 const ShaderParameterList& localParams = curFunction->getLocalParameters();
@@ -339,9 +366,9 @@ namespace Ogre {
                 {
                     os << "\t";
                     writeLocalParameter(os, *itParam);			
-                    os << ";" << ENDL;						
+                    os << ";" << std::endl;						
                 }
-                os << ENDL;			
+                os << std::endl;			
 
                 // Sort function atoms.
                 curFunction->sortAtomInstances();
@@ -360,7 +387,7 @@ namespace Ogre {
                     itOperand = pFuncInvoc->getOperandList().begin();
 
                     // Local string stream
-                    std::stringstream localOs;
+                    StringStream localOs;
 
                     // Write function name			
                     localOs << "\t" << pFuncInvoc->getFunctionName() << "(";
@@ -395,7 +422,7 @@ namespace Ogre {
                                     tempVar.replace(tempVar.begin(), tempVar.begin() + 1, "o");
 
                                     // Declare the copy variable and assign the original
-                                    os << "\t" << mGpuConstTypeMap[op.getParameter()->getType()] << " " << newVar << " = " << tempVar << ";\n" << ENDL;	
+                                    os << "\t" << mGpuConstTypeMap[op.getParameter()->getType()] << " " << newVar << " = " << tempVar << ";\n" << std::endl;	
 
                                     // From now on we replace it automatic 
                                     mInputToGLStatesMap[paramName] = newVar;
@@ -419,7 +446,7 @@ namespace Ogre {
                                     if(mInputToGLStatesMap.find(newVar) == mInputToGLStatesMap.end())
                                     {
                                         // Declare the copy variable and assign the original
-                                        os << "\t" << mGpuConstTypeMap[itFound->get()->getType()] << " " << newVar << " = " << paramName << ";\n" << ENDL;	
+                                        os << "\t" << mGpuConstTypeMap[itFound->get()->getType()] << " " << newVar << " = " << paramName << ";\n" << std::endl;	
 
                                         // From now on we replace it automatic 
                                         mInputToGLStatesMap[paramName] = newVar;
@@ -528,32 +555,37 @@ namespace Ogre {
                     }
 
                     // Write function call closer.
-                    localOs << ");" << ENDL;
-                    localOs << ENDL;
+                    localOs << ");" << std::endl;
+                    localOs << std::endl;
                     os << localOs.str();
                 }
-
-                if (gpuType == GPT_FRAGMENT_PROGRAM)
-                {
-                    os << "\tgl_FragColor = outputColor;" << ENDL;
-                }
-                else if (gpuType == GPT_VERTEX_PROGRAM)
-                {
-                    os << "\tgl_Position = outputPosition;" << ENDL;
-                }
-
-                os << "}" << ENDL;
+				
+				if (gpuType == GPT_FRAGMENT_PROGRAM)
+				{
+                    // GLSL fragment program has to write always gl_FragColor (but this is also deprecated after version 130)
+                    // Always add gl_FragColor as an output.  The name is for compatibility.
+                    if(mGLSLVersion > 100)
+                    {
+                        os << "\tfragColour = outputColor;" << std::endl;
+                    }
+                    else
+                    {
+                        os << "\tgl_FragColor = outputColor;" << std::endl;
+                    }
+				}
+				else if (gpuType == GPT_VERTEX_PROGRAM)
+				{
+					os << "\tgl_Position = outputPosition;" << std::endl;
+				}
+				
+                os << "}" << std::endl;
             }
-            os << ENDL;
+            os << std::endl;
         }
 
         //-----------------------------------------------------------------------
         void GLSLESProgramWriter::writeInputParameters(
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-			StringSerialiser& os,
-#else
 			std::ostream& os, 
-#endif
 			Function* function, 
 			GpuProgramType gpuType)
         {
@@ -580,11 +612,15 @@ namespace Ogre {
                     paramName.replace(paramName.begin(), paramName.begin() + 1, "o");	
                     mInputToGLStatesMap[pParam->getName()] = paramName;
 
-                    os << "varying\t";
+                    if(mGLSLVersion > 100)
+                        os << "in\t";
+                    else
+                        os << "varying\t";
+
                     os << mGpuConstTypeMap[pParam->getType()];
                     os << "\t";	
                     os << paramName;
-                    os << ";" << ENDL;	
+                    os << ";" << std::endl;	
                 }
                 else if (gpuType == GPT_VERTEX_PROGRAM && 
                          mContentToPerVertexAttributes.find(paramContent) != mContentToPerVertexAttributes.end())
@@ -592,7 +628,10 @@ namespace Ogre {
                     // Due the fact that glsl does not have register like cg we have to rename the params
                     // according their content.
                     mInputToGLStatesMap[paramName] = mContentToPerVertexAttributes[paramContent];
-                    os << "attribute\t"; 
+                    if(mGLSLVersion > 100)
+                        os << "in\t";
+                    else
+                        os << "attribute\t";
 
                     // All uv texcoords passed by OGRE are vec4
                     if (paramContent == Parameter::SPC_TEXTURE_COORDINATE0 ||
@@ -612,7 +651,7 @@ namespace Ogre {
                     }
                     os << "\t";	
                     os << mContentToPerVertexAttributes[paramContent];
-                    os << ";" << ENDL;	
+                    os << ";" << std::endl;	
                 }
                 else
                 {
@@ -620,18 +659,14 @@ namespace Ogre {
                     os << mGpuConstTypeMap[pParam->getType()];
                     os << "\t";	
                     os << paramName;
-                    os << ";" << ENDL;	
+                    os << ";" << std::endl;	
                 }							
             }
         }
 
         //-----------------------------------------------------------------------
         void GLSLESProgramWriter::writeOutParameters(
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-			StringSerialiser& os,
-#else
-			std::ostream& os, 
-#endif
+			std::ostream& os,
 			Function* function, 
 			GpuProgramType gpuType)
         {
@@ -653,7 +688,10 @@ namespace Ogre {
                     }
                     else
                     {
-                        os << "varying\t";
+                        if(mGLSLVersion > 100)
+                            os << "out\t";
+                        else
+                            os << "varying\t";
                         os << mGpuConstTypeMap[pParam->getType()];
                         os << "\t";
                         os << pParam->getName();
@@ -661,12 +699,15 @@ namespace Ogre {
 						{
 							os << "[" << pParam->getSize() << "]";	
 						}
-						os << ";" << ENDL;	
+						os << ";" << std::endl;	
                     }
                 }
                 else if(gpuType == GPT_FRAGMENT_PROGRAM &&
                         pParam->getSemantic() == Parameter::SPS_COLOR)
-                {					
+                {
+                    if(mGLSLVersion > 100)
+                        os << "out vec4 fragColour;" << std::endl;
+
                     // GLSL ES fragment program has to always write gl_FragColor
                     mInputToGLStatesMap[pParam->getName()] = "outputColor";
                 }
@@ -684,11 +725,7 @@ namespace Ogre {
         // 5. Go back to step 1 until we have found all the functions
         //
         void GLSLESProgramWriter::writeProgramDependencies(
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-			StringSerialiser& os,
-#else
 			std::ostream& os, 
-#endif 
 			Program* program)
         {
             for(unsigned int i = 0; i < program->getDependencyCount(); ++i)
@@ -698,9 +735,9 @@ namespace Ogre {
             }
 
 
-            os << "//-----------------------------------------------------------------------------" << ENDL;
-            os << "//                         PROGRAM DEPENDENCIES"                                 << ENDL;
-            os << "//-----------------------------------------------------------------------------" << ENDL;
+            os << "//-----------------------------------------------------------------------------" << std::endl;
+            os << "//                         PROGRAM DEPENDENCIES"                                 << std::endl;
+            os << "//-----------------------------------------------------------------------------" << std::endl;
 
             FunctionVector forwardDecl; // Holds all function declarations
             const ShaderFunctionList& functionList = program->getFunctions();
@@ -834,7 +871,7 @@ namespace Ogre {
                             os << ", ";
                         }
                     }
-                    os << ENDL << "{" << ENDL << body << ENDL << "}" << ENDL;
+                    os << std::endl << "{" << std::endl << body << std::endl << "}" << std::endl;
                 }
             }
         }
@@ -862,13 +899,7 @@ namespace Ogre {
         }
 
         //-----------------------------------------------------------------------
-        void GLSLESProgramWriter::writeLocalParameter(
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-			StringSerialiser& os,
-#else
-			std::ostream& os, 
-#endif
-			ParameterPtr parameter)
+        void GLSLESProgramWriter::writeLocalParameter(std::ostream& os, ParameterPtr parameter)
         {
             os << mGpuConstTypeMap[parameter->getType()];
             os << "\t";	
@@ -949,7 +980,7 @@ namespace Ogre {
                         // First, look for a return type
                         if(isBasicType(tokens[0]) && ((tokens.size() < 3) || (tokens[2] != "=")) )
                         {
-                            String functionSig = "";
+                            String functionSig;
                             String functionBody = "";
                             FunctionInvocation *functionInvoc = NULL;
 
