@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,10 @@ namespace Ogre {
     void * mac_getBundleSym(CFBundleRef bundle, const char *name);
     bool mac_unloadExeBundle(CFBundleRef bundle);
     void* mac_loadDylib(const char* name);
-    _OgreExport std::string macPluginPath();
-    _OgreExport std::string macBundlePath();
+    void* mac_loadFramework(String name);
+    _OgreExport String macPluginPath();
+    _OgreExport String macBundlePath();
+    _OgreExport String macFrameworksPath();
+    _OgreExport String macCachePath();
+    _OgreExport String macTempFileName(void);
 }
