@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -28,19 +28,37 @@ THE SOFTWARE.
 #ifndef __Ogre_Iterator_Range_H__
 #define __Ogre_Iterator_Range_H__
 
+#include "OgreHeaderPrefix.h"
 
-//#define OGRE_USE_BOOST 1 - picked up by CMake
 #if OGRE_USE_BOOST
-#include <boost/range.hpp>
+#   if OGRE_COMPILER == OGRE_COMPILER_CLANG || OGRE_COMPILER == OGRE_COMPILER_GNUC
+#       pragma GCC diagnostic push
+#if OGRE_COMPILER == OGRE_COMPILER_GNUC
+#       pragma GCC diagnostic ignored "-Wpragmas"
+#elif OGRE_COMPILER == OGRE_COMPILER_CLANG
+#       pragma GCC diagnostic ignored "-Wdocumentation"
+#endif
+#       pragma GCC diagnostic ignored "-Wshadow"
+#       pragma GCC diagnostic ignored "-Wpadded"
+#       pragma GCC diagnostic ignored "-Wweak-vtables"
+#       pragma GCC diagnostic ignored "-Wall"
+#       pragma GCC diagnostic ignored "-Wundef"
+#   endif
+
+#   include <boost/range.hpp>
+
+#   if OGRE_COMPILER == OGRE_COMPILER_CLANG || OGRE_COMPILER == OGRE_COMPILER_GNUC
+#       pragma GCC diagnostic pop
+#   endif
 #endif
 
 namespace Ogre {
 
 /** 
  * 
- * \brief Base for an iterator_range
+ * @brief Base for an iterator_range
  * 
- * \param T iterator type   
+ * @tparam T iterator type
  * 
  * This class implements the minimal interface of the (boost::iterator_)range concept
  *\n Also it prepairs for direct usage of boost::iterator_range by providing the real used type via iterator_range::type
@@ -83,17 +101,17 @@ class iterator_range{
 		bool operator!=( const iterator_range& rhs ) const { return !operator==(rhs); }
 
 		/**
-		\brief typedef to fulfill container interface
+		@brief typedef to fulfill container interface
 		
-		\note there is no distinction between const_iterator and iterator.
+		@note there is no distinction between const_iterator and iterator.
 		
 		*/				
 		typedef T iterator;
 		
 		/**
-		\brief typedef to fulfill container interface
+		@brief typedef to fulfill container interface
 		
-		\note there is no distinction between const_iterator and iterator.
+		@note there is no distinction between const_iterator and iterator.
 		
 		*/						
 		typedef T const_iterator;
@@ -114,9 +132,9 @@ class iterator_range{
 
 /** 
  * 
- * \brief Predefined type
+ * @brief Predefined type
  * 
- * \param T iterator type   
+ * @tparam T iterator type
  *
  * compatility class for VectorIterator
 */
@@ -157,9 +175,9 @@ struct VectorRange : public iterator_range<typename T::iterator>::type
 
 /** 
  * 
- * \brief Predefined type
+ * @brief Predefined type
  * 
- * \param T iterator type   
+ * @tparam T iterator type
  *
  * compatility class for ConstVectorIterator
 */
@@ -218,9 +236,9 @@ struct ConstVectorRange : public iterator_range<typename T::const_iterator>::typ
 
 /** 
  * 
- * \brief Predefined type
+ * @brief Predefined type
  * 
- * \param T iterator type   
+ * @tparam T iterator type
  *
  * compatility class for MapIterator
 */
@@ -260,9 +278,9 @@ struct MapRange : public iterator_range<typename T::iterator>::type
 
 /** 
  * 
- * \brief Predefined type
+ * @brief Predefined type
  * 
- * \param T iterator type   
+ * @tparam T iterator type   
  *
  * compatility class for ConstMapIterator
 */
@@ -315,7 +333,7 @@ struct ConstMapRange : public iterator_range<typename T::const_iterator>::type
 	
 };
 
-
+#include "OgreHeaderSuffix.h"
 
 }
 #endif
