@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -40,13 +40,11 @@ THE SOFTWARE.
 #include "OgrePlatform.h"
 
 
-#if OGRE_PLATFORM != OGRE_PLATFORM_WIN32
+#if OGRE_PLATFORM != OGRE_PLATFORM_WIN32 && OGRE_PLATFORM != OGRE_PLATFORM_WINRT
 
 #include <dirent.h>
 #include <unistd.h>
-#if OGRE_PLATFORM != OGRE_PLATFORM_SYMBIAN
 #include <fnmatch.h>
-#endif
 
 /* Our simplified data entry structure */
 struct _finddata_t
@@ -56,7 +54,6 @@ struct _finddata_t
     unsigned long size;
 };
 
-#if OGRE_PLATFORM != OGRE_PLATFORM_SYMBIAN
 #define _A_NORMAL 0x00  /* Normalfile-Noread/writerestrictions */
 #define _A_RDONLY 0x01  /* Read only file */
 #define _A_HIDDEN 0x02  /* Hidden file */
@@ -70,5 +67,3 @@ int _findnext(intptr_t id, struct _finddata_t *data);
 int _findclose(intptr_t id);
 
 #endif
-
-#endif
