@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -89,7 +89,7 @@ namespace Ogre {
             // Global keyframe index available, map to local keyframe index directly.
             assert(timeIndex.getKeyIndex() < mKeyFrameIndexMap.size());
             i = mKeyFrames.begin() + mKeyFrameIndexMap[timeIndex.getKeyIndex()];
-#ifdef _DEBUG
+#if OGRE_DEBUG_MODE
             KeyFrame timeKey(0, timePos);
             if (i != std::lower_bound(mKeyFrames.begin(), mKeyFrames.end(), &timeKey, KeyFrameTimeLess()))
             {
@@ -105,10 +105,8 @@ namespace Ogre {
             Real totalAnimationLength = mParent->getLength();
             assert(totalAnimationLength > 0.0f && "Invalid animation length!");
 
-            while (timePos > totalAnimationLength && totalAnimationLength > 0.0f)
-            {
-                timePos -= totalAnimationLength;
-            }
+            if( timePos > totalAnimationLength && totalAnimationLength > 0.0f )
+				timePos = fmod( timePos, totalAnimationLength );
 
             // No global keyframe index, need to search with local keyframes.
             KeyFrame timeKey(0, timePos);
@@ -678,18 +676,35 @@ namespace Ogre {
 		newTrack->mUseShortestRotationPath = mUseShortestRotationPath;
 		populateClone(newTrack);
 		return newTrack;
-	}	
+	}
+	//--------------------------------------------------------------------------
+	void NodeAnimationTrack::_applyBaseKeyFrame(const KeyFrame* b)
+	{
+		const TransformKeyFrame* base = static_cast<const TransformKeyFrame*>(b);
+		
+        for (KeyFrameList::iterator i = mKeyFrames.begin(); i != mKeyFrames.end(); ++i)
+        {
+			TransformKeyFrame* kf = static_cast<TransformKeyFrame*>(*i);
+			kf->setTranslate(kf->getTranslate() - base->getTranslate());
+			kf->setRotation(base->getRotation().Inverse() * kf->getRotation());
+			kf->setScale(kf->getScale() * (Vector3::UNIT_SCALE / base->getScale()));
+		}
+			
+	}
 	//--------------------------------------------------------------------------
 	VertexAnimationTrack::VertexAnimationTrack(Animation* parent,
 		unsigned short handle, VertexAnimationType animType)
-		: AnimationTrack(parent, handle), mAnimationType(animType)
+		: AnimationTrack(parent, handle)
+		, mAnimationType(animType)
 	{
 	}
 	//--------------------------------------------------------------------------
 	VertexAnimationTrack::VertexAnimationTrack(Animation* parent, unsigned short handle,
 		VertexAnimationType animType, VertexData* targetData, TargetMode target)
-		: AnimationTrack(parent, handle), mAnimationType(animType),
-		mTargetVertexData(targetData), mTargetMode(target)
+		: AnimationTrack(parent, handle)
+		, mAnimationType(animType)
+		, mTargetVertexData(targetData)
+		, mTargetMode(target)
 	{
 	}
 	//--------------------------------------------------------------------------
@@ -713,6 +728,104 @@ namespace Ogre {
 				"VertexAnimationTrack::createVertexPoseKeyFrame");
 		}
 		return static_cast<VertexPoseKeyFrame*>(createKeyFrame(timePos));
+	}
+	//--------------------------------------------------------------------------
+	void VertexAnimationTrack::getInterpolatedKeyFrame(const TimeIndex& timeIndex, KeyFrame* kf) const
+	{
+		// Only relevant for pose animation
+		if (mAnimationType == VAT_POSE)
+		{
+			// Get keyframes
+			KeyFrame *kf1, *kf2;
+			Real t = getKeyFramesAtTime(timeIndex, &kf1, &kf2);
+			
+			VertexPoseKeyFrame* vkfOut = static_cast<VertexPoseKeyFrame*>(kf);
+			VertexPoseKeyFrame* vkf1 = static_cast<VertexPoseKeyFrame*>(kf1);
+			VertexPoseKeyFrame* vkf2 = static_cast<VertexPoseKeyFrame*>(kf2);
+			
+			// For each pose reference in key 1, we need to locate the entry in
+			// key 2 and interpolate the influence
+			const VertexPoseKeyFrame::PoseRefList& poseList1 = vkf1->getPoseReferences();
+			const VertexPoseKeyFrame::PoseRefList& poseList2 = vkf2->getPoseReferences();
+			for (VertexPoseKeyFrame::PoseRefList::const_iterator p1 = poseList1.begin();
+				 p1 != poseList1.end(); ++p1)
+			{
+				Real startInfluence = p1->influence;
+				Real endInfluence = 0;
+				// Search for entry in keyframe 2 list (if not there, will be 0)
+				for (VertexPoseKeyFrame::PoseRefList::const_iterator p2 = poseList2.begin();
+					 p2 != poseList2.end(); ++p2)
+				{
+					if (p1->poseIndex == p2->poseIndex)
+					{
+						endInfluence = p2->influence;
+						break;
+					}
+				}
+				// Interpolate influence
+				Real influence = startInfluence + t*(endInfluence - startInfluence);
+				
+				vkfOut->addPoseReference(p1->poseIndex, influence);
+				
+				
+			}
+			// Now deal with any poses in key 2 which are not in key 1
+			for (VertexPoseKeyFrame::PoseRefList::const_iterator p2 = poseList2.begin();
+				 p2 != poseList2.end(); ++p2)
+			{
+				bool found = false;
+				for (VertexPoseKeyFrame::PoseRefList::const_iterator p1 = poseList1.begin();
+					 p1 != poseList1.end(); ++p1)
+				{
+					if (p1->poseIndex == p2->poseIndex)
+					{
+						found = true;
+						break;
+					}
+				}
+				if (!found)
+				{
+					// Need to apply this pose too, scaled from 0 start
+					Real influence = t * p2->influence;
+					
+					vkfOut->addPoseReference(p2->poseIndex, influence);
+
+				}
+			} // key 2 iteration
+			
+		}			
+		
+		
+	}
+	//--------------------------------------------------------------------------
+	bool VertexAnimationTrack::getVertexAnimationIncludesNormals() const
+	{
+		if (mAnimationType == VAT_NONE)
+			return false;
+		
+		if (mAnimationType == VAT_MORPH)
+		{
+			bool normals = false;
+			for (KeyFrameList::const_iterator i = mKeyFrames.begin(); i != mKeyFrames.end(); ++i)
+			{
+				VertexMorphKeyFrame* kf = static_cast<VertexMorphKeyFrame*>(*i);
+				bool thisnorm = kf->getVertexBuffer()->getVertexSize() > 12;
+				if (i == mKeyFrames.begin())
+					normals = thisnorm;
+				else
+					// Only support normals if ALL keyframes include them
+					normals = normals && thisnorm;
+
+			}
+			return normals;
+		}
+		else 
+		{
+			// needs to derive from Mesh::PoseList, can't tell here
+			return false;
+		}
+
+				
 	}
 	//--------------------------------------------------------------------------
 	void VertexAnimationTrack::apply(const TimeIndex& timeIndex, Real weight, Real scale)
@@ -744,7 +857,7 @@ namespace Ogre {
 					"Haven't set up hardware vertex animation elements!");
 
 				// no use for TempBlendedBufferInfo here btw
-				// NB we assume that position buffer is unshared
+				// NB we assume that position buffer is unshared, except for normals
 				// VertexDeclaration::getAutoOrganisedDeclaration should see to that
 				const VertexElement* posElem =
 					data->vertexDeclaration->findElementBySemantic(VES_POSITION);
@@ -753,7 +866,7 @@ namespace Ogre {
 					posElem->getSource(), vkf1->getVertexBuffer());
 				// Set keyframe2 data as derived
 				data->vertexBufferBinding->setBinding(
-					data->hwAnimationDataList[0].targetVertexElement->getSource(),
+					data->hwAnimationDataList[0].targetBufferIndex,
 					vkf2->getVertexBuffer());
 				// save T for use later
 				data->hwAnimationDataList[0].parametric = t;
@@ -798,7 +911,7 @@ namespace Ogre {
 				// Scale by animation weight
 				influence = weight * influence;
 				// Get pose
-				assert (p1->poseIndex <= poseList->size());
+				assert (poseList && p1->poseIndex < poseList->size());
 				Pose* pose = (*poseList)[p1->poseIndex];
 				// apply
 				applyPoseToVertexData(pose, data, influence);
@@ -824,7 +937,7 @@ namespace Ogre {
 					// Scale by animation weight
 					influence = weight * influence;
 					// Get pose
-					assert (p2->poseIndex <= poseList->size());
+					assert (poseList && p2->poseIndex <= poseList->size());
 					const Pose* pose = (*poseList)[p2->poseIndex];
 					// apply
 					applyPoseToVertexData(pose, data, influence);
@@ -851,8 +964,8 @@ namespace Ogre {
 			{
 				VertexData::HardwareAnimationData& animData = data->hwAnimationDataList[hwIndex];
 				data->vertexBufferBinding->setBinding(
-					animData.targetVertexElement->getSource(),
-					pose->_getHardwareVertexBuffer(data->vertexCount));
+					animData.targetBufferIndex,
+					pose->_getHardwareVertexBuffer(data));
 				// save final influence in parametric
 				animData.parametric = influence;
 
@@ -862,7 +975,7 @@ namespace Ogre {
 		else
 		{
 			// Software
-			Mesh::softwareVertexPoseBlend(influence, pose->getVertexOffsets(), data);
+			Mesh::softwareVertexPoseBlend(influence, pose->getVertexOffsets(), pose->getNormals(), data);
 		}
 
 	}
@@ -948,7 +1061,20 @@ namespace Ogre {
 		populateClone(newTrack);
 		return newTrack;
 	}	
-
+	//--------------------------------------------------------------------------
+	void VertexAnimationTrack::_applyBaseKeyFrame(const KeyFrame* b)
+	{
+		const VertexPoseKeyFrame* base = static_cast<const VertexPoseKeyFrame*>(b);
+		
+        for (KeyFrameList::iterator i = mKeyFrames.begin(); i != mKeyFrames.end(); ++i)
+        {
+			VertexPoseKeyFrame* kf = static_cast<VertexPoseKeyFrame*>(*i);
+			
+			kf->_applyBaseKeyFrame(base);
+		}
+		
+	}
+	
 
 }
 
