@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -45,7 +45,9 @@ namespace Ogre {
 
     //-----------------------------------------------------------------------------
     AutoParamDataSource::AutoParamDataSource()
-        : mWorldMatrixDirty(true),
+        : mWorldMatrixCount(0),
+         mWorldMatrixArray(0),
+         mWorldMatrixDirty(true),
          mViewMatrixDirty(true),
          mProjMatrixDirty(true),
          mWorldViewMatrixDirty(true),
@@ -58,6 +60,7 @@ namespace Ogre {
          mInverseTransposeWorldViewMatrixDirty(true),
 		 mCameraPositionDirty(true),
          mCameraPositionObjectSpaceDirty(true),
+         mPassNumber(0),
 		 mSceneDepthRangeDirty(true),
 		 mLodCameraPositionDirty(true),
 		 mLodCameraPositionObjectSpaceDirty(true),
@@ -593,6 +596,11 @@ namespace Ogre {
         return mCurrentPass->getShininess();
     }
     //-----------------------------------------------------------------------------
+    Real AutoParamDataSource::getSurfaceAlphaRejectionValue(void) const
+    {
+        return static_cast<Real>(static_cast<unsigned int>(mCurrentPass->getAlphaRejectValue())) / 255.0f;
+    }
+    //-----------------------------------------------------------------------------
     ColourValue AutoParamDataSource::getDerivedAmbientLightColour(void) const
     {
         return getAmbientLightColour() * getSurfaceAmbientColour();
@@ -1086,7 +1094,7 @@ namespace Ogre {
 			{
 				const VisibleObjectsBoundsInfo& info = 
 					mCurrentSceneManager->getVisibleObjectsBoundsInfo(
-						(Camera*)mCurrentTextureProjector[index]);
+						(const Camera*)mCurrentTextureProjector[index]);
 
 				Real depthRange = info.maxDistanceInFrustum - info.minDistanceInFrustum;
 				if (depthRange > std::numeric_limits<Real>::epsilon())
