@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -258,6 +258,12 @@ namespace Ogre {
 		mBillboardSet->setRenderQueueGroup(queueID);
 	}
 	//-----------------------------------------------------------------------
+	void BillboardParticleRenderer::setRenderQueueGroupAndPriority(uint8 queueID, ushort priority)
+	{
+		assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
+		mBillboardSet->setRenderQueueGroupAndPriority(queueID, priority);
+	}
+	//-----------------------------------------------------------------------
 	void BillboardParticleRenderer::setKeepParticlesInLocalSpace(bool keepLocal)
 	{
 		mBillboardSet->setBillboardsInWorldSpace(!keepLocal);
