@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -195,7 +195,7 @@ namespace Ogre {
             index < mActiveBillboards.size() &&
             "Billboard index out of bounds." );
 
-        /* We can't access it directly, so we check wether it's in the first
+        /* We can't access it directly, so we check whether it's in the first
            or the second half, then we start either from the beginning or the
            end of the list
         */
@@ -220,7 +220,7 @@ namespace Ogre {
             index < mActiveBillboards.size() &&
             "Billboard index out of bounds." );
 
-        /* We can't access it directly, so we check wether it's in the first
+        /* We can't access it directly, so we check whether it's in the first
            or the second half, then we start either from the beginning or the
            end of the list.
            We then remove the billboard form the 'used' list and add it to
@@ -305,16 +305,16 @@ namespace Ogre {
     {
         mMaterialName = name;
 
-        mpMaterial = MaterialManager::getSingleton().getByName(name, groupName);
+        mMaterial = MaterialManager::getSingleton().getByName(name, groupName);
 
-		if (mpMaterial.isNull())
+		if (mMaterial.isNull())
 			OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find material " + name,
 				"BillboardSet::setMaterialName" );
 
         /* Ensure that the new material was loaded (will not load again if
            already loaded anyway)
         */
-        mpMaterial->load();
+        mMaterial->load();
     }
 
     //-----------------------------------------------------------------------
@@ -640,23 +640,23 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     const MaterialPtr& BillboardSet::getMaterial(void) const
     {
-        return mpMaterial;
+        return mMaterial;
     }
 
     void BillboardSet::setMaterial( const MaterialPtr& material )
 	{
-		mpMaterial = material;
+		mMaterial = material;
 		
-        if (mpMaterial.isNull())
+        if (mMaterial.isNull())
         {
 			LogManager::getSingleton().logMessage("Can't assign material "  
                                                   " to BillboardSet of " + getName() + " because this "
                                                   "Material does not exist. Have you forgotten to define it in a "
-                                                  ".material script?");
+                                                  ".material script?", LML_CRITICAL);
 			
-            mpMaterial = MaterialManager::getSingleton().getByName("BaseWhite");
+			mMaterial = MaterialManager::getSingleton().getByName("BaseWhite");
 			
-            if (mpMaterial.isNull())
+            if (mMaterial.isNull())
             {
                 OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, "Can't assign default material "
                             "to BillboardSet " + getName() + ". Did "
@@ -665,10 +665,10 @@ namespace Ogre {
             }
         }
 		
-		mMaterialName = mpMaterial->getName();
+		mMaterialName = mMaterial->getName();
         
         // Ensure new material loaded (will not load again if already loaded)
-        mpMaterial->load();
+        mMaterial->load();
 	}
 
     //-----------------------------------------------------------------------
@@ -681,6 +681,7 @@ namespace Ogre {
 		{
 			op.operationType = RenderOperation::OT_POINT_LIST;
     	    op.useIndexes = false;
+            op.useGlobalInstancingVertexBufferIsAvailable = false;
 			op.indexData = 0;
 			op.vertexData->vertexCount = mNumVisibleBillboards;
 		}
@@ -781,7 +782,7 @@ namespace Ogre {
 			LogManager::getSingleton().logMessage("Warning: BillboardSet " +
 				mName + " has point rendering enabled but is using a type "
 				"other than BBT_POINT, this may not give you the results you "
-				"expect.");
+				"expect.", LML_CRITICAL);
 		}
 
         mVertexData = OGRE_NEW VertexData();
@@ -1224,7 +1225,7 @@ namespace Ogre {
             Vector3 axis = (offsets[3] - offsets[0]).crossProduct(offsets[2] - offsets[1]).normalisedCopy();
 
             Matrix3 rotation;
-			rotation.FromAxisAngle(axis, bb.mRotation);
+			rotation.FromAngleAxis(axis, bb.mRotation);
 
             Vector3 pt;
 
