@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -53,8 +53,10 @@ namespace Ogre {
 CompositorInstance::CompositorInstance(CompositionTechnique *technique,
     CompositorChain *chain):
     mCompositor(technique->getParent()), mTechnique(technique), mChain(chain),
-		mEnabled(false)
+		mEnabled(false),
+		mAlive(false)
 {
+	mEnabled = false;
 	const String& logicName = mTechnique->getCompositorLogicName();
 	if (!logicName.empty())
 	{
@@ -77,10 +79,25 @@ CompositorInstance::~CompositorInstance()
 //-----------------------------------------------------------------------
 void CompositorInstance::setEnabled(bool value)
 {
-    if (mEnabled != value)
+	if (mEnabled != value)
     {
         mEnabled = value;
 
+	    //Probably first time enabling, create resources.
+	    if( mEnabled && !mAlive )
+    		setAlive( true );
+
+		/// Notify chain state needs recompile.
+        mChain->_markDirty();
+	}
+}
+//-----------------------------------------------------------------------
+void CompositorInstance::setAlive(bool value)
+{
+    if (mAlive != value)
+    {
+        mAlive = value;
+
         // Create of free resource.
         if (value)
         {
@@ -89,6 +106,7 @@ void CompositorInstance::setEnabled(bool value)
         else
         {
             freeResources(false, true);
+			setEnabled(false);
         }
 
 		/// Notify chain state needs recompile.
@@ -96,11 +114,6 @@ void CompositorInstance::setEnabled(bool value)
     }
 }
 //-----------------------------------------------------------------------
-bool CompositorInstance::getEnabled()
-{
-    return mEnabled;
-}
-//-----------------------------------------------------------------------
 
 /** Clear framebuffer RenderSystem operation
  */
@@ -149,7 +162,7 @@ public:
 	virtual void execute(SceneManager *sm, RenderSystem *rs)
 	{
 		rs->setStencilCheckEnabled(stencilCheck);
-		rs->setStencilBufferParams(func, refValue, mask, stencilFailOp, depthFailOp, passOp, twoSidedOperation);
+		rs->setStencilBufferParams(func, refValue, mask, 0xFFFFFFFF, stencilFailOp, depthFailOp, passOp, twoSidedOperation);
 	}
 };
 
@@ -161,6 +174,8 @@ public:
 	RSQuadOperation(CompositorInstance *inInstance, uint32 inPass_id, MaterialPtr inMat):
 	  mat(inMat), instance(inInstance), pass_id(inPass_id),
       mQuadCornerModified(false),
+      mQuadFarCorners(false),
+      mQuadFarCornersViewSpace(false),
       mQuadLeft(-1),
       mQuadTop(1),
       mQuadRight(1),
@@ -245,7 +260,7 @@ public:
 class RSSetSchemeOperation: public CompositorInstance::RenderSystemOperation
 {
 public:
-	RSSetSchemeOperation(const String& schemeName) : mSchemeName(schemeName) {}
+	RSSetSchemeOperation(const String& schemeName) : mPreviousLateResolving(false), mSchemeName(schemeName) {}
 	
 	String mPreviousScheme;
 	bool mPreviousLateResolving;
@@ -319,7 +334,7 @@ void CompositorInstance::collectPasses(TargetOperation &finalState, CompositionT
 				LogManager::getSingleton().logMessage("Warning in compilation of Compositor "
 					+mCompositor->getName()+": Attempt to render queue "+
 					StringConverter::toString(pass->getFirstRenderQueue())+" before "+
-					StringConverter::toString(finalState.currentQueueGroupID));
+					StringConverter::toString(finalState.currentQueueGroupID), LML_CRITICAL);
 			}
 
 			RSSetSchemeOperation* setSchemeOperation = 0;
@@ -356,7 +371,7 @@ void CompositorInstance::collectPasses(TargetOperation &finalState, CompositionT
             {
                 /// No material -- warn user
 				LogManager::getSingleton().logMessage("Warning in compilation of Compositor "
-					+mCompositor->getName()+": No material defined for composition pass");
+					+mCompositor->getName()+": No material defined for composition pass", LML_CRITICAL);
                 break;
             }
 			srcmat->load();
@@ -364,7 +379,7 @@ void CompositorInstance::collectPasses(TargetOperation &finalState, CompositionT
 			{
 				/// No supported techniques -- warn user
 				LogManager::getSingleton().logMessage("Warning in compilation of Compositor "
-					+mCompositor->getName()+": material "+srcmat->getName()+" has no supported techniques");
+					+mCompositor->getName()+": material "+srcmat->getName()+" has no supported techniques", LML_CRITICAL);
                 break;
 			}
 			srctech = srcmat->getBestTechnique(0);
@@ -393,7 +408,7 @@ void CompositorInstance::collectPasses(TargetOperation &finalState, CompositionT
 							/// Texture unit not there
 							LogManager::getSingleton().logMessage("Warning in compilation of Compositor "
 								+mCompositor->getName()+": material "+srcmat->getName()+" texture unit "
-								+StringConverter::toString(x)+" out of bounds");
+								+StringConverter::toString(x)+" out of bounds", LML_CRITICAL);
 						}
 					}
 				}
@@ -439,7 +454,7 @@ void CompositorInstance::_compileTargetOperations(CompiledState &compiledState)
         if(target->getInputMode() == CompositionTargetPass::IM_PREVIOUS)
         {
             /// Collect target state for previous compositor
-            /// The TargetOperation for the final target is collected seperately as it is merged
+            /// The TargetOperation for the final target is collected separately as it is merged
             /// with later operations
             mPreviousInstance->_compileOutputOperation(ts);
         }
@@ -463,7 +478,7 @@ void CompositorInstance::_compileOutputOperation(TargetOperation &finalState)
     if(tpass->getInputMode() == CompositionTargetPass::IM_PREVIOUS)
     {
         /// Collect target state for previous compositor
-        /// The TargetOperation for the final target is collected seperately as it is merged
+        /// The TargetOperation for the final target is collected separately as it is merged
         /// with later operations
         mPreviousInstance->_compileOutputOperation(finalState);
     }
@@ -510,7 +525,7 @@ void CompositorInstance::setTechnique(CompositionTechnique* tech, bool reuseText
 		// replace technique
 		mTechnique = tech;
 
-		if (mEnabled)
+		if (mAlive)
 		{
 			// free up resources, but keep reserves if reusing
 			freeResources(false, !reuseTextures);
@@ -528,7 +543,6 @@ void CompositorInstance::setScheme(const String& schemeName, bool reuseTextures)
 	if (tech)
 	{
 		setTechnique(tech, reuseTextures);
-		mActiveScheme = tech->getSchemeName();
 	}
 }
 //-----------------------------------------------------------------------
@@ -600,11 +614,13 @@ void CompositorInstance::createResources(bool forResizeOnly)
     while(it.hasMoreElements())
     {
         CompositionTechnique::TextureDefinition *def = it.getNext();
-
+        
 		if (!def->refCompName.empty()) {
 			//This is a reference, isn't created in this compositor
 			continue;
 		}
+        
+        RenderTarget* rendTarget;
 		if (def->scope == CompositionTechnique::TS_GLOBAL) {
 			//This is a global texture, just link the created resources from the parent
 			Compositor* parentComp = mTechnique->getParent();
@@ -612,157 +628,169 @@ void CompositorInstance::createResources(bool forResizeOnly)
 			{
 				size_t atch = 0;
 				for (PixelFormatList::iterator p = def->formatList.begin(); 
-					p != def->formatList.end(); ++p, ++atch)
+                     p != def->formatList.end(); ++p, ++atch)
 				{
 					Ogre::TexturePtr tex = parentComp->getTextureInstance(def->name, atch);
 					mLocalTextures[getMRTTexLocalName(def->name, atch)] = tex;
 				}
 				MultiRenderTarget* mrt = static_cast<MultiRenderTarget*>
-					(parentComp->getRenderTarget(def->name));
+                (parentComp->getRenderTarget(def->name));
 				mLocalMRTs[def->name] = mrt;
-			}
-			else
-			{
+                
+                rendTarget = mrt;
+            } else {
 				Ogre::TexturePtr tex = parentComp->getTextureInstance(def->name, 0);
 				mLocalTextures[def->name] = tex;
-			}
-			continue;
-		}
-        /// Determine width and height
-        size_t width = def->width;
-        size_t height = def->height;
-		uint fsaa = 0;
-		String fsaaHint;
-		bool hwGamma = false;
-
-		// Skip this one if we're only (re)creating for a resize & it's not derived
-		// from the target size
-		if (forResizeOnly && width != 0 && height != 0)
-			continue;
-
-		deriveTextureRenderTargetOptions(def->name, &hwGamma, &fsaa, &fsaaHint);
-
-        if(width == 0)
-            width = static_cast<size_t>(
-				static_cast<float>(mChain->getViewport()->getActualWidth()) * def->widthFactor);
-        if(height == 0)
-			height = static_cast<size_t>(
-				static_cast<float>(mChain->getViewport()->getActualHeight()) * def->heightFactor);
-
-		// determine options as a combination of selected options and possible options
-		if (!def->fsaa)
-		{
-			fsaa = 0;
-			fsaaHint = StringUtil::BLANK;
-		}
-		hwGamma = hwGamma || def->hwGammaWrite;
-
-        /// Make the tetxure
-		RenderTarget* rendTarget;
-		if (def->formatList.size() > 1)
-		{
-			String MRTbaseName = "c" + StringConverter::toString(dummyCounter++) + 
+                
+                rendTarget = tex->getBuffer()->getRenderTarget();
+            }
+            
+        } else {
+            /// Determine width and height
+            size_t width = def->width;
+            size_t height = def->height;
+            uint fsaa = 0;
+            String fsaaHint;
+            bool hwGamma = false;
+            
+            // Skip this one if we're only (re)creating for a resize & it's not derived
+            // from the target size
+            if (forResizeOnly && width != 0 && height != 0)
+                continue;
+            
+            deriveTextureRenderTargetOptions(def->name, &hwGamma, &fsaa, &fsaaHint);
+            
+            if(width == 0)
+                width = static_cast<size_t>(
+                                            static_cast<float>(mChain->getViewport()->getActualWidth()) * def->widthFactor);
+            if(height == 0)
+                height = static_cast<size_t>(
+                                             static_cast<float>(mChain->getViewport()->getActualHeight()) * def->heightFactor);
+            
+            // determine options as a combination of selected options and possible options
+            if (!def->fsaa)
+            {
+                fsaa = 0;
+                fsaaHint = StringUtil::BLANK;
+            }
+            hwGamma = hwGamma || def->hwGammaWrite;
+            
+            /// Make the tetxure
+            if (def->formatList.size() > 1)
+            {
+                String MRTbaseName = "c" + StringConverter::toString(dummyCounter++) + 
 				"/" + def->name + "/" + mChain->getViewport()->getTarget()->getName();
-			MultiRenderTarget* mrt = 
+                MultiRenderTarget* mrt = 
 				Root::getSingleton().getRenderSystem()->createMultiRenderTarget(MRTbaseName);
-			mLocalMRTs[def->name] = mrt;
-
-			// create and bind individual surfaces
-			size_t atch = 0;
-			for (PixelFormatList::iterator p = def->formatList.begin(); 
-				p != def->formatList.end(); ++p, ++atch)
-			{
-
-				String texname = MRTbaseName + "/" + StringConverter::toString(atch);
-				String mrtLocalName = getMRTTexLocalName(def->name, atch);
-				TexturePtr tex;
-				if (def->pooled)
-				{
-					// get / create pooled texture
-					tex = CompositorManager::getSingleton().getPooledTexture(texname,
-						mrtLocalName, 
-						width, height, *p, fsaa, fsaaHint,  
-						hwGamma && !PixelUtil::isFloatingPoint(*p), 
-						assignedTextures, this, def->scope);
-				}
-				else
-				{
-					tex = TextureManager::getSingleton().createManual(
-						texname, 
-						ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, TEX_TYPE_2D, 
-						(uint)width, (uint)height, 0, *p, TU_RENDERTARGET, 0, 
-						hwGamma && !PixelUtil::isFloatingPoint(*p), fsaa, fsaaHint ); 
-				}
-				
-				RenderTexture* rt = tex->getBuffer()->getRenderTarget();
-				rt->setAutoUpdated(false);
-				mrt->bindSurface(atch, rt);
-
-				// Also add to local textures so we can look up
-				mLocalTextures[mrtLocalName] = tex;
-				
-			}
-
-			rendTarget = mrt;
-		}
-		else
-		{
-			String texName =  "c" + StringConverter::toString(dummyCounter++) + 
+                mLocalMRTs[def->name] = mrt;
+                
+                // create and bind individual surfaces
+                size_t atch = 0;
+                for (PixelFormatList::iterator p = def->formatList.begin(); 
+                     p != def->formatList.end(); ++p, ++atch)
+                {
+                    
+                    String texname = MRTbaseName + "/" + StringConverter::toString(atch);
+                    String mrtLocalName = getMRTTexLocalName(def->name, atch);
+                    TexturePtr tex;
+                    if (def->pooled)
+                    {
+                        // get / create pooled texture
+                        tex = CompositorManager::getSingleton().getPooledTexture(texname,
+                                                                                 mrtLocalName, 
+                                                                                 width, height, *p, fsaa, fsaaHint,  
+                                                                                 hwGamma && !PixelUtil::isFloatingPoint(*p), 
+                                                                                 assignedTextures, this, def->scope);
+                    }
+                    else
+                    {
+                        tex = TextureManager::getSingleton().createManual(
+                                                                          texname, 
+                                                                          ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, TEX_TYPE_2D, 
+                                                                          (uint)width, (uint)height, 0, *p, TU_RENDERTARGET, 0, 
+                                                                          hwGamma && !PixelUtil::isFloatingPoint(*p), fsaa, fsaaHint ); 
+                    }
+                    
+                    RenderTexture* rt = tex->getBuffer()->getRenderTarget();
+                    rt->setAutoUpdated(false);
+                    mrt->bindSurface(atch, rt);
+                    
+                    // Also add to local textures so we can look up
+                    mLocalTextures[mrtLocalName] = tex;
+                    
+                }
+                
+                rendTarget = mrt;
+            }
+            else
+            {
+                String texName =  "c" + StringConverter::toString(dummyCounter++) + 
 				"/" + def->name + "/" + mChain->getViewport()->getTarget()->getName();
-			
-			// space in the name mixup the cegui in the compositor demo
-			// this is an auto generated name - so no spaces can't hart us.
-			std::replace( texName.begin(), texName.end(), ' ', '_' ); 
-
-			TexturePtr tex;
-			if (def->pooled)
-			{
-				// get / create pooled texture
-				tex = CompositorManager::getSingleton().getPooledTexture(texName, 
-					def->name, width, height, def->formatList[0], fsaa, fsaaHint,
-					hwGamma && !PixelUtil::isFloatingPoint(def->formatList[0]), assignedTextures, 
-					this, def->scope);
-			}
-			else
-			{
-				tex = TextureManager::getSingleton().createManual(
-					texName, 
-					ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, TEX_TYPE_2D, 
-					(uint)width, (uint)height, 0, def->formatList[0], TU_RENDERTARGET, 0,
-					hwGamma && !PixelUtil::isFloatingPoint(def->formatList[0]), fsaa, fsaaHint); 
-			}
-
-			rendTarget = tex->getBuffer()->getRenderTarget();
-			mLocalTextures[def->name] = tex;
-		}
+                
+                // space in the name mixup the cegui in the compositor demo
+                // this is an auto generated name - so no spaces can't hart us.
+                std::replace( texName.begin(), texName.end(), ' ', '_' ); 
+                
+                TexturePtr tex;
+                if (def->pooled)
+                {
+                    // get / create pooled texture
+                    tex = CompositorManager::getSingleton().getPooledTexture(texName, 
+                                                                             def->name, width, height, def->formatList[0], fsaa, fsaaHint,
+                                                                             hwGamma && !PixelUtil::isFloatingPoint(def->formatList[0]), assignedTextures, 
+                                                                             this, def->scope);
+                }
+                else
+                {
+                    tex = TextureManager::getSingleton().createManual(
+                                                                      texName, 
+                                                                      ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, TEX_TYPE_2D, 
+                                                                      (uint)width, (uint)height, 0, def->formatList[0], TU_RENDERTARGET, 0,
+                                                                      hwGamma && !PixelUtil::isFloatingPoint(def->formatList[0]), fsaa, fsaaHint); 
+                }
+                
+                rendTarget = tex->getBuffer()->getRenderTarget();
+                mLocalTextures[def->name] = tex;
+            }
+        }
         
+		//Set DepthBuffer pool for sharing
+		rendTarget->setDepthBufferPool( def->depthBufferId );
         
         /// Set up viewport over entire texture
         rendTarget->setAutoUpdated( false );
-
+        
 		// We may be sharing / reusing this texture, so test before adding viewport
 		if (rendTarget->getNumViewports() == 0)
 		{
+			Viewport* v;
 			Camera* camera = mChain->getViewport()->getCamera();
-
-			// Save last viewport and current aspect ratio
-			Viewport* oldViewport = camera->getViewport();
-			Real aspectRatio = camera->getAspectRatio();
-
-			Viewport* v = rendTarget->addViewport( camera );
+			if (!camera)
+			{
+				v = rendTarget->addViewport( camera );
+			}
+			else
+			{
+				// Save last viewport and current aspect ratio
+				Viewport* oldViewport = camera->getViewport();
+				Real aspectRatio = camera->getAspectRatio();
+                
+				v = rendTarget->addViewport( camera );
+                
+				// Should restore aspect ratio, in case of auto aspect ratio
+				// enabled, it'll changed when add new viewport.
+				camera->setAspectRatio(aspectRatio);
+				// Should restore last viewport, i.e. never disturb user code
+				// which might based on that.
+				camera->_notifyViewport(oldViewport);
+			}
+            
 			v->setClearEveryFrame( false );
 			v->setOverlaysEnabled( false );
 			v->setBackgroundColour( ColourValue( 0, 0, 0, 0 ) );
-
-			// Should restore aspect ratio, in case of auto aspect ratio
-			// enabled, it'll changed when add new viewport.
-			camera->setAspectRatio(aspectRatio);
-			// Should restore last viewport, i.e. never disturb user code
-			// which might based on that.
-			camera->_notifyViewport(oldViewport);
 		}
     }
-
+    
 	_fireNotifyResourcesCreated(forResizeOnly);
 }
 //---------------------------------------------------------------------
@@ -958,27 +986,57 @@ RenderTarget *CompositorInstance::getTargetForTex(const String &name)
 	CompositionTechnique::TextureDefinition* texDef = mTechnique->getTextureDefinition(name);
 	if (texDef != 0 && !texDef->refCompName.empty()) 
 	{
-		//This is a reference - find the compositor and referenced texture definition
-		Ogre::CompositorInstance *refCompInst = mChain->getCompositor(texDef->refCompName);
-		if(refCompInst == 0)
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor",
-				"CompositorInstance::getTargetForTex");
-		}
-		Ogre::Compositor *refComp = refCompInst->getCompositor();
-		CompositionTechnique::TextureDefinition* refTexDef = refComp->getSupportedTechnique(refCompInst->getScheme())->getTextureDefinition(texDef->refTexName);
-		if (refTexDef == 0)
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor texture",
-				"CompositorInstance::getTargetForTex");
-		}
-
-		switch (refTexDef->scope) 
-		{
-			case CompositionTechnique::TS_CHAIN: 
+ 		//This TextureDefinition is reference.
+ 		//Since referenced TD's have no info except name we have to find original TD
+        
+ 		CompositionTechnique::TextureDefinition* refTexDef = 0;
+        
+ 		//Try chain first
+ 		if(mChain)
+  		{
+ 			CompositorInstance* refCompInst = mChain->getCompositor(texDef->refCompName);
+ 			if(refCompInst)
+ 			{
+                refTexDef = refCompInst->getCompositor()->getSupportedTechnique(
+                    refCompInst->getScheme())->getTextureDefinition(texDef->refTexName);
+                // if the texture with the reference name can not be found, try the name
+                if (refTexDef == 0)
+                {
+ 				    refTexDef = refCompInst->getCompositor()->getSupportedTechnique(
+                        refCompInst->getScheme())->getTextureDefinition(name);
+                }
+ 			}
+			else
 			{
-				//Find the instance and check if it is before us
-				CompositorInstance* refCompInst = 0;
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor",
+                            "CompositorInstance::getTargetForTex");
+			}
+  		}
+        
+ 		if(refTexDef == 0)
+  		{
+ 			//Still NULL. Try global search.
+            const CompositorPtr &refComp = CompositorManager::getSingleton().getByName(texDef->refCompName);
+ 			if(!refComp.isNull())
+ 			{
+ 				refTexDef = refComp->getSupportedTechnique()->getTextureDefinition(name);
+ 			}
+ 		}
+        
+ 		if(refTexDef == 0)
+ 		{
+ 			//Still NULL
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor texture",
+                        "CompositorInstance::getTargetForTex");
+  		}
+        
+ 		switch(refTexDef->scope) 
+  		{
+ 			case CompositionTechnique::TS_CHAIN:
+  			{
+  				//Find the instance and check if it is before us
+  				CompositorInstance* refCompInst = 0;
+                OgreAssert(mChain, "Undefined compositor chain");
 				CompositorChain::InstanceIterator it = mChain->getCompositors();
 				bool beforeMe = true;
 				while (it.hasMoreElements())
@@ -1010,8 +1068,16 @@ RenderTarget *CompositorInstance::getTargetForTex(const String &name)
 				return refCompInst->getRenderTarget(texDef->refTexName);
 			}
 			case CompositionTechnique::TS_GLOBAL:
+			{
 				//Chain and global case - the referenced compositor will know how to handle
+				const CompositorPtr& refComp = CompositorManager::getSingleton().getByName(texDef->refCompName);
+				if(refComp.isNull())
+				{
+					OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor",
+						"CompositorInstance::getTargetForTex");
+				}
 				return refComp->getRenderTarget(texDef->refTexName);
+			}
 			case CompositionTechnique::TS_LOCAL:
 			default:
 				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Referencing local compositor texture",
@@ -1026,30 +1092,61 @@ RenderTarget *CompositorInstance::getTargetForTex(const String &name)
 //-----------------------------------------------------------------------
 const String &CompositorInstance::getSourceForTex(const String &name, size_t mrtIndex)
 {
-	CompositionTechnique::TextureDefinition* texDef = mTechnique->getTextureDefinition(name);
-
-	if (!texDef->refCompName.empty()) 
-	{
-		Ogre::CompositorInstance *refCompInst = mChain->getCompositor(texDef->refCompName);
-		if(refCompInst == 0)
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor",
-			"CompositorInstance::getSourceForTex");
-		}
-		Ogre::Compositor *refComp = refCompInst->getCompositor();
-		CompositionTechnique::TextureDefinition* refTexDef = refComp->getSupportedTechnique(refCompInst->getScheme())->getTextureDefinition(texDef->refTexName);
-		if (refTexDef == 0)
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor texture",
-				"CompositorInstance::getSourceForTex");
-		}
-
-		switch (refTexDef->scope) 
-		{
-			case CompositionTechnique::TS_CHAIN: 
+  	CompositionTechnique::TextureDefinition* texDef = mTechnique->getTextureDefinition(name);
+ 	if(texDef == 0)
+ 	{
+ 		OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent TextureDefinition",
+                    "CompositorInstance::getSourceForTex");
+ 	}
+    
+ 	//Check if texture definition is reference
+ 	if(!texDef->refCompName.empty())
+  	{
+ 		//This TextureDefinition is reference.
+ 		//Since referenced TD's have no info except name we have to find original TD
+ 		
+ 		CompositionTechnique::TextureDefinition* refTexDef = 0;
+        
+ 		//Try chain first
+ 		if(mChain)
+  		{
+ 			CompositorInstance* refCompInst = mChain->getCompositor(texDef->refCompName);
+ 			if(refCompInst)
+ 			{
+ 				refTexDef = refCompInst->getCompositor()->
+                    getSupportedTechnique(refCompInst->getScheme())->getTextureDefinition(texDef->refTexName);
+ 			}
+			else
 			{
-				//Find the instance and check if it is before us
-				CompositorInstance* refCompInst = 0;
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor",
+                            "CompositorInstance::getSourceForTex");
+			}
+ 		}
+        
+ 		if(refTexDef == 0)
+ 		{
+ 			//Still NULL. Try global search.
+            const CompositorPtr &refComp = CompositorManager::getSingleton().getByName(texDef->refCompName);
+ 			if(!refComp.isNull())
+ 			{
+ 				refTexDef = refComp->getSupportedTechnique()->getTextureDefinition(texDef->refTexName);
+ 			}
+ 		}
+        
+ 		if(refTexDef == 0)
+ 		{
+ 			//Still NULL
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor texture",
+                        "CompositorInstance::getSourceForTex");
+  		}
+        
+ 		switch(refTexDef->scope)
+  		{
+ 			case CompositionTechnique::TS_CHAIN:
+  			{
+  				//Find the instance and check if it is before us
+  				CompositorInstance* refCompInst = 0;
+                OgreAssert(mChain, "Undefined compositor chain");
 				CompositorChain::InstanceIterator it = mChain->getCompositors();
 				bool beforeMe = true;
 				while (it.hasMoreElements())
@@ -1081,8 +1178,16 @@ const String &CompositorInstance::getSourceForTex(const String &name, size_t mrt
 				return refCompInst->getTextureInstanceName(texDef->refTexName, mrtIndex);
 			}
 			case CompositionTechnique::TS_GLOBAL:
+			{
 				//Chain and global case - the referenced compositor will know how to handle
+				const CompositorPtr& refComp = CompositorManager::getSingleton().getByName(texDef->refCompName);
+				if(refComp.isNull())
+				{
+					OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Referencing non-existent compositor",
+						"CompositorInstance::getSourceForTex");
+				}
 				return refComp->getTextureInstanceName(texDef->refTexName, mrtIndex);
+			}
 			case CompositionTechnique::TS_LOCAL:
 			default:
 				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Referencing local compositor texture",
@@ -1153,6 +1258,33 @@ void CompositorInstance::_fireNotifyResourcesCreated(bool forResizeOnly)
 		(*i)->notifyResourcesCreated(forResizeOnly);
 }
 //-----------------------------------------------------------------------
+void CompositorInstance::notifyCameraChanged(Camera* camera)
+{
+	// update local texture's viewports.
+	LocalTextureMap::iterator localTexIter = mLocalTextures.begin();
+	LocalTextureMap::iterator localTexIterEnd = mLocalTextures.end();
+	while (localTexIter != localTexIterEnd)
+	{
+		RenderTexture* target = localTexIter->second->getBuffer()->getRenderTarget();
+		// skip target that has no viewport (this means texture is under MRT)
+		if (target->getNumViewports() == 1)
+		{
+			target->getViewport(0)->setCamera(camera);
+		}
+		++localTexIter;
+	}
+
+	// update MRT's viewports.
+	LocalMRTMap::iterator localMRTIter = mLocalMRTs.begin();
+	LocalMRTMap::iterator localMRTIterEnd = mLocalMRTs.end();
+	while (localMRTIter != localMRTIterEnd)
+	{
+		MultiRenderTarget* target = localMRTIter->second;
+		target->getViewport(0)->setCamera(camera);
+		++localMRTIter;
+	}
+}
+//-----------------------------------------------------------------------
 CompositorInstance::RenderSystemOperation::~RenderSystemOperation()
 {
 }
