@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,8 +31,9 @@ THE SOFTWARE.
 
 #include "OgreException.h"
 #include "OgreLogManager.h"
+#include "OgreStringConverter.h"
 
-#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
+#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WINRT
 #  define WIN32_LEAN_AND_MEAN
 #  if !defined(NOMINMAX) && defined(_MSC_VER)
 #	define NOMINMAX // required to stop windows.h messing up std::min
@@ -40,8 +41,14 @@ THE SOFTWARE.
 #  include <windows.h>
 #endif
 
+#if OGRE_PLATFORM == OGRE_PLATFORM_WINRT
+#include "OgreUTFString.h"
+#endif
+
 #if OGRE_PLATFORM == OGRE_PLATFORM_APPLE || OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
 #   include "macUtils.h"
+#endif
+#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE || OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS || OGRE_PLATFORM == OGRE_PLATFORM_NACL || OGRE_PLATFORM == OGRE_PLATFORM_FLASHCC
 #   include <dlfcn.h>
 #endif
 
@@ -69,20 +76,31 @@ namespace Ogre {
 		String name = mName;
 #if OGRE_PLATFORM == OGRE_PLATFORM_LINUX || OGRE_PLATFORM == OGRE_PLATFORM_NACL
         // dlopen() does not add .so to the filename, like windows does for .dll
-	if (name.find(".so") == String::npos)
-           name += ".so";
+        if (name.find(".so") == String::npos)
+        {
+            name += ".so.";
+            name += StringConverter::toString(OGRE_VERSION_MAJOR) + ".";
+            name += StringConverter::toString(OGRE_VERSION_MINOR) + ".";
+            name += StringConverter::toString(OGRE_VERSION_PATCH);
+        }
 #elif OGRE_PLATFORM == OGRE_PLATFORM_APPLE
         // dlopen() does not add .dylib to the filename, like windows does for .dll
         if (name.substr(name.length() - 6, 6) != ".dylib")
 			name += ".dylib";
-#elif OGRE_PLATFORM == OGRE_PLATFORM_WIN32
+#elif OGRE_PLATFORM == OGRE_PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WINRT
 		// Although LoadLibraryEx will add .dll itself when you only specify the library name,
 		// if you include a relative path then it does not. So, add it to be sure.
 		if (name.substr(name.length() - 4, 4) != ".dll")
 			name += ".dll";
 #endif
         mInst = (DYNLIB_HANDLE)DYNLIB_LOAD( name.c_str() );
-
+#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE
+        if(!mInst)
+        {
+            // Try again as a framework
+            mInst = (DYNLIB_HANDLE)FRAMEWORK_LOAD( mName );
+        }
+#endif
         if( !mInst )
             OGRE_EXCEPT(
                 Exception::ERR_INTERNAL_ERROR, 
@@ -117,7 +135,7 @@ namespace Ogre {
     String DynLib::dynlibError( void ) 
     {
 #if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
-        LPVOID lpMsgBuf; 
+        LPTSTR lpMsgBuf; 
         FormatMessage( 
             FORMAT_MESSAGE_ALLOCATE_BUFFER | 
             FORMAT_MESSAGE_FROM_SYSTEM | 
@@ -125,13 +143,40 @@ namespace Ogre {
             NULL, 
             GetLastError(), 
             MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
-            (LPTSTR) &lpMsgBuf, 
+            (LPTSTR)&lpMsgBuf, 
             0, 
             NULL 
             ); 
-        String ret = (char*)lpMsgBuf;
+        String ret = lpMsgBuf;
         // Free the buffer.
         LocalFree( lpMsgBuf );
+        return ret;
+#elif OGRE_PLATFORM == OGRE_PLATFORM_WINRT
+        WCHAR wideMsgBuf[1024]; 
+        if(0 == FormatMessageW( 
+            FORMAT_MESSAGE_FROM_SYSTEM | 
+            FORMAT_MESSAGE_IGNORE_INSERTS, 
+            NULL, 
+            GetLastError(), 
+            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
+            wideMsgBuf, 
+            sizeof(wideMsgBuf) / sizeof(wideMsgBuf[0]), 
+            NULL 
+            ))
+		{
+			wideMsgBuf[0] = 0;
+		}
+		char narrowMsgBuf[2048] = "";
+		if(0 == WideCharToMultiByte(
+			CP_ACP, 0,
+			wideMsgBuf, -1,
+			narrowMsgBuf, sizeof(narrowMsgBuf) / sizeof(narrowMsgBuf[0]),
+			NULL, NULL))
+		{
+			narrowMsgBuf[0] = 0;
+		}
+        String ret = narrowMsgBuf;
+
         return ret;
 #elif OGRE_PLATFORM == OGRE_PLATFORM_LINUX || OGRE_PLATFORM == OGRE_PLATFORM_APPLE
         return String(dlerror());
