@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -51,12 +51,6 @@ THE SOFTWARE.
 #endif
 #endif
 
-// Color order is actually RGB on iOS
-#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
-#undef FREEIMAGE_COLORORDER
-#define FREEIMAGE_COLORORDER FREEIMAGE_COLORORDER_RGB
-#endif
-
 namespace Ogre {
 
 	FreeImageCodec::RegisteredCodecList FreeImageCodec::msCodecList;
@@ -151,7 +145,7 @@ namespace Ogre {
 		for (RegisteredCodecList::iterator i = msCodecList.begin();
 			i != msCodecList.end(); ++i)
 		{
-			Codec::unRegisterCodec(*i);
+			Codec::unregisterCodec(*i);
 			OGRE_DELETE *i;
 		}
 		msCodecList.clear();
@@ -164,7 +158,7 @@ namespace Ogre {
     { 
     }
 	//---------------------------------------------------------------------
-	FIBITMAP* FreeImageCodec::encode(MemoryDataStreamPtr& input, CodecDataPtr& pData) const
+	FIBITMAP* FreeImageCodec::encodeBitmap(MemoryDataStreamPtr& input, CodecDataPtr& pData) const
 	{
 		FIBITMAP* ret = 0;
 
@@ -354,11 +348,10 @@ namespace Ogre {
 
 
 		// Copy data, invert scanlines and respect FreeImage pitch
-		uchar* pSrc;
 		uchar* pDst = FreeImage_GetBits(ret);
 		for (size_t y = 0; y < pImgData->height; ++y)
 		{
-			pSrc = srcData + (pImgData->height - y - 1) * srcPitch;
+			uchar* pSrc = srcData + (pImgData->height - y - 1) * srcPitch;
 			memcpy(pDst, pSrc, srcPitch);
 			pDst += dstPitch;
 		}
@@ -372,9 +365,9 @@ namespace Ogre {
 		return ret;
 	}
     //---------------------------------------------------------------------
-    DataStreamPtr FreeImageCodec::code(MemoryDataStreamPtr& input, Codec::CodecDataPtr& pData) const
+    DataStreamPtr FreeImageCodec::encode(MemoryDataStreamPtr& input, Codec::CodecDataPtr& pData) const
     {        
-		FIBITMAP* fiBitmap = encode(input, pData);
+		FIBITMAP* fiBitmap = encodeBitmap(input, pData);
 
 		// open memory chunk allocated by FreeImage
 		FIMEMORY* mem = FreeImage_OpenMemory();
@@ -400,10 +393,10 @@ namespace Ogre {
 
     }
     //---------------------------------------------------------------------
-    void FreeImageCodec::codeToFile(MemoryDataStreamPtr& input, 
+    void FreeImageCodec::encodeToFile(MemoryDataStreamPtr& input,
         const String& outFileName, Codec::CodecDataPtr& pData) const
     {
-		FIBITMAP* fiBitmap = encode(input, pData);
+		FIBITMAP* fiBitmap = encodeBitmap(input, pData);
 
 		FreeImage_Save((FREE_IMAGE_FORMAT)mFreeImageType, fiBitmap, outFileName.c_str());
 		FreeImage_Unload(fiBitmap);
@@ -564,11 +557,10 @@ namespace Ogre {
         // Bind output buffer
         output.bind(OGRE_NEW MemoryDataStream(imgData->size));
 
-		uchar* pSrc;
 		uchar* pDst = output->getPtr();
 		for (size_t y = 0; y < imgData->height; ++y)
 		{
-			pSrc = srcData + (imgData->height - y - 1) * srcPitch;
+			uchar* pSrc = srcData + (imgData->height - y - 1) * srcPitch;
 			memcpy(pDst, pSrc, dstPitch);
 			pDst += dstPitch;
 		}
@@ -593,7 +585,7 @@ namespace Ogre {
 	String FreeImageCodec::magicNumberToFileExt(const char *magicNumberPtr, size_t maxbytes) const
 	{
 		FIMEMORY* fiMem = 
-			FreeImage_OpenMemory((BYTE*)magicNumberPtr, static_cast<DWORD>(maxbytes));
+                    FreeImage_OpenMemory((BYTE*)const_cast<char*>(magicNumberPtr), static_cast<DWORD>(maxbytes));
 
 		FREE_IMAGE_FORMAT fif = FreeImage_GetFileTypeFromMemory(fiMem, (int)maxbytes);
 		FreeImage_CloseMemory(fiMem);
