@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -89,8 +89,31 @@ namespace Ogre
         mLoadFromFile = false;
 		mCompileError = false;
     }
-		
+    size_t GpuProgram::calculateSize(void) const
+    {
+        size_t memSize = 0;
+        memSize += sizeof(bool) * 7;
+        memSize += mManualNamedConstantsFile.size() * sizeof(char);
+        memSize += mFilename.size() * sizeof(char);
+        memSize += mSource.size() * sizeof(char);
+        memSize += mSyntaxCode.size() * sizeof(char);
+        memSize += sizeof(GpuProgramType);
+        memSize += sizeof(ushort);
 
+        size_t paramsSize = 0;
+        if(!mDefaultParams.isNull())
+            paramsSize += mDefaultParams.getPointer()->calculateSize();
+        if(!mFloatLogicalToPhysical.isNull())
+            paramsSize += mFloatLogicalToPhysical.getPointer()->bufferSize;
+        if(!mDoubleLogicalToPhysical.isNull())
+            paramsSize += mDoubleLogicalToPhysical.getPointer()->bufferSize;
+        if(!mIntLogicalToPhysical.isNull())
+            paramsSize += mIntLogicalToPhysical.getPointer()->bufferSize;
+        if(!mConstantDefs.isNull())
+            paramsSize += mConstantDefs->calculateSize();
+
+        return memSize + paramsSize;
+    }
     //-----------------------------------------------------------------------------
     void GpuProgram::loadImpl(void)
     {
@@ -261,7 +284,7 @@ namespace Ogre
 			ret->_setNamedConstants(mConstantDefs);
 		}
 		// link shared logical / physical map for low-level use
-		ret->_setLogicalIndexes(mFloatLogicalToPhysical, mIntLogicalToPhysical);
+		ret->_setLogicalIndexes(mFloatLogicalToPhysical, mDoubleLogicalToPhysical, mIntLogicalToPhysical);
 
         // Copy in default parameters if present
         if (!mDefaultParams.isNull())
@@ -334,6 +357,18 @@ namespace Ogre
 		{
 			return "geometry_program";
 		}
+		else if (t->getType() == GPT_DOMAIN_PROGRAM)
+		{
+			return "domain_program";
+		}
+		else if (t->getType() == GPT_HULL_PROGRAM)
+		{
+			return "hull_program";
+		}
+		else if (t->getType() == GPT_COMPUTE_PROGRAM)
+		{
+			return "compute_program";
+		}
 		else
         {
             return "fragment_program";
@@ -350,6 +385,18 @@ namespace Ogre
 		{
 			t->setType(GPT_GEOMETRY_PROGRAM);
 		}
+		else if (val == "domain_program")
+		{
+			t->setType(GPT_DOMAIN_PROGRAM);
+		}
+		else if (val == "hull_program")
+		{
+			t->setType(GPT_HULL_PROGRAM);
+		}
+		else if (val == "compute_program")
+		{
+			t->setType(GPT_COMPUTE_PROGRAM);
+		}
 		else
         {
             t->setType(GPT_FRAGMENT_PROGRAM);
@@ -432,33 +479,5 @@ namespace Ogre
 		GpuProgram* t = static_cast<GpuProgram*>(target);
 		t->setAdjacencyInfoRequired(StringConverter::parseBool(val));
 	}
-    //-----------------------------------------------------------------------
-    GpuProgramPtr& GpuProgramPtr::operator=(const HighLevelGpuProgramPtr& r)
-    {
-        // Can assign direct
-        if (pRep == r.getPointer())
-            return *this;
-        release();
-		// lock & copy other mutex pointer
-        OGRE_MUTEX_CONDITIONAL(r.OGRE_AUTO_MUTEX_NAME)
-        {
-		    OGRE_LOCK_MUTEX(*r.OGRE_AUTO_MUTEX_NAME)
-		    OGRE_COPY_AUTO_SHARED_MUTEX(r.OGRE_AUTO_MUTEX_NAME)
-            pRep = r.getPointer();
-            pUseCount = r.useCountPointer();
-            if (pUseCount)
-            {
-                ++(*pUseCount);
-            }
-        }
-		else
-		{
-			// RHS must be a null pointer
-			assert(r.isNull() && "RHS must be null if it has no mutex!");
-			setNull();
-		}
-        return *this;
-    }
-
 }
 
