@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -112,21 +112,21 @@ namespace Ogre {
 		{
 			OGRE_EXCEPT( 
 				Exception::ERR_INTERNAL_ERROR,
-				"Can not flip an unitialized texture",
+				"Can not flip an uninitialised texture",
 				"Image::flipAroundY" );
 		}
         
-         mNumMipmaps = 0; // Image operations lose precomputed mipmaps
+        mNumMipmaps = 0; // Image operations lose precomputed mipmaps
 
 		uchar	*pTempBuffer1 = NULL;
 		ushort	*pTempBuffer2 = NULL;
 		uchar	*pTempBuffer3 = NULL;
 		uint	*pTempBuffer4 = NULL;
 
-		uchar	*src1 = mBuffer, *dst1 = NULL;
-		ushort	*src2 = (ushort *)mBuffer, *dst2 = NULL;
-		uchar	*src3 = mBuffer, *dst3 = NULL;
-		uint	*src4 = (uint *)mBuffer, *dst4 = NULL;
+		uchar	*src1 = mBuffer;
+		ushort	*src2 = (ushort *)mBuffer;
+		uchar	*src3 = mBuffer;
+		uint	*src4 = (uint *)mBuffer;
 
 		ushort y;
 		switch (mPixelSize)
@@ -135,7 +135,7 @@ namespace Ogre {
 			pTempBuffer1 = OGRE_ALLOC_T(uchar, mWidth * mHeight, MEMCATEGORY_GENERAL);
 			for (y = 0; y < mHeight; y++)
 			{
-				dst1 = (pTempBuffer1 + ((y * mWidth) + mWidth - 1));
+				uchar *dst1 = (pTempBuffer1 + ((y * mWidth) + mWidth - 1));
 				for (ushort x = 0; x < mWidth; x++)
 					memcpy(dst1--, src1++, sizeof(uchar));
 			}
@@ -148,7 +148,7 @@ namespace Ogre {
 			pTempBuffer2 = OGRE_ALLOC_T(ushort, mWidth * mHeight, MEMCATEGORY_GENERAL);
 			for (y = 0; y < mHeight; y++)
 			{
-				dst2 = (pTempBuffer2 + ((y * mWidth) + mWidth - 1));
+				ushort *dst2 = (pTempBuffer2 + ((y * mWidth) + mWidth - 1));
 				for (ushort x = 0; x < mWidth; x++)
 					memcpy(dst2--, src2++, sizeof(ushort));
 			}
@@ -162,7 +162,7 @@ namespace Ogre {
 			for (y = 0; y < mHeight; y++)
 			{
 				size_t offset = ((y * mWidth) + (mWidth - 1)) * 3;
-				dst3 = pTempBuffer3;
+				uchar *dst3 = pTempBuffer3;
 				dst3 += offset;
 				for (size_t x = 0; x < mWidth; x++)
 				{
@@ -179,7 +179,7 @@ namespace Ogre {
 			pTempBuffer4 = OGRE_ALLOC_T(uint, mWidth * mHeight, MEMCATEGORY_GENERAL);
 			for (y = 0; y < mHeight; y++)
 			{
-				dst4 = (pTempBuffer4 + ((y * mWidth) + mWidth - 1));
+				uint *dst4 = (pTempBuffer4 + ((y * mWidth) + mWidth - 1));
 				for (ushort x = 0; x < mWidth; x++)
 					memcpy(dst4--, src4++, sizeof(uint));
 			}
@@ -207,7 +207,7 @@ namespace Ogre {
 		{
 			OGRE_EXCEPT( 
 				Exception::ERR_INTERNAL_ERROR,
-				"Can not flip an unitialized texture",
+				"Can not flip an uninitialised texture",
 				"Image::flipAroundX" );
 		}
         
@@ -232,10 +232,10 @@ namespace Ogre {
 	}
 
 	//-----------------------------------------------------------------------------
-	Image& Image::loadDynamicImage( uchar* pData, size_t uWidth, size_t uHeight, 
-		size_t depth,
+	Image& Image::loadDynamicImage( uchar* pData, uint32 uWidth, uint32 uHeight,
+		uint32 depth,
 		PixelFormat eFormat, bool autoDelete, 
-		size_t numFaces, size_t numMipMaps)
+		size_t numFaces, uint8 numMipMaps)
 	{
 
 		freeMemory();
@@ -270,7 +270,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------------
 	Image & Image::loadRawData(
 		DataStreamPtr& stream, 
-		size_t uWidth, size_t uHeight, size_t uDepth,
+		uint32 uWidth, uint32 uHeight, uint32 uDepth,
 		PixelFormat eFormat,
 		size_t numFaces, size_t numMipMaps)
 	{
@@ -345,7 +345,7 @@ namespace Ogre {
 		// Wrap memory, be sure not to delete when stream destroyed
 		MemoryDataStreamPtr wrapper(OGRE_NEW MemoryDataStream(mBuffer, mBufSize, false));
 
-		pCodec->codeToFile(wrapper, filename, codeDataPtr);
+		pCodec->encodeToFile(wrapper, filename, codeDataPtr);
 	}
 	//---------------------------------------------------------------------
 	DataStreamPtr Image::encode(const String& formatextension)
@@ -373,7 +373,7 @@ namespace Ogre {
 		// Wrap memory, be sure not to delete when stream destroyed
 		MemoryDataStreamPtr wrapper(OGRE_NEW MemoryDataStream(mBuffer, mBufSize, false));
 
-		return pCodec->code(wrapper, codeDataPtr);
+		return pCodec->encode(wrapper, codeDataPtr);
 	}
 	//-----------------------------------------------------------------------------
 	Image & Image::load(DataStreamPtr& stream, const String& type )
@@ -466,7 +466,7 @@ namespace Ogre {
 	}
 
 	//-----------------------------------------------------------------------------
-	size_t Image::getNumMipmaps() const
+	uint8 Image::getNumMipmaps() const
 	{
 		return mNumMipmaps;
 	}
@@ -485,18 +485,18 @@ namespace Ogre {
 	}
 
 	//-----------------------------------------------------------------------------
-	size_t Image::getDepth() const
+	uint32 Image::getDepth() const
 	{
 		return mDepth;
 	}
 	//-----------------------------------------------------------------------------
-	size_t Image::getWidth() const
+	uint32 Image::getWidth() const
 	{
 		return mWidth;
 	}
 
 	//-----------------------------------------------------------------------------
-	size_t Image::getHeight() const
+	uint32 Image::getHeight() const
 	{
 		return mHeight;
 	}
@@ -540,26 +540,18 @@ namespace Ogre {
 		if( bpp != 24 && bpp != 32 ) return;
 
 		uint stride = bpp >> 3;
+		
+		uchar gammaramp[256];
+		const Real exponent = 1.0f / gamma;
+		for(int i = 0; i < 256; i++) {
+			gammaramp[i] = static_cast<uchar>(Math::Pow(i/255.0f, exponent)*255+0.5f);
+		}
 
 		for( size_t i = 0, j = size / stride; i < j; i++, buffer += stride )
 		{
-			Real r, g, b;
-
-			const Real rangeMult = 255.0f;
-			const Real rangeMultInv = 1.0f / rangeMult;
-			const Real gammaValue = 1.0f / gamma;
-
-			r = rangeMultInv * buffer[0];
-			g = rangeMultInv * buffer[1];
-			b = rangeMultInv * buffer[2];
-
-			Math::Pow(r, gammaValue);
-			Math::Pow(g, gammaValue);
-			Math::Pow(b, gammaValue);
-
-			buffer[0] = (uchar)(r * rangeMult);
-			buffer[1] = (uchar)(g * rangeMult);
-			buffer[2] = (uchar)(b * rangeMult);
+			buffer[0] = gammaramp[buffer[0]];
+			buffer[1] = gammaramp[buffer[1]];
+			buffer[2] = gammaramp[buffer[2]];
 		}
 	}
 	//-----------------------------------------------------------------------------
@@ -723,13 +715,13 @@ namespace Ogre {
         // Calculate mipmap offset and size
         uint8 *offset = const_cast<uint8*>(getData());
 		// Base offset is number of full faces
-        size_t width = getWidth(), height=getHeight(), depth=getDepth();
+        uint32 width = getWidth(), height=getHeight(), depth=getDepth();
 		size_t numMips = getNumMipmaps();
 
 		// Figure out the offsets 
 		size_t fullFaceSize = 0;
 		size_t finalFaceSize = 0;
-		size_t finalWidth = 0, finalHeight = 0, finalDepth = 0;
+		uint32 finalWidth = 0, finalHeight = 0, finalDepth = 0;
 		for(size_t mip=0; mip <= numMips; ++mip)
         {
 			if (mip == mipmap)
@@ -754,7 +746,7 @@ namespace Ogre {
 		return src;
 	}
     //-----------------------------------------------------------------------------    
-    size_t Image::calculateSize(size_t mipmaps, size_t faces, size_t width, size_t height, size_t depth, 
+    size_t Image::calculateSize(size_t mipmaps, size_t faces, uint32 width, uint32 height, uint32 depth, 
         PixelFormat format)
     {
         size_t size = 0;
@@ -852,7 +844,7 @@ namespace Ogre {
 
 		for (size_t face = 0; face < numFaces; ++face)
 		{
-			for (size_t mip = 0; mip <= mNumMipmaps; ++mip)
+			for (uint8 mip = 0; mip <= mNumMipmaps; ++mip)
 			{
 				// convert the RGB first
 				PixelBox srcRGB = rgb.getPixelBox(face, mip);
