@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -44,6 +44,7 @@ namespace Ogre {
 		  mSpotOuter(Degree(40.0f)),
           mSpotInner(Degree(30.0f)),
           mSpotFalloff(1.0f),
+          mSpotNearClip(0.0f),
 		  mRange(100000),
 		  mAttenuationConst(1.0f),
 		  mAttenuationLinear(0.0f),
@@ -63,6 +64,8 @@ namespace Ogre {
           mDerivedTransformDirty(false),
 		  mCustomShadowCameraSetup()
     {
+		//mMinPixelSize should always be zero for lights otherwise lights will disapear
+    	mMinPixelSize = 0;
     }
     //-----------------------------------------------------------------------
 	Light::Light(const String& name) : MovableObject(name),
@@ -74,6 +77,7 @@ namespace Ogre {
 		mSpotOuter(Degree(40.0f)),
         mSpotInner(Degree(30.0f)),
         mSpotFalloff(1.0f),
+        mSpotNearClip(0.0f),
 		mRange(100000),
 		mAttenuationConst(1.0f),
 		mAttenuationLinear(0.0f),
@@ -92,6 +96,8 @@ namespace Ogre {
         mDerivedTransformDirty(false),
 		mCustomShadowCameraSetup()
     {
+		//mMinPixelSize should always be zero for lights otherwise lights will disapear
+    	mMinPixelSize = 0;
     }
     //-----------------------------------------------------------------------
     Light::~Light()
@@ -114,15 +120,12 @@ namespace Ogre {
         mPosition.y = y;
         mPosition.z = z;
         mDerivedTransformDirty = true;
-		mDerivedCamRelativeDirty = true;
-
     }
     //-----------------------------------------------------------------------
     void Light::setPosition(const Vector3& vec)
     {
         mPosition = vec;
         mDerivedTransformDirty = true;
-		mDerivedCamRelativeDirty = true;
     }
     //-----------------------------------------------------------------------
     const Vector3& Light::getPosition(void) const
@@ -151,13 +154,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Light::setSpotlightRange(const Radian& innerAngle, const Radian& outerAngle, Real falloff)
     {
-
-        if (mLightType != LT_SPOTLIGHT)
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
-                "setSpotlightRange is only valid for spotlights.",
-                "Light::setSpotlightRange");
-
-        mSpotInner =innerAngle;
+        mSpotInner = innerAngle;
         mSpotOuter = outerAngle;
         mSpotFalloff = falloff;
     }
@@ -284,6 +281,8 @@ namespace Ogre {
             }
 
             mDerivedTransformDirty = false;
+            //if the position has been updated we must update also the relative position
+            mDerivedCamRelativeDirty = true;
         }
 		if (mCameraToBeRelativeTo && mDerivedCamRelativeDirty)
 		{
@@ -865,6 +864,76 @@ namespace Ogre {
         }
 	}
 	//-----------------------------------------------------------------------
+	bool Light::isInLightRange(const Ogre::Sphere& container) const
+	{
+		bool isIntersect = true;
+		//directional light always intersects (check only spotlight and point)
+		if (mLightType != LT_DIRECTIONAL)
+		{
+			//Check that the sphere is within the sphere of the light
+			isIntersect = container.intersects(Sphere(mDerivedPosition, mRange));
+			//If this is a spotlight, check that the sphere is within the cone of the spot light
+			if ((isIntersect) && (mLightType == LT_SPOTLIGHT))
+			{
+				//check first check of the sphere surrounds the position of the light
+				//(this covers the case where the center of the sphere is behind the position of the light
+				// something which is not covered in the next test).
+				isIntersect = container.intersects(mDerivedPosition);
+				//if not test cones
+				if (!isIntersect)
+				{
+					//Calculate the cone that exists between the sphere and the center position of the light
+					Ogre::Vector3 lightSphereConeDirection = container.getCenter() - mDerivedPosition;
+					Ogre::Radian halfLightSphereConeAngle = Math::ASin(container.getRadius() / lightSphereConeDirection.length());
+
+					//Check that the light cone and the light-position-to-sphere cone intersect)
+					Radian angleBetweenConeDirections = lightSphereConeDirection.angleBetween(mDerivedDirection);
+					isIntersect = angleBetweenConeDirections <=  halfLightSphereConeAngle + mSpotOuter * 0.5;
+				}
+			}
+		}
+		return isIntersect;
+	}
+
+	//-----------------------------------------------------------------------
+	bool Light::isInLightRange(const Ogre::AxisAlignedBox& container) const
+	{
+		bool isIntersect = true;
+		//Check the 2 simple / obvious situations. Light is directional or light source is inside the container
+		if ((mLightType != LT_DIRECTIONAL) && (container.intersects(mDerivedPosition) == false))
+		{
+			//Check that the container is within the sphere of the light
+			isIntersect = Math::intersects(Sphere(mDerivedPosition, mRange),container);
+			//If this is a spotlight, do a more specific check
+			if ((isIntersect) && (mLightType == LT_SPOTLIGHT) && (mSpotOuter.valueRadians() <= Math::PI))
+			{
+				//Create a rough bounding box around the light and check if
+				Quaternion localToWorld = Vector3::NEGATIVE_UNIT_Z.getRotationTo(mDerivedDirection);
+
+				Real boxOffset = Math::Sin(mSpotOuter * 0.5) * mRange;
+				AxisAlignedBox lightBoxBound;
+				lightBoxBound.merge(Vector3::ZERO);
+				lightBoxBound.merge(localToWorld * Vector3(boxOffset, boxOffset, -mRange));
+				lightBoxBound.merge(localToWorld * Vector3(-boxOffset, boxOffset, -mRange));
+				lightBoxBound.merge(localToWorld * Vector3(-boxOffset, -boxOffset, -mRange));
+				lightBoxBound.merge(localToWorld * Vector3(boxOffset, -boxOffset, -mRange));
+				lightBoxBound.setMaximum(lightBoxBound.getMaximum() + mDerivedPosition);
+				lightBoxBound.setMinimum(lightBoxBound.getMinimum() + mDerivedPosition);
+				isIntersect = lightBoxBound.intersects(container);
+				
+				//If the bounding box check succeeded do one more test
+				if (isIntersect)
+				{
+					//Check intersection again with the bounding sphere of the container
+					//Helpful for when the light is at an angle near one of the vertexes of the bounding box
+					isIntersect = isInLightRange(Sphere(container.getCenter(), 
+						container.getHalfSize().length()));
+				}
+			}
+		}
+		return isIntersect;
+	}
+	//-----------------------------------------------------------------------
 	//-----------------------------------------------------------------------
 	String LightFactory::FACTORY_TYPE_NAME = "Light";
 	//-----------------------------------------------------------------------
