@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -54,7 +54,7 @@ namespace Ogre
     //-----------------------------------------------------------------------
     Log::~Log()
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 		if (!mSuppressFile)
 		{
 	        mLog.close();
@@ -63,7 +63,7 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void Log::logMessage( const String& message, LogMessageLevel lml, bool maskDebug )
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
         if ((mLogLevel + lml) >= OGRE_LOG_THRESHOLD)
         {
 			bool skipThisMessage = false;
@@ -79,7 +79,19 @@ namespace Ogre
                 }
 #else
                 if (mDebugOut && !maskDebug)
-                    std::cerr << message << std::endl;
+                {
+
+#	if OGRE_DEBUG_MODE && (OGRE_PLATFORM == OGRE_PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WINRT)
+					String logMessageString(message);
+					logMessageString.append( "\n" );
+                    Ogre_OutputCString( logMessageString.c_str());
+#	else
+					if (lml == LML_CRITICAL)
+						std::cerr << message << std::endl;
+					else
+						std::cout << message << std::endl;
+#	endif
+				}
 #endif
 
 				// Write time into log
@@ -107,35 +119,35 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void Log::setTimeStampEnabled(bool timeStamp)
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
         mTimeStamp = timeStamp;
     }
 
     //-----------------------------------------------------------------------
     void Log::setDebugOutputEnabled(bool debugOutput)
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
         mDebugOut = debugOutput;
     }
 
 	//-----------------------------------------------------------------------
     void Log::setLogDetail(LoggingLevel ll)
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
         mLogLevel = ll;
     }
 
     //-----------------------------------------------------------------------
     void Log::addListener(LogListener* listener)
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
         mListeners.push_back(listener);
     }
 
     //-----------------------------------------------------------------------
     void Log::removeListener(LogListener* listener)
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
         mListeners.erase(std::find(mListeners.begin(), mListeners.end(), listener));
     }
 	//---------------------------------------------------------------------
