@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -174,6 +174,27 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
+    size_t Material::calculateSize(void) const
+    {
+        size_t memSize = 0;
+
+        // Tally up techniques
+        Techniques::const_iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            memSize += (*i)->calculateSize();
+        }
+
+        memSize += sizeof(bool) * 3;
+        memSize += mUnsupportedReasons.size() * sizeof(char);
+        memSize += sizeof(LodStrategy);
+
+        memSize += Resource::calculateSize();
+
+        return memSize;
+    }
+    //-----------------------------------------------------------------------
     MaterialPtr Material::clone(const String& newName, bool changeGroup, 
 		const String& newGroup) const
     {
@@ -495,7 +516,7 @@ namespace Ogre {
         // Did we find any?
         if (mSupportedTechniques.empty())
         {
-			LogManager::getSingleton().stream()
+            LogManager::getSingleton().stream(LML_CRITICAL)
 				<< "WARNING: material " << mName << " has no supportable "
 				<< "Techniques and will be blank. Explanation: \n" << mUnsupportedReasons;
         }
@@ -769,6 +790,7 @@ namespace Ogre {
 		iend = lodValues.end();
 		// First, clear and add single zero entry
 		mLodValues.clear();
+        mUserLodValues.clear();
         mUserLodValues.push_back(0);
 		mLodValues.push_back(mLodStrategy->getBaseValue());
 		for (i = lodValues.begin(); i != iend; ++i)
@@ -789,6 +811,11 @@ namespace Ogre {
     {
         return LodValueIterator(mLodValues.begin(), mLodValues.end());
     }
+    // --------------------------------------------------------------------
+    Material::LodValueIterator Material::getUserLodValueIterator(void) const
+    {
+        return LodValueIterator(mUserLodValues.begin(), mUserLodValues.end());
+    }
 
     //-----------------------------------------------------------------------
     bool Material::applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply) const
@@ -819,7 +846,7 @@ namespace Ogre {
         assert(mLodValues.size());
         mLodValues[0] = mLodStrategy->getBaseValue();
 
-        // Re-transform all user lod values (starting at index 1, no need to transform base value)
+        // Re-transform all user LOD values (starting at index 1, no need to transform base value)
         for (size_t i = 1; i < mUserLodValues.size(); ++i)
             mLodValues[i] = mLodStrategy->transformUserValue(mUserLodValues[i]);
     }
