@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -43,59 +43,10 @@ THE SOFTWARE.
 #include "OgreOptimisedUtil.h"
 #include "OgreTangentSpaceCalc.h"
 #include "OgreLodStrategyManager.h"
-
+#include "OgreLodConfig.h"
+#include "OgrePixelCountLodStrategy.h"
 
 namespace Ogre {
-    //-----------------------------------------------------------------------
-    MeshPtr::MeshPtr(const ResourcePtr& r) : SharedPtr<Mesh>()
-    {
-		// lock & copy other mutex pointer
-        OGRE_MUTEX_CONDITIONAL(r.OGRE_AUTO_MUTEX_NAME)
-        {
-		    OGRE_LOCK_MUTEX(*r.OGRE_AUTO_MUTEX_NAME)
-		    OGRE_COPY_AUTO_SHARED_MUTEX(r.OGRE_AUTO_MUTEX_NAME)
-            pRep = static_cast<Mesh*>(r.getPointer());
-            pUseCount = r.useCountPointer();
-            if (pUseCount)
-            {
-                ++(*pUseCount);
-            }
-        }
-    }
-    //-----------------------------------------------------------------------
-    MeshPtr& MeshPtr::operator=(const ResourcePtr& r)
-    {
-        if (pRep == static_cast<Mesh*>(r.getPointer()))
-            return *this;
-        release();
-		// lock & copy other mutex pointer
-        OGRE_MUTEX_CONDITIONAL(r.OGRE_AUTO_MUTEX_NAME)
-        {
-		    OGRE_LOCK_MUTEX(*r.OGRE_AUTO_MUTEX_NAME)
-		    OGRE_COPY_AUTO_SHARED_MUTEX(r.OGRE_AUTO_MUTEX_NAME)
-            pRep = static_cast<Mesh*>(r.getPointer());
-            pUseCount = r.useCountPointer();
-            if (pUseCount)
-            {
-                ++(*pUseCount);
-            }
-        }
-		else
-		{
-			// RHS must be a null pointer
-			assert(r.isNull() && "RHS must be null if it has no mutex!");
-			setNull();
-		}
-        return *this;
-    }
-    //-----------------------------------------------------------------------
-    void MeshPtr::destroy(void)
-    {
-        // We're only overriding so that we can destroy after full definition of Mesh
-        SharedPtr<Mesh>::destroy();
-    }
-    //-----------------------------------------------------------------------
-    //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     Mesh::Mesh(ResourceManager* creator, const String& name, ResourceHandle handle,
         const String& group, bool isManual, ManualResourceLoader* loader)
@@ -123,7 +74,7 @@ namespace Ogre {
 
 		// Init first (manual) lod
 		MeshLodUsage lod;
-        lod.userValue = 0; // User value not used for base lod level
+        lod.userValue = 0; // User value not used for base LOD level
 		lod.value = mLodStrategy->getBaseValue();
         lod.edgeData = NULL;
         lod.manualMesh.setNull();
@@ -259,10 +210,10 @@ namespace Ogre {
 			}
 		}
 
-        // The loading process accesses lod usages directly, so
+        // The loading process accesses LOD usages directly, so
         // transformation of user values must occur after loading is complete.
 
-        // Transform user lod values (starting at index 1, no need to transform base value)
+        // Transform user LOD values (starting at index 1, no need to transform base value)
 		for (MeshLodUsageList::iterator i = mMeshLodUsageList.begin(); i != mMeshLodUsageList.end(); ++i)
             i->value = mLodStrategy->transformUserValue(i->userValue);
 	}
@@ -364,40 +315,9 @@ namespace Ogre {
 
         // Copy submeshes first
         vector<SubMesh*>::type::iterator subi;
-        SubMesh* newSub;
         for (subi = mSubMeshList.begin(); subi != mSubMeshList.end(); ++subi)
         {
-            newSub = newMesh->createSubMesh();
-            newSub->mMaterialName = (*subi)->mMaterialName;
-            newSub->mMatInitialised = (*subi)->mMatInitialised;
-            newSub->operationType = (*subi)->operationType;
-            newSub->useSharedVertices = (*subi)->useSharedVertices;
-            newSub->extremityPoints = (*subi)->extremityPoints;
-
-            if (!(*subi)->useSharedVertices)
-            {
-                // Copy unique vertex data
-				newSub->vertexData = (*subi)->vertexData->clone();
-                // Copy unique index map
-                newSub->blendIndexToBoneIndexMap = (*subi)->blendIndexToBoneIndexMap;
-            }
-
-            // Copy index data
-            OGRE_DELETE newSub->indexData;
-			newSub->indexData = (*subi)->indexData->clone();
-            // Copy any bone assignments
-            newSub->mBoneAssignments = (*subi)->mBoneAssignments;
-            newSub->mBoneAssignmentsOutOfDate = (*subi)->mBoneAssignmentsOutOfDate;
-            // Copy texture aliases
-            newSub->mTextureAliases = (*subi)->mTextureAliases;
-
-            // Copy lod face lists
-            newSub->mLodFaceList.reserve((*subi)->mLodFaceList.size());
-            SubMesh::LODFaceList::const_iterator facei;
-            for (facei = (*subi)->mLodFaceList.begin(); facei != (*subi)->mLodFaceList.end(); ++facei) {
-                IndexData* newIndexData = (*facei)->clone();
-                newSub->mLodFaceList.push_back(newIndexData);
-            }
+            (*subi)->clone("", newMesh.get());
         }
 
         // Copy shared geometry and index map, if any
@@ -420,12 +340,13 @@ namespace Ogre {
 		newMesh->mIsLodManual = mIsLodManual;
 		newMesh->mNumLods = mNumLods;
 		newMesh->mMeshLodUsageList = mMeshLodUsageList;
+        newMesh->mAutoBuildEdgeLists = mAutoBuildEdgeLists;
         // Unreference edge lists, otherwise we'll delete the same lot twice, build on demand
         MeshLodUsageList::iterator lodi;
         for (lodi = newMesh->mMeshLodUsageList.begin(); lodi != newMesh->mMeshLodUsageList.end(); ++lodi) {
             MeshLodUsage& lod = *lodi;
             lod.edgeData = NULL;
-            // TODO: Copy manual lod meshes
+            // TODO: Copy manual LOD meshes
         }
 
 		newMesh->mVertexBufferUsage = mVertexBufferUsage;
@@ -459,12 +380,10 @@ namespace Ogre {
 		newMesh->mSharedVertexDataAnimationType = mSharedVertexDataAnimationType;
 		newMesh->mAnimationTypesDirty = true;
 
-
         newMesh->load();
         newMesh->touch();
 
         return newMesh;
-
     }
     //-----------------------------------------------------------------------
     const AxisAlignedBox& Mesh::getBounds(void) const
@@ -513,7 +432,7 @@ namespace Ogre {
 			{
 				// Load skeleton
 				try {
-					mSkeleton = SkeletonManager::getSingleton().load(skelName, mGroup);
+					mSkeleton = SkeletonManager::getSingleton().load(skelName, mGroup).staticCast<Skeleton>();
 				}
 				catch (...)
 				{
@@ -709,7 +628,7 @@ namespace Ogre {
                 "The lowest weighted assignments beyond this limit have been removed, so "
                 "your animation may look slightly different. To eliminate this, reduce "
                 "the number of bone assignments per vertex on your mesh to " +
-                StringConverter::toString(OGRE_MAX_BLEND_WEIGHTS) + ".");
+                StringConverter::toString(OGRE_MAX_BLEND_WEIGHTS) + ".", LML_CRITICAL);
             // we've adjusted them down to the max
             maxBones = OGRE_MAX_BLEND_WEIGHTS;
 
@@ -722,7 +641,7 @@ namespace Ogre {
                 "includes vertices without bone assignments. Those vertices will "
 				"transform to wrong position when skeletal animation enabled. "
 				"To eliminate this, assign at least one bone assignment per vertex "
-				"on your mesh.");
+				"on your mesh.", LML_CRITICAL);
 		}
 
         return maxBones;
@@ -730,15 +649,16 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void  Mesh::_compileBoneAssignments(void)
     {
-        unsigned short maxBones =
-            _rationaliseBoneAssignments(sharedVertexData->vertexCount, mBoneAssignments);
-
-        if (maxBones != 0)
-        {
-            compileBoneAssignments(mBoneAssignments, maxBones, 
-                sharedBlendIndexToBoneIndexMap, sharedVertexData);
-        }
+		if (sharedVertexData)
+		{
+			unsigned short maxBones = _rationaliseBoneAssignments(sharedVertexData->vertexCount, mBoneAssignments);
 
+			if (maxBones != 0)
+			{
+				compileBoneAssignments(mBoneAssignments, maxBones, 
+					sharedBlendIndexToBoneIndexMap, sharedVertexData);
+			}
+		}
         mBoneAssignmentsOutOfDate = false;
     }
     //---------------------------------------------------------------------
@@ -924,7 +844,7 @@ namespace Ogre {
     //---------------------------------------------------------------------
     const MeshLodUsage& Mesh::getLodLevel(ushort index) const
     {
-        assert(index < mMeshLodUsageList.size());
+        index = std::min(index, (ushort)(mMeshLodUsageList.size() - 1));
         if (mIsLodManual && index > 0 && mMeshLodUsageList[index].manualMesh.isNull())
         {
             // Load the mesh now
@@ -980,8 +900,8 @@ namespace Ogre {
 
 		// Basic prerequisites
 		assert(mIsLodManual && "Not using manual LODs!");
-		assert(index != 0 && "Can't modify first lod level (full detail)");
-		assert(index < mMeshLodUsageList.size() && "Index out of bounds");
+		assert(index != 0 && "Can't modify first LOD level (full detail)");
+		assert(index < mMeshLodUsageList.size() && "Idndex out of bounds");
 		// get lod
 		MeshLodUsage* lod = &(mMeshLodUsageList[index]);
 
@@ -1019,7 +939,7 @@ namespace Ogre {
         assert(!mEdgeListsBuilt && "Can't modify LOD after edge lists built");
 
 		// Basic prerequisites
-		assert(level != 0 && "Can't modify first lod level (full detail)");
+		assert(level != 0 && "Can't modify first LOD level (full detail)");
 		assert(level < mMeshLodUsageList.size() && "Index out of bounds");
 
 		mMeshLodUsageList[level] = usage;
@@ -1033,7 +953,7 @@ namespace Ogre {
 		// Basic prerequisites
 		assert(!mIsLodManual && "Not using generated LODs!");
         assert(subIdx <= mSubMeshList.size() && "Index out of bounds");
-		assert(level != 0 && "Can't modify first lod level (full detail)");
+		assert(level != 0 && "Can't modify first LOD level (full detail)");
 		assert(level <= mSubMeshList[subIdx]->mLodFaceList.size() && "Index out of bounds");
 
 		SubMesh* sm = mSubMeshList[subIdx];
@@ -1097,6 +1017,70 @@ namespace Ogre {
 		mIndexBufferUsage = vbUsage;
 		mIndexBufferShadowBuffer = shadowBuffer;
 	}
+	//---------------------------------------------------------------------
+	void Mesh::mergeAdjacentTexcoords( unsigned short finalTexCoordSet,
+										unsigned short texCoordSetToDestroy )
+	{
+		if( sharedVertexData )
+			mergeAdjacentTexcoords( finalTexCoordSet, texCoordSetToDestroy, sharedVertexData );
+
+		SubMeshList::const_iterator itor = mSubMeshList.begin();
+		SubMeshList::const_iterator end  = mSubMeshList.end();
+
+		while( itor != end )
+		{
+			if( !(*itor)->useSharedVertices )
+				mergeAdjacentTexcoords( finalTexCoordSet, texCoordSetToDestroy, (*itor)->vertexData );
+			++itor;
+		}
+	}
+	//---------------------------------------------------------------------
+	void Mesh::mergeAdjacentTexcoords( unsigned short finalTexCoordSet,
+										unsigned short texCoordSetToDestroy,
+										VertexData *vertexData )
+	{
+		VertexDeclaration *vDecl	= vertexData->vertexDeclaration;
+
+	    const VertexElement *uv0 = vDecl->findElementBySemantic( VES_TEXTURE_COORDINATES,
+																	finalTexCoordSet );
+		const VertexElement *uv1 = vDecl->findElementBySemantic( VES_TEXTURE_COORDINATES,
+																	texCoordSetToDestroy );
+
+		if( uv0 && uv1 )
+		{
+			//Check that both base types are compatible (mix floats w/ shorts) and there's enough space
+			VertexElementType baseType0 = VertexElement::getBaseType( uv0->getType() );
+			VertexElementType baseType1 = VertexElement::getBaseType( uv1->getType() );
+
+			unsigned short totalTypeCount = VertexElement::getTypeCount( uv0->getType() ) +
+											VertexElement::getTypeCount( uv1->getType() );
+			if( baseType0 == baseType1 && totalTypeCount <= 4 )
+			{
+				const VertexDeclaration::VertexElementList &veList = vDecl->getElements();
+				VertexDeclaration::VertexElementList::const_iterator uv0Itor = std::find( veList.begin(),
+																					veList.end(), *uv0 );
+				unsigned short elem_idx		= std::distance( veList.begin(), uv0Itor );
+				VertexElementType newType	= VertexElement::multiplyTypeCount( baseType0,
+																				totalTypeCount );
+
+				if( ( uv0->getOffset() + uv0->getSize() == uv1->getOffset() ||
+					  uv1->getOffset() + uv1->getSize() == uv0->getOffset() ) &&
+					uv0->getSource() == uv1->getSource() )
+				{
+					//Special case where they adjacent, just change the declaration & we're done.
+					size_t newOffset		= std::min( uv0->getOffset(), uv1->getOffset() );
+					unsigned short newIdx	= std::min( uv0->getIndex(), uv1->getIndex() );
+
+					vDecl->modifyElement( elem_idx, uv0->getSource(), newOffset, newType,
+											VES_TEXTURE_COORDINATES, newIdx );
+					vDecl->removeElement( VES_TEXTURE_COORDINATES, texCoordSetToDestroy );
+					uv1 = 0;
+				}
+
+				vDecl->closeGapsInSource();
+			}
+		}
+	}
     //---------------------------------------------------------------------
     void Mesh::organiseTangentsBuffer(VertexData *vertexData,
         VertexElementSemantic targetSemantic, unsigned short index, 
@@ -1264,7 +1248,7 @@ namespace Ogre {
 			{
 				tangentsCalc.clear();
 				tangentsCalc.setVertexData(sm->vertexData);
-				tangentsCalc.addIndexData(sm->indexData);
+                tangentsCalc.addIndexData(sm->indexData, sm->operationType);
 				TangentSpaceCalc::Result res = 
 					tangentsCalc.build(targetSemantic, sourceTexCoordSet, index);
 
@@ -1423,8 +1407,6 @@ namespace Ogre {
             // use getLodLevel to enforce loading of manual mesh lods
             MeshLodUsage& usage = const_cast<MeshLodUsage&>(getLodLevel(lodIndex));
 
-			bool atLeastOneIndexSet = false;
-
             if (mIsLodManual && lodIndex != 0)
             {
                 // Delegate edge building to manual mesh
@@ -1439,6 +1421,7 @@ namespace Ogre {
                 // Build
                 EdgeListBuilder eb;
                 size_t vertexSetCount = 0;
+                bool atLeastOneIndexSet = false;
 
                 if (sharedVertexData)
                 {
@@ -2222,12 +2205,48 @@ namespace Ogre {
         assert(mMeshLodUsageList.size());
         mMeshLodUsageList[0].value = mLodStrategy->getBaseValue();
 
-        // Re-transform user lod values (starting at index 1, no need to transform base value)
+        // Re-transform user LOD values (starting at index 1, no need to transform base value)
 		for (MeshLodUsageList::iterator i = mMeshLodUsageList.begin(); i != mMeshLodUsageList.end(); ++i)
             i->value = mLodStrategy->transformUserValue(i->userValue);
 
     }
-    //---------------------------------------------------------------------
+    //--------------------------------------------------------------------
+    void Mesh::_configureMeshLodUsage( const LodConfig& lodConfig )
+    {
+        // In theory every mesh should have a submesh.
+        assert(getNumSubMeshes() > 0);
+        setLodStrategy(lodConfig.strategy);
+        SubMesh* submesh = getSubMesh(0);
+        mNumLods = submesh->mLodFaceList.size() + 1;
+        mMeshLodUsageList.resize(mNumLods);
+        for (size_t n = 0, i = 0; i < lodConfig.levels.size(); i++) {
+            // Record usages. First LOD usage is the mesh itself.
 
+            // Skip LODs, which have the same amount of vertices. No buffer generated for them.
+            if (!lodConfig.levels[i].outSkipped) {
+                // Generated buffers are less then the reported by ProgressiveMesh.
+                // This would fail if you use QueuedProgressiveMesh and the MeshPtr is force unloaded before LOD generation completes.
+                assert(mMeshLodUsageList.size() > n + 1);
+                MeshLodUsage& lod = mMeshLodUsageList[++n];
+                lod.userValue = lodConfig.levels[i].distance;
+                lod.value = getLodStrategy()->transformUserValue(lod.userValue);
+                lod.edgeData = 0;
+                lod.manualMesh.setNull();
+            }
+        }
+
+        // TODO: Fix this in PixelCountLodStrategy::getIndex()
+        // Fix bug in Ogre with pixel count LOD strategy.
+        // Changes [0, 20, 15, 10, 5] to [max, 20, 15, 10, 5].
+        // Fixes PixelCountLodStrategy::getIndex() function, which returned always 0 index.
+        if (lodConfig.strategy == AbsolutePixelCountLodStrategy::getSingletonPtr()) {
+            mMeshLodUsageList[0].userValue = std::numeric_limits<Real>::max();
+            mMeshLodUsageList[0].value = std::numeric_limits<Real>::max();
+        } else {
+            mMeshLodUsageList[0].userValue = 0;
+            mMeshLodUsageList[0].value = 0;
+        }
+    }
+    //---------------------------------------------------------------------
 }
 
