@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -486,36 +486,44 @@ namespace Ogre {
     {
         // Add up requests for emission
         static vector<unsigned>::type requested;
+        static vector<unsigned>::type emittedRequested;
+
         if( requested.size() != mEmitters.size() )
             requested.resize( mEmitters.size() );
+        if( emittedRequested.size() != mEmittedEmitterPoolSize)
+            emittedRequested.resize( mEmittedEmitterPoolSize );
 
-        size_t totalRequested, emitterCount, i, emissionAllowed;
+        size_t totalRequested, emitterCount, emittedEmitterCount, i, emissionAllowed;
         ParticleEmitterList::iterator itEmit, iEmitEnd;
-		ActiveEmittedEmitterList::iterator itActiveEmit;
+        ActiveEmittedEmitterList::iterator itActiveEmit, itActiveEnd;
+
         iEmitEnd = mEmitters.end();
         emitterCount = mEmitters.size();
+        emittedEmitterCount=mActiveEmittedEmitters.size();
+        itActiveEnd=mActiveEmittedEmitters.end();
         emissionAllowed = mFreeParticles.size();
         totalRequested = 0;
 
         // Count up total requested emissions for regular emitters (and exclude the ones that are used as
-		// a template for emitted emitters)
+        // a template for emitted emitters)
         for (itEmit = mEmitters.begin(), i = 0; itEmit != iEmitEnd; ++itEmit, ++i)
         {
-			if (!(*itEmit)->isEmitted())
-			{
-				requested[i] = (*itEmit)->_getEmissionCount(timeElapsed);
-				totalRequested += requested[i];
-			}
+            if (!(*itEmit)->isEmitted())
+            {
+                requested[i] = (*itEmit)->_getEmissionCount(timeElapsed);
+                totalRequested += requested[i];
+            }
         }
 
-		// Add up total requested emissions for (active) emitted emitters
-		for (itActiveEmit = mActiveEmittedEmitters.begin(); itActiveEmit != mActiveEmittedEmitters.end(); ++itActiveEmit)
-		{
-			totalRequested += (*itActiveEmit)->_getEmissionCount(timeElapsed);
-		}
+        // Add up total requested emissions for (active) emitted emitters
+        for (itActiveEmit = mActiveEmittedEmitters.begin(), i=0; itActiveEmit != itActiveEnd; ++itActiveEmit, ++i)
+        {
+            emittedRequested[i] = (*itActiveEmit)->_getEmissionCount(timeElapsed);
+            totalRequested += emittedRequested[i];
+        }
 
         // Check if the quota will be exceeded, if so reduce demand
-		Real ratio =  1.0f;
+        Real ratio =  1.0f;
         if (totalRequested > emissionAllowed)
         {
             // Apportion down requested values to allotted values
@@ -524,24 +532,28 @@ namespace Ogre {
             {
                 requested[i] = static_cast<unsigned>(requested[i] * ratio);
             }
+            for (i = 0; i < emittedEmitterCount; ++i)
+            {
+                emittedRequested[i] = static_cast<unsigned>(emittedRequested[i] * ratio);
+            }
         }
 
         // Emit
-		// For each emission, apply a subset of the motion for the frame
-		// this ensures an even distribution of particles when many are
-		// emitted in a single frame
+        // For each emission, apply a subset of the motion for the frame
+        // this ensures an even distribution of particles when many are
+        // emitted in a single frame
         for (itEmit = mEmitters.begin(), i = 0; itEmit != iEmitEnd; ++itEmit, ++i)
         {
-			// Trigger the emitters, but exclude the emitters that are already in the emitted emitters list; 
-			// they are handled in a separate loop
-			if (!(*itEmit)->isEmitted())
-				_executeTriggerEmitters (*itEmit, static_cast<unsigned>(requested[i]), timeElapsed);
+            // Trigger the emitters, but exclude the emitters that are already in the emitted emitters list; 
+            // they are handled in a separate loop
+            if (!(*itEmit)->isEmitted())
+                _executeTriggerEmitters (*itEmit, static_cast<unsigned>(requested[i]), timeElapsed);
         }
 
-		// Do the same with all active emitted emitters
-		for (itActiveEmit = mActiveEmittedEmitters.begin(), i = 0; itActiveEmit != mActiveEmittedEmitters.end(); ++itActiveEmit, ++i)
-			_executeTriggerEmitters (*itActiveEmit, static_cast<unsigned>((*itActiveEmit)->_getEmissionCount(timeElapsed) * ratio), timeElapsed);
-	}
+        // Do the same with all active emitted emitters
+        for (itActiveEmit = mActiveEmittedEmitters.begin(), i = 0; itActiveEmit != mActiveEmittedEmitters.end(); ++itActiveEmit, ++i)
+            _executeTriggerEmitters (*itActiveEmit, emittedRequested[i], timeElapsed);
+    }
     //-----------------------------------------------------------------------
     void ParticleSystem::_executeTriggerEmitters(ParticleEmitter* emitter, unsigned requested, Real timeElapsed)
     {
@@ -1013,7 +1025,7 @@ namespace Ogre {
         if (mIsRendererConfigured)
         {
             MaterialPtr mat = MaterialManager::getSingleton().load(
-                mMaterialName, mResourceGroupName);
+                mMaterialName, mResourceGroupName).staticCast<Material>();
             mRenderer->_setMaterial(mat);
         }
     }
@@ -1090,7 +1102,7 @@ namespace Ogre {
             mRenderer->_notifyDefaultDimensions(mDefaultWidth, mDefaultHeight);
             createVisualParticles(0, mParticlePool.size());
             MaterialPtr mat = MaterialManager::getSingleton().load(
-                mMaterialName, mResourceGroupName);
+                mMaterialName, mResourceGroupName).staticCast<Material>();
             mRenderer->_setMaterial(mat);
 			if (mRenderQueueIDSet)
 				mRenderer->setRenderQueueGroup(mRenderQueueID);
@@ -1174,6 +1186,15 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------
+	void ParticleSystem::setRenderQueueGroupAndPriority(uint8 queueID, ushort priority)
+	{
+		MovableObject::setRenderQueueGroupAndPriority(queueID, priority);
+		if (mRenderer)
+		{
+			mRenderer->setRenderQueueGroupAndPriority(queueID, priority);
+		}
+	}
+	//-----------------------------------------------------------------------
 	void ParticleSystem::setKeepParticlesInLocalSpace(bool keepLocal)
 	{
 		mLocalSpace = keepLocal;
@@ -1279,12 +1300,11 @@ namespace Ogre {
 		// Run through mEmitters and add keys to the pool
 		ParticleEmitterList::iterator emitterIterator;
 		ParticleEmitterList::iterator emitterIteratorInner;
-		ParticleEmitter* emitter = 0;
 		ParticleEmitter* emitterInner = 0;
 		for (emitterIterator = mEmitters.begin(); emitterIterator != mEmitters.end(); ++emitterIterator)
 		{
 			// Determine the names of all emitters that are emitted
-			emitter = *emitterIterator ;
+			ParticleEmitter* emitter = *emitterIterator ;
 			if (emitter && emitter->getEmittedEmitter() != StringUtil::BLANK)
 			{
 				// This one will be emitted, register its name and leave the vector empty!
@@ -1304,7 +1324,7 @@ namespace Ogre {
 					emitter->setEmitted(true);
 					break;
 				}
-				else
+				else if(emitter)
 				{
 					// Set explicitly to 'false' although the default value is already 'false'
 					emitter->setEmitted(false);
@@ -1323,7 +1343,6 @@ namespace Ogre {
 
 		EmittedEmitterPool::iterator emittedEmitterPoolIterator;
 		ParticleEmitterList::iterator emitterIterator;
-		ParticleEmitter* emitter = 0;
 		ParticleEmitter* clonedEmitter = 0;
 		String name = StringUtil::BLANK;
 		EmittedEmitterList* e = 0;
@@ -1337,7 +1356,7 @@ namespace Ogre {
 			e = &emittedEmitterPoolIterator->second;
 
 			// Search the correct emitter in the mEmitters vector
-			emitter = 0;
+			ParticleEmitter* emitter = 0;
 			for (emitterIterator = mEmitters.begin(); emitterIterator != mEmitters.end(); ++emitterIterator)
 			{
 				emitter = *emitterIterator;
@@ -1355,7 +1374,7 @@ namespace Ogre {
 
 						// Initially deactivate the emitted emitter if duration/repeat_delay are set
 						if (clonedEmitter->getDuration() > 0.0f && 
-							(clonedEmitter->getRepeatDelay() > 0.0f || clonedEmitter->getMinRepeatDelay() > 0.0f || clonedEmitter->getMinRepeatDelay() > 0.0f))
+							(clonedEmitter->getRepeatDelay() > 0.0f || clonedEmitter->getMinRepeatDelay() > 0.0f))
 							clonedEmitter->setEnabled(false);
 
 						// Add cloned emitters to the pool
@@ -1421,7 +1440,7 @@ namespace Ogre {
 			e->clear();
         }
 
-		// Dont leave any references behind
+		// Don't leave any references behind
 		mEmittedEmitterPool.clear();
 		mFreeEmittedEmitters.clear();
 		mActiveEmittedEmitters.clear();
@@ -1468,7 +1487,7 @@ namespace Ogre {
 	void ParticleSystem::_notifyReorganiseEmittedEmitterData (void)
 	{
 		removeAllEmittedEmitters();
-		mEmittedEmitterPoolInitialised = false; // Dont rearrange immediately; it will be performed in the regular flow
+		mEmittedEmitterPoolInitialised = false; // Don't rearrange immediately; it will be performed in the regular flow
 	}
     //-----------------------------------------------------------------------
     String ParticleSystem::CmdCull::doGet(const void* target) const
