@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -51,7 +51,7 @@ namespace Ogre {
         removeAll();
     }
 	//-----------------------------------------------------------------------
-    ResourcePtr ResourceManager::create(const String& name, const String& group, 
+	ResourcePtr ResourceManager::createResource(const String& name, const String& group,
 		bool isManual, ManualResourceLoader* loader, const NameValuePairList* params)
 	{
 		// Call creation implementation
@@ -74,14 +74,14 @@ namespace Ogre {
 		const NameValuePairList* params)
 	{
 		// Lock for the whole get / insert
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 
-		ResourcePtr res = getByName(name, group);
+		ResourcePtr res = getResourceByName(name, group);
 		bool created = false;
 		if (res.isNull())
 		{
 			created = true;
-			res = create(name, group, isManual, loader, params);
+			res = createResource(name, group, isManual, loader, params);
 		}
 
 		return ResourceCreateOrRetrieveResult(res, created);
@@ -104,12 +104,13 @@ namespace Ogre {
         ResourcePtr r = createOrRetrieve(name,group,isManual,loader,loadParams).first;
 		// ensure loaded
         r->load(backgroundThread);
+
         return r;
     }
     //-----------------------------------------------------------------------
 	void ResourceManager::addImpl( ResourcePtr& res )
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 
 			std::pair<ResourceMap::iterator, bool> result;
 		if(ResourceGroupManager::getSingleton().isResourceGroupInGlobalPool(res->getGroup()))
@@ -182,7 +183,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceManager::removeImpl( ResourcePtr& res )
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 
 		if(ResourceGroupManager::getSingleton().isResourceGroupInGlobalPool(res->getGroup()))
 		{
@@ -233,7 +234,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceManager::unload(const String& name)
 	{
-		ResourcePtr res = getByName(name);
+		ResourcePtr res = getResourceByName(name);
 
 		if (!res.isNull())
 		{
@@ -257,7 +258,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceManager::unloadAll(bool reloadableOnly)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 
 		ResourceMap::iterator i, iend;
 		iend = mResources.end();
@@ -273,7 +274,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceManager::reloadAll(bool reloadableOnly)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 
 		ResourceMap::iterator i, iend;
 		iend = mResources.end();
@@ -289,7 +290,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void ResourceManager::unloadUnreferencedResources(bool reloadableOnly)
     {
-        OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 
         ResourceMap::iterator i, iend;
         iend = mResources.end();
@@ -310,7 +311,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void ResourceManager::reloadUnreferencedResources(bool reloadableOnly)
     {
-        OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 
         ResourceMap::iterator i, iend;
         iend = mResources.end();
@@ -336,7 +337,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceManager::remove(const String& name)
 	{
-		ResourcePtr res = getByName(name);
+		ResourcePtr res = getResourceByName(name);
 
 		if (!res.isNull())
 		{
@@ -356,7 +357,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceManager::removeAll(void)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 
 		mResources.clear();
 		mResourcesWithGroup.clear();
@@ -367,7 +368,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void ResourceManager::removeUnreferencedResources(bool reloadableOnly)
     {
-        OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 
         ResourceMap::iterator i, iend;
         iend = mResources.end();
@@ -390,14 +391,14 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    ResourcePtr ResourceManager::getByName(const String& name, const String& groupName /* = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME */)
+    ResourcePtr ResourceManager::getResourceByName(const String& name, const String& groupName /* = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME */)
     {		
 		ResourcePtr res;
 
 		// if not in the global pool - get it from the grouped pool 
 		if(!ResourceGroupManager::getSingleton().isResourceGroupInGlobalPool(groupName))
 		{
-			OGRE_LOCK_AUTO_MUTEX
+                    OGRE_LOCK_AUTO_MUTEX;
 			ResourceWithGroupMap::iterator itGroup = mResourcesWithGroup.find(groupName);
 
 			if( itGroup != mResourcesWithGroup.end())
@@ -414,7 +415,7 @@ namespace Ogre {
 		// if didn't find it the grouped pool - get it from the global pool 
 		if (res.isNull())
 		{
-			OGRE_LOCK_AUTO_MUTEX
+                    OGRE_LOCK_AUTO_MUTEX;
 
 			ResourceMap::iterator it = mResources.find(name);
 
@@ -429,7 +430,7 @@ namespace Ogre {
 				{
 					ResourceWithGroupMap::iterator iter = mResourcesWithGroup.begin();
 					ResourceWithGroupMap::iterator iterE = mResourcesWithGroup.end();
-					for ( ; iter != iterE ; iter++ )
+					for ( ; iter != iterE ; ++iter )
 					{
 						ResourceMap::iterator resMapIt = iter->second.find(name);
 
@@ -448,7 +449,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     ResourcePtr ResourceManager::getByHandle(ResourceHandle handle)
     {
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 
         ResourceHandleMap::iterator it = mResourcesByHandle.find(handle);
         if (it == mResourcesByHandle.end())
@@ -463,15 +464,34 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     ResourceHandle ResourceManager::getNextHandle(void)
     {
-		OGRE_LOCK_AUTO_MUTEX
-
+        // This is an atomic operation and hence needs no locking
         return mNextHandle++;
     }
     //-----------------------------------------------------------------------
     void ResourceManager::checkUsage(void)
     {
-        // TODO Page out here?
-    }
+		if (getMemoryUsage() > mMemoryBudget)
+		{
+                    OGRE_LOCK_AUTO_MUTEX;
+			// unload unreferenced resources until we are within our budget again
+			const bool reloadableOnly = true;
+			ResourceMap::iterator i, iend;
+			iend = mResources.end();
+			for (i = mResources.begin(); i != iend && getMemoryUsage() > mMemoryBudget; ++i)
+			{
+				// A use count of 3 means that only RGM and RM have references
+				// RGM has one (this one) and RM has 2 (by name and by handle)
+				if (i->second.useCount() == ResourceGroupManager::RESOURCE_SYSTEM_NUM_REFERENCE_COUNTS)
+				{
+					Resource* res = i->second.get();
+					if (!reloadableOnly || res->isReloadable())
+					{
+						res->unload();
+					}
+				}
+			}
+		}
+	}
 	//-----------------------------------------------------------------------
 	void ResourceManager::_notifyResourceTouched(Resource* res)
 	{
@@ -480,21 +500,18 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceManager::_notifyResourceLoaded(Resource* res)
 	{
-		OGRE_LOCK_AUTO_MUTEX
-
 		mMemoryUsage += res->getSize();
+		checkUsage();
 	}
 	//-----------------------------------------------------------------------
 	void ResourceManager::_notifyResourceUnloaded(Resource* res)
 	{
-		OGRE_LOCK_AUTO_MUTEX
-
 		mMemoryUsage -= res->getSize();
 	}
 	//---------------------------------------------------------------------
 	ResourceManager::ResourcePool* ResourceManager::getResourcePool(const String& name)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 
 		ResourcePoolMap::iterator i = mResourcePoolMap.find(name);
 		if (i == mResourcePoolMap.end())
@@ -508,7 +525,10 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void ResourceManager::destroyResourcePool(ResourcePool* pool)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+        if(!pool)
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Cannot destroy a null ResourcePool.", "ResourceManager::destroyResourcePool");
+
+        OGRE_LOCK_AUTO_MUTEX;
 
 		ResourcePoolMap::iterator i = mResourcePoolMap.find(pool->getName());
 		if (i != mResourcePoolMap.end())
@@ -520,7 +540,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void ResourceManager::destroyResourcePool(const String& name)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 
 		ResourcePoolMap::iterator i = mResourcePoolMap.find(name);
 		if (i != mResourcePoolMap.end())
@@ -533,7 +553,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void ResourceManager::destroyAllResourcePools()
 	{
-		OGRE_LOCK_AUTO_MUTEX
+        OGRE_LOCK_AUTO_MUTEX;
 
 		for (ResourcePoolMap::iterator i = mResourcePoolMap.begin(); 
 			i != mResourcePoolMap.end(); ++i)
@@ -561,7 +581,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void ResourceManager::ResourcePool::clear()
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 		for (ItemList::iterator i = mItems.begin(); i != mItems.end(); ++i)
 		{
 			(*i)->getCreator()->remove((*i)->getHandle());
