@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -233,7 +233,7 @@ namespace Ogre
 			Ogre::StringConverter::toString(line) + ")";
 		if(!msg.empty())
 			str = str + ": " + msg;
-		Ogre::LogManager::getSingleton().logMessage(str);
+		Ogre::LogManager::getSingleton().logMessage(str, LML_CRITICAL);
 	}
 
 	bool ScriptCompilerListener::handleEvent(ScriptCompiler *compiler, ScriptCompilerEvent *evt, void *retval)
@@ -287,57 +287,57 @@ namespace Ogre
 		return compile(nodes, group);
 	}
 
-	static void logAST(int tabs, const AbstractNodePtr &node)
-	{
-		String msg = "";
-		for(int i = 0; i < tabs; ++i)
-			msg += "\t";
-
-		switch(node->type)
-		{
-		case ANT_ATOM:
-			{
-				AtomAbstractNode *atom = reinterpret_cast<AtomAbstractNode*>(node.get());
-				msg = msg + atom->value;
-			}
-			break;
-		case ANT_PROPERTY:
-			{
-				PropertyAbstractNode *prop = reinterpret_cast<PropertyAbstractNode*>(node.get());
-				msg = msg + prop->name + " =";
-				for(AbstractNodeList::iterator i = prop->values.begin(); i != prop->values.end(); ++i)
-				{
-					if((*i)->type == ANT_ATOM)
-						msg = msg + " " + reinterpret_cast<AtomAbstractNode*>((*i).get())->value;
-				}
-			}
-			break;
-		case ANT_OBJECT:
-			{
-				ObjectAbstractNode *obj = reinterpret_cast<ObjectAbstractNode*>(node.get());
-				msg = msg + node->file + " - " + StringConverter::toString(node->line) + " - " + obj->cls + " \"" + obj->name + "\" =";
-				for(AbstractNodeList::iterator i = obj->values.begin(); i != obj->values.end(); ++i)
-				{
-					if((*i)->type == ANT_ATOM)
-						msg = msg + " " + reinterpret_cast<AtomAbstractNode*>((*i).get())->value;
-				}
-			}
-			break;
-		default:
-			msg = msg + "Unacceptable node type: " + StringConverter::toString(node->type);
-		}
-
-		LogManager::getSingleton().logMessage(msg);
-
-		if(node->type == ANT_OBJECT)
-		{
-			ObjectAbstractNode *obj = reinterpret_cast<ObjectAbstractNode*>(node.get());
-			for(AbstractNodeList::iterator i = obj->children.begin(); i != obj->children.end(); ++i)
-			{
-				logAST(tabs + 1, *i);
-			}
-		}
-	}
+//	static void logAST(int tabs, const AbstractNodePtr &node)
+//	{
+//		String msg = "";
+//		for(int i = 0; i < tabs; ++i)
+//			msg += "\t";
+//
+//		switch(node->type)
+//		{
+//		case ANT_ATOM:
+//			{
+//				AtomAbstractNode *atom = reinterpret_cast<AtomAbstractNode*>(node.get());
+//				msg = msg + atom->value;
+//			}
+//			break;
+//		case ANT_PROPERTY:
+//			{
+//				PropertyAbstractNode *prop = reinterpret_cast<PropertyAbstractNode*>(node.get());
+//				msg = msg + prop->name + " =";
+//				for(AbstractNodeList::iterator i = prop->values.begin(); i != prop->values.end(); ++i)
+//				{
+//					if((*i)->type == ANT_ATOM)
+//						msg = msg + " " + reinterpret_cast<AtomAbstractNode*>((*i).get())->value;
+//				}
+//			}
+//			break;
+//		case ANT_OBJECT:
+//			{
+//				ObjectAbstractNode *obj = reinterpret_cast<ObjectAbstractNode*>(node.get());
+//				msg = msg + node->file + " - " + StringConverter::toString(node->line) + " - " + obj->cls + " \"" + obj->name + "\" =";
+//				for(AbstractNodeList::iterator i = obj->values.begin(); i != obj->values.end(); ++i)
+//				{
+//					if((*i)->type == ANT_ATOM)
+//						msg = msg + " " + reinterpret_cast<AtomAbstractNode*>((*i).get())->value;
+//				}
+//			}
+//			break;
+//		default:
+//			msg = msg + "Unacceptable node type: " + StringConverter::toString(node->type);
+//		}
+//
+//		LogManager::getSingleton().logMessage(msg);
+//
+//		if(node->type == ANT_OBJECT)
+//		{
+//			ObjectAbstractNode *obj = reinterpret_cast<ObjectAbstractNode*>(node.get());
+//			for(AbstractNodeList::iterator i = obj->children.begin(); i != obj->children.end(); ++i)
+//			{
+//				logAST(tabs + 1, *i);
+//			}
+//		}
+//	}
 
 	bool ScriptCompiler::compile(const ConcreteNodeListPtr &nodes, const String &group)
 	{
@@ -365,13 +365,14 @@ namespace Ogre
 		// Allows early bail-out through the listener
 		if(mListener && !mListener->postConversion(this, ast))
 			return mErrors.empty();
-
+		
 		// Translate the nodes
 		for(AbstractNodeList::iterator i = ast->begin(); i != ast->end(); ++i)
 		{
 			//logAST(0, *i);
 			if((*i)->type == ANT_OBJECT && reinterpret_cast<ObjectAbstractNode*>((*i).get())->abstract)
 				continue;
+			//LogManager::getSingleton().logMessage(reinterpret_cast<ObjectAbstractNode*>((*i).get())->name);
 			ScriptTranslator *translator = ScriptCompilerManager::getSingleton().getTranslator(*i);
 			if(translator)
 				translator->translate(this, *i);
@@ -464,7 +465,7 @@ namespace Ogre
 				Ogre::StringConverter::toString(line) + ")";
 			if(!msg.empty())
 				str = str + ": " + msg;
-			Ogre::LogManager::getSingleton().logMessage(str);
+			Ogre::LogManager::getSingleton().logMessage(str, LML_CRITICAL);
 		}
 
 		mErrors.push_back(err);
@@ -502,7 +503,7 @@ namespace Ogre
 	void ScriptCompiler::processImports(Ogre::AbstractNodeListPtr &nodes)
 	{
 		// We only need to iterate over the top-level of nodes
-		AbstractNodeList::iterator i = nodes->begin(), last = nodes->end();
+		AbstractNodeList::iterator i = nodes->begin();
 		while(i != nodes->end())
 		{
 			// We move to the next node here and save the current one.
@@ -638,7 +639,7 @@ namespace Ogre
 				ObjectAbstractNode *obj = (ObjectAbstractNode*)(*i).get();
 
 				// Overlay base classes in order.
-                for (std::vector<String>::const_iterator baseIt = obj->bases.begin(), end_it = obj->bases.end(); baseIt != end_it; ++baseIt)
+                for (vector<String>::const_iterator baseIt = obj->bases.begin(), end_it = obj->bases.end(); baseIt != end_it; ++baseIt)
 				{
                     const String& base = *baseIt;
 					// Check the top level first, then check the import table
@@ -992,13 +993,20 @@ namespace Ogre
 		mIds["vertex_program"] = ID_VERTEX_PROGRAM;
 		mIds["geometry_program"] = ID_GEOMETRY_PROGRAM;
 		mIds["fragment_program"] = ID_FRAGMENT_PROGRAM;
+		mIds["tesselation_hull_program"] = ID_TESSELATION_HULL_PROGRAM;
+		mIds["tesselation_domain_program"] = ID_TESSELATION_DOMAIN_PROGRAM;
+		mIds["compute_program"] = ID_COMPUTE_PROGRAM;
 		mIds["technique"] = ID_TECHNIQUE;
 		mIds["pass"] = ID_PASS;
 		mIds["texture_unit"] = ID_TEXTURE_UNIT;
 		mIds["vertex_program_ref"] = ID_VERTEX_PROGRAM_REF;
 		mIds["geometry_program_ref"] = ID_GEOMETRY_PROGRAM_REF;
 		mIds["fragment_program_ref"] = ID_FRAGMENT_PROGRAM_REF;
+		mIds["tesselation_hull_program_ref"] = ID_TESSELATION_HULL_PROGRAM_REF;
+		mIds["tesselation_domain_program_ref"] = ID_TESSELATION_DOMAIN_PROGRAM_REF;
+		mIds["compute_program_ref"] = ID_COMPUTE_PROGRAM_REF;
 		mIds["shadow_caster_vertex_program_ref"] = ID_SHADOW_CASTER_VERTEX_PROGRAM_REF;
+		mIds["shadow_caster_fragment_program_ref"] = ID_SHADOW_CASTER_FRAGMENT_PROGRAM_REF;
 		mIds["shadow_receiver_vertex_program_ref"] = ID_SHADOW_RECEIVER_VERTEX_PROGRAM_REF;
 		mIds["shadow_receiver_fragment_program_ref"] = ID_SHADOW_RECEIVER_FRAGMENT_PROGRAM_REF;
 
@@ -1026,84 +1034,83 @@ namespace Ogre
 		mIds["include"] = ID_INCLUDE;
 		mIds["exclude"] = ID_EXCLUDE;
 
-
-
 		mIds["ambient"] = ID_AMBIENT;
 		mIds["diffuse"] = ID_DIFFUSE;
 		mIds["specular"] = ID_SPECULAR;
 		mIds["emissive"] = ID_EMISSIVE;
-			mIds["vertexcolour"] = ID_VERTEXCOLOUR;
+        mIds["vertexcolour"] = ID_VERTEXCOLOUR;
 		mIds["scene_blend"] = ID_SCENE_BLEND;
 		mIds["colour_blend"] = ID_COLOUR_BLEND;
-			mIds["one"] = ID_ONE;
-			mIds["zero"] = ID_ZERO;
-			mIds["dest_colour"] = ID_DEST_COLOUR;
-			mIds["src_colour"] = ID_SRC_COLOUR;
-			mIds["one_minus_src_colour"] = ID_ONE_MINUS_SRC_COLOUR;
-			mIds["one_minus_dest_colour"] = ID_ONE_MINUS_DEST_COLOUR;
-			mIds["dest_alpha"] = ID_DEST_ALPHA;
-			mIds["src_alpha"] = ID_SRC_ALPHA;
-			mIds["one_minus_dest_alpha"] = ID_ONE_MINUS_DEST_ALPHA;
-			mIds["one_minus_src_alpha"] = ID_ONE_MINUS_SRC_ALPHA;
+        mIds["one"] = ID_ONE;
+        mIds["zero"] = ID_ZERO;
+        mIds["dest_colour"] = ID_DEST_COLOUR;
+        mIds["src_colour"] = ID_SRC_COLOUR;
+        mIds["one_minus_src_colour"] = ID_ONE_MINUS_SRC_COLOUR;
+        mIds["one_minus_dest_colour"] = ID_ONE_MINUS_DEST_COLOUR;
+        mIds["dest_alpha"] = ID_DEST_ALPHA;
+        mIds["src_alpha"] = ID_SRC_ALPHA;
+        mIds["one_minus_dest_alpha"] = ID_ONE_MINUS_DEST_ALPHA;
+        mIds["one_minus_src_alpha"] = ID_ONE_MINUS_SRC_ALPHA;
 		mIds["separate_scene_blend"] = ID_SEPARATE_SCENE_BLEND;
 		mIds["scene_blend_op"] = ID_SCENE_BLEND_OP;
-			mIds["reverse_subtract"] = ID_REVERSE_SUBTRACT;
-			mIds["min"] = ID_MIN;
-			mIds["max"] = ID_MAX;
+        mIds["reverse_subtract"] = ID_REVERSE_SUBTRACT;
+        mIds["min"] = ID_MIN;
+        mIds["max"] = ID_MAX;
 		mIds["separate_scene_blend_op"] = ID_SEPARATE_SCENE_BLEND_OP;
 		mIds["depth_check"] = ID_DEPTH_CHECK;
 		mIds["depth_write"] = ID_DEPTH_WRITE;
 		mIds["depth_func"] = ID_DEPTH_FUNC;
 		mIds["depth_bias"] = ID_DEPTH_BIAS;
 		mIds["iteration_depth_bias"] = ID_ITERATION_DEPTH_BIAS;
-			mIds["always_fail"] = ID_ALWAYS_FAIL;
-			mIds["always_pass"] = ID_ALWAYS_PASS;
-			mIds["less_equal"] = ID_LESS_EQUAL;
-			mIds["less"] = ID_LESS;
-			mIds["equal"] = ID_EQUAL;
-			mIds["not_equal"] = ID_NOT_EQUAL;
-			mIds["greater_equal"] = ID_GREATER_EQUAL;
-			mIds["greater"] = ID_GREATER;
+        mIds["always_fail"] = ID_ALWAYS_FAIL;
+        mIds["always_pass"] = ID_ALWAYS_PASS;
+        mIds["less_equal"] = ID_LESS_EQUAL;
+        mIds["less"] = ID_LESS;
+        mIds["equal"] = ID_EQUAL;
+        mIds["not_equal"] = ID_NOT_EQUAL;
+        mIds["greater_equal"] = ID_GREATER_EQUAL;
+        mIds["greater"] = ID_GREATER;
 		mIds["alpha_rejection"] = ID_ALPHA_REJECTION;
 		mIds["alpha_to_coverage"] = ID_ALPHA_TO_COVERAGE;
 		mIds["light_scissor"] = ID_LIGHT_SCISSOR;
 		mIds["light_clip_planes"] = ID_LIGHT_CLIP_PLANES;
 		mIds["transparent_sorting"] = ID_TRANSPARENT_SORTING;
 		mIds["illumination_stage"] = ID_ILLUMINATION_STAGE;
-			mIds["decal"] = ID_DECAL;
+        mIds["decal"] = ID_DECAL;
 		mIds["cull_hardware"] = ID_CULL_HARDWARE;
-			mIds["clockwise"] = ID_CLOCKWISE;
-			mIds["anticlockwise"] = ID_ANTICLOCKWISE;
+        mIds["clockwise"] = ID_CLOCKWISE;
+        mIds["anticlockwise"] = ID_ANTICLOCKWISE;
 		mIds["cull_software"] = ID_CULL_SOFTWARE;
-			mIds["back"] = ID_BACK;
-			mIds["front"] = ID_FRONT;
+        mIds["back"] = ID_BACK;
+        mIds["front"] = ID_FRONT;
 		mIds["normalise_normals"] = ID_NORMALISE_NORMALS;
 		mIds["lighting"] = ID_LIGHTING;
 		mIds["shading"] = ID_SHADING;
-			mIds["flat"] = ID_FLAT;
-			mIds["gouraud"] = ID_GOURAUD;
-			mIds["phong"] = ID_PHONG;
+        mIds["flat"] = ID_FLAT;
+        mIds["gouraud"] = ID_GOURAUD;
+        mIds["phong"] = ID_PHONG;
 		mIds["polygon_mode"] = ID_POLYGON_MODE;
-			mIds["solid"] = ID_SOLID;
-			mIds["wireframe"] = ID_WIREFRAME;
-			mIds["points"] = ID_POINTS;
+        mIds["solid"] = ID_SOLID;
+        mIds["wireframe"] = ID_WIREFRAME;
+        mIds["points"] = ID_POINTS;
 		mIds["polygon_mode_overrideable"] = ID_POLYGON_MODE_OVERRIDEABLE;
 		mIds["fog_override"] = ID_FOG_OVERRIDE;
-			mIds["none"] = ID_NONE;
-			mIds["linear"] = ID_LINEAR;
-			mIds["exp"] = ID_EXP;
-			mIds["exp2"] = ID_EXP2;
+        mIds["none"] = ID_NONE;
+        mIds["linear"] = ID_LINEAR;
+        mIds["exp"] = ID_EXP;
+        mIds["exp2"] = ID_EXP2;
 		mIds["colour_write"] = ID_COLOUR_WRITE;
 		mIds["max_lights"] = ID_MAX_LIGHTS;
 		mIds["start_light"] = ID_START_LIGHT;
 		mIds["iteration"] = ID_ITERATION;
-			mIds["once"] = ID_ONCE;
-			mIds["once_per_light"] = ID_ONCE_PER_LIGHT;
-			mIds["per_n_lights"] = ID_PER_N_LIGHTS;
-			mIds["per_light"] = ID_PER_LIGHT;
-			mIds["point"] = ID_POINT;
-			mIds["spot"] = ID_SPOT;
-			mIds["directional"] = ID_DIRECTIONAL;
+        mIds["once"] = ID_ONCE;
+        mIds["once_per_light"] = ID_ONCE_PER_LIGHT;
+        mIds["per_n_lights"] = ID_PER_N_LIGHTS;
+        mIds["per_light"] = ID_PER_LIGHT;
+        mIds["point"] = ID_POINT;
+        mIds["spot"] = ID_SPOT;
+        mIds["directional"] = ID_DIRECTIONAL;
+		mIds["light_mask"] = ID_LIGHT_MASK;
 		mIds["point_size"] = ID_POINT_SIZE;
 		mIds["point_sprites"] = ID_POINT_SPRITES;
 		mIds["point_size_min"] = ID_POINT_SIZE_MIN;
@@ -1112,91 +1119,97 @@ namespace Ogre
 
 		mIds["texture_alias"] = ID_TEXTURE_ALIAS;
 		mIds["texture"] = ID_TEXTURE;
-			mIds["1d"] = ID_1D;
-			mIds["2d"] = ID_2D;
-			mIds["3d"] = ID_3D;
-			mIds["cubic"] = ID_CUBIC;
-			mIds["unlimited"] = ID_UNLIMITED;
-			mIds["alpha"] = ID_ALPHA;
-			mIds["gamma"] = ID_GAMMA;
+        mIds["1d"] = ID_1D;
+        mIds["2d"] = ID_2D;
+        mIds["3d"] = ID_3D;
+        mIds["cubic"] = ID_CUBIC;
+        mIds["unlimited"] = ID_UNLIMITED;
+		mIds["2darray"] = ID_2DARRAY;
+        mIds["alpha"] = ID_ALPHA;
+        mIds["gamma"] = ID_GAMMA;
 		mIds["anim_texture"] = ID_ANIM_TEXTURE;
 		mIds["cubic_texture"] = ID_CUBIC_TEXTURE;
-			mIds["separateUV"] = ID_SEPARATE_UV;
-			mIds["combinedUVW"] = ID_COMBINED_UVW;
+        mIds["separateUV"] = ID_SEPARATE_UV;
+        mIds["combinedUVW"] = ID_COMBINED_UVW;
 		mIds["tex_coord_set"] = ID_TEX_COORD_SET;
 		mIds["tex_address_mode"] = ID_TEX_ADDRESS_MODE;
-			mIds["wrap"] = ID_WRAP;
-			mIds["clamp"] = ID_CLAMP;
-			mIds["mirror"] = ID_MIRROR;
-			mIds["border"] = ID_BORDER;
+        mIds["wrap"] = ID_WRAP;
+        mIds["clamp"] = ID_CLAMP;
+        mIds["mirror"] = ID_MIRROR;
+        mIds["border"] = ID_BORDER;
 		mIds["tex_border_colour"] = ID_TEX_BORDER_COLOUR;
 		mIds["filtering"] = ID_FILTERING;
-			mIds["bilinear"] = ID_BILINEAR;
-			mIds["trilinear"] = ID_TRILINEAR;
-			mIds["anisotropic"] = ID_ANISOTROPIC;
+        mIds["bilinear"] = ID_BILINEAR;
+        mIds["trilinear"] = ID_TRILINEAR;
+        mIds["anisotropic"] = ID_ANISOTROPIC;
+		mIds["compare_test"] = ID_CMPTEST;
+		mIds["compare_func"] = ID_CMPFUNC;
 		mIds["max_anisotropy"] = ID_MAX_ANISOTROPY;
 		mIds["mipmap_bias"] = ID_MIPMAP_BIAS;
 		mIds["colour_op"] = ID_COLOUR_OP;
-			mIds["replace"] = ID_REPLACE;
-			mIds["add"] = ID_ADD;
-			mIds["modulate"] = ID_MODULATE;
-			mIds["alpha_blend"] = ID_ALPHA_BLEND;
+        mIds["replace"] = ID_REPLACE;
+        mIds["add"] = ID_ADD;
+        mIds["modulate"] = ID_MODULATE;
+        mIds["alpha_blend"] = ID_ALPHA_BLEND;
 		mIds["colour_op_ex"] = ID_COLOUR_OP_EX;
-			mIds["source1"] = ID_SOURCE1;
-			mIds["source2"] = ID_SOURCE2;
-			mIds["modulate"] = ID_MODULATE;
-			mIds["modulate_x2"] = ID_MODULATE_X2;
-			mIds["modulate_x4"] = ID_MODULATE_X4;
-			mIds["add"] = ID_ADD;
-			mIds["add_signed"] = ID_ADD_SIGNED;
-			mIds["add_smooth"] = ID_ADD_SMOOTH;
-			mIds["subtract"] = ID_SUBTRACT;
-			mIds["blend_diffuse_alpha"] = ID_BLEND_DIFFUSE_ALPHA;
-			mIds["blend_texture_alpha"] = ID_BLEND_TEXTURE_ALPHA;
-			mIds["blend_current_alpha"] = ID_BLEND_CURRENT_ALPHA;
-			mIds["blend_manual"] = ID_BLEND_MANUAL;
-			mIds["dotproduct"] = ID_DOT_PRODUCT;
-			mIds["blend_diffuse_colour"] = ID_BLEND_DIFFUSE_COLOUR;
-			mIds["src_current"] = ID_SRC_CURRENT;
-			mIds["src_texture"] = ID_SRC_TEXTURE;
-			mIds["src_diffuse"] = ID_SRC_DIFFUSE;
-			mIds["src_specular"] = ID_SRC_SPECULAR;
-			mIds["src_manual"] = ID_SRC_MANUAL;
+        mIds["source1"] = ID_SOURCE1;
+        mIds["source2"] = ID_SOURCE2;
+        mIds["modulate"] = ID_MODULATE;
+        mIds["modulate_x2"] = ID_MODULATE_X2;
+        mIds["modulate_x4"] = ID_MODULATE_X4;
+        mIds["add"] = ID_ADD;
+        mIds["add_signed"] = ID_ADD_SIGNED;
+        mIds["add_smooth"] = ID_ADD_SMOOTH;
+        mIds["subtract"] = ID_SUBTRACT;
+        mIds["blend_diffuse_alpha"] = ID_BLEND_DIFFUSE_ALPHA;
+        mIds["blend_texture_alpha"] = ID_BLEND_TEXTURE_ALPHA;
+        mIds["blend_current_alpha"] = ID_BLEND_CURRENT_ALPHA;
+        mIds["blend_manual"] = ID_BLEND_MANUAL;
+        mIds["dotproduct"] = ID_DOT_PRODUCT;
+        mIds["blend_diffuse_colour"] = ID_BLEND_DIFFUSE_COLOUR;
+        mIds["src_current"] = ID_SRC_CURRENT;
+        mIds["src_texture"] = ID_SRC_TEXTURE;
+        mIds["src_diffuse"] = ID_SRC_DIFFUSE;
+        mIds["src_specular"] = ID_SRC_SPECULAR;
+        mIds["src_manual"] = ID_SRC_MANUAL;
 		mIds["colour_op_multipass_fallback"] = ID_COLOUR_OP_MULTIPASS_FALLBACK;
 		mIds["alpha_op_ex"] = ID_ALPHA_OP_EX;
 		mIds["env_map"] = ID_ENV_MAP;
-			mIds["spherical"] = ID_SPHERICAL;
-			mIds["planar"] = ID_PLANAR;
-			mIds["cubic_reflection"] = ID_CUBIC_REFLECTION;
-			mIds["cubic_normal"] = ID_CUBIC_NORMAL;
+        mIds["spherical"] = ID_SPHERICAL;
+        mIds["planar"] = ID_PLANAR;
+        mIds["cubic_reflection"] = ID_CUBIC_REFLECTION;
+        mIds["cubic_normal"] = ID_CUBIC_NORMAL;
 		mIds["scroll"] = ID_SCROLL;
 		mIds["scroll_anim"] = ID_SCROLL_ANIM;
 		mIds["rotate"] = ID_ROTATE;
 		mIds["rotate_anim"] = ID_ROTATE_ANIM;
 		mIds["scale"] = ID_SCALE;
 		mIds["wave_xform"] = ID_WAVE_XFORM;
-			mIds["scroll_x"] = ID_SCROLL_X;
-			mIds["scroll_y"] = ID_SCROLL_Y;
-			mIds["scale_x"] = ID_SCALE_X;
-			mIds["scale_y"] = ID_SCALE_Y;
-			mIds["sine"] = ID_SINE;
-			mIds["triangle"] = ID_TRIANGLE;
-			mIds["sawtooth"] = ID_SAWTOOTH;
-			mIds["square"] = ID_SQUARE;
-			mIds["inverse_sawtooth"] = ID_INVERSE_SAWTOOTH;
+        mIds["scroll_x"] = ID_SCROLL_X;
+        mIds["scroll_y"] = ID_SCROLL_Y;
+        mIds["scale_x"] = ID_SCALE_X;
+        mIds["scale_y"] = ID_SCALE_Y;
+        mIds["sine"] = ID_SINE;
+        mIds["triangle"] = ID_TRIANGLE;
+        mIds["sawtooth"] = ID_SAWTOOTH;
+        mIds["square"] = ID_SQUARE;
+        mIds["inverse_sawtooth"] = ID_INVERSE_SAWTOOTH;
 		mIds["transform"] = ID_TRANSFORM;
 		mIds["binding_type"] = ID_BINDING_TYPE;
-			mIds["vertex"] = ID_VERTEX;
-			mIds["fragment"] = ID_FRAGMENT;
+        mIds["vertex"] = ID_VERTEX;
+        mIds["fragment"] = ID_FRAGMENT;
+		mIds["geometry"] = ID_GEOMETRY;
+		mIds["tesselation_hull"] = ID_TESSELATION_HULL;
+		mIds["tesselation_domain"] = ID_TESSELATION_DOMAIN;
+		mIds["compute"] = ID_COMPUTE;
 		mIds["content_type"] = ID_CONTENT_TYPE;
-			mIds["named"] = ID_NAMED;
-			mIds["shadow"] = ID_SHADOW;
+        mIds["named"] = ID_NAMED;
+        mIds["shadow"] = ID_SHADOW;
 		mIds["texture_source"] = ID_TEXTURE_SOURCE;
 		mIds["shared_params"] = ID_SHARED_PARAMS;
 		mIds["shared_param_named"] = ID_SHARED_PARAM_NAMED;
 		mIds["shared_params_ref"] = ID_SHARED_PARAMS_REF;
 
-
 		// Particle system
 		mIds["particle_system"] = ID_PARTICLE_SYSTEM;
 		mIds["emitter"] = ID_EMITTER;
@@ -1208,15 +1221,16 @@ namespace Ogre
 		mIds["target_output"] = ID_TARGET_OUTPUT;
 
 		mIds["input"] = ID_INPUT;
-			mIds["none"] = ID_NONE;
-			mIds["previous"] = ID_PREVIOUS;
-			mIds["target_width"] = ID_TARGET_WIDTH;
-			mIds["target_height"] = ID_TARGET_HEIGHT;
-			mIds["target_width_scaled"] = ID_TARGET_WIDTH_SCALED;
-			mIds["target_height_scaled"] = ID_TARGET_HEIGHT_SCALED;
-			mIds["pooled"] = ID_POOLED;
-			//mIds["gamma"] = ID_GAMMA; - already registered
-			mIds["no_fsaa"] = ID_NO_FSAA;
+        mIds["none"] = ID_NONE;
+        mIds["previous"] = ID_PREVIOUS;
+        mIds["target_width"] = ID_TARGET_WIDTH;
+        mIds["target_height"] = ID_TARGET_HEIGHT;
+        mIds["target_width_scaled"] = ID_TARGET_WIDTH_SCALED;
+        mIds["target_height_scaled"] = ID_TARGET_HEIGHT_SCALED;
+        mIds["pooled"] = ID_POOLED;
+        //mIds["gamma"] = ID_GAMMA; - already registered
+        mIds["no_fsaa"] = ID_NO_FSAA;
+        mIds["depth_pool"] = ID_DEPTH_POOL;
 
 		mIds["texture_ref"] = ID_TEXTURE_REF;
 		mIds["local_scope"] = ID_SCOPE_LOCAL;
@@ -1238,12 +1252,12 @@ namespace Ogre
 		mIds["first_render_queue"] = ID_FIRST_RENDER_QUEUE;
 		mIds["last_render_queue"] = ID_LAST_RENDER_QUEUE;
 		mIds["quad_normals"] = ID_QUAD_NORMALS;
-			mIds["camera_far_corners_view_space"] = ID_CAMERA_FAR_CORNERS_VIEW_SPACE;
-			mIds["camera_far_corners_world_space"] = ID_CAMERA_FAR_CORNERS_WORLD_SPACE;
+        mIds["camera_far_corners_view_space"] = ID_CAMERA_FAR_CORNERS_VIEW_SPACE;
+        mIds["camera_far_corners_world_space"] = ID_CAMERA_FAR_CORNERS_WORLD_SPACE;
 
 		mIds["buffers"] = ID_BUFFERS;
-			mIds["colour"] = ID_COLOUR;
-			mIds["depth"] = ID_DEPTH;
+        mIds["colour"] = ID_COLOUR;
+        mIds["depth"] = ID_DEPTH;
 		mIds["colour_value"] = ID_COLOUR_VALUE;
 		mIds["depth_value"] = ID_DEPTH_VALUE;
 		mIds["stencil_value"] = ID_STENCIL_VALUE;
@@ -1253,15 +1267,20 @@ namespace Ogre
 		mIds["ref_value"] = ID_REF_VALUE;
 		mIds["mask"] = ID_MASK;
 		mIds["fail_op"] = ID_FAIL_OP;
-			mIds["keep"] = ID_KEEP;
-			mIds["increment"] = ID_INCREMENT;
-			mIds["decrement"] = ID_DECREMENT;
-			mIds["increment_wrap"] = ID_INCREMENT_WRAP;
-			mIds["decrement_wrap"] = ID_DECREMENT_WRAP;
-			mIds["invert"] = ID_INVERT;
+        mIds["keep"] = ID_KEEP;
+        mIds["increment"] = ID_INCREMENT;
+        mIds["decrement"] = ID_DECREMENT;
+        mIds["increment_wrap"] = ID_INCREMENT_WRAP;
+        mIds["decrement_wrap"] = ID_DECREMENT_WRAP;
+        mIds["invert"] = ID_INVERT;
 		mIds["depth_fail_op"] = ID_DEPTH_FAIL_OP;
 		mIds["pass_op"] = ID_PASS_OP;
 		mIds["two_sided"] = ID_TWO_SIDED;
+#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
+		mIds["rtshader_system"] = ID_RT_SHADER_SYSTEM;
+#endif
+
+		mIds["subroutine"] = ID_SUBROUTINE;
 	}
 
 	// AbstractTreeeBuilder
@@ -1450,7 +1469,13 @@ namespace Ogre
 				// Finally try to map the cls to an id
 				ScriptCompiler::IdMap::const_iterator iter2 = mCompiler->mIds.find(impl->cls);
 				if(iter2 != mCompiler->mIds.end())
+                {
 					impl->id = iter2->second;
+                }
+                else
+                {
+                    mCompiler->addError(CE_UNEXPECTEDTOKEN, impl->file, impl->line, "token class, " + impl->cls + ", unrecognized.");
+                }
 
 				asn = AbstractNodePtr(impl);
 				mCurrent = impl;
@@ -1529,28 +1554,26 @@ namespace Ogre
 	
 
 	// ScriptCompilerManager
-	template<> ScriptCompilerManager *Singleton<ScriptCompilerManager>::ms_Singleton = 0;
+	template<> ScriptCompilerManager *Singleton<ScriptCompilerManager>::msSingleton = 0;
 	
 	ScriptCompilerManager* ScriptCompilerManager::getSingletonPtr(void)
     {
-        return ms_Singleton;
+        return msSingleton;
     }
 	//-----------------------------------------------------------------------
     ScriptCompilerManager& ScriptCompilerManager::getSingleton(void)
     {  
-        assert( ms_Singleton );  return ( *ms_Singleton );  
+        assert( msSingleton );  return ( *msSingleton );  
     }
 	//-----------------------------------------------------------------------
 	ScriptCompilerManager::ScriptCompilerManager()
 		:mListener(0), OGRE_THREAD_POINTER_INIT(mScriptCompiler)
 	{
-		OGRE_LOCK_AUTO_MUTEX
-#if OGRE_USE_NEW_COMPILERS == 1
+            OGRE_LOCK_AUTO_MUTEX;
 		mScriptPatterns.push_back("*.program");
 		mScriptPatterns.push_back("*.material");
 		mScriptPatterns.push_back("*.particle");
 		mScriptPatterns.push_back("*.compositor");
-#endif
         mScriptPatterns.push_back("*.os");
 		ResourceGroupManager::getSingleton()._registerScriptLoader(this);
 
@@ -1568,7 +1591,7 @@ namespace Ogre
 	//-----------------------------------------------------------------------
 	void ScriptCompilerManager::setListener(ScriptCompilerListener *listener)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 		mListener = listener;
 	}
 	//-----------------------------------------------------------------------
@@ -1579,13 +1602,13 @@ namespace Ogre
 	//-----------------------------------------------------------------------
 	void ScriptCompilerManager::addTranslatorManager(Ogre::ScriptTranslatorManager *man)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 		mManagers.push_back(man);
 	}
 	//-----------------------------------------------------------------------
 	void ScriptCompilerManager::removeTranslatorManager(Ogre::ScriptTranslatorManager *man)
 	{
-		OGRE_LOCK_AUTO_MUTEX
+            OGRE_LOCK_AUTO_MUTEX;
 		
 		for(vector<ScriptTranslatorManager*>::type::iterator i = mManagers.begin(); i != mManagers.end(); ++i)
 		{
@@ -1606,7 +1629,7 @@ namespace Ogre
 	{
 		ScriptTranslator *translator = 0;
 		{
-			OGRE_LOCK_AUTO_MUTEX
+                    OGRE_LOCK_AUTO_MUTEX;
 			
 			// Start looking from the back
 			for(vector<ScriptTranslatorManager*>::type::reverse_iterator i = mManagers.rbegin(); i != mManagers.rend(); ++i)
@@ -1648,7 +1671,7 @@ namespace Ogre
 #endif
 		// Set the listener on the compiler before we continue
 		{
-			OGRE_LOCK_AUTO_MUTEX
+                    OGRE_LOCK_AUTO_MUTEX;
 			OGRE_THREAD_POINTER_GET(mScriptCompiler)->setListener(mListener);
 		}
         OGRE_THREAD_POINTER_GET(mScriptCompiler)->compile(stream->getAsString(), stream->getName(), groupName);
