@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -41,13 +41,8 @@ namespace Ogre{
 		enum{ READY = 0, COMMENT, MULTICOMMENT, WORD, QUOTE, VAR, POSSIBLECOMMENT };
 
 		// Set up some constant characters of interest
-#if OGRE_WCHAR_T_STRINGS
-		const wchar_t varopener = L'$', quote = L'\"', slash = L'/', backslash = L'\\', openbrace = L'{', closebrace = L'}', colon = L':', star = L'*';
-		wchar_t c = 0, lastc = 0;
-#else
-		const wchar_t varopener = '$', quote = '\"', slash = '/', backslash = '\\', openbrace = '{', closebrace = '}', colon = ':', star = '*';
+		const wchar_t varopener = '$', quote = '\"', slash = '/', backslash = '\\', openbrace = '{', closebrace = '}', colon = ':', star = '*', cr = '\r', lf = '\n';
 		char c = 0, lastc = 0;
-#endif
 
 		String lexeme;
 		uint32 line = 1, state = READY, lastQuote = 0;
@@ -206,7 +201,7 @@ namespace Ogre{
 			}
 
 			// Separate check for newlines just to track line numbers
-			if(isNewline(c))
+			if(c == cr || (c == lf && lastc != cr))
 				line++;
 			
 			i++;
@@ -234,13 +229,8 @@ namespace Ogre{
 
 	void ScriptLexer::setToken(const Ogre::String &lexeme, Ogre::uint32 line, const String &source, Ogre::ScriptTokenList *tokens)
 	{
-#if OGRE_WCHAR_T_STRINGS
-		const wchar_t openBracket = L'{', closeBracket = L'}', colon = L':', 
-			quote = L'\"', var = L'$';
-#else
-		const char openBracket = '{', closeBracket = '}', colon = ':', 
+		const char openBracket = '{', closeBracket = '}', colon = ':',
 			quote = '\"', var = '$';
-#endif
 
 		ScriptTokenPtr token(OGRE_NEW_T(ScriptToken, MEMCATEGORY_GENERAL)(), SPFM_DELETE_T);
 		token->lexeme = lexeme;
@@ -282,20 +272,12 @@ namespace Ogre{
 
 	bool ScriptLexer::isWhitespace(Ogre::String::value_type c) const
 	{
-#ifdef OGRE_WCHAR_T_STRINGS
-		return c == L' ' || c == L'\r' || c == L'\t';
-#else
 		return c == ' ' || c == '\r' || c == '\t';
-#endif
 	}
 
 	bool ScriptLexer::isNewline(Ogre::String::value_type c) const
 	{
-#ifdef OGRE_WCHAR_T_STRINGS
-		return c == L'\n' || c == L'\r';
-#else
 		return c == '\n' || c == '\r';
-#endif
 	}
 
 }
