@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -30,16 +30,34 @@ THE SOFTWARE.
 
 namespace Ogre {
 
+    String StringConverter::msDefaultStringLocale = OGRE_DEFAULT_LOCALE;
+    std::locale StringConverter::msLocale = std::locale(msDefaultStringLocale.c_str());
+    bool StringConverter::msUseLocale = false;
+
     //-----------------------------------------------------------------------
     String StringConverter::toString(Real val, unsigned short precision, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f", val);
-		return String(buffer, n);
-#else
-        stringstream stream;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream.precision(precision);
+        stream.width(width);
+        stream.fill(fill);
+        if (flags)
+            stream.setf(flags);
+        stream << val;
+
+        return stream.str();
+    }
+#if OGRE_DOUBLE_PRECISION == 1
+    //-----------------------------------------------------------------------
+    String StringConverter::toString(float val, unsigned short precision,
+                                     unsigned short width, char fill, std::ios::fmtflags flags)
+    {
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
         stream.precision(precision);
         stream.width(width);
         stream.fill(fill);
@@ -47,14 +65,32 @@ namespace Ogre {
             stream.setf(flags);
         stream << val;
         return stream.str();
-#endif
     }
+#else
+    //-----------------------------------------------------------------------
+    String StringConverter::toString(double val, unsigned short precision,
+                                     unsigned short width, char fill, std::ios::fmtflags flags)
+    {
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream.precision(precision);
+        stream.width(width);
+        stream.fill(fill);
+        if (flags)
+            stream.setf(flags);
+        stream << val;
+        return stream.str();
+    }
+#endif
     //-----------------------------------------------------------------------
     String StringConverter::toString(int val, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-        stringstream stream;
-		stream.width(width);
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream.width(width);
         stream.fill(fill);
         if (flags)
             stream.setf(flags);
@@ -66,7 +102,9 @@ namespace Ogre {
     String StringConverter::toString(unsigned int val, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-        stringstream stream;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
         stream.width(width);
         stream.fill(fill);
         if (flags)
@@ -78,7 +116,9 @@ namespace Ogre {
     String StringConverter::toString(size_t val, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-        stringstream stream;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
         stream.width(width);
         stream.fill(fill);
         if (flags)
@@ -91,7 +131,9 @@ namespace Ogre {
     String StringConverter::toString(unsigned long val, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-        stringstream stream;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
         stream.width(width);
         stream.fill(fill);
         if (flags)
@@ -106,8 +148,10 @@ namespace Ogre {
     String StringConverter::toString(size_t val, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-        stringstream stream;
-		stream.width(width);
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream.width(width);
         stream.fill(fill);
         if (flags)
             stream.setf(flags);
@@ -118,8 +162,10 @@ namespace Ogre {
     String StringConverter::toString(unsigned long val, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-        stringstream stream;
-		stream.width(width);
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream.width(width);
         stream.fill(fill);
         if (flags)
             stream.setf(flags);
@@ -131,65 +177,49 @@ namespace Ogre {
     String StringConverter::toString(long val, 
         unsigned short width, char fill, std::ios::fmtflags flags)
     {
-        stringstream stream;
-		stream.width(width);
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream.width(width);
         stream.fill(fill);
         if (flags)
             stream.setf(flags);
         stream << val;
         return stream.str();
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     String StringConverter::toString(const Vector2& val)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f %f", val.x, val.y);
-		return String(buffer, n);
-#else
-        stringstream stream;
-		stream << val.x << " " << val.y;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream << val.x << " " << val.y;
         return stream.str();
-#endif
     }
     //-----------------------------------------------------------------------
     String StringConverter::toString(const Vector3& val)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f %f %f", val.x, val.y, val.z);
-		return String(buffer, n);
-#else
-        stringstream stream;
-		stream << val.x << " " << val.y << " " << val.z;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream << val.x << " " << val.y << " " << val.z;
         return stream.str();
-#endif
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     String StringConverter::toString(const Vector4& val)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f %f %f %f", val.x, val.y, val.z, val.w);
-		return String(buffer, n);
-#else
-        stringstream stream;
-		stream << val.x << " " << val.y << " " << val.z << " " << val.w;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
+        stream << val.x << " " << val.y << " " << val.z << " " << val.w;
         return stream.str();
-#endif
     }
     //-----------------------------------------------------------------------
     String StringConverter::toString(const Matrix3& val)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f %f %f %f %f %f %f %f %f", val[0][0], 
-			val[0][1], val[0][2], val[1][0], val[1][1], val[1][2],
-			val[2][0], val[2][1], val[2][2]);
-		return String(buffer, n);
-#else
-		stringstream stream;
-        stream << val[0][0] << " " 
+        StringStream stream;
+        stream.imbue(msLocale);
+        stream << val[0][0] << " "
             << val[0][1] << " "             
             << val[0][2] << " "             
             << val[1][0] << " "             
@@ -199,7 +229,6 @@ namespace Ogre {
             << val[2][1] << " "             
             << val[2][2];
         return stream.str();
-#endif
     }
     //-----------------------------------------------------------------------
     String StringConverter::toString(bool val, bool yesNo)
@@ -228,17 +257,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     String StringConverter::toString(const Matrix4& val)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f", 
-			val[0][0], val[0][1], val[0][2], val[0][3], 
-			val[1][0], val[1][1], val[1][2], val[1][3], 
-			val[2][0], val[2][1], val[2][2], val[2][3],
-			val[3][0], val[3][1], val[3][2], val[3][3]);
-		return String(buffer, n);
-#else
-		stringstream stream;
-        stream << val[0][0] << " " 
+        StringStream stream;
+        stream.imbue(msLocale);
+        stream << val[0][0] << " "
             << val[0][1] << " "             
             << val[0][2] << " "             
             << val[0][3] << " "             
@@ -255,38 +276,31 @@ namespace Ogre {
             << val[3][2] << " "             
             << val[3][3];
         return stream.str();
-#endif
     }
     //-----------------------------------------------------------------------
     String StringConverter::toString(const Quaternion& val)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f %f %f %f", val.w, val.x, val.y, val.z);
-		return String(buffer, n);
-#else
-		stringstream stream;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
         stream  << val.w << " " << val.x << " " << val.y << " " << val.z;
         return stream.str();
-#endif
     }
     //-----------------------------------------------------------------------
     String StringConverter::toString(const ColourValue& val)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		static char buffer[128] = "";
-		int n = snprintf(buffer, 128, "%f %f %f %f", val.r, val.g, val.b, val.a);
-		return String(buffer, n);
-#else
-		stringstream stream;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
         stream << val.r << " " << val.g << " " << val.b << " " << val.a;
         return stream.str();
-#endif
     }
     //-----------------------------------------------------------------------
     String StringConverter::toString(const StringVector& val)
     {
-		stringstream stream;
+        StringStream stream;
+        if (msUseLocale)
+            stream.imbue(msLocale);
         StringVector::const_iterator i, iend, ibegin;
         ibegin = val.begin();
         iend = val.end();
@@ -302,76 +316,92 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Real StringConverter::parseReal(const String& val, Real defaultValue)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		Real ret = 0;
-		int n = sscanf(val.c_str(), "%f", &ret);
-		
-		if(n == 0){
-			// Nothing read, so try integer format
-			int ret2 = 0;
-			n = sscanf(val.c_str(), "%d", &ret2);
-			if(n == 1)
-				ret = (Real)ret2;
-		}
-#else
-		// Use istringstream for direct correspondence with toString
-		StringStream str(val);
-		Real ret = defaultValue;
-		str >> ret;
-#endif
+        // Use iStringStream for direct correspondence with toString
+        StringStream str(val);
+        if (msUseLocale)
+            str.imbue(msLocale);
+        Real ret = defaultValue;
+        if( !(str >> ret) )
+            return defaultValue;
+
         return ret;
     }
     //-----------------------------------------------------------------------
     int StringConverter::parseInt(const String& val, int defaultValue)
     {
-		// Use istringstream for direct correspondence with toString
-		StringStream str(val);
-		int ret = defaultValue;
-		str >> ret;
+        // Use iStringStream for direct correspondence with toString
+        StringStream str(val);
+        if (msUseLocale)
+            str.imbue(msLocale);
+        int ret = defaultValue;
+        if( !(str >> ret) )
+            return defaultValue;
 
         return ret;
     }
     //-----------------------------------------------------------------------
     unsigned int StringConverter::parseUnsignedInt(const String& val, unsigned int defaultValue)
     {
-		// Use istringstream for direct correspondence with toString
-		StringStream str(val);
-		unsigned int ret = defaultValue;
-		str >> ret;
+        // Use iStringStream for direct correspondence with toString
+        StringStream str(val);
+        if (msUseLocale)
+            str.imbue(msLocale);
+        unsigned int ret = defaultValue;
+        if( !(str >> ret) )
+            return defaultValue;
 
-		return ret;
+        return ret;
     }
     //-----------------------------------------------------------------------
     long StringConverter::parseLong(const String& val, long defaultValue)
     {
-		// Use istringstream for direct correspondence with toString
-		StringStream str(val);
-		long ret = defaultValue;
-		str >> ret;
+        // Use iStringStream for direct correspondence with toString
+        StringStream str(val);
+        if (msUseLocale)
+            str.imbue(msLocale);
+        long ret = defaultValue;
+        if( !(str >> ret) )
+            return defaultValue;
 
-		return ret;
+        return ret;
     }
     //-----------------------------------------------------------------------
     unsigned long StringConverter::parseUnsignedLong(const String& val, unsigned long defaultValue)
     {
-		// Use istringstream for direct correspondence with toString
-		StringStream str(val);
-		unsigned long ret = defaultValue;
-		str >> ret;
+        // Use iStringStream for direct correspondence with toString
+        StringStream str(val);
+        if (msUseLocale)
+            str.imbue(msLocale);
+        unsigned long ret = defaultValue;
+        if( !(str >> ret) )
+            return defaultValue;
 
-		return ret;
+        return ret;
+    }
+    //-----------------------------------------------------------------------
+    size_t StringConverter::parseSizeT(const String& val, size_t defaultValue)
+    {
+        // Use iStringStream for direct correspondence with toString
+        StringStream str(val);
+        if (msUseLocale)
+            str.imbue(msLocale);
+        size_t ret = defaultValue;
+        if( !(str >> ret) )
+            return defaultValue;
+
+        return ret;
     }
     //-----------------------------------------------------------------------
     bool StringConverter::parseBool(const String& val, bool defaultValue)
     {
-		if ((StringUtil::startsWith(val, "true") || StringUtil::startsWith(val, "yes")
-			|| StringUtil::startsWith(val, "1")))
-			return true;
-		else if ((StringUtil::startsWith(val, "false") || StringUtil::startsWith(val, "no")
-			|| StringUtil::startsWith(val, "0")))
-			return false;
-		else
-			return defaultValue;
+        if ((StringUtil::startsWith(val, "true") || StringUtil::startsWith(val, "yes")
+            || StringUtil::startsWith(val, "1")))
+            return true;
+        else if ((StringUtil::startsWith(val, "false") || StringUtil::startsWith(val, "no")
+            || StringUtil::startsWith(val, "0")))
+            return false;
+        else
+            return defaultValue;
     }
     //-----------------------------------------------------------------------
     Vector2 StringConverter::parseVector2(const String& val, const Vector2& defaultValue)
@@ -385,10 +415,10 @@ namespace Ogre {
         }
         else
         {
-            return Vector2(parseReal(vec[0]),parseReal(vec[1]));
+            return Vector2(parseReal(vec[0], defaultValue[0]), parseReal(vec[1], defaultValue[1]));
         }
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     Vector3 StringConverter::parseVector3(const String& val, const Vector3& defaultValue)
     {
         // Split on space
@@ -400,10 +430,12 @@ namespace Ogre {
         }
         else
         {
-            return Vector3(parseReal(vec[0]),parseReal(vec[1]),parseReal(vec[2]));
+            return Vector3(parseReal(vec[0], defaultValue[0]),
+                           parseReal(vec[1], defaultValue[1]),
+                           parseReal(vec[2], defaultValue[2]));
         }
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     Vector4 StringConverter::parseVector4(const String& val, const Vector4& defaultValue)
     {
         // Split on space
@@ -415,7 +447,10 @@ namespace Ogre {
         }
         else
         {
-            return Vector4(parseReal(vec[0]),parseReal(vec[1]),parseReal(vec[2]),parseReal(vec[3]));
+            return Vector4(parseReal(vec[0], defaultValue[0]),
+                           parseReal(vec[1], defaultValue[1]),
+                           parseReal(vec[2], defaultValue[2]),
+                           parseReal(vec[3], defaultValue[3]));
         }
     }
     //-----------------------------------------------------------------------
@@ -430,9 +465,17 @@ namespace Ogre {
         }
         else
         {
-            return Matrix3(parseReal(vec[0]),parseReal(vec[1]),parseReal(vec[2]),
-                parseReal(vec[3]),parseReal(vec[4]),parseReal(vec[5]),
-                parseReal(vec[6]),parseReal(vec[7]),parseReal(vec[8]));
+            return Matrix3(parseReal(vec[0], defaultValue[0][0]),
+                           parseReal(vec[1], defaultValue[0][1]),
+                           parseReal(vec[2], defaultValue[0][2]),
+
+                           parseReal(vec[3], defaultValue[1][0]),
+                           parseReal(vec[4], defaultValue[1][1]),
+                           parseReal(vec[5], defaultValue[1][2]),
+
+                           parseReal(vec[6], defaultValue[2][0]),
+                           parseReal(vec[7], defaultValue[2][1]),
+                           parseReal(vec[8], defaultValue[2][2]));
         }
     }
     //-----------------------------------------------------------------------
@@ -447,10 +490,25 @@ namespace Ogre {
         }
         else
         {
-            return Matrix4(parseReal(vec[0]),parseReal(vec[1]),parseReal(vec[2]), parseReal(vec[3]),
-                parseReal(vec[4]),parseReal(vec[5]), parseReal(vec[6]), parseReal(vec[7]),
-                parseReal(vec[8]),parseReal(vec[9]), parseReal(vec[10]), parseReal(vec[11]),
-                parseReal(vec[12]),parseReal(vec[13]), parseReal(vec[14]), parseReal(vec[15]));
+            return Matrix4(parseReal(vec[0], defaultValue[0][0]),
+                           parseReal(vec[1], defaultValue[0][1]),
+                           parseReal(vec[2], defaultValue[0][2]),
+                           parseReal(vec[3], defaultValue[0][3]),
+                           
+                           parseReal(vec[4], defaultValue[1][0]),
+                           parseReal(vec[5], defaultValue[1][1]),
+                           parseReal(vec[6], defaultValue[1][2]),
+                           parseReal(vec[7], defaultValue[1][3]),
+                           
+                           parseReal(vec[8], defaultValue[2][0]),
+                           parseReal(vec[9], defaultValue[2][1]),
+                           parseReal(vec[10], defaultValue[2][2]),
+                           parseReal(vec[11], defaultValue[2][3]),
+                           
+                           parseReal(vec[12], defaultValue[3][0]),
+                           parseReal(vec[13], defaultValue[3][1]),
+                           parseReal(vec[14], defaultValue[3][2]),
+                           parseReal(vec[15], defaultValue[3][3]));
         }
     }
     //-----------------------------------------------------------------------
@@ -465,7 +523,10 @@ namespace Ogre {
         }
         else
         {
-            return Quaternion(parseReal(vec[0]),parseReal(vec[1]),parseReal(vec[2]), parseReal(vec[3]));
+            return Quaternion(parseReal(vec[0], defaultValue[0]),
+                              parseReal(vec[1], defaultValue[1]),
+                              parseReal(vec[2], defaultValue[2]),
+                              parseReal(vec[3], defaultValue[3]));
         }
     }
     //-----------------------------------------------------------------------
@@ -476,11 +537,17 @@ namespace Ogre {
 
         if (vec.size() == 4)
         {
-            return ColourValue(parseReal(vec[0]),parseReal(vec[1]),parseReal(vec[2]), parseReal(vec[3]));
+            return ColourValue(parseReal(vec[0], defaultValue[0]),
+                               parseReal(vec[1], defaultValue[1]),
+                               parseReal(vec[2], defaultValue[2]),
+                               parseReal(vec[3], defaultValue[3]));
         }
         else if (vec.size() == 3)
         {
-            return ColourValue(parseReal(vec[0]),parseReal(vec[1]),parseReal(vec[2]), 1.0f);
+            return ColourValue(parseReal(vec[0], defaultValue[0]),
+                               parseReal(vec[1], defaultValue[1]),
+                               parseReal(vec[2], defaultValue[2]),
+                               1.0f);
         }
         else
         {
@@ -492,20 +559,16 @@ namespace Ogre {
     {
         return StringUtil::split(val);
     }
-	//-----------------------------------------------------------------------
-	bool StringConverter::isNumber(const String& val)
-	{
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
-		float test;
-		int n = sscanf(val.c_str(), "%f", &test);
-		return n == 1;
-#else
-		StringStream str(val);
-		float tst;
-		str >> tst;
-		return !str.fail() && str.eof();
-#endif
-	}
+    //-----------------------------------------------------------------------
+    bool StringConverter::isNumber(const String& val)
+    {
+        StringStream str(val);
+        if (msUseLocale)
+            str.imbue(msLocale);
+        float tst;
+        str >> tst;
+        return !str.fail() && str.eof();
+    }
 }
 
 
