@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -42,19 +42,22 @@ THE SOFTWARE.
 namespace Ogre {
     //-----------------------------------------------------------------------
     SubEntity::SubEntity (Entity* parent, SubMesh* subMeshBasis)
-        : Renderable(), mParentEntity(parent), mMaterialName("BaseWhite"),
+        : Renderable(), mParentEntity(parent), //mMaterialName("BaseWhite"),
 		mSubMesh(subMeshBasis), mCachedCamera(0)
     {
-        mpMaterial = MaterialManager::getSingleton().getByName(mMaterialName, subMeshBasis->parent->getGroup());
+        //mMaterialPtr = MaterialManager::getSingleton().getByName(mMaterialName, subMeshBasis->parent->getGroup());
         mMaterialLodIndex = 0;
         mVisible = true;
+        mRenderQueueID = 0;
+        mRenderQueueIDSet = false;
+        mRenderQueuePrioritySet = false;
         mSkelAnimVertexData = 0;
+        mVertexAnimationAppliedThisFrame = false;
 		mSoftwareVertexAnimVertexData = 0;
 		mHardwareVertexAnimVertexData = 0;
 		mHardwarePoseCount = 0;
-
-
-
+		mIndexStart = 0;
+        mIndexEnd = 0;
     }
     //-----------------------------------------------------------------------
     SubEntity::~SubEntity()
@@ -74,7 +77,8 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     const String& SubEntity::getMaterialName(void) const
     {
-        return mMaterialName;
+		return !mMaterialPtr.isNull() ? mMaterialPtr->getName() : StringUtil::BLANK;
+        //return mMaterialName;
     }
     //-----------------------------------------------------------------------
     void SubEntity::setMaterialName( const String& name, const String& groupName /* = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME */)
@@ -88,7 +92,7 @@ namespace Ogre {
 			LogManager::getSingleton().logMessage("Can't assign material " + name +
 				" to SubEntity of " + mParentEntity->getName() + " because this "
 				"Material does not exist. Have you forgotten to define it in a "
-				".material script?");
+				".material script?", LML_CRITICAL);
 
 			material = MaterialManager::getSingleton().getByName("BaseWhite");
 
@@ -106,18 +110,18 @@ namespace Ogre {
 
 	void SubEntity::setMaterial( const MaterialPtr& material )
 	{
-		mpMaterial = material;
+		mMaterialPtr = material;
 		
-        if (mpMaterial.isNull())
+        if (mMaterialPtr.isNull())
         {
 			LogManager::getSingleton().logMessage("Can't assign material "  
                 " to SubEntity of " + mParentEntity->getName() + " because this "
                 "Material does not exist. Have you forgotten to define it in a "
-                ".material script?");
+                ".material script?", LML_CRITICAL);
 			
-            mpMaterial = MaterialManager::getSingleton().getByName("BaseWhite");
+			mMaterialPtr = MaterialManager::getSingleton().getByName("BaseWhite");
 			
-            if (mpMaterial.isNull())
+            if (mMaterialPtr.isNull())
             {
                 OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, "Can't assign default material "
                     "to SubEntity of " + mParentEntity->getName() + ". Did "
@@ -126,25 +130,22 @@ namespace Ogre {
             }
         }
 		
-		mMaterialName = mpMaterial->getName();
-
         // Ensure new material loaded (will not load again if already loaded)
-        mpMaterial->load();
+        mMaterialPtr->load();
 
         // tell parent to reconsider material vertex processing options
         mParentEntity->reevaluateVertexProcessing();
-
 	}
 
     //-----------------------------------------------------------------------
     const MaterialPtr& SubEntity::getMaterial(void) const
     {
-        return mpMaterial;
+        return mMaterialPtr;
     }
     //-----------------------------------------------------------------------
     Technique* SubEntity::getTechnique(void) const
     {
-        return mpMaterial->getBestTechnique(mMaterialLodIndex, this);
+        return mMaterialPtr->getBestTechnique(mMaterialLodIndex, this);
     }
     //-----------------------------------------------------------------------
     void SubEntity::getRenderOperation(RenderOperation& op)
@@ -154,8 +155,42 @@ namespace Ogre {
 		// Deal with any vertex data overrides
 		op.vertexData = getVertexDataForBinding();
 
+		// If we use custom index position the client is responsible to set meaningful values 
+		if(mIndexStart != mIndexEnd)
+		{
+			op.indexData->indexStart = mIndexStart;
+			op.indexData->indexCount = mIndexEnd;
+		}
     }
 	//-----------------------------------------------------------------------
+    void SubEntity::setIndexDataStartIndex(size_t start_index)
+    {
+		if(start_index < mSubMesh->indexData->indexCount)
+	        mIndexStart = start_index;
+    }
+    //-----------------------------------------------------------------------
+    size_t SubEntity::getIndexDataStartIndex() const
+    {
+        return mIndexStart;
+    }
+    //-----------------------------------------------------------------------
+    void SubEntity::setIndexDataEndIndex(size_t end_index)
+    {
+		if(end_index > 0 && end_index <= mSubMesh->indexData->indexCount)
+	        mIndexEnd = end_index;
+    }
+    //-----------------------------------------------------------------------
+    size_t SubEntity::getIndexDataEndIndex() const
+    {
+        return mIndexEnd;
+    }
+    //-----------------------------------------------------------------------
+	void SubEntity::resetIndexDataStartEndIndex()
+	{
+		mIndexStart = 0;
+		mIndexEnd = 0;
+	}
+	//-----------------------------------------------------------------------
 	VertexData* SubEntity::getVertexDataForBinding(void)
 	{
 		if (mSubMesh->useSharedVertices)
@@ -382,14 +417,15 @@ namespace Ogre {
 			// Pack into 4-element constants offset based on constant data index
 			// If there are more than 4 entries, this will be called more than once
 			Vector4 val(0.0f,0.0f,0.0f,0.0f);
-
+			const VertexData* vd = mHardwareVertexAnimVertexData ? mHardwareVertexAnimVertexData : mParentEntity->mHardwareVertexAnimVertexData;
+			
 			size_t animIndex = constantEntry.data * 4;
 			for (size_t i = 0; i < 4 && 
-				animIndex < mHardwareVertexAnimVertexData->hwAnimationDataList.size();
+				animIndex < vd->hwAnimationDataList.size();
 				++i, ++animIndex)
 			{
 				val[i] = 
-					mHardwareVertexAnimVertexData->hwAnimationDataList[animIndex].parametric;
+					vd->hwAnimationDataList[animIndex].parametric;
 			}
 			// set the parametric morph value
 			params->_writeRawConstant(constantEntry.physicalIndex, val);
@@ -422,6 +458,8 @@ namespace Ogre {
 			!mVertexAnimationAppliedThisFrame &&
 			(!hardwareAnimation || mSubMesh->getVertexAnimationType() == VAT_MORPH))
 		{
+			// Note, VES_POSITION is specified here but if normals are included in animation
+			// then these will be re-bound too (buffers must be shared)
 			const VertexElement* srcPosElem = 
 				mSubMesh->vertexData->vertexDeclaration->findElementBySemantic(VES_POSITION);
 			HardwareVertexBufferSharedPtr srcBuf = 
@@ -433,7 +471,7 @@ namespace Ogre {
 				mSoftwareVertexAnimVertexData->vertexDeclaration->findElementBySemantic(VES_POSITION);
 			mSoftwareVertexAnimVertexData->vertexBufferBinding->setBinding(
 				destPosElem->getSource(), srcBuf);
-
+			
 		}
 
 		// rebind any missing hardware pose buffers
@@ -448,4 +486,37 @@ namespace Ogre {
 
 	}
 
+    void SubEntity::setRenderQueueGroup(uint8 queueID)
+    {
+        mRenderQueueIDSet = true;
+        mRenderQueueID = queueID;
+    }
+
+    void SubEntity::setRenderQueueGroupAndPriority(uint8 queueID, ushort priority)
+    {
+        setRenderQueueGroup(queueID);
+        mRenderQueuePrioritySet = true;
+        mRenderQueuePriority = priority;
+    }
+
+    uint8 SubEntity::getRenderQueueGroup(void) const
+    {
+        return mRenderQueueID;
+    }
+
+    ushort SubEntity::getRenderQueuePriority(void) const
+    {
+        return mRenderQueuePriority;
+    }
+
+    bool SubEntity::isRenderQueueGroupSet(void) const
+    {
+        return mRenderQueueIDSet;
+    }
+
+    bool SubEntity::isRenderQueuePrioritySet(void) const
+    {
+        return mRenderQueuePrioritySet;
+    }
+
 }
