@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -40,9 +40,11 @@ namespace Ogre {
         : useSharedVertices(true)
         , operationType(RenderOperation::OT_TRIANGLE_LIST)
         , vertexData(0)
+        , parent(0)
         , mMatInitialised(false)
         , mBoneAssignmentsOutOfDate(false)
 		, mVertexAnimationType(VAT_NONE)
+		, mVertexAnimationIncludesNormals(false)
 		, mBuildEdgesEnabled(true)
     {
 		indexData = OGRE_NEW IndexData();
@@ -211,7 +213,7 @@ namespace Ogre {
     //---------------------------------------------------------------------
     void SubMesh::removeLodLevels(void)
     {
-        ProgressiveMesh::LODFaceList::iterator lodi, lodend;
+        LODFaceList::iterator lodi, lodend;
 		lodend = mLodFaceList.end();
 		for (lodi = mLodFaceList.begin(); lodi != lodend; ++lodi)
 		{
@@ -360,7 +362,7 @@ namespace Ogre {
 			// just insert all indexes
 			for (size_t i = vertexData->vertexStart; i < vertexData->vertexCount; i++)
 			{
-				boxes [0].mIndices.insert (i);
+				boxes [0].mIndices.insert (static_cast<int>(i));
 			}
 
 		}
@@ -443,7 +445,7 @@ namespace Ogre {
 
         vbuf->unlock ();
     }
-	 //---------------------------------------------------------------------
+    //---------------------------------------------------------------------
 	void SubMesh::setBuildEdgesEnabled(bool b)
 	{
 		mBuildEdgesEnabled = b;
@@ -453,6 +455,50 @@ namespace Ogre {
 			parent->setAutoBuildEdgeLists(true);
 		}
 	}
+    //---------------------------------------------------------------------
+    SubMesh * SubMesh::clone(const String& newName, Mesh *parentMesh)
+    {
+        // This is a bit like a copy constructor, but with the additional aspect of registering the clone with
+        //  the MeshManager
+
+        SubMesh* newSub;
+        if(!parentMesh)
+            newSub = parent->createSubMesh(newName);
+        else
+            newSub = parentMesh->createSubMesh(newName);
+
+        newSub->mMaterialName = this->mMaterialName;
+        newSub->mMatInitialised = this->mMatInitialised;
+        newSub->operationType = this->operationType;
+        newSub->useSharedVertices = this->useSharedVertices;
+        newSub->extremityPoints = this->extremityPoints;
+
+        if (!this->useSharedVertices)
+        {
+            // Copy unique vertex data
+            newSub->vertexData = this->vertexData->clone();
+            // Copy unique index map
+            newSub->blendIndexToBoneIndexMap = this->blendIndexToBoneIndexMap;
+        }
+
+        // Copy index data
+        OGRE_DELETE newSub->indexData;
+        newSub->indexData = this->indexData->clone();
+        // Copy any bone assignments
+        newSub->mBoneAssignments = this->mBoneAssignments;
+        newSub->mBoneAssignmentsOutOfDate = this->mBoneAssignmentsOutOfDate;
+        // Copy texture aliases
+        newSub->mTextureAliases = this->mTextureAliases;
+
+        // Copy lod face lists
+        newSub->mLodFaceList.reserve(this->mLodFaceList.size());
+        SubMesh::LODFaceList::const_iterator facei;
+        for (facei = this->mLodFaceList.begin(); facei != this->mLodFaceList.end(); ++facei) {
+            IndexData* newIndexData = (*facei)->clone();
+            newSub->mLodFaceList.push_back(newIndexData);
+        }
+        return newSub;
+    }
 }
 
 
