@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -62,6 +62,20 @@ namespace Ogre {
         return mIsSupported;
     }
     //-----------------------------------------------------------------------------
+    size_t Technique::calculateSize(void) const
+    {
+        size_t memSize = 0;
+
+        // Tally up passes
+        Passes::const_iterator i, iend;
+        iend = mPasses.end();
+        for (i = mPasses.begin(); i != iend; ++i)
+        {
+            memSize += (*i)->calculateSize();
+        }
+        return memSize;
+    }
+    //-----------------------------------------------------------------------------
     String Technique::_compile(bool autoManageTextureUnits)
     {
 		StringUtil::StrStreamType errors;
@@ -129,7 +143,24 @@ namespace Ogre {
 					}
 				}
 			}
+			if (currPass->hasComputeProgram())
+			{
+				// Check fragment program version
+				if (!currPass->getComputeProgram()->isSupported())
+				{
+					// Can't do this one
+					compileErrors << "Pass " << passNum << 
+						": Compute program " << currPass->getComputeProgram()->getName()
+						<< " cannot be used - ";
+					if (currPass->getComputeProgram()->hasCompileError())
+						compileErrors << "compile error.";
+					else
+						compileErrors << "not supported.";
 
+					compileErrors << std::endl;
+					return false;
+				}
+			}
 			if (currPass->hasVertexProgram())
 			{
 				// Check vertex program version
@@ -148,6 +179,42 @@ namespace Ogre {
 					return false;
 				}
 			}
+			if (currPass->hasTesselationHullProgram())
+			{
+				// Check tesselation control program version
+				if (!currPass->getTesselationHullProgram()->isSupported() )
+				{
+					// Can't do this one
+					compileErrors << "Pass " << passNum << 
+						": Tesselation Hull program " << currPass->getTesselationHullProgram()->getName()
+						<< " cannot be used - ";
+					if (currPass->getTesselationHullProgram()->hasCompileError())
+						compileErrors << "compile error.";
+					else
+						compileErrors << "not supported.";
+
+					compileErrors << std::endl;
+					return false;
+				}
+			}
+			if (currPass->hasTesselationDomainProgram())
+			{
+				// Check tesselation control program version
+				if (!currPass->getTesselationDomainProgram()->isSupported() )
+				{
+					// Can't do this one
+					compileErrors << "Pass " << passNum << 
+						": Tesselation Domain program " << currPass->getTesselationDomainProgram()->getName()
+						<< " cannot be used - ";
+					if (currPass->getTesselationDomainProgram()->hasCompileError())
+						compileErrors << "compile error.";
+					else
+						compileErrors << "not supported.";
+
+					compileErrors << std::endl;
+					return false;
+				}
+			}
 			if (currPass->hasGeometryProgram())
 			{
 				// Check geometry program version
@@ -207,7 +274,8 @@ namespace Ogre {
 					// Any 3D textures? NB we make the assumption that any
 					// card capable of running fragment programs can support
 					// 3D textures, which has to be true, surely?
-					if (tex->getTextureType() == TEX_TYPE_3D && !caps->hasCapability(RSC_TEXTURE_3D))
+					if (((tex->getTextureType() == TEX_TYPE_3D) || (tex->getTextureType() == TEX_TYPE_2D_ARRAY)) && 
+                         !caps->hasCapability(RSC_TEXTURE_3D))
 					{
 						// Fail
 						compileErrors << "Pass " << passNum << 
@@ -610,7 +678,8 @@ namespace Ogre {
 		{
 			// in case we could not get material as it wasn't yet parsed/existent at that time.
 			mShadowCasterMaterial = MaterialManager::getSingleton().getByName(mShadowCasterMaterialName);
-			mShadowCasterMaterial->load();
+            if (!mShadowCasterMaterial.isNull())
+			    mShadowCasterMaterial->load();
 		}
 		if (!mShadowReceiverMaterial.isNull())
 		{
@@ -620,7 +689,8 @@ namespace Ogre {
 		{
 			// in case we could not get material as it wasn't yet parsed/existent at that time.
 			mShadowReceiverMaterial = MaterialManager::getSingleton().getByName(mShadowReceiverMaterialName);
-			mShadowReceiverMaterial->load();
+            if (!mShadowReceiverMaterial.isNull())
+			    mShadowReceiverMaterial->load();
 		}
     }
     //-----------------------------------------------------------------------------
@@ -1257,7 +1327,7 @@ namespace Ogre {
 	void  Technique::setShadowCasterMaterial(const Ogre::String &name) 
 	{ 
 		mShadowCasterMaterialName = name;
-		mShadowCasterMaterial = MaterialManager::getSingleton().getByName(name); 
+		mShadowCasterMaterial = MaterialManager::getSingleton().getByName(name);
 	}
 	//-----------------------------------------------------------------------
 	Ogre::MaterialPtr  Technique::getShadowReceiverMaterial() const 
@@ -1282,7 +1352,7 @@ namespace Ogre {
 	void  Technique::setShadowReceiverMaterial(const Ogre::String &name)  
 	{ 
 		mShadowReceiverMaterialName = name;
-		mShadowReceiverMaterial = MaterialManager::getSingleton().getByName(name); 
+		mShadowReceiverMaterial = MaterialManager::getSingleton().getByName(name);
 	}
 	//---------------------------------------------------------------------
 	void Technique::addGPUVendorRule(GPUVendor vendor, Technique::IncludeOrExclude includeOrExclude)
