@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	uint16 WorkQueue::getChannel(const String& channelName)
 	{
-		OGRE_LOCK_MUTEX(mChannelMapMutex)
+            OGRE_LOCK_MUTEX(mChannelMapMutex);
 
 		ChannelMap::iterator i = mChannelMap.find(channelName);
 		if (i == mChannelMap.end())
@@ -79,6 +79,8 @@ namespace Ogre {
 		, mRequestCount(0)
 		, mPaused(false)
 		, mAcceptRequests(true)
+        , mShuttingDown(false)
+        , mIdleProcessed(0)
 	{
 	}
 	//---------------------------------------------------------------------
@@ -126,7 +128,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void DefaultWorkQueueBase::addRequestHandler(uint16 channel, RequestHandler* rh)
 	{
-		OGRE_LOCK_RW_MUTEX_WRITE(mRequestHandlerMutex);
+            OGRE_LOCK_RW_MUTEX_WRITE(mRequestHandlerMutex);
 
 		RequestHandlerListByChannel::iterator i = mRequestHandlers.find(channel);
 		if (i == mRequestHandlers.end())
@@ -197,14 +199,14 @@ namespace Ogre {
 	}
 	//---------------------------------------------------------------------
 	WorkQueue::RequestID DefaultWorkQueueBase::addRequest(uint16 channel, uint16 requestType, 
-		const Any& rData, uint8 retryCount, bool forceSynchronous)
+		const Any& rData, uint8 retryCount, bool forceSynchronous, bool idleThread)
 	{
 		Request* req = 0;
 		RequestID rid = 0;
 
 		{
 			// lock to acquire rid and push request to the queue
-			OGRE_LOCK_MUTEX(mRequestMutex)
+                    OGRE_LOCK_MUTEX(mRequestMutex);
 
 			if (!mAcceptRequests || mShuttingDown)
 				return 0;
@@ -222,7 +224,7 @@ namespace Ogre {
 				<< "): ID=" << rid
 			    << " channel=" << channel << " requestType=" << requestType;
 #if OGRE_THREAD_SUPPORT
-			if (!forceSynchronous)
+			if (!forceSynchronous&& !idleThread)
 			{
 				mRequestQueue.push_back(req);
 				notifyWorkers();
@@ -230,10 +232,16 @@ namespace Ogre {
 			}
 #endif
 		}
-		
-
-		processRequestResponse(req, true);
-
+		if(idleThread){
+			OGRE_LOCK_MUTEX(mIdleMutex);
+			mIdleRequestQueue.push_back(req);
+			if(!mIdleThreadRunning)
+			{
+				notifyWorkers();
+			}
+		} else { //forceSynchronous
+			processRequestResponse(req, true);
+		}
 		return rid;
 
 	}
@@ -242,7 +250,7 @@ namespace Ogre {
 		uint16 requestType, const Any& rData, uint8 retryCount)
 	{
 		// lock to push request to the queue
-		OGRE_LOCK_MUTEX(mRequestMutex)
+            OGRE_LOCK_MUTEX(mRequestMutex);
 
 		if (mShuttingDown)
 			return;
@@ -268,7 +276,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void DefaultWorkQueueBase::abortRequest(RequestID id)
 	{
-		OGRE_LOCK_MUTEX(mProcessMutex)
+            OGRE_LOCK_MUTEX(mProcessMutex);
 
 		// NOTE: Pending requests are exist any of RequestQueue, ProcessQueue and
 		// ResponseQueue when keeping ProcessMutex, so we check all of these queues.
@@ -283,7 +291,7 @@ namespace Ogre {
 		}
 
 		{
-			OGRE_LOCK_MUTEX(mRequestMutex)
+                    OGRE_LOCK_MUTEX(mRequestMutex);
 
 			for (RequestQueue::iterator i = mRequestQueue.begin(); i != mRequestQueue.end(); ++i)
 			{
@@ -296,7 +304,20 @@ namespace Ogre {
 		}
 
 		{
-			OGRE_LOCK_MUTEX(mResponseMutex)
+			if(mIdleProcessed)
+			{
+				mIdleProcessed->abortRequest();
+			}
+
+			OGRE_LOCK_MUTEX(mIdleMutex);
+			for (RequestQueue::iterator i = mIdleRequestQueue.begin(); i != mIdleRequestQueue.end(); ++i)
+			{
+				(*i)->abortRequest();
+			}
+		}
+
+		{
+                    OGRE_LOCK_MUTEX(mResponseMutex);
 
 			for (ResponseQueue::iterator i = mResponseQueue.begin(); i != mResponseQueue.end(); ++i)
 			{
@@ -311,7 +332,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void DefaultWorkQueueBase::abortRequestsByChannel(uint16 channel)
 	{
-		OGRE_LOCK_MUTEX(mProcessMutex)
+            OGRE_LOCK_MUTEX(mProcessMutex);
 
 		for (RequestQueue::iterator i = mProcessQueue.begin(); i != mProcessQueue.end(); ++i)
 		{
@@ -322,7 +343,7 @@ namespace Ogre {
 		}
 
 		{
-			OGRE_LOCK_MUTEX(mRequestMutex)
+                    OGRE_LOCK_MUTEX(mRequestMutex);
 
 			for (RequestQueue::iterator i = mRequestQueue.begin(); i != mRequestQueue.end(); ++i)
 			{
@@ -332,9 +353,25 @@ namespace Ogre {
 				}
 			}
 		}
+		{
+			if (mIdleProcessed && mIdleProcessed->getChannel() == channel)
+			{
+				mIdleProcessed->abortRequest();
+			}
+
+			OGRE_LOCK_MUTEX(mIdleMutex);
+
+			for (RequestQueue::iterator i = mIdleRequestQueue.begin(); i != mIdleRequestQueue.end(); ++i)
+			{
+				if ((*i)->getChannel() == channel)
+				{
+					(*i)->abortRequest();
+				}
+			}
+		}
 
 		{
-			OGRE_LOCK_MUTEX(mResponseMutex)
+                    OGRE_LOCK_MUTEX(mResponseMutex);
 
 			for (ResponseQueue::iterator i = mResponseQueue.begin(); i != mResponseQueue.end(); ++i)
 			{
@@ -346,17 +383,44 @@ namespace Ogre {
 		}
 	}
 	//---------------------------------------------------------------------
+	void DefaultWorkQueueBase::abortPendingRequestsByChannel(uint16 channel)
+	{
+		{
+                    OGRE_LOCK_MUTEX(mRequestMutex);
+			for (RequestQueue::iterator i = mRequestQueue.begin(); i != mRequestQueue.end(); ++i)
+			{
+				if ((*i)->getChannel() == channel)
+				{
+					(*i)->abortRequest();
+				}
+			}
+		}
+		{
+                    OGRE_LOCK_MUTEX(mIdleMutex);
+
+				for (RequestQueue::iterator i = mIdleRequestQueue.begin(); i != mIdleRequestQueue.end(); ++i)
+				{
+					if ((*i)->getChannel() == channel)
+					{
+						(*i)->abortRequest();
+					}
+				}
+		}
+	}
+	//---------------------------------------------------------------------
 	void DefaultWorkQueueBase::abortAllRequests()
 	{
-		OGRE_LOCK_MUTEX(mProcessMutex)
-
-		for (RequestQueue::iterator i = mProcessQueue.begin(); i != mProcessQueue.end(); ++i)
+            OGRE_LOCK_MUTEX(mProcessMutex);
 		{
-			(*i)->abortRequest();
+			for (RequestQueue::iterator i = mProcessQueue.begin(); i != mProcessQueue.end(); ++i)
+			{
+				(*i)->abortRequest();
+			}
 		}
+		
 
 		{
-			OGRE_LOCK_MUTEX(mRequestMutex)
+                    OGRE_LOCK_MUTEX(mRequestMutex);
 
 			for (RequestQueue::iterator i = mRequestQueue.begin(); i != mRequestQueue.end(); ++i)
 			{
@@ -365,18 +429,34 @@ namespace Ogre {
 		}
 
 		{
-			OGRE_LOCK_MUTEX(mResponseMutex)
+
+			if(mIdleProcessed)
+			{
+				mIdleProcessed->abortRequest();
+			}
+
+			OGRE_LOCK_MUTEX(mIdleMutex);
+
+			for (RequestQueue::iterator i = mIdleRequestQueue.begin(); i != mIdleRequestQueue.end(); ++i)
+			{
+				(*i)->abortRequest();
+			}
+		}
+
+		{
+                    OGRE_LOCK_MUTEX(mResponseMutex);
 
 			for (ResponseQueue::iterator i = mResponseQueue.begin(); i != mResponseQueue.end(); ++i)
 			{
 				(*i)->abortRequest();
 			}
 		}
+
 	}
 	//---------------------------------------------------------------------
 	void DefaultWorkQueueBase::setPaused(bool pause)
 	{
-		OGRE_LOCK_MUTEX(mRequestMutex)
+            OGRE_LOCK_MUTEX(mRequestMutex);
 
 		mPaused = pause;
 	}
@@ -388,7 +468,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void DefaultWorkQueueBase::setRequestsAccepted(bool accept)
 	{
-		OGRE_LOCK_MUTEX(mRequestMutex)
+            OGRE_LOCK_MUTEX(mRequestMutex);
 
 		mAcceptRequests = accept;
 	}
@@ -400,12 +480,16 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	void DefaultWorkQueueBase::_processNextRequest()
 	{
+		if(processIdleRequests()){
+			// Found idle requests.
+			return;
+		}
 		Request* request = 0;
 		{
 			// scoped to only lock while retrieving the next request
-			OGRE_LOCK_MUTEX(mProcessMutex)
+                    OGRE_LOCK_MUTEX(mProcessMutex);
 			{
-				OGRE_LOCK_MUTEX(mRequestMutex)
+                            OGRE_LOCK_MUTEX(mRequestMutex);
 
 				if (!mRequestQueue.empty())
 				{
@@ -428,7 +512,7 @@ namespace Ogre {
 	{
 		Response* response = processRequest(r);
 
-		OGRE_LOCK_MUTEX(mProcessMutex)
+		OGRE_LOCK_MUTEX(mProcessMutex);
 
 		RequestQueue::iterator it;
 		for( it = mProcessQueue.begin(); it != mProcessQueue.end(); ++it )
@@ -439,7 +523,10 @@ namespace Ogre {
 				break;
 			}
 		}
-
+		if( mIdleProcessed == r )
+		{
+			mIdleProcessed = 0;
+		}
 		if (response)
 		{
 			if (!response->succeeded())
@@ -468,7 +555,7 @@ namespace Ogre {
 					response->abortRequest();
 				}
 				// Queue response
-				OGRE_LOCK_MUTEX(mResponseMutex)
+				OGRE_LOCK_MUTEX(mResponseMutex);
 				mResponseQueue.push_back(response);
 				// no need to wake thread, this is processed by the main thread
 			}
@@ -496,7 +583,7 @@ namespace Ogre {
 		{
 			Response* response = 0;
 			{
-				OGRE_LOCK_MUTEX(mResponseMutex)
+                            OGRE_LOCK_MUTEX(mResponseMutex);
 
 				if (mResponseQueue.empty())
 					break; // exit loop
@@ -530,7 +617,7 @@ namespace Ogre {
 		RequestHandlerListByChannel handlerListCopy;
 		{
 			// lock the list only to make a copy of it, to maximise parallelism
-			OGRE_LOCK_RW_MUTEX_READ(mRequestHandlerMutex);
+                    OGRE_LOCK_RW_MUTEX_READ(mRequestHandlerMutex);
 			
 			handlerListCopy = mRequestHandlers;
 			
@@ -605,12 +692,66 @@ namespace Ogre {
 			"DefaultWorkQueueBase('" << mName << "') - PROCESS_RESPONSE_END(" << dbgMsg.str();
 
 	}
+
+	bool DefaultWorkQueueBase::processIdleRequests()
+	{
+		{
+                    OGRE_LOCK_MUTEX(mIdleMutex);
+			if(mIdleRequestQueue.empty() || mIdleThreadRunning){
+				return false;
+			} else {
+				mIdleThreadRunning = true;
+			}
+		}
+		try {
+			while(1){
+				{
+                                    OGRE_LOCK_MUTEX(mProcessMutex); // mProcessMutex needs to be the top mutex to prevent livelocks
+					{
+                                            OGRE_LOCK_MUTEX(mIdleMutex);
+						if(!mIdleRequestQueue.empty()){
+							mIdleProcessed = mIdleRequestQueue.front();
+							mIdleRequestQueue.pop_front();
+						} else {
+							mIdleProcessed = 0;
+							mIdleThreadRunning = false;
+							return true;
+						}
+					}
+				}
+				processRequestResponse(mIdleProcessed, false);
+			}
+		} catch (...) { // Normally this should not happen.
+			{
+				// It is very important to clean up or the idle thread will be locked forever!
+                            OGRE_LOCK_MUTEX(mProcessMutex);
+				{
+                                    OGRE_LOCK_MUTEX(mIdleMutex);
+					if(mIdleProcessed){
+						mIdleProcessed->abortRequest();
+					}
+					mIdleProcessed = 0;
+					mIdleThreadRunning = false;
+				}
+			}
+			Ogre::LogManager::getSingleton().stream() << "Exception caught in top of worker thread!";
+
+			return true;
+		}
+	}
+
+
 	//---------------------------------------------------------------------
 
 	void DefaultWorkQueueBase::WorkerFunc::operator()()
 	{
 		mQueue->_threadMain();
 	}
+	
+	void DefaultWorkQueueBase::WorkerFunc::operator()() const
+	{
+		mQueue->_threadMain();
+	}
 
 	void DefaultWorkQueueBase::WorkerFunc::run()
 	{
