@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -92,7 +92,7 @@ namespace Ogre
 	void DefaultWorkQueue::_registerThreadWithRenderSystem()
 	{
 		{
-			OGRE_LOCK_MUTEX(mRegisterRSMutex);
+                    OGRE_LOCK_MUTEX(mRegisterRSMutex);;
 			tbb::tbb_thread::id cur = tbb::this_tbb_thread::get_id();
 			if (mRegisteredThreads.find(cur) == mRegisteredThreads.end())
 			{
