@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -33,10 +33,12 @@ THE SOFTWARE.
 
 extern "C" {
 	// MSVC libs use _chkstk for stack-probing. MinGW equivalent is _alloca.
-	void _alloca();
+	// But calling it to ensure Ogre build with D3D9 static does cause a crash.
+	// Might have to investige further here, later.
+	//void _alloca();
 	void _chkstk()
 	{
-		_alloca();
+		//_alloca();
 	}
 	
 	// MSVC uses security cookies to prevent some buffer overflow attacks.
