@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -103,17 +103,19 @@ namespace Ogre
     void OctreeZone::removeNode( PCZSceneNode * n )
     {
 		if ( n != 0 )
+        {
 			removeNodeFromOctree( n );
 
-		if (n->getHomeZone() == this)
-		{
-			mHomeNodeList.erase( n );
+            if (n->getHomeZone() == this)
+            {
+                mHomeNodeList.erase( n );
 
-		}
-		else
-		{
-			mVisitorNodeList.erase( n );
-		}
+            }
+            else
+            {
+                mVisitorNodeList.erase( n );
+            }
+        }
     }
 
 	/** Remove all nodes from the node reference list and clear it
@@ -552,9 +554,9 @@ namespace Ogre
     {
 
         //return immediately if nothing is in the zone.
-		if (mHomeNodeList.size() == 0 &&
-			mVisitorNodeList.size() == 0 &&
-			mPortals.size() == 0)
+		if (mHomeNodeList.empty() &&
+			mVisitorNodeList.empty() &&
+			mPortals.empty())
             return ;
 
         // Else, the zone is automatically assumed to be visible since either
