@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,7 +31,6 @@ begin                : Mon Sep 30 2002
 copyright            : (C) 2002 by Jon Anderson
 email                : janders@users.sf.net
 
-Enhancements 2003 - 2004 (C) The OGRE Team
 Modified to use with PCZones 2007 by Eric Cha
 -----------------------------------------------------------------------------
 */
@@ -282,9 +281,6 @@ namespace Ogre
     */
     void Octree::_getChildIndexes( const AxisAlignedBox &box, int *x, int *y, int *z ) const
     {
-        Vector3 max = mBox.getMaximum();
-        Vector3 min = box.getMinimum();
-
         Vector3 center = mBox.getMaximum().midPoint( mBox.getMinimum() );
 
         Vector3 ncenter = box.getMaximum().midPoint( box.getMinimum() );
