@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -160,7 +160,7 @@ namespace Ogre
 		if (!portal->getEnabled()) return false;
 
 		// if the frustum has no planes, just return true
-        if (mActiveCullingPlanes.size() == 0)
+        if (mActiveCullingPlanes.empty())
         {
             return true;
         }
@@ -339,7 +339,7 @@ namespace Ogre
 		if (!portal->getEnabled()) return false;
 
 		// if the frustum has no planes, just return true
-		if (mActiveCullingPlanes.size() == 0)
+		if (mActiveCullingPlanes.empty())
 		{
 			return true;
 		}
