@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -396,14 +396,24 @@ namespace Ogre
 
         if ( on != 0 )
 		{
-			// remove references to the node from zones
-            removeSceneNode( on );
+            // destroy the node
+            destroySceneNode( on );
 		}
-
-		// destroy the node
-        SceneManager::destroySceneNode( name );
-
     }
+    
+    // Destroy a scene node
+    void PCZSceneManager::destroySceneNode(SceneNode* sn)
+    {
+        if ( sn != 0 )
+		{
+			// remove references to the node from zones
+            removeSceneNode( sn );
+        
+            // destroy the node
+			SceneManager::destroySceneNode( sn->getName() );
+		}
+    }
+
 	//-----------------------------------------------------------------------
 	void PCZSceneManager::clearScene(void)
 	{
@@ -612,7 +622,7 @@ namespace Ogre
         MovableObjectCollection* lights =
             getMovableObjectCollection(PCZLightFactory::FACTORY_TYPE_NAME);
 	    {
-		    OGRE_LOCK_MUTEX(lights->mutex)
+                OGRE_LOCK_MUTEX(lights->mutex);
 
 		    MovableObjectIterator it(lights->map.begin(), lights->map.end());
 
@@ -710,10 +720,11 @@ namespace Ogre
 		{
 			// add to the global list of zones
 			mZones[instanceName] = newZone;
-		}
-		if (newZone->requiresZoneSpecificNodeData())
-		{
-			createZoneSpecificNodeData(newZone);
+
+            if (newZone->requiresZoneSpecificNodeData())
+            {
+                createZoneSpecificNodeData(newZone);
+            }
 		}
 		return newZone;
 	}
@@ -732,7 +743,7 @@ namespace Ogre
 		MovableObjectCollection* lights =
 		getMovableObjectCollection(PCZLightFactory::FACTORY_TYPE_NAME);
 		{
-			OGRE_LOCK_MUTEX(lights->mutex) // Is locking necessary in destroyZone? I don't know..
+                    OGRE_LOCK_MUTEX(lights->mutex); // Is locking necessary in destroyZone? I don't know..
 
 			MovableObjectIterator it(lights->map.begin(), lights->map.end());
 
@@ -974,7 +985,7 @@ namespace Ogre
 
 
 		{
-			OGRE_LOCK_MUTEX(lights->mutex)
+                    OGRE_LOCK_MUTEX(lights->mutex);
 
 			// Pre-allocate memory
 			mTestLightInfos.clear();
@@ -1392,7 +1403,7 @@ namespace Ogre
 
     //---------------------------------------------------------------------
     AxisAlignedBoxSceneQuery*
-    PCZSceneManager::createAABBQuery(const AxisAlignedBox& box, unsigned long mask)
+    PCZSceneManager::createAABBQuery(const AxisAlignedBox& box, uint32 mask)
     {
         PCZAxisAlignedBoxSceneQuery* q = OGRE_NEW PCZAxisAlignedBoxSceneQuery(this);
         q->setBox(box);
@@ -1401,7 +1412,7 @@ namespace Ogre
     }
     //---------------------------------------------------------------------
     SphereSceneQuery*
-    PCZSceneManager::createSphereQuery(const Sphere& sphere, unsigned long mask)
+    PCZSceneManager::createSphereQuery(const Sphere& sphere, uint32 mask)
     {
         PCZSphereSceneQuery* q = OGRE_NEW PCZSphereSceneQuery(this);
         q->setSphere(sphere);
@@ -1411,7 +1422,7 @@ namespace Ogre
     //---------------------------------------------------------------------
     PlaneBoundedVolumeListSceneQuery*
     PCZSceneManager::createPlaneBoundedVolumeQuery(const PlaneBoundedVolumeList& volumes,
-            unsigned long mask)
+            uint32 mask)
     {
         PCZPlaneBoundedVolumeListSceneQuery* q = OGRE_NEW PCZPlaneBoundedVolumeListSceneQuery(this);
         q->setVolumes(volumes);
@@ -1421,7 +1432,7 @@ namespace Ogre
 
     //---------------------------------------------------------------------
     RaySceneQuery*
-    PCZSceneManager::createRayQuery(const Ray& ray, unsigned long mask)
+    PCZSceneManager::createRayQuery(const Ray& ray, uint32 mask)
     {
         PCZRaySceneQuery* q = OGRE_NEW PCZRaySceneQuery(this);
         q->setRay(ray);
@@ -1430,7 +1441,7 @@ namespace Ogre
     }
     //---------------------------------------------------------------------
     IntersectionSceneQuery*
-    PCZSceneManager::createIntersectionQuery(unsigned long mask)
+    PCZSceneManager::createIntersectionQuery(uint32 mask)
     {
 
         PCZIntersectionSceneQuery* q = OGRE_NEW PCZIntersectionSceneQuery(this);
