@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -45,8 +45,8 @@ namespace Ogre
 		D3DDEVTYPE devType, 
 		DWORD behaviorFlags)
 	{
-		mpDeviceManager				= deviceManager;
-		mpDevice					= NULL;		
+		mDeviceManager				= deviceManager;
+		mDevice					= NULL;		
 		mAdapterNumber				= adapterNumber;
 		mMonitor					= hMonitor;
 		mDeviceType					= devType;
@@ -67,11 +67,11 @@ namespace Ogre
 	}
 
 	//---------------------------------------------------------------------
-	D3D9Device::RenderWindowToResorucesIterator D3D9Device::getRenderWindowIterator(D3D9RenderWindow* renderWindow)
+	D3D9Device::RenderWindowToResourcesIterator D3D9Device::getRenderWindowIterator(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.find(renderWindow);
+		RenderWindowToResourcesIterator it = mMapRenderWindowToResources.find(renderWindow);
 
-		if (it == mMapRenderWindowToResoruces.end())
+		if (it == mMapRenderWindowToResources.end())
 		{
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
 				"Render window was not attached to this device !!", 
@@ -85,16 +85,16 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::attachRenderWindow(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.find(renderWindow);
+		RenderWindowToResourcesIterator it = mMapRenderWindowToResources.find(renderWindow);
 
-		if (it == mMapRenderWindowToResoruces.end())
+		if (it == mMapRenderWindowToResources.end())
 		{
-			RenderWindowResources* renderWindowResources = new RenderWindowResources;
+			RenderWindowResources* renderWindowResources = OGRE_NEW_T(RenderWindowResources, MEMCATEGORY_RENDERSYS);
 
 			memset(renderWindowResources, 0, sizeof(RenderWindowResources));						
 			renderWindowResources->adapterOrdinalInGroupIndex = 0;					
 			renderWindowResources->acquired = false;
-			mMapRenderWindowToResoruces[renderWindow] = renderWindowResources;			
+			mMapRenderWindowToResources[renderWindow] = renderWindowResources;			
 		}
 		updateRenderWindowsIndices();
 	}
@@ -102,9 +102,9 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::detachRenderWindow(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.find(renderWindow);
+		RenderWindowToResourcesIterator it = mMapRenderWindowToResources.find(renderWindow);
 
-		if (it != mMapRenderWindowToResoruces.end())
+		if (it != mMapRenderWindowToResources.end())
 		{		
 			// The focus window in which the d3d9 device created on is detached.
 			// resources must be acquired again.
@@ -121,9 +121,9 @@ namespace Ogre
 
 			releaseRenderWindowResources(renderWindowResources);
 
-			SAFE_DELETE(renderWindowResources);
+			OGRE_DELETE_T(renderWindowResources, RenderWindowResources, MEMCATEGORY_RENDERSYS);
 			
-			mMapRenderWindowToResoruces.erase(it);		
+			mMapRenderWindowToResources.erase(it);		
 		}
 		updateRenderWindowsIndices();
 	}
@@ -136,7 +136,7 @@ namespace Ogre
 		bool resetDevice = false;
 			
 		// Create device if need to.
-		if (mpDevice == NULL)
+		if (mDevice == NULL)
 		{			
 			createD3D9Device();
 		}
@@ -144,7 +144,7 @@ namespace Ogre
 		// Case device already exists.
 		else
 		{
-			RenderWindowToResorucesIterator itPrimary = getRenderWindowIterator(getPrimaryWindow());
+			RenderWindowToResourcesIterator itPrimary = getRenderWindowIterator(getPrimaryWindow());
 
 			if (itPrimary->second->swapChain != NULL)
 			{
@@ -172,7 +172,7 @@ namespace Ogre
 			// grabbed by the primary window using the GetDepthStencilSurface method.
 			if (resetDevice == false)
 			{
-				mpDevice->SetDepthStencilSurface(itPrimary->second->depthBuffer);
+				mDevice->SetDepthStencilSurface(itPrimary->second->depthBuffer);
 			}
 			
 		}
@@ -187,9 +187,9 @@ namespace Ogre
 		else
 		{
 			// Update resources of each window.
-			RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
+			RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 
-			while (it != mMapRenderWindowToResoruces.end())
+			while (it != mMapRenderWindowToResources.end())
 			{
 				acquireRenderWindowResources(it);
 				++it;
@@ -202,16 +202,13 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::release()
 	{
-		if (mpDevice != NULL)
+		if (mDevice != NULL)
 		{
 			D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
 
-			// Clean up depth stencil surfaces
-			renderSystem->_cleanupDepthStencils(mpDevice);	
+			RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 
-			RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
-
-			while (it != mMapRenderWindowToResoruces.end())
+			while (it != mMapRenderWindowToResources.end())
 			{
 				RenderWindowResources* renderWindowResources = it->second;
 
@@ -226,7 +223,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	bool D3D9Device::acquire(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 		
 		acquireRenderWindowResources(it);
 
@@ -251,7 +248,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	IDirect3DSurface9* D3D9Device::getDepthBuffer(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);		
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);		
 
 		return it->second->depthBuffer;
 	}
@@ -259,7 +256,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	IDirect3DSurface9* D3D9Device::getBackBuffer(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 	
 		return it->second->backBuffer;		
 	}
@@ -267,22 +264,22 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	uint D3D9Device::getRenderWindowCount() const
 	{
-		return static_cast<uint>(mMapRenderWindowToResoruces.size());
+		return static_cast<uint>(mMapRenderWindowToResources.size());
 	}
 
 	//---------------------------------------------------------------------
 	D3D9RenderWindow* D3D9Device::getRenderWindow(uint index)
 	{
-		if (index >= mMapRenderWindowToResoruces.size())
+		if (index >= mMapRenderWindowToResources.size())
 		{
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
 				"Index of render window is out of bounds!", 
 				"D3D9RenderWindow::getRenderWindow");
 		}
 		
-		RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
+		RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 
-		while (it != mMapRenderWindowToResoruces.end())
+		while (it != mMapRenderWindowToResources.end())
 		{
 			if (index == 0)			
 				break;			
@@ -297,7 +294,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::setAdapterOrdinalIndex(D3D9RenderWindow* renderWindow, uint adapterOrdinalInGroupIndex)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 
 		it->second->adapterOrdinalInGroupIndex = adapterOrdinalInGroupIndex;
 
@@ -309,29 +306,38 @@ namespace Ogre
 	{	
 		// Lock access to rendering device.
 		D3D9RenderSystem::getResourceManager()->lockDeviceAccess();
-		
+
+		//Remove _all_ depth buffers created by this device
+		D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
+		renderSystem->_cleanupDepthBuffers( mDevice );
+
 		release();
 		
-		RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
+		RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 
-		if (it != mMapRenderWindowToResoruces.end())
+		while (it != mMapRenderWindowToResources.end())
 		{	
 			if (it->first->getWindowHandle() == msSharedFocusWindow)
 				setSharedWindowHandle(NULL);
 
-			SAFE_DELETE(it->second);
+			OGRE_DELETE_T(it->second, RenderWindowResources, MEMCATEGORY_RENDERSYS);
 			++it;
 		}
-		mMapRenderWindowToResoruces.clear();
+		mMapRenderWindowToResources.clear();
 		
 		// Reset dynamic attributes.		
 		mFocusWindow			= NULL;		
 		mD3D9DeviceCapsValid	= false;
-		SAFE_DELETE_ARRAY(mPresentationParams);
+
+		if (mPresentationParams != NULL)
+		{
+			OGRE_FREE (mPresentationParams, MEMCATEGORY_RENDERSYS);
+			mPresentationParams = NULL;
+		}		
 		mPresentationParamsCount = 0;
 
 		// Notify the device manager on this instance destruction.	
-		mpDeviceManager->notifyOnDeviceDestroy(this);
+		mDeviceManager->notifyOnDeviceDestroy(this);
 
 		// UnLock access to rendering device.
 		D3D9RenderSystem::getResourceManager()->unlockDeviceAccess();
@@ -342,7 +348,7 @@ namespace Ogre
 	{		
 		HRESULT hr;
 
-		hr = mpDevice->TestCooperativeLevel();
+		hr = mDevice->TestCooperativeLevel();
 
 		if (hr == D3DERR_DEVICELOST ||
 			hr == D3DERR_DEVICENOTRESET)
@@ -359,7 +365,7 @@ namespace Ogre
 		HRESULT hr;
 
 		// Check that device is in valid state for reset.
-		hr = mpDevice->TestCooperativeLevel();
+		hr = mDevice->TestCooperativeLevel();
 		if (hr == D3DERR_DEVICELOST ||
 			hr == D3DERR_DRIVERINTERNALERROR)
 		{
@@ -372,7 +378,7 @@ namespace Ogre
 		D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
 
 		// Inform all resources that device lost.
-		D3D9RenderSystem::getResourceManager()->notifyOnDeviceLost(mpDevice);
+		D3D9RenderSystem::getResourceManager()->notifyOnDeviceLost(mDevice);
 
 		// Notify all listener before device is rested
 		renderSystem->notifyOnDeviceLost(this);
@@ -386,13 +392,13 @@ namespace Ogre
 
 
 		// Cleanup depth stencils surfaces.
-		renderSystem->_cleanupDepthStencils(mpDevice);
+		renderSystem->_cleanupDepthBuffers();
 
 		updatePresentationParameters();
 
-		RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
+		RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 
-		while (it != mMapRenderWindowToResoruces.end())
+		while (it != mMapRenderWindowToResources.end())
 		{
 			RenderWindowResources* renderWindowResources = it->second;
 
@@ -402,10 +408,13 @@ namespace Ogre
 
 		clearDeviceStreams();
 
+		renderSystem->fireDeviceEvent(this, "BeforeDeviceReset");
 
 		// Reset the device using the presentation parameters.
-		hr = mpDevice->Reset(mPresentationParams);
+		hr = mDevice->Reset(mPresentationParams);
 	
+		renderSystem->fireDeviceEvent(this, "AfterDeviceReset");
+
 		if (hr == D3DERR_DEVICELOST)
 		{
 			// UnLock access to rendering device.
@@ -427,22 +436,22 @@ namespace Ogre
 		setupDeviceStates();
 
 		// Update resources of each window.
-		it = mMapRenderWindowToResoruces.begin();
+		it = mMapRenderWindowToResources.begin();
 
-		while (it != mMapRenderWindowToResoruces.end())
+		while (it != mMapRenderWindowToResources.end())
 		{
 			acquireRenderWindowResources(it);
 			++it;
 		}		
 
-		D3D9Device* pCurActiveDevice = mpDeviceManager->getActiveDevice();
+		D3D9Device* pCurActiveDevice = mDeviceManager->getActiveDevice();
 
-		mpDeviceManager->setActiveDevice(this);
+		mDeviceManager->setActiveDevice(this);
 
 		// Inform all resources that device has been reset.
-		D3D9RenderSystem::getResourceManager()->notifyOnDeviceReset(mpDevice);
+		D3D9RenderSystem::getResourceManager()->notifyOnDeviceReset(mDevice);
 
-		mpDeviceManager->setActiveDevice(pCurActiveDevice);
+		mDeviceManager->setActiveDevice(pCurActiveDevice);
 		
 		renderSystem->notifyOnDeviceReset(this);
 
@@ -475,6 +484,15 @@ namespace Ogre
 		return true;
 	}
 
+	//---------------------------------------------------------------------
+	bool D3D9Device::isFullScreen() const
+	{		
+		if (mPresentationParamsCount > 0 && mPresentationParams[0].Windowed == FALSE)
+			return true;
+				
+		return false;
+	}
+
 	//---------------------------------------------------------------------
 	const D3DCAPS9& D3D9Device::getD3D9DeviceCaps() const
 	{
@@ -504,23 +522,27 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	IDirect3DDevice9* D3D9Device::getD3D9Device()
 	{
-		return mpDevice;
+		return mDevice;
 	}
 
 	//---------------------------------------------------------------------
 	void D3D9Device::updatePresentationParameters()
 	{		
 		// Clear old presentation parameters.
-		SAFE_DELETE_ARRAY(mPresentationParams);
+		if (mPresentationParams != NULL)
+		{
+			OGRE_FREE (mPresentationParams, MEMCATEGORY_RENDERSYS);
+			mPresentationParams = NULL;
+		}	
 		mPresentationParamsCount = 0;		
 
-		if (mMapRenderWindowToResoruces.size() > 0)
+		if (mMapRenderWindowToResources.size() > 0)
 		{
-			mPresentationParams = new D3DPRESENT_PARAMETERS[mMapRenderWindowToResoruces.size()];
+			mPresentationParams = OGRE_ALLOC_T(D3DPRESENT_PARAMETERS, mMapRenderWindowToResources.size(), MEMCATEGORY_RENDERSYS);
 
-			RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
+			RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 
-			while (it != mMapRenderWindowToResoruces.end())
+			while (it != mMapRenderWindowToResources.end())
 			{
 				D3D9RenderWindow* renderWindow = it->first;
 				RenderWindowResources* renderWindowResources = it->second;
@@ -572,9 +594,9 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	bool D3D9Device::isMultihead() const
 	{
-		RenderWindowToResorucesMap::const_iterator it = mMapRenderWindowToResoruces.begin();
+		RenderWindowToResourcesMap::const_iterator it = mMapRenderWindowToResources.begin();
 
-		while (it != mMapRenderWindowToResoruces.end())				
+		while (it != mMapRenderWindowToResources.end())				
 		{
 			RenderWindowResources* renderWindowResources = it->second;
 			
@@ -601,18 +623,18 @@ namespace Ogre
 			DWORD   dwCurValue = D3DTOP_FORCE_DWORD;
 			HRESULT hr;
 
-			hr = mpDevice->SetTexture(stage, NULL);
+			hr = mDevice->SetTexture(stage, NULL);
 			if( hr != S_OK )
 			{
 				String str = "Unable to disable texture '" + StringConverter::toString((unsigned int)stage) + "' in D3D9";
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, str, "D3D9Device::clearDeviceStreams" );
 			}
 		
-			mpDevice->GetTextureStageState(static_cast<DWORD>(stage), D3DTSS_COLOROP, &dwCurValue);
+			mDevice->GetTextureStageState(static_cast<DWORD>(stage), D3DTSS_COLOROP, &dwCurValue);
 
 			if (dwCurValue != D3DTOP_DISABLE)
 			{
-				hr = mpDevice->SetTextureStageState(static_cast<DWORD>(stage), D3DTSS_COLOROP, D3DTOP_DISABLE);
+				hr = mDevice->SetTextureStageState(static_cast<DWORD>(stage), D3DTSS_COLOROP, D3DTOP_DISABLE);
 				if( hr != S_OK )
 				{
 					String str = "Unable to disable texture '" + StringConverter::toString((unsigned)stage) + "' in D3D9";
@@ -631,7 +653,7 @@ namespace Ogre
 		// Unbind any vertex streams to avoid memory leaks				
 		for (unsigned int i = 0; i < mD3D9DeviceCaps.MaxStreams; ++i)
 		{
-			mpDevice->SetStreamSource(i, NULL, 0, 0);
+			mDevice->SetStreamSource(i, NULL, 0, 0);
 		}
 	}
 
@@ -663,14 +685,14 @@ namespace Ogre
 		// Try to create the device with hardware vertex processing. 
 		mBehaviorFlags |= D3DCREATE_HARDWARE_VERTEXPROCESSING;
 		hr = pD3D9->CreateDevice(mAdapterNumber, mDeviceType, mFocusWindow,
-			mBehaviorFlags, mPresentationParams, &mpDevice);
+			mBehaviorFlags, mPresentationParams, &mDevice);
 
 		if (FAILED(hr))
 		{
 			// Try a second time, may fail the first time due to back buffer count,
 			// which will be corrected down to 1 by the runtime
 			hr = pD3D9->CreateDevice(mAdapterNumber, mDeviceType, mFocusWindow,
-				mBehaviorFlags, mPresentationParams, &mpDevice);
+				mBehaviorFlags, mPresentationParams, &mDevice);
 		}
 
 		// Case hardware vertex processing failed.
@@ -681,7 +703,7 @@ namespace Ogre
 			mBehaviorFlags |= D3DCREATE_MIXED_VERTEXPROCESSING;
 
 			hr = pD3D9->CreateDevice(mAdapterNumber, mDeviceType, mFocusWindow,
-				mBehaviorFlags, mPresentationParams, &mpDevice);
+				mBehaviorFlags, mPresentationParams, &mDevice);
 		}
 
 		if( FAILED( hr ) )
@@ -690,7 +712,7 @@ namespace Ogre
 			mBehaviorFlags &= ~D3DCREATE_MIXED_VERTEXPROCESSING;
 			mBehaviorFlags |= D3DCREATE_SOFTWARE_VERTEXPROCESSING;
 			hr = pD3D9->CreateDevice(mAdapterNumber, mDeviceType, mFocusWindow,
-				mBehaviorFlags, mPresentationParams, &mpDevice);
+				mBehaviorFlags, mPresentationParams, &mDevice);
 		}
 
 		if ( FAILED( hr ) )
@@ -698,7 +720,7 @@ namespace Ogre
 			// try reference device
 			mDeviceType = D3DDEVTYPE_REF;
 			hr = pD3D9->CreateDevice(mAdapterNumber, mDeviceType, mFocusWindow,
-				mBehaviorFlags, mPresentationParams, &mpDevice);
+				mBehaviorFlags, mPresentationParams, &mDevice);
 
 			if ( FAILED( hr ) )
 			{
@@ -709,7 +731,7 @@ namespace Ogre
 		}
 
 		// Get current device caps.
-		hr = mpDevice->GetDeviceCaps(&mD3D9DeviceCaps);
+		hr = mDevice->GetDeviceCaps(&mD3D9DeviceCaps);
 		if( FAILED( hr ) )
 		{
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
@@ -718,7 +740,7 @@ namespace Ogre
 		}
 
 		// Get current creation parameters caps.
-		hr = mpDevice->GetCreationParameters(&mCreationParams);
+		hr = mDevice->GetCreationParameters(&mCreationParams);
 		if ( FAILED(hr) )
 		{
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
@@ -734,14 +756,17 @@ namespace Ogre
 		// Lock access to rendering device.
 		D3D9RenderSystem::getResourceManager()->lockDeviceAccess();
 
-		D3D9Device* pCurActiveDevice = mpDeviceManager->getActiveDevice();
+		D3D9Device* pCurActiveDevice = mDeviceManager->getActiveDevice();
 
-		mpDeviceManager->setActiveDevice(this);
+		mDeviceManager->setActiveDevice(this);
 
 		// Inform all resources that new device created.
-		D3D9RenderSystem::getResourceManager()->notifyOnDeviceCreate(mpDevice);
+		D3D9RenderSystem::getResourceManager()->notifyOnDeviceCreate(mDevice);
 
-		mpDeviceManager->setActiveDevice(pCurActiveDevice);
+		mDeviceManager->setActiveDevice(pCurActiveDevice);
+
+		D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
+		renderSystem->fireDeviceEvent(this, "DeviceCreated");
 
 		// UnLock access to rendering device.
 		D3D9RenderSystem::getResourceManager()->unlockDeviceAccess();
@@ -750,24 +775,27 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::releaseD3D9Device()
 	{
-		if (mpDevice != NULL)
+		if (mDevice != NULL)
 		{
+			D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
+			renderSystem->fireDeviceEvent(this, "DeviceReleased");
+
 			// Lock access to rendering device.
 			D3D9RenderSystem::getResourceManager()->lockDeviceAccess();
 
-			D3D9Device* pCurActiveDevice = mpDeviceManager->getActiveDevice();
+			D3D9Device* pCurActiveDevice = mDeviceManager->getActiveDevice();
 
-			mpDeviceManager->setActiveDevice(this);
+			mDeviceManager->setActiveDevice(this);
 
 			// Inform all resources that device is going to be destroyed.
-			D3D9RenderSystem::getResourceManager()->notifyOnDeviceDestroy(mpDevice);
+			D3D9RenderSystem::getResourceManager()->notifyOnDeviceDestroy(mDevice);
 
-			mpDeviceManager->setActiveDevice(pCurActiveDevice);
+			mDeviceManager->setActiveDevice(pCurActiveDevice);
 			
 			clearDeviceStreams();		
 
 			// Release device.
-			SAFE_RELEASE(mpDevice);	
+			SAFE_RELEASE(mDevice);	
 			
 			// UnLock access to rendering device.
 			D3D9RenderSystem::getResourceManager()->unlockDeviceAccess();
@@ -777,6 +805,13 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::releaseRenderWindowResources(RenderWindowResources* renderWindowResources)
 	{
+		if( renderWindowResources->depthBuffer )
+		{
+			D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>
+													(Root::getSingleton().getRenderSystem());
+			renderSystem->_cleanupDepthBuffers( renderWindowResources->depthBuffer );
+		}
+
 		SAFE_RELEASE(renderWindowResources->backBuffer);
 		SAFE_RELEASE(renderWindowResources->depthBuffer);
 		SAFE_RELEASE(renderWindowResources->swapChain);
@@ -786,7 +821,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::invalidate(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 
 		it->second->acquired = false;		
 	}
@@ -832,11 +867,11 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	bool D3D9Device::validateDeviceState(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);		
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);		
 		RenderWindowResources* renderWindowResources =  it->second;
 		HRESULT hr;
 
-		hr = mpDevice->TestCooperativeLevel();	
+		hr = mDevice->TestCooperativeLevel();	
 
 		// Case device is not valid for rendering. 
 		if (FAILED(hr))
@@ -895,7 +930,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::validateBackBufferSize(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 		RenderWindowResources*	renderWindowResources = it->second;
 	
 
@@ -922,7 +957,7 @@ namespace Ogre
 		if (renderWindow->isFullScreen())
 			return true;
 
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 		HMONITOR	hRenderWindowMonitor = NULL;
 
 		// Find the monitor this render window belongs to.
@@ -939,7 +974,7 @@ namespace Ogre
 			// Lock access to rendering device.
 			D3D9RenderSystem::getResourceManager()->lockDeviceAccess();
 
-			mpDeviceManager->linkRenderWindow(renderWindow);
+			mDeviceManager->linkRenderWindow(renderWindow);
 
 			// UnLock access to rendering device.
 			D3D9RenderSystem::getResourceManager()->unlockDeviceAccess();
@@ -953,7 +988,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::present(D3D9RenderWindow* renderWindow)
 	{		
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 		RenderWindowResources*	renderWindowResources = it->second;				
 
 
@@ -966,12 +1001,15 @@ namespace Ogre
 
 		HRESULT hr;
 
+		D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
+		renderSystem->fireDeviceEvent(this, "BeforeDevicePresent");
+
 		if (isMultihead())
 		{
 			// Only the master will call present method results in synchronized
 			// buffer swap for the rest of the implicit swap chain.
 			if (getPrimaryWindow() == renderWindow)
-				hr = mpDevice->Present( NULL, NULL, NULL, NULL );
+				hr = mDevice->Present( NULL, NULL, NULL, NULL );
 			else
 				hr = S_OK;
 		}
@@ -997,7 +1035,7 @@ namespace Ogre
 	}
 
 	//---------------------------------------------------------------------
-	void D3D9Device::acquireRenderWindowResources(RenderWindowToResorucesIterator it)
+	void D3D9Device::acquireRenderWindowResources(RenderWindowToResourcesIterator it)
 	{
 		RenderWindowResources*	renderWindowResources = it->second;
 		D3D9RenderWindow*		renderWindow = it->first;			
@@ -1008,14 +1046,14 @@ namespace Ogre
 		if (isSwapChainWindow(renderWindow) && !isMultihead())
 		{
 			// Create swap chain
-			HRESULT hr = mpDevice->CreateAdditionalSwapChain(&renderWindowResources->presentParameters, 
+			HRESULT hr = mDevice->CreateAdditionalSwapChain(&renderWindowResources->presentParameters, 
 				&renderWindowResources->swapChain);
 
 			if (FAILED(hr))
 			{
 				// Try a second time, may fail the first time due to back buffer count,
 				// which will be corrected by the runtime
-				hr = mpDevice->CreateAdditionalSwapChain(&renderWindowResources->presentParameters, 
+				hr = mDevice->CreateAdditionalSwapChain(&renderWindowResources->presentParameters, 
 					&renderWindowResources->swapChain);
 			}
 
@@ -1029,7 +1067,7 @@ namespace Ogre
 		else
 		{
 			// The swap chain is already created by the device
-			HRESULT hr = mpDevice->GetSwapChain(renderWindowResources->presentParametersIndex, 
+			HRESULT hr = mDevice->GetSwapChain(renderWindowResources->presentParametersIndex, 
 				&renderWindowResources->swapChain);
 			if (FAILED(hr)) 
 			{
@@ -1053,7 +1091,7 @@ namespace Ogre
 			if (isMultihead() && isAutoDepthStencil() || 
 			    isMultihead() == false && isSwapChainWindow(renderWindow) == false)
 			{
-				mpDevice->GetDepthStencilSurface(&renderWindowResources->depthBuffer);
+				mDevice->GetDepthStencilSurface(&renderWindowResources->depthBuffer);
 			}
 			else
 			{
@@ -1066,7 +1104,7 @@ namespace Ogre
 				if (targetHeight == 0)
 					targetHeight = 1;
 
-				HRESULT hr = mpDevice->CreateDepthStencilSurface(
+				HRESULT hr = mDevice->CreateDepthStencilSurface(
 					targetWidth, targetHeight,
 					renderWindowResources->presentParameters.AutoDepthStencilFormat,
 					renderWindowResources->presentParameters.MultiSampleType,
@@ -1084,9 +1122,23 @@ namespace Ogre
 
 				if (isSwapChainWindow(renderWindow) == false)
 				{
-					mpDevice->SetDepthStencilSurface(renderWindowResources->depthBuffer);
+					mDevice->SetDepthStencilSurface(renderWindowResources->depthBuffer);
 				}
 			}
+
+			if (renderWindowResources->depthBuffer)
+			{
+				//Tell the RS we have a depth buffer we created it needs to add to the default pool
+				D3D9RenderSystem* renderSystem = static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem());
+				DepthBuffer *depthBuf = renderSystem->_addManualDepthBuffer( mDevice, renderWindowResources->depthBuffer );
+
+				//Don't forget we want this window to use _this_ depth buffer
+				renderWindow->attachDepthBuffer( depthBuf );
+			}
+			else
+			{
+				LogManager::getSingleton().logMessage("D3D9 : WARNING - Depth buffer could not be acquired.");
+			}
 		}
 
 		renderWindowResources->acquired = true; 
@@ -1095,7 +1147,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9Device::setupDeviceStates()
 	{
-		HRESULT hr = mpDevice->SetRenderState(D3DRS_SPECULARENABLE, TRUE);
+		HRESULT hr = mDevice->SetRenderState(D3DRS_SPECULARENABLE, TRUE);
 		
 		if (FAILED(hr)) 
 		{
@@ -1108,7 +1160,7 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	bool D3D9Device::isSwapChainWindow(D3D9RenderWindow* renderWindow)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 		
 		if (it->second->presentParametersIndex == 0 || renderWindow->isFullScreen())			
 			return false;
@@ -1119,12 +1171,12 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	D3D9RenderWindow* D3D9Device::getPrimaryWindow()
 	{		
-		RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
+		RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 	
-		while (it != mMapRenderWindowToResoruces.end() && it->second->presentParametersIndex != 0)				
+		while (it != mMapRenderWindowToResources.end() && it->second->presentParametersIndex != 0)				
 			++it;		
 
-		assert(it != mMapRenderWindowToResoruces.end());
+		assert(it != mMapRenderWindowToResources.end());
 
 		return it->first;
 	}
@@ -1142,11 +1194,11 @@ namespace Ogre
 		// Update present parameters index attribute per render window.
 		if (isMultihead())
 		{
-			RenderWindowToResorucesIterator it = mMapRenderWindowToResoruces.begin();
+			RenderWindowToResourcesIterator it = mMapRenderWindowToResources.begin();
 
 			// Multi head device case -  
 			// Present parameter index is based on adapter ordinal in group index.
-			while (it != mMapRenderWindowToResoruces.end())			
+			while (it != mMapRenderWindowToResources.end())			
 			{
 				it->second->presentParametersIndex = it->second->adapterOrdinalInGroupIndex;
 				++it;
@@ -1162,32 +1214,39 @@ namespace Ogre
 
 			uint nextPresParamIndex = 0;
 
-			RenderWindowToResorucesIterator it;
+			RenderWindowToResourcesIterator it;
 			D3D9RenderWindow* deviceFocusWindow = NULL;
 
 			// In case a d3d9 device exists - try to keep the present parameters order
 			// so that the window that the device is focused on will stay the same and we
 			// will avoid device re-creation.
-			if (mpDevice != NULL)
+			if (mDevice != NULL)
 			{
-				it = mMapRenderWindowToResoruces.begin();
-				while (it != mMapRenderWindowToResoruces.end())			
+				it = mMapRenderWindowToResources.begin();
+				while (it != mMapRenderWindowToResources.end())			
 				{
+					//This "if" handles the common case of a single device
 					if (it->first->getWindowHandle() == mCreationParams.hFocusWindow)
 					{
 						deviceFocusWindow = it->first;
 						it->second->presentParametersIndex = nextPresParamIndex;
 						++nextPresParamIndex;
 						break;
-					}					
+					}
+					//This "if" handles multiple devices when a shared window is used
+					if ((it->second->presentParametersIndex == 0) && (it->second->acquired == true))
+					{
+						deviceFocusWindow = it->first;
+						++nextPresParamIndex;
+					}
 					++it;
 				}
 			}
 			
 		
 
-			it = mMapRenderWindowToResoruces.begin();
-			while (it != mMapRenderWindowToResoruces.end())			
+			it = mMapRenderWindowToResources.begin();
+			while (it != mMapRenderWindowToResources.end())			
 			{
 				if (it->first != deviceFocusWindow)
 				{
@@ -1202,7 +1261,7 @@ namespace Ogre
 	void D3D9Device::copyContentsToMemory(D3D9RenderWindow* renderWindow, 
 		const PixelBox &dst, RenderTarget::FrameBuffer buffer)
 	{
-		RenderWindowToResorucesIterator it = getRenderWindowIterator(renderWindow);
+		RenderWindowToResourcesIterator it = getRenderWindowIterator(renderWindow);
 		RenderWindowResources* resources = it->second;
 		bool swapChain = isSwapChainWindow(renderWindow);
 
@@ -1227,14 +1286,14 @@ namespace Ogre
 		if (buffer == RenderTarget::FB_AUTO)
 		{
 			//buffer = mIsFullScreen? FB_FRONT : FB_BACK;
-			buffer = RenderTarget::FB_FRONT;
+			buffer = RenderTarget::FB_BACK;
 		}
 
 		if (buffer == RenderTarget::FB_FRONT)
 		{
 			D3DDISPLAYMODE dm;
 
-			if (FAILED(hr = mpDevice->GetDisplayMode(0, &dm)))
+			if (FAILED(hr = mDevice->GetDisplayMode(0, &dm)))
 			{
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
 					"Can't get display mode: " + Root::getSingleton().getErrorDescription(hr),
@@ -1244,7 +1303,7 @@ namespace Ogre
 			desc.Width = dm.Width;
 			desc.Height = dm.Height;
 			desc.Format = D3DFMT_A8R8G8B8;
-			if (FAILED(hr = mpDevice->CreateOffscreenPlainSurface(desc.Width, desc.Height,
+			if (FAILED(hr = mDevice->CreateOffscreenPlainSurface(desc.Width, desc.Height,
 				desc.Format,
 				D3DPOOL_SYSTEMMEM,
 				&pTempSurf,
@@ -1256,7 +1315,7 @@ namespace Ogre
 			}
 
 			if (FAILED(hr = swapChain ? resources->swapChain->GetFrontBufferData(pTempSurf) :
-				mpDevice->GetFrontBufferData(0, pTempSurf)))
+				mDevice->GetFrontBufferData(0, pTempSurf)))
 			{
 				SAFE_RELEASE(pTempSurf);
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
@@ -1322,7 +1381,7 @@ namespace Ogre
 		{
 			SAFE_RELEASE(pSurf);
 			if(FAILED(hr = swapChain? resources->swapChain->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &pSurf) :
-				mpDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pSurf)))
+				mDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pSurf)))
 			{
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
 					"Can't get back buffer: " + Root::getSingleton().getErrorDescription(hr),
@@ -1336,7 +1395,7 @@ namespace Ogre
 					"D3D9Device::copyContentsToMemory");
 			}
 
-			if (FAILED(hr = mpDevice->CreateOffscreenPlainSurface(desc.Width, desc.Height,
+			if (FAILED(hr = mDevice->CreateOffscreenPlainSurface(desc.Width, desc.Height,
 				desc.Format,
 				D3DPOOL_SYSTEMMEM,
 				&pTempSurf,
@@ -1349,7 +1408,7 @@ namespace Ogre
 
 			if (desc.MultiSampleType == D3DMULTISAMPLE_NONE)
 			{
-				if (FAILED(hr = mpDevice->GetRenderTargetData(pSurf, pTempSurf)))
+				if (FAILED(hr = mDevice->GetRenderTargetData(pSurf, pTempSurf)))
 				{
 					SAFE_RELEASE(pTempSurf);
 					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
@@ -1361,7 +1420,7 @@ namespace Ogre
 			{
 				IDirect3DSurface9* pStretchSurf = 0;
 
-				if (FAILED(hr = mpDevice->CreateRenderTarget(desc.Width, desc.Height,
+				if (FAILED(hr = mDevice->CreateRenderTarget(desc.Width, desc.Height,
 					desc.Format,
 					D3DMULTISAMPLE_NONE,
 					0,
@@ -1375,7 +1434,7 @@ namespace Ogre
 						"D3D9Device::copyContentsToMemory");
 				}
 
-				if (FAILED(hr = mpDevice->StretchRect(pSurf, 0, pStretchSurf, 0, D3DTEXF_NONE)))
+				if (FAILED(hr = mDevice->StretchRect(pSurf, 0, pStretchSurf, 0, D3DTEXF_NONE)))
 				{
 					SAFE_RELEASE(pTempSurf);
 					SAFE_RELEASE(pStretchSurf);
@@ -1383,7 +1442,7 @@ namespace Ogre
 						"Can't stretch rect: " + Root::getSingleton().getErrorDescription(hr),
 						"D3D9Device::copyContentsToMemory");
 				}
-				if (FAILED(hr = mpDevice->GetRenderTargetData(pStretchSurf, pTempSurf)))
+				if (FAILED(hr = mDevice->GetRenderTargetData(pStretchSurf, pTempSurf)))
 				{
 					SAFE_RELEASE(pTempSurf);
 					SAFE_RELEASE(pStretchSurf);
