@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -37,7 +37,7 @@ THE SOFTWARE.
 
 namespace Ogre {
 
-OGRE_STATIC_MUTEX_INSTANCE(D3D9HardwarePixelBuffer::msDeviceAccessMutex)
+    OGRE_STATIC_MUTEX_INSTANCE(D3D9HardwarePixelBuffer::msDeviceAccessMutex);
 //-----------------------------------------------------------------------------  
 
 D3D9HardwarePixelBuffer::D3D9HardwarePixelBuffer(HardwareBuffer::Usage usage, 
@@ -58,8 +58,12 @@ D3D9HardwarePixelBuffer::~D3D9HardwarePixelBuffer()
 	while (it != mMapDeviceToBufferResources.end())
 	{
 		SAFE_RELEASE(it->second->surface);
-		SAFE_RELEASE(it->second->volume);
-		SAFE_DELETE(it->second);
+		SAFE_RELEASE(it->second->volume);		
+		if (it->second != NULL)
+		{
+			OGRE_FREE (it->second, MEMCATEGORY_RENDERSYS);
+			it->second = NULL;
+		}
 		DeviceToBufferResourcesIterator deadi = it++;
 		mMapDeviceToBufferResources.erase(deadi);
 	}
@@ -118,10 +122,10 @@ void D3D9HardwarePixelBuffer::bind(IDirect3DDevice9 *dev, IDirect3DSurface9 *sur
 				Box fullBufferBox(0,0,0,mWidth,mHeight,mDepth);
 				PixelBox dstBox(fullBufferBox, mFormat);
 
-				dstBox.data = new char[getSizeInBytes()];
+				dstBox.data = OGRE_MALLOC (getSizeInBytes(), MEMCATEGORY_RESOURCE);
 				blitToMemory(fullBufferBox, dstBox, it->second, it->first);
 				blitFromMemory(dstBox, fullBufferBox, bufferResources);
-				SAFE_DELETE_ARRAY(dstBox.data);
+				OGRE_FREE(dstBox.data, MEMCATEGORY_RESOURCE);
 				break;
 			}
 			++it;			
@@ -174,10 +178,10 @@ void D3D9HardwarePixelBuffer::bind(IDirect3DDevice9 *dev, IDirect3DVolume9 *volu
 				Box fullBufferBox(0,0,0,mWidth,mHeight,mDepth);
 				PixelBox dstBox(fullBufferBox, mFormat);
 
-				dstBox.data = new char[getSizeInBytes()];
+				dstBox.data = OGRE_MALLOC(getSizeInBytes(), MEMCATEGORY_RESOURCE);
 				blitToMemory(fullBufferBox, dstBox, it->second, it->first);
 				blitFromMemory(dstBox, fullBufferBox, bufferResources);
-				SAFE_DELETE(dstBox.data);
+				OGRE_FREE(dstBox.data, MEMCATEGORY_RESOURCE);
 				break;
 			}
 			++it;			
@@ -199,7 +203,7 @@ D3D9HardwarePixelBuffer::BufferResources* D3D9HardwarePixelBuffer::getBufferReso
 //-----------------------------------------------------------------------------  
 D3D9HardwarePixelBuffer::BufferResources* D3D9HardwarePixelBuffer::createBufferResources()
 {
-	BufferResources* newResources = new BufferResources;
+	BufferResources* newResources = OGRE_ALLOC_T(BufferResources, 1, MEMCATEGORY_RENDERSYS);
 
 	memset(newResources, 0, sizeof(BufferResources));
 
@@ -216,8 +220,12 @@ void D3D9HardwarePixelBuffer::destroyBufferResources(IDirect3DDevice9* d3d9Devic
 	if (it != mMapDeviceToBufferResources.end())
 	{
 		SAFE_RELEASE(it->second->surface);
-		SAFE_RELEASE(it->second->volume);	
-		SAFE_DELETE(it->second);
+		SAFE_RELEASE(it->second->volume);			
+		if (it->second != NULL)
+		{
+			OGRE_FREE (it->second, MEMCATEGORY_RENDERSYS);
+			it->second = NULL;
+		}
 		mMapDeviceToBufferResources.erase(it);
 	}
 }
@@ -404,12 +412,24 @@ Ogre::PixelBox D3D9HardwarePixelBuffer::lockBuffer(BufferResources* bufferResour
 	else if(bufferResources->volume) 
 	{
 		// Volume
-		D3DBOX pbox = toD3DBOX(lockBox); // specify range to lock
 		D3DLOCKED_BOX lbox; // Filled in by D3D
+		HRESULT hr;
 
-		if(bufferResources->volume->LockBox(&lbox, &pbox, flags) != D3D_OK)
+		if (lockBox.left == 0 && lockBox.top == 0 && lockBox.front == 0 && 
+			lockBox.right == mWidth && lockBox.bottom == mHeight && lockBox.back == mDepth)
+		{
+			hr = bufferResources->volume->LockBox(&lbox, NULL, flags);
+		}
+		else
+		{
+			D3DBOX pbox = toD3DBOX(lockBox); // specify range to lock
+			hr = bufferResources->volume->LockBox(&lbox, &pbox, flags);
+		}
+		if (FAILED(hr))		
+		{
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Volume locking failed",
-			"D3D9HardwarePixelBuffer::lockImpl");
+				"D3D9HardwarePixelBuffer::lockImpl");
+		}
 		fromD3DLock(rval, lbox);
 	}
 
@@ -620,7 +640,7 @@ void D3D9HardwarePixelBuffer::blitFromMemory(const PixelBox &src, const Image::B
 	// convert to pixelbuffer's native format if necessary
 	if (D3D9Mappings::_getPF(src.format) == D3DFMT_UNKNOWN)
 	{
-		buf.bind(new MemoryDataStream(
+		buf.bind(OGRE_NEW MemoryDataStream(
 			PixelUtil::getMemorySize(src.getWidth(), src.getHeight(), src.getDepth(),
 			mFormat)));
 		converted = PixelBox(src.getWidth(), src.getHeight(), src.getDepth(), mFormat, buf->getPtr());
@@ -968,7 +988,7 @@ void D3D9HardwarePixelBuffer::updateRenderTexture(bool writeGamma, uint fsaa, co
 		String name;
 		name = "rtt/" +Ogre::StringConverter::toString((size_t)this) + "/" + srcName;
 
-		mRenderTexture = new D3D9RenderTexture(name, this, writeGamma, fsaa);		
+		mRenderTexture = OGRE_NEW D3D9RenderTexture(name, this, writeGamma, fsaa);		
 		Root::getSingleton().getRenderSystem()->attachRenderTarget(*mRenderTexture);
 	}
 }
