@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -37,6 +37,7 @@ THE SOFTWARE.
 #include "OgreRoot.h"
 #include "OgreWindowEventUtilities.h"
 #include "OgreD3D9DeviceManager.h"
+#include "OgreDepthBuffer.h"
 
 namespace Ogre
 {
@@ -49,10 +50,13 @@ namespace Ogre
 		mHWnd = 0;
 		mActive = false;		
 		mClosed = false;
+		mHidden = false;
 		mSwitchingFullscreen = false;
 		mDisplayFrequency = 0;
 		mDeviceValid = false;
 		mUseNVPerfHUD = false;
+		mWindowedWinStyle = 0;
+		mFullscreenWinStyle = 0;
 	}
 
 	D3D9RenderWindow::~D3D9RenderWindow()
@@ -82,7 +86,9 @@ namespace Ogre
 		mUseNVPerfHUD = false;
 		size_t fsaaSamples = 0;
 		String fsaaHint;
+		bool enableDoubleClick = false;
 		int monitorIndex = -1;	//Default by detecting the adapter from left / top position
+		
 
 		if(miscParams)
 		{
@@ -103,15 +109,19 @@ namespace Ogre
 			// parentWindowHandle		-> parentHWnd
 			opt = miscParams->find("parentWindowHandle");
 			if(opt != miscParams->end())
-				parentHWnd = (HWND)StringConverter::parseUnsignedInt(opt->second);
+				parentHWnd = (HWND)StringConverter::parseSizeT(opt->second);
 			// externalWindowHandle		-> externalHandle
 			opt = miscParams->find("externalWindowHandle");
 			if(opt != miscParams->end())
-				externalHandle = (HWND)StringConverter::parseUnsignedInt(opt->second);
+				externalHandle = (HWND)StringConverter::parseSizeT(opt->second);
 			// vsync	[parseBool]
 			opt = miscParams->find("vsync");
 			if(opt != miscParams->end())
 				mVSync = StringConverter::parseBool(opt->second);
+			// hidden	[parseBool]
+			opt = miscParams->find("hidden");
+			if(opt != miscParams->end())
+				mHidden = StringConverter::parseBool(opt->second);
 			// vsyncInterval	[parseUnsignedInt]
 			opt = miscParams->find("vsyncInterval");
 			if(opt != miscParams->end())
@@ -160,8 +170,16 @@ namespace Ogre
 			opt = miscParams->find("monitorIndex");
 			if(opt != miscParams->end())
 				monitorIndex = StringConverter::parseInt(opt->second);
+			opt = miscParams->find("show");
+			if(opt != miscParams->end())
+				mHidden = !StringConverter::parseBool(opt->second);
+			// enable double click messages
+			opt = miscParams->find("enableDoubleClick");
+			if(opt != miscParams->end())
+				enableDoubleClick = StringConverter::parseBool(opt->second);
 
 		}
+		mIsFullScreen = fullScreen;
 
 		// Destroy current window if any
 		if( mHWnd )
@@ -169,13 +187,11 @@ namespace Ogre
 
 		if (!externalHandle)
 		{
-			DWORD		dwStyle = WS_VISIBLE | WS_CLIPCHILDREN;
 			DWORD		dwStyleEx = 0;
 			HMONITOR    hMonitor = NULL;		
 			MONITORINFO monitorInfo;
 			RECT		rc;
 
-
 			// If we specified which adapter we want to use - find it's monitor.
 			if (monitorIndex != -1)
 			{
@@ -210,7 +226,31 @@ namespace Ogre
 			monitorInfo.cbSize = sizeof(MONITORINFO);
 			GetMonitorInfo(hMonitor, &monitorInfo);
 
+			// Update window style flags.
+			mFullscreenWinStyle = WS_CLIPCHILDREN | WS_POPUP;
+			mWindowedWinStyle   = WS_CLIPCHILDREN;
 
+			if (!mHidden)
+			{
+				mFullscreenWinStyle |= WS_VISIBLE;
+				mWindowedWinStyle |= WS_VISIBLE;
+			}
+
+			if (parentHWnd)
+			{
+				mWindowedWinStyle |= WS_CHILD;
+			}
+			else
+			{
+				if (border == "none")
+					mWindowedWinStyle |= WS_POPUP;
+				else if (border == "fixed")
+					mWindowedWinStyle |= WS_OVERLAPPED | WS_BORDER | WS_CAPTION |
+					WS_SYSMENU | WS_MINIMIZEBOX;
+				else
+					mWindowedWinStyle |= WS_OVERLAPPEDWINDOW;
+			}
+					
 			unsigned int winWidth, winHeight;
 			winWidth = width;
 			winHeight = height;
@@ -219,12 +259,12 @@ namespace Ogre
 			// No specified top left -> Center the window in the middle of the monitor
 			if (left == INT_MAX || top == INT_MAX)
 			{				
-				int screenw = monitorInfo.rcWork.right  - monitorInfo.rcWork.left;
-				int screenh = monitorInfo.rcWork.bottom - monitorInfo.rcWork.top;
+				uint32 screenw = monitorInfo.rcWork.right  - monitorInfo.rcWork.left;
+				uint32 screenh = monitorInfo.rcWork.bottom - monitorInfo.rcWork.top;
 
 				// clamp window dimensions to screen size
-				int outerw = (winWidth < screenw)? winWidth : screenw;
-				int outerh = (winHeight < screenh)? winHeight : screenh;
+				uint32 outerw = (winWidth < screenw)? winWidth : screenw;
+				uint32 outerh = (winHeight < screenh)? winHeight : screenh;
 
 				if (left == INT_MAX)
 					left = monitorInfo.rcWork.left + (screenw - outerw) / 2;
@@ -249,36 +289,20 @@ namespace Ogre
 
 			if (fullScreen)
 			{
-				dwStyleEx |= WS_EX_TOPMOST;
-				dwStyle |= WS_POPUP;
+				dwStyleEx |= WS_EX_TOPMOST;				
 				mTop = monitorInfo.rcMonitor.top;
 				mLeft = monitorInfo.rcMonitor.left;		
 			}
 			else
-			{
-				if (parentHWnd)
-				{
-					dwStyle |= WS_CHILD;
-				}
-				else
-				{
-					if (border == "none")
-						dwStyle |= WS_POPUP;
-					else if (border == "fixed")
-						dwStyle |= WS_OVERLAPPED | WS_BORDER | WS_CAPTION |
-						WS_SYSMENU | WS_MINIMIZEBOX;
-					else
-						dwStyle |= WS_OVERLAPPEDWINDOW;
-				}
-
-				adjustWindow(width, height, dwStyle, &winWidth, &winHeight);
+			{				
+                adjustWindow(width, height, &winWidth, &winHeight);
 
 				if (!outerSize)
 				{
 					// Calculate window dimensions required
 					// to get the requested client area
 					SetRect(&rc, 0, 0, mWidth, mHeight);
-					AdjustWindowRect(&rc, dwStyle, false);
+					AdjustWindowRect(&rc, getWindowStyle(fullScreen), false);
 					mWidth = rc.right - rc.left;
 					mHeight = rc.bottom - rc.top;
 
@@ -297,10 +321,14 @@ namespace Ogre
 				}
 			}
 			
+			UINT classStyle = 0;
+			if (enableDoubleClick)
+				classStyle |= CS_DBLCLKS;
+
 
 			// Register the window class
 			// NB allow 4 bytes of window data for D3D9RenderWindow pointer
-			WNDCLASS wc = { 0, WindowEventUtilities::_WndProc, 0, 0, hInst,
+			WNDCLASS wc = { classStyle, WindowEventUtilities::_WndProc, 0, 0, hInst,
 				LoadIcon(0, IDI_APPLICATION), LoadCursor(NULL, IDC_ARROW),
 				(HBRUSH)GetStockObject(BLACK_BRUSH), 0, "OgreD3D9Wnd" };
 			RegisterClass(&wc);
@@ -308,9 +336,8 @@ namespace Ogre
 			// Create our main window
 			// Pass pointer to self
 			mIsExternal = false;
-			mHWnd = CreateWindowEx(dwStyleEx, "OgreD3D9Wnd", title.c_str(), dwStyle,
+			mHWnd = CreateWindowEx(dwStyleEx, "OgreD3D9Wnd", title.c_str(), getWindowStyle(fullScreen),
 				mLeft, mTop, winWidth, winHeight, parentHWnd, 0, hInst, this);
-			mStyle = dwStyle;
 
 			WindowEventUtilities::_addRenderWindow(this);
 		}
@@ -331,8 +358,8 @@ namespace Ogre
 		mHeight = rc.bottom;
 
 		mName = name;
-		mIsDepthBuffered = depthBuffer;
-		mIsFullScreen = fullScreen;
+		mDepthBufferPoolId = depthBuffer ? DepthBuffer::POOL_DEFAULT : DepthBuffer::POOL_NO_DEPTH;
+		mDepthBuffer = 0;
 		mColourDepth = colourDepth;
 
 		LogManager::getSingleton().stream()
@@ -342,6 +369,7 @@ namespace Ogre
 									
 		mActive = true;
 		mClosed = false;
+		setHidden(mHidden);
 	}
 
 	void D3D9RenderWindow::setFullscreen(bool fullScreen, unsigned int width, unsigned int height)
@@ -351,8 +379,6 @@ namespace Ogre
 			if (fullScreen != mIsFullScreen)
 				mSwitchingFullscreen = true;
 
-			mStyle = WS_VISIBLE | WS_CLIPCHILDREN;
-
 			bool oldFullscreen = mIsFullScreen;
 			mIsFullScreen = fullScreen;
 			mWidth = mDesiredWidth = width;
@@ -360,8 +386,6 @@ namespace Ogre
 
 			if (fullScreen)
 			{
-				mStyle |= WS_POPUP;
-				
 				// Get the nearest monitor to this window.
 				HMONITOR hMonitor = MonitorFromWindow(mHWnd, MONITOR_DEFAULTTONEAREST);
 
@@ -386,63 +410,50 @@ namespace Ogre
 				{
 					SetWindowPos(mHWnd, HWND_TOPMOST, mLeft, mTop, width, height, SWP_NOACTIVATE);
 					//MoveWindow(mHWnd, mLeft, mTop, mWidth, mHeight, FALSE);
-					SetWindowLong(mHWnd, GWL_STYLE, mStyle);
+					SetWindowLong(mHWnd, GWL_STYLE, getWindowStyle(mIsFullScreen));
 					SetWindowPos(mHWnd, 0, 0,0, 0,0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
 				}
 			}
 			else
 			{
-				mStyle |= WS_OVERLAPPEDWINDOW;
 				// Calculate window dimensions required
 				// to get the requested client area
 				unsigned int winWidth, winHeight;
-				adjustWindow(mWidth, mHeight, mStyle, &winWidth, &winHeight);
+				winWidth = mWidth;
+				winHeight = mHeight;
+				
+				adjustWindow(mWidth, mHeight, &winWidth, &winHeight);
 
-				SetWindowLong(mHWnd, GWL_STYLE, mStyle);
+				SetWindowLong(mHWnd, GWL_STYLE, getWindowStyle(mIsFullScreen));
 				SetWindowPos(mHWnd, HWND_NOTOPMOST, 0, 0, winWidth, winHeight,
 					SWP_DRAWFRAME | SWP_FRAMECHANGED | SWP_NOMOVE | SWP_NOACTIVATE);
 				// Note that we also set the position in the restoreLostDevice method
 				// via _finishSwitchingFullScreen
+
+				// Update the current rect.
+				updateWindowRect();
 			}
-						
+								
 			// Have to release & trigger device reset
 			// NB don't use windowMovedOrResized since Win32 doesn't know
 			// about the size change yet				
 			mDevice->invalidate(this);
 			// Notify viewports of resize
 			ViewportList::iterator it = mViewportList.begin();
-			while(it != mViewportList.end()) (*it++).second->_updateDimensions();
+			while( it != mViewportList.end() )
+				(*it++).second->_updateDimensions();	
 		}
 	} 
 
 	void D3D9RenderWindow::adjustWindow(unsigned int clientWidth, unsigned int clientHeight, 
-		DWORD style, unsigned int* winWidth, unsigned int* winHeight)
+		unsigned int* winWidth, unsigned int* winHeight)
 	{
 		// NB only call this for non full screen
 		RECT rc;
 		SetRect(&rc, 0, 0, clientWidth, clientHeight);
-		AdjustWindowRect(&rc, style, false);
+		AdjustWindowRect(&rc, getWindowStyle(mIsFullScreen), false);
 		*winWidth = rc.right - rc.left;
 		*winHeight = rc.bottom - rc.top;
-
-		// adjust to monitor
-		HMONITOR hMonitor = MonitorFromWindow(mHWnd, MONITOR_DEFAULTTONEAREST);
-
-		// Get monitor info	
-		MONITORINFO monitorInfo;
-
-		memset(&monitorInfo, 0, sizeof(MONITORINFO));
-		monitorInfo.cbSize = sizeof(MONITORINFO);
-		GetMonitorInfo(hMonitor, &monitorInfo);
-
-		LONG maxW = monitorInfo.rcWork.right  - monitorInfo.rcWork.left;
-		LONG maxH = monitorInfo.rcWork.bottom - monitorInfo.rcWork.top;
-
-		if (*winWidth > (unsigned int)maxW)
-			*winWidth = maxW;
-		if (*winHeight > (unsigned int)maxH)
-			*winHeight = maxH;
-
 	}
 
 	void D3D9RenderWindow::_finishSwitchingFullscreen()
@@ -455,40 +466,35 @@ namespace Ogre
 			SetWindowRgn(mHWnd, hRgn, FALSE);
 		}
 		else
-		{
+		{			
 			// When switching back to windowed mode, need to reset window size 
 			// after device has been restored
 			// We may have had a resize event which polluted our desired sizes
+			if (mWidth != mDesiredWidth ||
+				mHeight != mDesiredHeight)
+			{
+				mWidth = mDesiredWidth;
+				mHeight = mDesiredHeight;				
+			}
 			unsigned int winWidth, winHeight;
-			adjustWindow(mDesiredWidth, mDesiredHeight, mStyle, &winWidth, &winHeight);
-
-			// deal with centreing when switching down to smaller resolution
+			adjustWindow(mWidth, mHeight, &winWidth, &winHeight);
 
+			// deal with centering when switching down to smaller resolution
 			HMONITOR hMonitor = MonitorFromWindow(mHWnd, MONITOR_DEFAULTTONEAREST);
 			MONITORINFO monitorInfo;
 			memset(&monitorInfo, 0, sizeof(MONITORINFO));
 			monitorInfo.cbSize = sizeof(MONITORINFO);
 			GetMonitorInfo(hMonitor, &monitorInfo);
 
-			LONG screenw = monitorInfo.rcWork.right  - monitorInfo.rcWork.left;
-			LONG screenh = monitorInfo.rcWork.bottom - monitorInfo.rcWork.top;
-
+			ULONG screenw = monitorInfo.rcWork.right  - monitorInfo.rcWork.left;
+			ULONG screenh = monitorInfo.rcWork.bottom - monitorInfo.rcWork.top;
 
 			int left = screenw > winWidth ? ((screenw - winWidth) / 2) : 0;
 			int top = screenh > winHeight ? ((screenh - winHeight) / 2) : 0;
 			SetWindowPos(mHWnd, HWND_NOTOPMOST, left, top, winWidth, winHeight,
 				SWP_DRAWFRAME | SWP_FRAMECHANGED | SWP_NOACTIVATE);
 
-			if (mWidth != mDesiredWidth ||
-				mHeight != mDesiredHeight)
-			{
-				mWidth = mDesiredWidth;
-				mHeight = mDesiredHeight;
-				// Notify viewports of resize
-				ViewportList::iterator it = mViewportList.begin();
-				while( it != mViewportList.end() )
-					(*it++).second->_updateDimensions();			
-			}
+			updateWindowRect();
 		}
 		mSwitchingFullscreen = false;
 	}
@@ -506,9 +512,9 @@ namespace Ogre
 		ZeroMemory( presentParams, sizeof(D3DPRESENT_PARAMETERS) );
 		presentParams->Windowed					= !mIsFullScreen;
 		presentParams->SwapEffect				= D3DSWAPEFFECT_DISCARD;
-		// triple buffer if VSync is on
+		// triple buffer if VSync is on or if flip swap is used. Otherwise we may get a performance penalty.
 		presentParams->BackBufferCount			= mVSync ? 2 : 1;
-		presentParams->EnableAutoDepthStencil	= mIsDepthBuffered;
+		presentParams->EnableAutoDepthStencil	= (mDepthBufferPoolId != DepthBuffer::POOL_NO_DEPTH);
 		presentParams->hDeviceWindow			= mHWnd;
 		presentParams->BackBufferWidth			= mWidth;
 		presentParams->BackBufferHeight			= mHeight;
@@ -666,6 +672,46 @@ namespace Ogre
 		return (mHWnd && !IsIconic(mHWnd));
 	}
 
+	void D3D9RenderWindow::setHidden(bool hidden)
+	{
+		mHidden = hidden;
+		if (!mIsExternal)
+		{
+			if (hidden)
+				ShowWindow(mHWnd, SW_HIDE);
+			else
+				ShowWindow(mHWnd, SW_SHOWNORMAL);
+		}
+	}
+
+	void D3D9RenderWindow::setVSyncEnabled(bool vsync)
+	{
+		mVSync = vsync;
+		if (!mIsExternal)
+		{
+			// we need to reset the device with new vsync params
+			// invalidate the window to trigger this
+			mDevice->invalidate(this);
+		}
+	}
+
+	bool D3D9RenderWindow::isVSyncEnabled() const
+	{
+		return mVSync;
+	}
+
+	void D3D9RenderWindow::setVSyncInterval(unsigned int interval)
+	{
+		mVSyncInterval = interval;
+		if (mVSync)
+			setVSyncEnabled(true);
+	}
+
+	unsigned int D3D9RenderWindow::getVSyncInterval() const
+	{
+		return mVSyncInterval;
+	}
+
 	void D3D9RenderWindow::reposition(int top, int left)
 	{
 		if (mHWnd && !mIsFullScreen)
@@ -677,13 +723,18 @@ namespace Ogre
 
 	void D3D9RenderWindow::resize(unsigned int width, unsigned int height)
 	{
-		if (mHWnd && !mIsFullScreen)
+		if (!mIsExternal)
 		{
-			unsigned int winWidth, winHeight;
-			adjustWindow(width, height, mStyle, &winWidth, &winHeight);
-			SetWindowPos(mHWnd, 0, 0, 0, winWidth, winHeight,
-				SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
+			if (mHWnd && !mIsFullScreen)
+			{
+				unsigned int winWidth, winHeight;
+				adjustWindow(width, height, &winWidth, &winHeight);
+				SetWindowPos(mHWnd, 0, 0, 0, winWidth, winHeight,
+					SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
+			}
 		}
+		else
+			updateWindowRect();
 	}
 
 	void D3D9RenderWindow::windowMovedOrResized()
@@ -694,7 +745,7 @@ namespace Ogre
 		updateWindowRect();
 	}
 
-	void D3D9RenderWindow::swapBuffers( bool waitForVSync )
+	void D3D9RenderWindow::swapBuffers( )
 	{
 		if (mDeviceValid)
 			mDevice->present(this);		
@@ -832,7 +883,7 @@ namespace Ogre
 	//-----------------------------------------------------------------------------
 	bool D3D9RenderWindow::isDepthBuffered() const
 	{
-		return mIsDepthBuffered;
+		return (mDepthBufferPoolId != DepthBuffer::POOL_NO_DEPTH);
 	}
 
 	//-----------------------------------------------------------------------------
