@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -46,7 +46,7 @@ namespace Ogre
         ResourceHandle handle, const String& group, bool isManual, 
         ManualResourceLoader* loader)
         :Texture(creator, name, handle, group, isManual, loader),              
-        mD3DPool(D3DPOOL_MANAGED),
+		mD3DPool(D3D9RenderSystem::isDirectX9Ex() ? D3DPOOL_DEFAULT : D3DPOOL_MANAGED),
 		mDynamicTextures(false),
 		mHwGammaReadSupported(false),
 		mHwGammaWriteSupported(false),	
@@ -77,7 +77,7 @@ namespace Ogre
 		{
 			TextureResources* textureResource = it->second;
 			
-			SAFE_DELETE(textureResource);
+			OGRE_DELETE_T(textureResource, TextureResources, MEMCATEGORY_RENDERSYS);
 			++it;
 		}		
 		mMapDeviceToTextureResources.clear();
@@ -201,7 +201,9 @@ namespace Ogre
 			// NB: Need to initialise pool to some value other than D3DPOOL_DEFAULT,
 			// otherwise, if the texture loading failed, it might re-create as empty
 			// texture when device lost/restore. The actual pool will determine later.
-			mD3DPool = D3DPOOL_MANAGED;
+			//
+			// In directX9Ex this is not the case there is no managed pool and no device loss
+			mD3DPool = D3D9RenderSystem::isDirectX9Ex() ? D3DPOOL_DEFAULT : D3DPOOL_MANAGED;
 		}
 
 		D3D9_DEVICE_ACCESS_CRITICAL_SECTION
@@ -294,7 +296,7 @@ namespace Ogre
 	{
 		assert(getTextureType() == TEX_TYPE_CUBE_MAP);
 
-        LoadedStreams loadedStreams = LoadedStreams(new vector<MemoryDataStreamPtr>::type());
+        LoadedStreams loadedStreams = LoadedStreams(OGRE_NEW_T (vector<MemoryDataStreamPtr>::type, MEMCATEGORY_GENERAL), SPFM_DELETE_T );
         // DDS load?
 		if (getSourceFileType() == "dds")
 		{
@@ -302,7 +304,7 @@ namespace Ogre
 			DataStreamPtr dstream = 
 				ResourceGroupManager::getSingleton().openResource(
 					mName, mGroup, true, this);
-            loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+            loadedStreams->push_back(MemoryDataStreamPtr(OGRE_NEW MemoryDataStream(dstream)));
         }
         else
         {
@@ -327,7 +329,7 @@ namespace Ogre
 					ResourceGroupManager::getSingleton().openResource(
 						fullName, mGroup, true, this);
 
-                loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+                loadedStreams->push_back(MemoryDataStreamPtr(OGRE_NEW MemoryDataStream(dstream)));
 			}
         }
 
@@ -343,8 +345,8 @@ namespace Ogre
 			ResourceGroupManager::getSingleton().openResource(
 				mName, mGroup, true, this);
 
-        LoadedStreams loadedStreams = LoadedStreams(new vector<MemoryDataStreamPtr>::type());
-        loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+        LoadedStreams loadedStreams = LoadedStreams(OGRE_NEW_T (vector<MemoryDataStreamPtr>::type, MEMCATEGORY_GENERAL), SPFM_DELETE_T);
+        loadedStreams->push_back(MemoryDataStreamPtr(OGRE_NEW MemoryDataStream(dstream)));
         return loadedStreams;
     }
 	/****************************************************************************************/
@@ -357,8 +359,8 @@ namespace Ogre
 			ResourceGroupManager::getSingleton().openResource(
 				mName, mGroup, true, this);
 
-        LoadedStreams loadedStreams = LoadedStreams(new vector<MemoryDataStreamPtr>::type());
-        loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+        LoadedStreams loadedStreams = LoadedStreams(OGRE_NEW_T (vector<MemoryDataStreamPtr>::type, MEMCATEGORY_GENERAL), SPFM_DELETE_T);
+        loadedStreams->push_back(MemoryDataStreamPtr(OGRE_NEW MemoryDataStream(dstream)));
         return loadedStreams;
 	}
 	/****************************************************************************************/
@@ -415,7 +417,7 @@ namespace Ogre
 	{
 		assert(mMapDeviceToTextureResources.find(d3d9Device) == mMapDeviceToTextureResources.end());
 
-		TextureResources* textureResources = new TextureResources;
+		TextureResources* textureResources = OGRE_NEW_T(TextureResources, MEMCATEGORY_RENDERSYS);
 
 		textureResources->pNormTex		= NULL;
 		textureResources->pCubeTex		= NULL;
@@ -506,7 +508,22 @@ namespace Ogre
 				numMips = 1;
 			}
 
-            // Determine D3D pool to use
+			// Check dynamic textures
+			if (mUsage & TU_DYNAMIC)
+			{
+				D3DFORMAT d3dPF = _chooseD3DFormat(d3d9Device);
+				if (_canUseDynamicTextures(d3d9Device, usage, D3DRTYPE_TEXTURE, d3dPF))
+				{
+					usage |= D3DUSAGE_DYNAMIC;
+					mDynamicTextures = true;
+				}
+				else
+				{
+					mDynamicTextures = false;
+				}
+			}
+
+			// Determine D3D pool to use
             D3DPOOL pool;
             if (useDefaultPool())
             {
@@ -625,7 +642,22 @@ namespace Ogre
 				numMips = 1;
 			}
 
-            // Determine D3D pool to use
+			// Check dynamic textures
+			if (mUsage & TU_DYNAMIC)
+			{
+				D3DFORMAT d3dPF = _chooseD3DFormat(d3d9Device);
+				if (_canUseDynamicTextures(d3d9Device, usage, D3DRTYPE_TEXTURE, d3dPF))
+				{
+					usage |= D3DUSAGE_DYNAMIC;
+					mDynamicTextures = true;
+				}
+				else
+				{
+					mDynamicTextures = false;
+				}
+			}
+			
+			// Determine D3D pool to use
             D3DPOOL pool;
             if (useDefaultPool())
             {
@@ -769,6 +801,21 @@ namespace Ogre
 				numMips = 1;
 			}
 
+			// Check dynamic textures
+			if (mUsage & TU_DYNAMIC)
+			{
+				D3DFORMAT d3dPF = _chooseD3DFormat(d3d9Device);
+				if (_canUseDynamicTextures(d3d9Device, usage, D3DRTYPE_VOLUMETEXTURE, d3dPF))
+				{
+					usage |= D3DUSAGE_DYNAMIC;
+					mDynamicTextures = true;
+				}
+				else
+				{
+					mDynamicTextures = false;
+				}
+			}
+
             // Determine D3D pool to use
             D3DPOOL pool;
             if (useDefaultPool())
@@ -791,11 +838,14 @@ namespace Ogre
 
 			HRESULT hr;
 
+			UINT autoWidthHeight = Root::getSingleton().getRenderSystem()->getCapabilities()->hasCapability(RSC_NON_POWER_OF_2_TEXTURES) ? 
+				D3DX_DEFAULT_NONPOW2 : D3DX_DEFAULT;
+
 			hr = D3DXCreateTextureFromFileInMemoryEx(
 				d3d9Device,
 				(*loadedStreams)[0]->getPtr(),
 				static_cast<UINT>((*loadedStreams)[0]->size()),
-				D3DX_DEFAULT, D3DX_DEFAULT, // dims
+				autoWidthHeight, autoWidthHeight, // dims
 				numMips,
 				usage,
 				D3DFMT_UNKNOWN,
@@ -960,7 +1010,7 @@ namespace Ogre
 		// we must have those defined here
 		assert(mSrcWidth > 0 || mSrcHeight > 0);
 
-		// determine wich D3D9 pixel format we'll use
+		// determine which D3D9 pixel format we'll use
 		HRESULT hr;
 		D3DFORMAT d3dPF = _chooseD3DFormat(d3d9Device);
 
@@ -1120,7 +1170,7 @@ namespace Ogre
 		// we must have those defined here
 		assert(mSrcWidth > 0 || mSrcHeight > 0);
 
-		// determine wich D3D9 pixel format we'll use
+		// determine which D3D9 pixel format we'll use
 		HRESULT hr;
 		D3DFORMAT d3dPF = _chooseD3DFormat(d3d9Device);
 
@@ -1697,7 +1747,7 @@ namespace Ogre
 			{
 				for(size_t mip=0; mip<=mNumMipmaps; ++mip)
 				{
-					buffer = new D3D9HardwarePixelBuffer((HardwareBuffer::Usage)bufusage, this);
+					buffer = OGRE_NEW D3D9HardwarePixelBuffer((HardwareBuffer::Usage)bufusage, this);
 					mSurfaceList.push_back(HardwarePixelBufferSharedPtr(buffer));
 				}
 			}
@@ -1810,7 +1860,8 @@ namespace Ogre
 		// Use managed unless we're a render target or user has asked for 
 		// a dynamic texture, and device supports D3DUSAGE_DYNAMIC (because default pool
 		// resources without the dynamic flag are not lockable)
-		return (mUsage & TU_RENDERTARGET) || ((mUsage & TU_DYNAMIC) && mDynamicTextures);
+		// or use if we are using directX9Ex as there is no managed pool under it.
+		return (D3D9RenderSystem::isDirectX9Ex()) || (mUsage & TU_RENDERTARGET) || ((mUsage & TU_DYNAMIC) && mDynamicTextures);
 	}
 	
 	//---------------------------------------------------------------------
@@ -1819,7 +1870,18 @@ namespace Ogre
 		D3D9_DEVICE_ACCESS_CRITICAL_SECTION
 
 		if (D3D9RenderSystem::getResourceManager()->getCreationPolicy() == RCP_CREATE_ON_ALL_DEVICES)
-			createTextureResources(d3d9Device);
+        {
+            try
+            {
+			    createTextureResources(d3d9Device);
+            }
+            catch (...)
+            {
+                mLoadingState.set(LOADSTATE_UNLOADED);
+                LogManager::getSingleton().stream() << "Warning: Failed to restore texture " << getName()
+                    << " on DeviceCreate.";
+            }
+        }
 	}
 
 	//---------------------------------------------------------------------
@@ -1831,7 +1893,7 @@ namespace Ogre
 
 		if (it != mMapDeviceToTextureResources.end())
 		{			
-			std::stringstream ss;
+			StringStream ss;
 
 			ss << "D3D9 device: 0x[" << d3d9Device << "] destroy. Releasing D3D9 texture: " << mName;
 			LogManager::getSingleton().logMessage(ss.str());
@@ -1851,8 +1913,8 @@ namespace Ogre
 			// after device reset.
 			freeTextureResources(d3d9Device, textureResource);
 
-			SAFE_DELETE(textureResource);
-
+			OGRE_DELETE_T(textureResource, TextureResources, MEMCATEGORY_RENDERSYS);
+			
 			mMapDeviceToTextureResources.erase(it);
 
 			LogManager::getSingleton().logMessage("Released D3D9 texture: " + mName);	
@@ -1870,7 +1932,7 @@ namespace Ogre
 
 			if (it != mMapDeviceToTextureResources.end())
 			{
-				std::stringstream ss;
+				StringStream ss;
 
 				ss << "D3D9 device: 0x[" << d3d9Device << "] lost. Releasing D3D9 texture: " << mName;
 				LogManager::getSingleton().logMessage(ss.str());
@@ -2020,7 +2082,7 @@ namespace Ogre
 		}
 	}    
 	//---------------------------------------------------------------------
-	void D3D9RenderTexture::swapBuffers(bool waitForVSync /* = true */)
+	void D3D9RenderTexture::swapBuffers()
 	{
 		// Only needed if we have to blit from AA surface
 		if (mFSAA > 0)
