@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,7 @@ THE SOFTWARE.
 #define __OgreOSXCocoaContext_H__
 
 #include "OgreOSXContext.h"
-#import <Cocoa/Cocoa.h>
+#import <AppKit/NSOpenGL.h>
 
 namespace Ogre {
 
@@ -63,6 +63,10 @@ namespace Ogre {
 		/** Grab the NSOpenGLPixelFormat if it exists */
 		NSOpenGLPixelFormat* getPixelFormat();
 		
+        /* The pixel dimensions of the backbuffer */
+        GLint mBackingWidth;
+        GLint mBackingHeight;
+
 	private:
 		NSOpenGLContext* mNSGLContext;
 		NSOpenGLPixelFormat *mNSGLPixelFormat;
