@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -33,8 +33,10 @@ THE SOFTWARE.
 #include "OgreGLSLProgram.h"
 #include "OgreGLSLLinkProgramManager.h"
 #include "OgreException.h"
+#include "OgreGpuProgramManager.h"
 
 namespace Ogre {
+    namespace GLSL {
 
 	//  a  builtin				custom attrib name
 	// ----------------------------------------------
@@ -103,9 +105,10 @@ namespace Ogre {
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
 				"Geometry shader output operation type can only be point list,"
 				"line strip or triangle strip",
-				"GLSLLinkProgram::activate");
+                            "GLSLLinkProgram::getGLGeometryOutputPrimitiveType");
 		}
 	}
+
 	//-----------------------------------------------------------------------
 	GLSLLinkProgram::GLSLLinkProgram(GLSLGpuProgram* vertexProgram, GLSLGpuProgram* geometryProgram, GLSLGpuProgram* fragmentProgram)
         : mVertexProgram(vertexProgram)
@@ -113,31 +116,10 @@ namespace Ogre {
 		, mFragmentProgram(fragmentProgram)
 		, mUniformRefsBuilt(false)
         , mLinked(false)
-
+		, mTriedToLinkAndFailed(false)
 	{
-			//checkForGLSLError( "GLSLLinkProgram::GLSLLinkProgram", "Error prior to Creating GLSL Program Object", 0);
-		    glGetError(); //Clean up the error. Otherwise will flood log.
-		    mGLHandle = glCreateProgramObjectARB();
-			checkForGLSLError( "GLSLLinkProgram::GLSLLinkProgram", "Error Creating GLSL Program Object", 0 );
-
-
-			// tell shaders to attach themselves to the LinkProgram
-			// let the shaders do the attaching since they may have several children to attach
-			if (mVertexProgram)
-			{
-				mVertexProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
-				setSkeletalAnimationIncluded(mVertexProgram->isSkeletalAnimationIncluded());
-			}
-			if (mGeometryProgram)
-			{
-				mGeometryProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
-				//Don't set adjacency flag. We handle it internally and expose "false"
-			}
-			if (mFragmentProgram)
-			{
-				mFragmentProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
-			}
-
+        // Initialise uniform cache
+		mUniformCache = new GLUniformCache();
 	}
 
 	//-----------------------------------------------------------------------
@@ -145,104 +127,83 @@ namespace Ogre {
 	{
 		glDeleteObjectARB(mGLHandle);
 
+        delete mUniformCache;
+        mUniformCache = 0;
 	}
 
 	//-----------------------------------------------------------------------
 	void GLSLLinkProgram::activate(void)
 	{
-		if (!mLinked)
-		{
-			if (mVertexProgram)
+		if (!mLinked && !mTriedToLinkAndFailed)
+		{			
+			glGetError(); //Clean up the error. Otherwise will flood log.
+
+			mGLHandle = glCreateProgramObjectARB();
+
+            GLenum glErr = glGetError();
+            if(glErr != GL_NO_ERROR)
+            {
+			    reportGLSLError( glErr, "GLSLLinkProgram::activate", "Error Creating GLSL Program Object", 0 );
+            }
+
+			if ( GpuProgramManager::getSingleton().canGetCompiledShaderBuffer() &&
+				 GpuProgramManager::getSingleton().isMicrocodeAvailableInCache(getCombinedName()) )
 			{
-				// Some drivers (e.g. OS X on nvidia) incorrectly determine the attribute binding automatically
-				// and end up aliasing existing built-ins. So avoid! 
-				// Bind all used attribs - not all possible ones otherwise we'll get 
-				// lots of warnings in the log, and also may end up aliasing names used
-				// as varyings by accident
-				// Because we can't ask GL whether an attribute is used in the shader
-				// until it is linked (chicken and egg!) we have to parse the source
-				
-				size_t numAttribs = sizeof(msCustomAttributes)/sizeof(CustomAttribute);
-				const String& vpSource = mVertexProgram->getGLSLProgram()->getSource();
-				for (size_t i = 0; i < numAttribs; ++i)
-				{
-					const CustomAttribute& a = msCustomAttributes[i];
-					
-					// we're looking for either: 
-					//   attribute vec<n> <semantic_name>
-					//   in vec<n> <semantic_name>
-					// The latter is recommended in GLSL 1.3 onwards 
-					// be slightly flexible about formatting
-					String::size_type pos = vpSource.find(a.name);
-					if (pos != String::npos)
-					{
-						String::size_type startpos = vpSource.find("attribute", pos < 20 ? 0 : pos-20);
-						if (startpos == String::npos)
-							startpos = vpSource.find("in", pos-20);
-						if (startpos != String::npos && startpos < pos)
-						{
-							// final check 
-							String expr = vpSource.substr(startpos, pos + a.name.length() - startpos);
-							StringVector vec = StringUtil::split(expr);
-							if ((vec[0] == "in" || vec[0] == "attribute") && vec[2] == a.name)
-								glBindAttribLocationARB(mGLHandle, a.attrib, a.name.c_str());
-						}
-
-					}
-				}
+				getMicrocodeFromCache();
 			}
-
-			if (mGeometryProgram)
+			else
 			{
-				RenderOperation::OperationType inputOperationType = mGeometryProgram->getGLSLProgram()->getInputOperationType();
-				glProgramParameteriEXT(mGLHandle,GL_GEOMETRY_INPUT_TYPE_EXT,
-					getGLGeometryInputPrimitiveType(inputOperationType, mGeometryProgram->isAdjacencyInfoRequired()));
-				
-				RenderOperation::OperationType outputOperationType = mGeometryProgram->getGLSLProgram()->getOutputOperationType();
-				switch (outputOperationType)
-				{
-				case RenderOperation::OT_POINT_LIST:
-				case RenderOperation::OT_LINE_STRIP:
-				case RenderOperation::OT_TRIANGLE_STRIP:
-                case RenderOperation::OT_LINE_LIST:
-                case RenderOperation::OT_TRIANGLE_LIST:
-                case RenderOperation::OT_TRIANGLE_FAN:
-					break;
-				
-				}
-				glProgramParameteriEXT(mGLHandle,GL_GEOMETRY_OUTPUT_TYPE_EXT,
-					getGLGeometryOutputPrimitiveType(outputOperationType));
+				compileAndLink();
 
-				glProgramParameteriEXT(mGLHandle,GL_GEOMETRY_VERTICES_OUT_EXT,
-					mGeometryProgram->getGLSLProgram()->getMaxOutputVertices());
 			}
-
-			glLinkProgramARB( mGLHandle );
-			glGetObjectParameterivARB( mGLHandle, GL_OBJECT_LINK_STATUS_ARB, &mLinked );
-			// force logging and raise exception if not linked
-			checkForGLSLError( "GLSLLinkProgram::Activate",
-				"Error linking GLSL Program Object : ", mGLHandle, !mLinked, !mLinked );
-			if(mLinked)
-			{
-				logObjectInfo( String("GLSL link result : "), mGLHandle );
-				buildGLUniformReferences();
-				extractAttributes();
-			}
-
+			buildGLUniformReferences();
+			extractAttributes();
 		}
-
 		if (mLinked)
 		{
-			checkForGLSLError( "GLSLLinkProgram::Activate",
-				"Error prior to using GLSL Program Object : ", mGLHandle, false, false);
+            GLenum glErr = glGetError();
+            if(glErr != GL_NO_ERROR)
+            {
+			    reportGLSLError( glErr, "GLSLLinkProgram::Activate",
+				    "Error prior to using GLSL Program Object : ", mGLHandle, false, false);
+            }
 
-		    glUseProgramObjectARB( mGLHandle );
+			glUseProgramObjectARB( mGLHandle );
 
-			checkForGLSLError( "GLSLLinkProgram::Activate",
-				"Error using GLSL Program Object : ", mGLHandle, false, false);
-		}
+            glErr = glGetError();
+            if(glErr != GL_NO_ERROR)
+            {
+			    reportGLSLError( glErr, "GLSLLinkProgram::Activate",
+				    "Error using GLSL Program Object : ", mGLHandle, false, false);
+            }
+        }
 	}
+    //-----------------------------------------------------------------------
+    void GLSLLinkProgram::getMicrocodeFromCache(void)
+    {
+		GpuProgramManager::Microcode cacheMicrocode = 
+			GpuProgramManager::getSingleton().getMicrocodeFromCache(getCombinedName());
+		
+		GLenum binaryFormat = *((GLenum *)(cacheMicrocode->getPtr()));
+		uint8 * programBuffer = cacheMicrocode->getPtr() + sizeof(GLenum);
+		size_t sizeOfBuffer = cacheMicrocode->size() - sizeof(GLenum);
+		glProgramBinary(mGLHandle, 
+						binaryFormat, 
+						programBuffer,
+						static_cast<GLsizei>(sizeOfBuffer)
+						);
 
+        glGetProgramiv(mGLHandle, GL_LINK_STATUS, &mLinked);
+        if (!mLinked)
+        {
+            //
+            // Something must have changed since the program binaries
+            // were cached away.  Fallback to source shader loading path,
+            // and then retrieve and cache new program binaries once again.
+            //
+			compileAndLink();
+        }
+	}
 	//-----------------------------------------------------------------------
 	void GLSLLinkProgram::extractAttributes(void)
 	{
@@ -305,6 +266,15 @@ namespace Ogre {
 		GLUniformReferenceIterator currentUniform = mGLUniformReferences.begin();
 		GLUniformReferenceIterator endUniform = mGLUniformReferences.end();
 
+        // determine if we need to transpose matrices when binding
+        int transpose = GL_TRUE;
+        if ((fromProgType == GPT_FRAGMENT_PROGRAM && mVertexProgram && (!mVertexProgram->getGLSLProgram()->getColumnMajorMatrices())) ||
+            (fromProgType == GPT_VERTEX_PROGRAM && mFragmentProgram && (!mFragmentProgram->getGLSLProgram()->getColumnMajorMatrices())) ||
+            (fromProgType == GPT_GEOMETRY_PROGRAM && mGeometryProgram && (!mGeometryProgram->getGLSLProgram()->getColumnMajorMatrices())))
+        {
+            transpose = GL_FALSE;
+        }
+
 		for (;currentUniform != endUniform; ++currentUniform)
 		{
 			// Only pull values from buffer it's supposed to be in (vertex or fragment)
@@ -318,6 +288,35 @@ namespace Ogre {
 
 					GLsizei glArraySize = (GLsizei)def->arraySize;
 
+                    bool shouldUpdate = true;
+
+                    switch (def->constType)
+                    {
+                        case GCT_INT1:
+                        case GCT_INT2:
+                        case GCT_INT3:
+                        case GCT_INT4:
+                        case GCT_SAMPLER1D:
+                        case GCT_SAMPLER1DSHADOW:
+                        case GCT_SAMPLER2D:
+                        case GCT_SAMPLER2DSHADOW:
+                        case GCT_SAMPLER3D:
+                        case GCT_SAMPLERCUBE:
+                            shouldUpdate = mUniformCache->updateUniform(currentUniform->mLocation,
+                                                                        params->getIntPointer(def->physicalIndex),
+                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(int)));
+                            break;
+                        default:
+                            shouldUpdate = mUniformCache->updateUniform(currentUniform->mLocation,
+                                                                        params->getFloatPointer(def->physicalIndex),
+                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(float)));
+                            break;
+
+                    }
+
+                    if(!shouldUpdate)
+                        continue;
+
 					// get the index in the parameter real list
 					switch (def->constType)
 					{
@@ -339,57 +338,57 @@ namespace Ogre {
 						break;
 					case GCT_MATRIX_2X2:
 						glUniformMatrix2fvARB(currentUniform->mLocation, glArraySize, 
-							GL_TRUE, params->getFloatPointer(def->physicalIndex));
+							transpose, params->getFloatPointer(def->physicalIndex));
 						break;
 					case GCT_MATRIX_2X3:
 						if (GLEW_VERSION_2_1)
 						{
 							glUniformMatrix2x3fv(currentUniform->mLocation, glArraySize, 
-								GL_TRUE, params->getFloatPointer(def->physicalIndex));
+								transpose, params->getFloatPointer(def->physicalIndex));
 						}
 						break;
 					case GCT_MATRIX_2X4:
 						if (GLEW_VERSION_2_1)
 						{
 							glUniformMatrix2x4fv(currentUniform->mLocation, glArraySize, 
-								GL_TRUE, params->getFloatPointer(def->physicalIndex));
+								transpose, params->getFloatPointer(def->physicalIndex));
 						}
 						break;
 					case GCT_MATRIX_3X2:
 						if (GLEW_VERSION_2_1)
 						{
 							glUniformMatrix3x2fv(currentUniform->mLocation, glArraySize, 
-								GL_TRUE, params->getFloatPointer(def->physicalIndex));
+								transpose, params->getFloatPointer(def->physicalIndex));
 						}
 						break;
 					case GCT_MATRIX_3X3:
 						glUniformMatrix3fvARB(currentUniform->mLocation, glArraySize, 
-							GL_TRUE, params->getFloatPointer(def->physicalIndex));
+							transpose, params->getFloatPointer(def->physicalIndex));
 						break;
 					case GCT_MATRIX_3X4:
 						if (GLEW_VERSION_2_1)
 						{
 							glUniformMatrix3x4fv(currentUniform->mLocation, glArraySize, 
-								GL_TRUE, params->getFloatPointer(def->physicalIndex));
+								transpose, params->getFloatPointer(def->physicalIndex));
 						}
 						break;
 					case GCT_MATRIX_4X2:
 						if (GLEW_VERSION_2_1)
 						{
 							glUniformMatrix4x2fv(currentUniform->mLocation, glArraySize, 
-								GL_TRUE, params->getFloatPointer(def->physicalIndex));
+								transpose, params->getFloatPointer(def->physicalIndex));
 						}
 						break;
 					case GCT_MATRIX_4X3:
 						if (GLEW_VERSION_2_1)
 						{
 							glUniformMatrix4x3fv(currentUniform->mLocation, glArraySize, 
-								GL_TRUE, params->getFloatPointer(def->physicalIndex));
+								transpose, params->getFloatPointer(def->physicalIndex));
 						}
 						break;
 					case GCT_MATRIX_4X4:
 						glUniformMatrix4fvARB(currentUniform->mLocation, glArraySize, 
-							GL_TRUE, params->getFloatPointer(def->physicalIndex));
+							transpose, params->getFloatPointer(def->physicalIndex));
 						break;
 					case GCT_INT1:
 						glUniform1ivARB(currentUniform->mLocation, glArraySize, 
@@ -411,6 +410,7 @@ namespace Ogre {
 					case GCT_SAMPLER1DSHADOW:
 					case GCT_SAMPLER2D:
 					case GCT_SAMPLER2DSHADOW:
+                    case GCT_SAMPLER2DARRAY:
 					case GCT_SAMPLER3D:
 					case GCT_SAMPLERCUBE:
 						// samplers handled like 1-element ints
@@ -418,19 +418,22 @@ namespace Ogre {
 							(GLint*)params->getIntPointer(def->physicalIndex));
 						break;
                     case GCT_UNKNOWN:
+                    default:
                         break;
 
 					} // end switch
 	#if OGRE_DEBUG_MODE
-					checkForGLSLError( "GLSLLinkProgram::updateUniforms", "Error updating uniform", 0 );
+                    GLenum glErr = glGetError();
+                    if(glErr != GL_NO_ERROR)
+                    {
+					    reportGLSLError( glErr, "GLSLLinkProgram::updateUniforms", "Error updating uniform", 0 );
+                    }
 	#endif
 				} // variability & mask
 			} // fromProgType == currentUniform->mSourceProgType
   
   		} // end for
 	}
-
-
 	//-----------------------------------------------------------------------
 	void GLSLLinkProgram::updatePassIterationUniforms(GpuProgramParametersSharedPtr params)
 	{
@@ -447,12 +450,196 @@ namespace Ogre {
 				// get the index in the parameter real list
 				if (index == currentUniform->mConstantDef->physicalIndex)
 				{
-					glUniform1fvARB( currentUniform->mLocation, 1, params->getFloatPointer(index));
-					// there will only be one multipass entry
-					return;
+                    if(!mUniformCache->updateUniform(currentUniform->mLocation,
+                                                     params->getFloatPointer(index),
+                                                     static_cast<GLsizei>(currentUniform->mConstantDef->elementSize *
+                                                     currentUniform->mConstantDef->arraySize *
+                                                     sizeof(float))))
+                        return;
 				}
 			}
 		}
 
     }
+	//-----------------------------------------------------------------------
+	Ogre::String GLSLLinkProgram::getCombinedName()
+	{
+		String name;
+		if (mVertexProgram)
+		{
+			name += "Vertex Program:" ;
+			name += mVertexProgram->getName();
+		}
+		if (mFragmentProgram)
+		{
+			name += " Fragment Program:" ;
+			name += mFragmentProgram->getName();
+		}
+		if (mGeometryProgram)
+		{
+			name += " Geometry Program:" ;
+			name += mGeometryProgram->getName();
+		}
+		return name;
+	}
+	//-----------------------------------------------------------------------
+	void GLSLLinkProgram::compileAndLink()
+	{
+		if (mVertexProgram)
+		{
+			// compile and attach Vertex Program
+			if (!mVertexProgram->getGLSLProgram()->compile(true))
+			{
+				// todo error
+				return;
+			}
+			mVertexProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
+			setSkeletalAnimationIncluded(mVertexProgram->isSkeletalAnimationIncluded());
+
+			// Some drivers (e.g. OS X on nvidia) incorrectly determine the attribute binding automatically
+
+			// and end up aliasing existing built-ins. So avoid! 
+			// Bind all used attribs - not all possible ones otherwise we'll get 
+			// lots of warnings in the log, and also may end up aliasing names used
+			// as varyings by accident
+			// Because we can't ask GL whether an attribute is used in the shader
+			// until it is linked (chicken and egg!) we have to parse the source
+
+			size_t numAttribs = sizeof(msCustomAttributes)/sizeof(CustomAttribute);
+			const String& vpSource = mVertexProgram->getGLSLProgram()->getSource();
+			for (size_t i = 0; i < numAttribs; ++i)
+			{
+				const CustomAttribute& a = msCustomAttributes[i];
+
+				// we're looking for either: 
+				//   attribute vec<n> <semantic_name>
+				//   in vec<n> <semantic_name>
+				// The latter is recommended in GLSL 1.3 onwards 
+				// be slightly flexible about formatting
+				String::size_type pos = vpSource.find(a.name);
+                bool foundAttr = false;
+				while (pos != String::npos && !foundAttr)
+				{
+					String::size_type startpos = vpSource.find("attribute", pos < 20 ? 0 : pos-20);
+					if (startpos == String::npos)
+						startpos = vpSource.find("in", pos-20);
+					if (startpos != String::npos && startpos < pos)
+					{
+						// final check 
+						String expr = vpSource.substr(startpos, pos + a.name.length() - startpos);
+						StringVector vec = StringUtil::split(expr);
+						if ((vec[0] == "in" || vec[0] == "attribute") && vec[2] == a.name)
+                        {
+							glBindAttribLocationARB(mGLHandle, a.attrib, a.name.c_str());
+                            foundAttr = true;
+                        }
+					}
+                    // Find the position of the next occurrence if needed
+                    pos = vpSource.find(a.name, pos + a.name.length());
+				}
+			}
+		}
+
+		if (mGeometryProgram)
+		{
+			// compile and attach Geometry Program
+			if (!mGeometryProgram->getGLSLProgram()->compile(true))
+			{
+				// todo error
+				return;
+			}
+
+			mGeometryProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
+
+			//Don't set adjacency flag. We handle it internally and expose "false"
+
+			RenderOperation::OperationType inputOperationType = mGeometryProgram->getGLSLProgram()->getInputOperationType();
+			glProgramParameteriEXT(mGLHandle, GL_GEOMETRY_INPUT_TYPE_EXT,
+				getGLGeometryInputPrimitiveType(inputOperationType, mGeometryProgram->isAdjacencyInfoRequired()));
+
+			RenderOperation::OperationType outputOperationType = mGeometryProgram->getGLSLProgram()->getOutputOperationType();
+
+			glProgramParameteriEXT(mGLHandle, GL_GEOMETRY_OUTPUT_TYPE_EXT,
+				getGLGeometryOutputPrimitiveType(outputOperationType));
+
+			glProgramParameteriEXT(mGLHandle, GL_GEOMETRY_VERTICES_OUT_EXT,
+				mGeometryProgram->getGLSLProgram()->getMaxOutputVertices());
+		}
+
+		if (mFragmentProgram)
+		{
+			// compile and attach Fragment Program
+			if (!mFragmentProgram->getGLSLProgram()->compile(true))
+			{
+				// todo error
+				return;
+			}		
+			mFragmentProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
+		}
+
+		
+		// now the link
+
+		glLinkProgramARB( mGLHandle );
+		glGetObjectParameterivARB( mGLHandle, GL_OBJECT_LINK_STATUS_ARB, &mLinked );
+		mTriedToLinkAndFailed = !mLinked;
+
+		if(mTriedToLinkAndFailed == true)
+		{
+			GLchar *msg = NULL;
+			GLint length;
+			glGetProgramiv(mGLHandle, GL_INFO_LOG_LENGTH, &length);
+			msg = new GLchar[length];
+			glGetProgramInfoLog(mGLHandle, length, NULL, msg);
+			logObjectInfo( getCombinedName() + String(", Error linking program: ") + msg, mGLHandle );
+			delete msg;
+		}
+		
+		// force logging and raise exception if not linked
+		GLenum glErr = glGetError();
+        if(glErr != GL_NO_ERROR)
+        {
+		    reportGLSLError( glErr, "GLSLLinkProgram::compileAndLink",
+			    "Error linking GLSL Program Object : ", mGLHandle, !mLinked, !mLinked );
+        }
+		
+		if(mLinked)
+		{
+			logObjectInfo(  getCombinedName() + String(" GLSL link result : "), mGLHandle );
+		}
+
+		if (mLinked)
+		{
+			if ( GpuProgramManager::getSingleton().getSaveMicrocodesToCache() )
+			{
+				// add to the microcode to the cache
+				String name;
+				name = getCombinedName();
+
+				// get buffer size
+				GLint binaryLength = 0;
+				glGetProgramiv(mGLHandle, GL_PROGRAM_BINARY_LENGTH, &binaryLength);
+
+				// turns out we need this param when loading
+				// it will be the first bytes of the array in the microcode
+				GLenum binaryFormat = 0; 
+
+                // create microcode
+                GpuProgramManager::Microcode newMicrocode = 
+                    GpuProgramManager::getSingleton().createMicrocode(binaryLength + sizeof(GLenum));
+
+        		// get binary
+				uint8 * programBuffer = newMicrocode->getPtr() + sizeof(GLenum);
+				glGetProgramBinary(mGLHandle, binaryLength, NULL, &binaryFormat, programBuffer);
+
+                // save binary format
+				memcpy(newMicrocode->getPtr(), &binaryFormat, sizeof(GLenum));
+
+        		// add to the microcode to the cache
+				GpuProgramManager::getSingleton().addMicrocodeToCache(name, newMicrocode);
+			}
+		}
+	}
+	//-----------------------------------------------------------------------
+} // namespace GLSL
 } // namespace Ogre
