@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -35,18 +35,20 @@ THE SOFTWARE.
 namespace Ogre {
 
 	//-----------------------------------------------------------------------
-	template<> GLSLLinkProgramManager* Singleton<GLSLLinkProgramManager>::ms_Singleton = 0;
+	template<> GLSL::GLSLLinkProgramManager* Singleton<GLSL::GLSLLinkProgramManager>::msSingleton = 0;
+
+    namespace GLSL {
 
 	//-----------------------------------------------------------------------
     GLSLLinkProgramManager* GLSLLinkProgramManager::getSingletonPtr(void)
     {
-        return ms_Singleton;
+        return msSingleton;
     }
 
 	//-----------------------------------------------------------------------
     GLSLLinkProgramManager& GLSLLinkProgramManager::getSingleton(void)
     {  
-        assert( ms_Singleton );  return ( *ms_Singleton );  
+        assert( msSingleton );  return ( *msSingleton );  
     }
 
 	//-----------------------------------------------------------------------
@@ -61,6 +63,7 @@ namespace Ogre {
 		mTypeEnumMap.insert(StringToEnumMap::value_type("sampler1D", GL_SAMPLER_1D));
 		mTypeEnumMap.insert(StringToEnumMap::value_type("sampler2D", GL_SAMPLER_2D));
 		mTypeEnumMap.insert(StringToEnumMap::value_type("sampler3D", GL_SAMPLER_3D));
+		mTypeEnumMap.insert(StringToEnumMap::value_type("sampler2DArray", GL_SAMPLER_2D_ARRAY_EXT));
 		mTypeEnumMap.insert(StringToEnumMap::value_type("samplerCube", GL_SAMPLER_CUBE));
 		mTypeEnumMap.insert(StringToEnumMap::value_type("sampler1DShadow", GL_SAMPLER_1D_SHADOW));
 		mTypeEnumMap.insert(StringToEnumMap::value_type("sampler2DShadow", GL_SAMPLER_2D_SHADOW));
@@ -213,6 +216,9 @@ namespace Ogre {
 		case GL_SAMPLER_2D_RECT_ARB:
 			defToUpdate.constType = GCT_SAMPLER2D;
 			break;
+        case GL_SAMPLER_2D_ARRAY_EXT:
+            defToUpdate.constType = GCT_SAMPLER2DARRAY;
+            break;
 		case GL_SAMPLER_3D:
 			defToUpdate.constType = GCT_SAMPLER3D;
 			break;
@@ -357,7 +363,7 @@ namespace Ogre {
 				// user defined uniform found, add it to the reference list
 				String paramName = String( uniformName );
 
-				// currant ATI drivers (Catalyst 7.2 and earlier) and older NVidia drivers will include all array elements as uniforms but we only want the root array name and location
+				// Current ATI drivers (Catalyst 7.2 and earlier) and older NVidia drivers will include all array elements as uniforms but we only want the root array name and location
 				// Also note that ATI Catalyst 6.8 to 7.2 there is a bug with glUniform that does not allow you to update a uniform array past the first uniform array element
 				// ie you can't start updating an array starting at element 1, must always be element 0.
 
@@ -435,7 +441,7 @@ namespace Ogre {
 				line = src.substr(currPos, endPos - currPos);
 
 				// Remove spaces before opening square braces, otherwise
-				// the following split() can split the line at inappropiate
+				// the following split() can split the line at inappropriate
 				// places (e.g. "vec3 something [3]" won't work).
 				for (String::size_type sqp = line.find (" ["); sqp != String::npos;
 					 sqp = line.find (" ["))
@@ -489,7 +495,7 @@ namespace Ogre {
 						{
 							LogManager::getSingleton().logMessage(
 								"Problem parsing the following GLSL Uniform: '"
-								+ line + "' in file " + filename);
+								+ line + "' in file " + filename, LML_CRITICAL);
 							// next uniform
 							break;
 						}
@@ -525,3 +531,4 @@ namespace Ogre {
 	}
 
 }
+}
