@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,9 @@ THE SOFTWARE.
 
 #include <X11/extensions/Xrandr.h>
 
+#if OGRE_THREAD_SUPPORT != 1
 GLenum glxewContextInit(Ogre::GLSupport *glSupport);
+#endif
 
 static Display *_currentDisplay;
 static Display *_getCurrentDisplay(void) { return _currentDisplay; }
@@ -160,6 +162,7 @@ namespace Ogre
 		ConfigOption optFSAA;
 		ConfigOption optRTTMode;
 		ConfigOption optSRGB;
+		ConfigOption optEnableFixedPipeline;
 		
 		optFullScreen.name = "Full Screen";
 		optFullScreen.immutable = false;
@@ -182,6 +185,12 @@ namespace Ogre
 		optSRGB.name = "sRGB Gamma Conversion";
 		optSRGB.immutable = false;
 
+		optEnableFixedPipeline.name = "Fixed Pipeline Enabled";
+		optEnableFixedPipeline.possibleValues.push_back( "Yes" );
+		optEnableFixedPipeline.possibleValues.push_back( "No" );
+		optEnableFixedPipeline.currentValue = "Yes";
+		optEnableFixedPipeline.immutable = false;
+
 		optFullScreen.possibleValues.push_back("No");
 		optFullScreen.possibleValues.push_back("Yes");
 		
@@ -225,12 +234,12 @@ namespace Ogre
 		
 		if (! mSampleLevels.empty())
 		{
-			StringVector::const_iterator value = mSampleLevels.begin();
-			StringVector::const_iterator end = mSampleLevels.end();
+			StringVector::const_iterator value_ = mSampleLevels.begin();
+			StringVector::const_iterator end_ = mSampleLevels.end();
 			
-			for (; value != end; value++)
+			for (; value_ != end_; value_++)
 			{
-				optFSAA.possibleValues.push_back(*value);
+				optFSAA.possibleValues.push_back(*value_);
 			}
 			
 			optFSAA.currentValue = optFSAA.possibleValues[0];
@@ -251,6 +260,7 @@ namespace Ogre
 		mOptions[optRTTMode.name] = optRTTMode;
 		mOptions[optFSAA.name] = optFSAA;
 		mOptions[optSRGB.name] = optSRGB;
+		mOptions[optEnableFixedPipeline.name] = optEnableFixedPipeline;
 		
 		refreshConfig();
 	}
@@ -274,7 +284,7 @@ namespace Ogre
 				
 				if (mode == optVideoMode->second.currentValue)
 				{
-					String frequency = StringConverter::toString(value->second) + " MHz";
+					String frequency = StringConverter::toString(value->second) + " Hz";
 					
 					optDisplayFrequency->second.possibleValues.push_back(frequency);
 				}
@@ -287,7 +297,7 @@ namespace Ogre
 			else
 			{
 				optVideoMode->second.currentValue = StringConverter::toString(mVideoModes[0].first.first,4) + " x " + StringConverter::toString(mVideoModes[0].first.second,4);
-				optDisplayFrequency->second.currentValue = StringConverter::toString(mVideoModes[0].second) + " MHz";
+				optDisplayFrequency->second.currentValue = StringConverter::toString(mVideoModes[0].second) + " Hz";
 			}
 		}
 	}
@@ -365,6 +375,12 @@ namespace Ogre
 			if((opt = mOptions.find("sRGB Gamma Conversion")) != end)
 				miscParams["gamma"] = opt->second.currentValue;
 			
+			opt = mOptions.find("Fixed Pipeline Enabled");
+			if (opt == mOptions.end())
+				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find Fixed Pipeline enabled options!", "Win32GLSupport::createWindow");
+			bool enableFixedPipeline = (opt->second.currentValue == "Yes");
+			renderSystem->setFixedPipelineEnabled(enableFixedPipeline);
+
 			window = renderSystem->_createRenderWindow(windowTitle, w, h, fullscreen, &miscParams);
 		} 
 		
@@ -424,7 +440,7 @@ namespace Ogre
 		
 		LogManager::getSingleton().stream() << "Supported GLX extensions: " << extensionsString;
 		
-		std::stringstream ext;
+		StringStream ext;
 		String instr;
 		
 		ext << extensionsString;
@@ -683,12 +699,15 @@ namespace Ogre
 	}
 
 	//-------------------------------------------------------------------------------------------------//
-	bool GLXGLSupport::loadIcon(const std::string &name, Pixmap *pixmap, Pixmap *bitmap)
+	bool GLXGLSupport::loadIcon(const String &name, Pixmap *pixmap, Pixmap *bitmap)
 	{
 		Image image;
 		int width, height;
 		char* imageData;
-		
+                
+                if (! Ogre::ResourceGroupManager::getSingleton().resourceExists(ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, name)) 
+                    return false;
+                
 		try 
 		{
 			// Try to load image
@@ -891,8 +910,8 @@ namespace Ogre
 		
 		for(mode = mVideoModes.begin(); mode != end; size++)
 		{
-			if (mode->first.first >= static_cast<int>(width) &&
-				mode->first.second >= static_cast<int>(height))
+			if (mode->first.first >= width &&
+				mode->first.second >= height)
 			{
 				if (! newMode || 
 					mode->first.first < newMode->first.first ||
@@ -930,7 +949,7 @@ namespace Ogre
 				
 				mCurrentMode = *newMode;
 				
-				LogManager::getSingleton().logMessage("Entered video mode " + StringConverter::toString(mCurrentMode.first.first) + "x" + StringConverter::toString(mCurrentMode.first.second) + " @ " + StringConverter::toString(mCurrentMode.second) + "MHz");
+				LogManager::getSingleton().logMessage("Entered video mode " + StringConverter::toString(mCurrentMode.first.first) + "x" + StringConverter::toString(mCurrentMode.first.second) + " @ " + StringConverter::toString(mCurrentMode.second) + "Hz");
 			}
 		}
 	}
@@ -954,7 +973,11 @@ namespace Ogre
 		
 		glXGetCurrentDisplay = (PFNGLXGETCURRENTDISPLAYPROC)_getCurrentDisplay;
 		
+#if OGRE_THREAD_SUPPORT != 1
 		if (glxewContextInit(this) != GLEW_OK)
+#else
+		if (glxewContextInit(glxewGetContext()) != GLEW_OK)
+#endif
 		{
 			XCloseDisplay (mGLDisplay);
 			XCloseDisplay (mXDisplay);
@@ -963,4 +986,23 @@ namespace Ogre
 		
 		glXGetCurrentDisplay = (PFNGLXGETCURRENTDISPLAYPROC)getProcAddress("glXGetCurrentDisplay");
 	}
+
+#if OGRE_THREAD_SUPPORT == 1
+	GLXEWContext* glxewGetContext()
+	{
+		using namespace Ogre;
+		static OGRE_THREAD_POINTER_VAR(GLXEWContext, GLXEWContextsPtr);
+
+		GLXEWContext * currentGLXEWContextsPtr = OGRE_THREAD_POINTER_GET(GLXEWContextsPtr);
+		if (currentGLXEWContextsPtr == NULL)
+		{
+			currentGLXEWContextsPtr = new GLXEWContext();
+			OGRE_THREAD_POINTER_SET(GLXEWContextsPtr, currentGLXEWContextsPtr);
+			memset(currentGLXEWContextsPtr, 0, sizeof(GLXEWContext));
+			glxewInit();
+		}
+		return currentGLXEWContextsPtr;
+	}
+#endif
 }
+
