@@ -4,7 +4,7 @@ This source file is part of OGRE
 	(Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -73,6 +73,9 @@ namespace Ogre
 		mIsExternalGLControl = false;
 		mClosed = false;
 		mActive = false;
+		mHidden = false;
+		mVSync = false;
+		mVSyncInterval = 1;
 	}
 	
 	//-------------------------------------------------------------------------------------------------//
@@ -110,6 +113,7 @@ namespace Ogre
 		uint samples = 0;
 		short frequency = 0;
 		bool vsync = false;
+		bool hidden = false;
 		unsigned int vsyncInterval = 1;
 		int gamma = 0;
 		::GLXContext glxContext = 0;
@@ -118,6 +122,7 @@ namespace Ogre
 		Window parentWindow = DefaultRootWindow(xDisplay);
 		int left = DisplayWidth(xDisplay, DefaultScreen(xDisplay))/2 - width/2;
 		int top  = DisplayHeight(xDisplay, DefaultScreen(xDisplay))/2 - height/2;
+		String border;
 		
 		mIsFullScreen = fullScreen;
 		
@@ -155,6 +160,9 @@ namespace Ogre
 			if((opt = miscParams->find("vsync")) != end) 
 				vsync = StringConverter::parseBool(opt->second);
 
+			if((opt = miscParams->find("hidden")) != end)
+				hidden = StringConverter::parseBool(opt->second);
+
 			if((opt = miscParams->find("vsyncInterval")) != end) 
 				vsyncInterval = StringConverter::parseUnsignedInt(opt->second);
 
@@ -213,6 +221,9 @@ namespace Ogre
 					externalWindow = StringConverter::parseUnsignedLong(tokens[0]);
 				}
 			}
+
+			if ((opt = miscParams->find("border")) != end)
+				border = opt->second;
 		}
 		
 		// Ignore fatal XErrorEvents during parameter validation:
@@ -274,7 +285,7 @@ namespace Ogre
 			};
 			
 			int maxAttribs[] = {
-				GLX_SAMPLES,		samples,
+				GLX_SAMPLES,		static_cast<int>(samples),
 				GLX_DOUBLEBUFFER,   1,
 				GLX_STENCIL_SIZE,   INT_MAX,
 				GLX_FRAMEBUFFER_SRGB_CAPABLE_EXT, 1,
@@ -283,6 +294,11 @@ namespace Ogre
 
 			fbConfig = mGLSupport->selectFBConfig(minAttribs, maxAttribs);
 
+			// Now check the actual supported fsaa value
+			GLint maxSamples;
+			mGLSupport->getFBConfigAttrib(fbConfig, GLX_SAMPLES, &maxSamples);
+			mFSAA = maxSamples;
+
 			if (gamma != 0)
 			{
 				mGLSupport->getFBConfigAttrib(fbConfig, GL_FRAMEBUFFER_SRGB_CAPABLE_EXT, &gamma);
@@ -367,14 +383,24 @@ namespace Ogre
 					}
 				}
 				
-				// Is this really necessary ? Which broken WM might need it?
 				if ((sizeHints = XAllocSizeHints()) != NULL)
 				{
+					// Is this really necessary ? Which broken WM might need it?
 					sizeHints->flags = USPosition;
+
+					if(!fullScreen && border == "fixed")
+					{
+						sizeHints->min_width = sizeHints->max_width = width;
+						sizeHints->min_height = sizeHints->max_height = height;
+						sizeHints->flags |= PMaxSize | PMinSize;
+					}
 				}
 				
 				XTextProperty titleprop;
-				char *lst = (char*)title.c_str();
+				vector<char>::type  title_ (title.begin(), title.end());
+				title_.push_back(0);
+
+				char *lst = &title_[0];
 				XStringListToTextProperty((char **)&lst, 1, &titleprop);
 				XSetWMProperties(xDisplay, mWindow, &titleprop, NULL, NULL, 0, sizeHints, wmHints, NULL);
 				
@@ -396,32 +422,20 @@ namespace Ogre
 			
 			glxDrawable = mWindow;
 			
-			XMapWindow(xDisplay, mWindow);
-			
-			if (mIsFullScreen)
-			{
-				switchFullScreen (true);
-			}
+			// setHidden takes care of mapping or unmapping the window
+			// and also calls setFullScreen if appropriate.
+			setHidden(hidden);
 			XFlush(xDisplay);
 			
 			WindowEventUtilities::_addRenderWindow(this);
 		}
 		
-    mContext = new GLXContext(mGLSupport, fbConfig, glxDrawable, glxContext);
+		mContext = new GLXContext(mGLSupport, fbConfig, glxDrawable, glxContext);
 		
-		::GLXDrawable oldDrawable = glXGetCurrentDrawable();
-		::GLXContext  oldContext  = glXGetCurrentContext();
-		
-		mContext->setCurrent();
-		
-		if (! mIsExternalGLControl && GLXEW_SGI_swap_control)
-		{
-			glXSwapIntervalSGI (vsync ? vsyncInterval : 0);
-		}
-		
-		mContext->endCurrent();
-		
-		glXMakeCurrent (mGLSupport->getGLDisplay(), oldDrawable, oldContext);
+		// apply vsync settings. call setVSyncInterval first to avoid 
+		// setting vsync more than once.
+		setVSyncInterval(vsyncInterval);
+		setVSyncEnabled(vsync);
 		
 		int fbConfigID;
 		
@@ -513,6 +527,77 @@ namespace Ogre
 		mVisible = visible;
 	}
 	
+	//-------------------------------------------------------------------------------------------------//
+	void GLXWindow::setHidden(bool hidden)
+	{
+		mHidden = hidden;
+		// ignore for external windows as these should handle
+		// this externally
+		if (mIsExternal)
+			return;
+
+		if (hidden)
+		{
+			XUnmapWindow(mGLSupport->getXDisplay(), mWindow);
+		}
+		else
+		{
+			XMapWindow(mGLSupport->getXDisplay(), mWindow);
+			if (mIsFullScreen)
+			{
+				switchFullScreen(true);
+			}
+		}
+	}
+
+	//-------------------------------------------------------------------------------------------------//
+	void GLXWindow::setVSyncInterval(unsigned int interval)
+	{
+		mVSyncInterval = interval;
+		if (mVSync)
+			setVSyncEnabled(true);
+	}
+
+	//-------------------------------------------------------------------------------------------------//
+	void GLXWindow::setVSyncEnabled(bool vsync)
+	{
+		mVSync = vsync;
+		// we need to make our context current to set vsync
+		// store previous context to restore when finished.
+		::GLXDrawable oldDrawable = glXGetCurrentDrawable();
+		::GLXContext  oldContext  = glXGetCurrentContext();
+		
+		mContext->setCurrent();
+		
+		if (! mIsExternalGLControl)
+		{
+			if (GLXEW_MESA_swap_control)
+				glXSwapIntervalMESA (vsync ? mVSyncInterval : 0);
+			else if (GLXEW_EXT_swap_control)
+				glXSwapIntervalEXT (mGLSupport->getGLDisplay(), glXGetCurrentDrawable(),
+									vsync ? mVSyncInterval : 0);
+			else if (GLXEW_SGI_swap_control)
+				if (vsync && mVSyncInterval)
+					glXSwapIntervalSGI (mVSyncInterval);
+		}
+		
+		mContext->endCurrent();
+		
+		glXMakeCurrent (mGLSupport->getGLDisplay(), oldDrawable, oldContext);
+	}
+
+	//-------------------------------------------------------------------------------------------------//
+	bool GLXWindow::isVSyncEnabled() const
+	{
+		return mVSync;
+	}
+
+	//-------------------------------------------------------------------------------------------------//
+	unsigned int GLXWindow::getVSyncInterval() const
+	{
+		return mVSyncInterval;
+	}
+
 	//-------------------------------------------------------------------------------------------------//
 	void GLXWindow::reposition(int left, int top)
 	{
@@ -575,7 +660,7 @@ namespace Ogre
 		
 		XGetWindowAttributes(xDisplay, mWindow, &windowAttrib);
 		
-		if (mWidth == windowAttrib.width && mHeight == windowAttrib.height)
+		if (mWidth == (unsigned)windowAttrib.width && mHeight == (unsigned)windowAttrib.height)
 			return;
 		
 		mWidth = windowAttrib.width;
@@ -586,7 +671,7 @@ namespace Ogre
 	}
 	
 	//-------------------------------------------------------------------------------------------------//
-	void GLXWindow::swapBuffers(bool waitForVSync)
+	void GLXWindow::swapBuffers()
 	{
 		if (mClosed || mIsExternalGLControl) 
 			return;
@@ -635,8 +720,8 @@ namespace Ogre
 		if (mClosed)
 			return;
 		
-		if ((dst.left < 0) || (dst.right > mWidth) ||
-			(dst.top < 0) || (dst.bottom > mHeight) ||
+		if ((dst.right > mWidth) ||
+			(dst.bottom > mHeight) ||
 			(dst.front != 0) || (dst.back != 1))
 		{
 			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid box.", "GLXWindow::copyContentsToMemory" );
@@ -655,28 +740,25 @@ namespace Ogre
 			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Unsupported format.", "GLXWindow::copyContentsToMemory" );
 		}
 		
-		glReadBuffer((buffer == FB_FRONT)? GL_FRONT : GL_BACK);
-		glReadPixels((GLint)dst.left, (GLint)dst.top,
-					 (GLsizei)dst.getWidth(), (GLsizei)dst.getHeight(),
-					 format, type, dst.data);
+		// Switch context if different from current one
+		RenderSystem* rsys = Root::getSingleton().getRenderSystem();
+		rsys->_setViewport(this->getViewport(0));
 		
-		//vertical flip
-		{
-			size_t rowSpan = dst.getWidth() * PixelUtil::getNumElemBytes(dst.format);
-			size_t height = dst.getHeight();
-			uchar *tmpData = new uchar[rowSpan * height];
-			uchar *srcRow = (uchar *)dst.data, *tmpRow = tmpData + (height - 1) * rowSpan;
-			
-			while (tmpRow >= tmpData)
-			{
-				memcpy(tmpRow, srcRow, rowSpan);
-				srcRow += rowSpan;
-				tmpRow -= rowSpan;
-			}
-			memcpy(dst.data, tmpData, rowSpan * height);
-			
-			delete [] tmpData;
-		}
+        if(dst.getWidth() != dst.rowPitch)
+            glPixelStorei(GL_PACK_ROW_LENGTH, dst.rowPitch);
+		// Must change the packing to ensure no overruns!
+		glPixelStorei(GL_PACK_ALIGNMENT, 1);
+		
+		glReadBuffer((buffer == FB_FRONT)? GL_FRONT : GL_BACK);
+        glReadPixels((GLint)0, (GLint)(mHeight - dst.getHeight()),
+                     (GLsizei)dst.getWidth(), (GLsizei)dst.getHeight(),
+                     format, type, dst.getTopLeftFrontPixelPtr());
+		
+		// restore default alignment
+		glPixelStorei(GL_PACK_ALIGNMENT, 4);
+        glPixelStorei(GL_PACK_ROW_LENGTH, 0);
+        
+        PixelUtil::bulkPixelVerticalFlip(dst);
 	}
 
 	//-------------------------------------------------------------------------------------------------//
