@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -42,15 +42,35 @@ THE SOFTWARE.
 #include <dlfcn.h>
 
 #include <OpenGL/OpenGL.h>
-
+#include <AppKit/NSScreen.h>
+#include <Foundation/NSString.h>
 namespace Ogre {
 
+    class OSXGLSupportImpl
+    {
+    public:
+        OSXGLSupportImpl(){}
+        NSString * mCurrentOSVersion;
+    };
+    
+bool OSXGLSupport::OSVersionIsAtLeast(String newVersion)
+{
+    return [mImpl->mCurrentOSVersion compare:[NSString stringWithCString:newVersion.c_str() encoding:NSASCIIStringEncoding]
+                              options:NSNumericSearch] >= NSOrderedSame;
+}
+
+
 OSXGLSupport::OSXGLSupport() : mAPI(""), mContextType("")
 {
+    
+    NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:@"/System/Library/CoreServices/SystemVersion.plist"];
+    mImpl = new OSXGLSupportImpl();
+    mImpl->mCurrentOSVersion = [dict objectForKey:@"ProductVersion"];
 }
 
 OSXGLSupport::~OSXGLSupport()
 {
+    delete mImpl;
 }
 
 void OSXGLSupport::addConfig( void )
@@ -64,9 +84,8 @@ void OSXGLSupport::addConfig( void )
 	ConfigOption optHiddenWindow;
 	ConfigOption optVsync;
 	ConfigOption optSRGB;
-#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
+    ConfigOption optContentScalingFactor;
 	ConfigOption optEnableFixedPipeline;
-#endif
 
 	// FS setting possibilities
 	optFullScreen.name = "Full Screen";
@@ -109,20 +128,22 @@ void OSXGLSupport::addConfig( void )
 	optSRGB.currentValue = "No";
 	optSRGB.immutable = false;
 
-#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
-		optEnableFixedPipeline.name = "Fixed Pipeline Enabled";
-		optEnableFixedPipeline.possibleValues.push_back( "Yes" );
-		optEnableFixedPipeline.possibleValues.push_back( "No" );
-		optEnableFixedPipeline.currentValue = "Yes";
-		optEnableFixedPipeline.immutable = false;
-#endif
+    optContentScalingFactor.name = "Content Scaling Factor";
+    optContentScalingFactor.possibleValues.push_back( "1.0" );
+    optContentScalingFactor.possibleValues.push_back( "1.33" );
+    optContentScalingFactor.possibleValues.push_back( "1.5" );
+    optContentScalingFactor.possibleValues.push_back( "2.0" );
+    if(OSVersionIsAtLeast("10.7"))
+        optContentScalingFactor.currentValue = StringConverter::toString((float)[NSScreen mainScreen].backingScaleFactor);
+    else
+        optContentScalingFactor.currentValue = "1.0";
+    optContentScalingFactor.immutable = false;
 
-    mOptions[ optFullScreen.name ] = optFullScreen;
-	mOptions[ optBitDepth.name ] = optBitDepth;
-
-#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
-		mOptions[optEnableFixedPipeline.name] = optEnableFixedPipeline;
-#endif
+    optEnableFixedPipeline.name = "Fixed Pipeline Enabled";
+    optEnableFixedPipeline.possibleValues.push_back( "Yes" );
+    optEnableFixedPipeline.possibleValues.push_back( "No" );
+    optEnableFixedPipeline.currentValue = "Yes";
+    optEnableFixedPipeline.immutable = false;
 
 	CGLRendererInfoObj rend;
 
@@ -286,6 +307,10 @@ void OSXGLSupport::addConfig( void )
 	mOptions[optHiddenWindow.name] = optHiddenWindow;
 	mOptions[optVsync.name] = optVsync;
 	mOptions[optSRGB.name] = optSRGB;
+    mOptions[optBitDepth.name] = optBitDepth;
+    mOptions[optContentScalingFactor.name] = optContentScalingFactor;
+
+    mOptions[optEnableFixedPipeline.name] = optEnableFixedPipeline;
 }
 
 String OSXGLSupport::validateConfig( void )
@@ -333,17 +358,21 @@ RenderWindow* OSXGLSupport::createWindow( bool autoCreateWindow, GLRenderSystem*
             winOptions[ "vsync" ] = opt->second.currentValue;
         }
 
+        opt = mOptions.find( "Content Scaling Factor" );
+        if( opt != mOptions.end() )
+        {
+            winOptions["contentScalingFactor"] = opt->second.currentValue;
+        }
+
         opt = mOptions.find( "sRGB Gamma Conversion" );
         if( opt != mOptions.end() )
             winOptions["gamma"] = opt->second.currentValue;
 
-#ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
 			opt = mOptions.find("Fixed Pipeline Enabled");
 			if (opt == mOptions.end())
 				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find Fixed Pipeline enabled options!", "Win32GLSupport::createWindow");
 			bool enableFixedPipeline = (opt->second.currentValue == "Yes");
 			renderSystem->setFixedPipelineEnabled(enableFixedPipeline);
-#endif
 
         opt = mOptions.find( "macAPI" );
         if( opt != mOptions.end() )
@@ -427,7 +456,7 @@ void* OSXGLSupport::getProcAddress( const char* name )
     void *symbol;
     symbol = NULL;
     
-    std::string fullPath = macPluginPath() + "RenderSystem_GL.dylib";
+    String fullPath = macPluginPath() + "RenderSystem_GL.dylib";
     void *handle = dlopen(fullPath.c_str(), RTLD_LAZY | RTLD_GLOBAL);
     if(handle) {
         symbol = dlsym (handle, name);
@@ -447,7 +476,7 @@ bool OSXGLSupport::supportsPBuffers()
 	return true;
 }
 
-GLPBuffer* OSXGLSupport::createPBuffer(PixelComponentType format, size_t width, size_t height)
+GLPBuffer* OSXGLSupport::createPBuffer(PixelComponentType format, uint32 width, uint32 height)
 {
 //	if(mContextType == "NSOpenGL")
 //		return OGRE_NEW OSXCocoaPBuffer(format, width, height);
