@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -43,7 +43,7 @@ THE SOFTWARE.
 
 namespace Ogre
 {
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
     uint32 OSXWindow::bitDepthFromDisplayMode(CGDisplayModeRef mode)
     {
         uint32 depth = 0;
@@ -55,6 +55,7 @@ namespace Ogre
         else if(CFStringCompare(pixEnc, CFSTR(IO8BitIndexedPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo)
             depth = 8;
 
+        CFRelease(pixEnc);
         return depth;
     }
 #endif
@@ -66,13 +67,17 @@ namespace Ogre
     //-------------------------------------------------------------------------------------------------//
     OSXWindow::~OSXWindow()
     {
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
+        if (mOriginalDisplayMode != NULL)
+            CGDisplayModeRelease(mOriginalDisplayMode);
+#endif
     }
     
     //-------------------------------------------------------------------------------------------------//
     void OSXWindow::copyContentsToMemory(const PixelBox &dst, FrameBuffer buffer)
     {
-        if ((dst.left < 0) || (dst.right > mWidth) ||
-            (dst.top < 0) || (dst.bottom > mHeight) ||
+        if ((dst.right > mWidth) ||
+            (dst.bottom > mHeight) ||
             (dst.front != 0) || (dst.back != 1))
         {
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
@@ -95,6 +100,14 @@ namespace Ogre
                         "OSXWindow::copyContentsToMemory" );
         }
         
+        // Switch context if different from current one
+		RenderSystem* rsys = Root::getSingleton().getRenderSystem();
+		rsys->_setViewport(this->getViewport(0));
+
+        if(dst.getWidth() != dst.rowPitch)
+        {
+            glPixelStorei(GL_PACK_ROW_LENGTH, static_cast<GLint>(dst.rowPitch));
+        }
         if((dst.getWidth()*Ogre::PixelUtil::getNumElemBytes(dst.format)) & 3)
         {
             // Standard alignment of 4 is not right
@@ -102,29 +115,14 @@ namespace Ogre
         }
         
         glReadBuffer((buffer == FB_FRONT)? GL_FRONT : GL_BACK);
-        glReadPixels((GLint)dst.left, (GLint)dst.top,
+        glReadPixels((GLint)0, (GLint)(mHeight - dst.getHeight()),
                      (GLsizei)dst.getWidth(), (GLsizei)dst.getHeight(),
-                     format, type, dst.data);
+                     format, type, dst.getTopLeftFrontPixelPtr());
         
         glPixelStorei(GL_PACK_ALIGNMENT, 4);
+        glPixelStorei(GL_PACK_ROW_LENGTH, 0);
         
-        //vertical flip
-        {
-            size_t rowSpan = dst.getWidth() * PixelUtil::getNumElemBytes(dst.format);
-            size_t height = dst.getHeight();
-            uchar *tmpData = (uchar *)OGRE_MALLOC_ALIGN(rowSpan * height, MEMCATEGORY_GENERAL, false);
-            uchar *srcRow = (uchar *)dst.data, *tmpRow = tmpData + (height - 1) * rowSpan;
-            
-            while (tmpRow >= tmpData)
-            {
-                memcpy(tmpRow, srcRow, rowSpan);
-                srcRow += rowSpan;
-                tmpRow -= rowSpan;
-            }
-            memcpy(dst.data, tmpData, rowSpan * height);
-            
-            OGRE_FREE_ALIGN(tmpData, MEMCATEGORY_GENERAL, false);
-        }
+        PixelUtil::bulkPixelVerticalFlip(dst);
     }
     
     //-------------------------------------------------------------------------------------------------//
@@ -132,11 +130,11 @@ namespace Ogre
     {
         // Find the best match to what was requested
         boolean_t exactMatch = 0;
-        unsigned int reqWidth = 0, reqHeight = 0, reqDepth = 0;
+        size_t reqWidth = 0, reqHeight = 0, reqDepth = 0;
         CGLError cglErr = kCGLNoError;
         CGError cgErr = kCGErrorSuccess;
         
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
         // Get a copy of the current display mode
         CGDisplayModeRef displayMode = CGDisplayCopyDisplayMode(kCGDirectMainDisplay);
         
@@ -159,7 +157,10 @@ namespace Ogre
             
             if((CGDisplayModeGetWidth(mode) == width) && (CGDisplayModeGetHeight(mode) == height))
             {
+                // Release memory
+                CGDisplayModeRelease(displayMode);
                 displayMode = mode;
+                CGDisplayModeRetain(displayMode);
                 exactMatch = 1;
                 break;
             }
@@ -176,13 +177,19 @@ namespace Ogre
                 
                 if((CGDisplayModeGetWidth(mode) >= width) && (CGDisplayModeGetHeight(mode) >= height))
                 {
+                    // Release memory
+                    CGDisplayModeRelease(displayMode);
                     displayMode = mode;
+                    CGDisplayModeRetain(displayMode);
                     exactMatch = 1;
                     break;
                 }
             }
         }
-        
+
+        // Release memory
+        CFRelease(allModes);
+
         reqWidth = CGDisplayModeGetWidth(displayMode);
         reqHeight = CGDisplayModeGetHeight(displayMode);
         reqDepth = bitDepthFromDisplayMode(displayMode);
@@ -241,9 +248,12 @@ namespace Ogre
         CG_CHECK_ERROR(cgErr)
         
         // Switch to the correct resolution
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
         mOriginalDisplayMode = CGDisplayCopyDisplayMode(kCGDirectMainDisplay);
         cgErr = CGDisplaySetDisplayMode(kCGDirectMainDisplay, displayMode, NULL);
+
+        // Release memory
+        CFRelease(displayMode);
 #else
         mOriginalDisplayMode = CGDisplayCurrentMode(kCGDirectMainDisplay);
         cgErr = CGDisplaySwitchToMode(kCGDirectMainDisplay, displayMode);
@@ -332,9 +342,9 @@ namespace Ogre
         }
 
         // Set some other variables.  Just in case we got a different value from CGDisplayBestModeForParameters than we requested
-        mWidth = reqWidth;
-        mHeight = reqHeight;
-        mColourDepth = reqDepth;
+        mWidth = static_cast<uint32>(reqWidth);
+        mHeight = static_cast<uint32>(reqHeight);
+        mColourDepth = static_cast<uint32>(reqDepth);
 
         cgErr = CGAcquireDisplayFadeReservation(kCGMaxDisplayReservationInterval,
                                                 &reservationToken);
@@ -403,7 +413,7 @@ namespace Ogre
         }
 
         // Switch back to the original screen resolution
-#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
+#if defined(MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
         CGDisplaySetDisplayMode(kCGDirectMainDisplay, mOriginalDisplayMode, NULL);
 #else
         CGDisplaySwitchToMode(kCGDirectMainDisplay, mOriginalDisplayMode);
