@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -174,4 +174,22 @@ namespace Ogre {
 				"Cannot create RenderToVertexBuffer in GLDefaultHardwareBufferManagerBase", 
 				"GLDefaultHardwareBufferManagerBase::createRenderToVertexBuffer");
 	}
+	//-----------------------------------------------------------------------
+	HardwareUniformBufferSharedPtr 
+		GLDefaultHardwareBufferManagerBase::createUniformBuffer(size_t sizeBytes, HardwareBuffer::Usage usage,bool useShadowBuffer, const String& name)
+	{
+		OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
+				"Cannot create UniformBuffer in GLDefaultHardwareBufferManagerBase", 
+				"GLDefaultHardwareBufferManagerBase::createUniformBuffer");
+	}
+	//-----------------------------------------------------------------------
+    HardwareCounterBufferSharedPtr
+        GLDefaultHardwareBufferManagerBase::createCounterBuffer(size_t sizeBytes,
+                                                                HardwareBuffer::Usage usage,
+                                                                bool useShadowBuffer, const String& name)
+	{
+		OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+                    "Counter buffers not supported in OpenGL RenderSystem.",
+                    "GLDefaultHardwareBufferManagerBase::createCounterBuffer");
+	}
 }
