@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -38,8 +38,6 @@ namespace Ogre {
     {
         // register with group manager
         ResourceGroupManager::getSingleton()._registerResourceManager(mResourceType, this);
-
-		createWarningTexture();
     }
     //-----------------------------------------------------------------------------
     GLTextureManager::~GLTextureManager()
@@ -61,8 +59,8 @@ namespace Ogre {
 	void GLTextureManager::createWarningTexture()
 	{
 		// Generate warning texture
-		size_t width = 8;
-		size_t height = 8;
+		uint32 width = 8;
+		uint32 height = 8;
 		uint32 *data = new uint32[width*height];		// 0xXXRRGGBB
 		// Yellow/black stripes
 		for(size_t y=0; y<height; ++y)
