@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -1286,9 +1286,9 @@ bool PS_1_4::BuildMachineInst()
 
 	
 
-	passed = true; // assume everything will go okay untill proven otherwise
+	passed = true; // assume everything will go okay until proven otherwise
 
-	// start with machine NOP instuction
+	// start with machine NOP instruction
 	// this is used after the switch to see if an instruction was set up
 	// determine which MachineInstID is required based on the op instruction
 	mOpType = mi_NOP;
