@@ -5,7 +5,7 @@ This source file is part of OGRE
 For the latest info, see http://www.ogre3d.org/
 
 Copyright (c) 2008 Renato Araujo Oliveira Filho <renatox@gmail.com>
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -137,26 +137,24 @@ namespace Ogre {
             scaled = mBuffer.getSubVolume(dstBox);
             Image::scale(src, scaled, Image::FILTER_BILINEAR);
         }
+#if OGRE_PLATFORM != OGRE_PLATFORM_APPLE_IOS
         else if ((src.format != mFormat) ||
                  ((GLESPixelUtil::getGLOriginFormat(src.format) == 0) && (src.format != PF_R8G8B8)))
+#else
+        else if (GLESPixelUtil::getGLOriginFormat(src.format) == 0)
+#endif
         {
             // Extents match, but format is not accepted as valid source format for GL
             // do conversion in temporary buffer
             allocateBuffer();
             scaled = mBuffer.getSubVolume(dstBox);
             PixelUtil::bulkPixelConversion(src, scaled);
-
-            if(mFormat == PF_A4R4G4B4)
-            {
-                // ARGB->BGRA
-                GLESPixelUtil::convertToGLformat(scaled, scaled);
-            }
         }
         else
         {
             allocateBuffer();
+            // No scaling or conversion needed
             scaled = src;
-
             if (src.format == PF_R8G8B8)
             {
                 scaled.format = PF_B8G8R8;
@@ -241,7 +239,7 @@ namespace Ogre {
         mTarget(target), mTextureID(id), mFace(face), mLevel(level), mSoftwareMipmap(crappyCard)
     {
         GL_CHECK_ERROR;
-        glBindTexture(GL_TEXTURE_2D, mTextureID);
+        glBindTexture(mTarget, mTextureID);
         GL_CHECK_ERROR;
 
         // Get face identifier
@@ -267,8 +265,8 @@ namespace Ogre {
 
         // Log a message
 //        std::stringstream str;
-//        str << "GLESHardwarePixelBuffer constructed for texture " << mTextureID 
-//            << " face " << mFace << " level " << mLevel << ": "
+//        str << "GLESHardwarePixelBuffer constructed for texture " << baseName
+//            << " id " << mTextureID << " face " << mFace << " level " << mLevel << ": "
 //            << "width=" << mWidth << " height="<< mHeight << " depth=" << mDepth
 //            << "format=" << PixelUtil::getFormatName(mFormat);
 //        LogManager::getSingleton().logMessage(LML_NORMAL, str.str());
@@ -312,6 +310,13 @@ namespace Ogre {
         }
     }
 
+#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
+    void GLESTextureBuffer::updateTextureId(GLuint textureID)
+    {
+        mTextureID = textureID;
+    }
+#endif
+    
     void GLESTextureBuffer::upload(const PixelBox &data, const Image::Box &dest)
     {
         glBindTexture(mTarget, mTextureID);
@@ -329,7 +334,7 @@ namespace Ogre {
             // for compressed formats
             if (dest.left == 0 && dest.top == 0)
             {
-                glCompressedTexImage2D(GL_TEXTURE_2D, mLevel,
+                glCompressedTexImage2D(mFaceTarget, mLevel,
                                        format,
                                        dest.getWidth(),
                                        dest.getHeight(),
@@ -340,7 +345,7 @@ namespace Ogre {
             }
             else
             {
-                glCompressedTexSubImage2D(GL_TEXTURE_2D, mLevel,
+                glCompressedTexSubImage2D(mFaceTarget, mLevel,
                                           dest.left, dest.top,
                                           dest.getWidth(), dest.getHeight(),
                                           format, data.getConsecutiveSize(),
@@ -395,6 +400,11 @@ namespace Ogre {
                 GL_CHECK_ERROR;
             }
 
+//            LogManager::getSingleton().logMessage("GLESTextureBuffer::upload - ID: " + StringConverter::toString(mTextureID) +
+//                                                  " Format: " + PixelUtil::getFormatName(data.format) +
+//                                                  " Origin format: " + StringConverter::toString(GLESPixelUtil::getGLOriginFormat(data.format, 0, ' ', std::ios::hex)) +
+//                                                  " Data type: " + StringConverter::toString(GLESPixelUtil::getGLOriginDataType(data.format, 0, ' ', std::ios::hex))
+//                                                  );
             glTexSubImage2D(mFaceTarget,
                             mLevel,
                             dest.left, dest.top,
@@ -427,9 +437,9 @@ namespace Ogre {
     
     void GLESTextureBuffer::copyFromFramebuffer(size_t zoffset)
     {
-        glBindTexture(GL_TEXTURE_2D, mTextureID);
+        glBindTexture(mFaceTarget, mTextureID);
         GL_CHECK_ERROR;
-        glCopyTexSubImage2D(GL_TEXTURE_2D, mLevel, 0, 0, 0, 0, mWidth, mHeight);
+        glCopyTexSubImage2D(mFaceTarget, mLevel, 0, 0, 0, 0, mWidth, mHeight);
         GL_CHECK_ERROR;
     }
 
@@ -602,6 +612,9 @@ namespace Ogre {
                 switch(mTarget)
                 {
                     case GL_TEXTURE_2D:
+#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
+                    case GL_TEXTURE_CUBE_MAP_OES:
+#endif
                         glCopyTexSubImage2D(mFaceTarget, mLevel, 
                                             dstBox.left, dstBox.top, 
                                             0, 0, dstBox.getWidth(), dstBox.getHeight());
@@ -759,7 +772,7 @@ namespace Ogre {
             GLenum glFormat = GLESPixelUtil::getGLOriginFormat(scaled.format);
             GLenum dataType = GLESPixelUtil::getGLOriginDataType(scaled.format);
 
-            glTexImage2D(GL_TEXTURE_2D,
+            glTexImage2D(mFaceTarget,
                          mip,
                          glFormat,
                          width, height,
