@@ -4,7 +4,7 @@
  (Object-oriented Graphics Rendering Engine)
  For the latest info, see http://www.ogre3d.org/
  
- Copyright (c) 2000-2012 Torus Knot Software Ltd
+ Copyright (c) 2000-2013 Torus Knot Software Ltd
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,12 @@
 #include "OgreGLSLESProgram.h"
 #include "OgreGLSLESProgramPipelineManager.h"
 #include "OgreGpuProgramManager.h"
+#include "OgreGLES2RenderSystem.h"
+#include "OgreGLES2UniformCache.h"
+#include "OgreGLES2HardwareUniformBuffer.h"
+#include "OgreHardwareBufferManager.h"
+#include "OgreGLES2Util.h"
+#include "OgreRoot.h"
 
 namespace Ogre
 {
@@ -40,38 +46,38 @@ namespace Ogre
 
     GLSLESProgramPipeline::~GLSLESProgramPipeline()
     {
-#if GL_EXT_separate_shader_objects
-        glDeleteProgramPipelinesEXT(1, &mGLProgramPipelineHandle);
-        GL_CHECK_ERROR;
+#if OGRE_PLATFORM != OGRE_PLATFORM_NACL
+        OGRE_IF_IOS_VERSION_IS_GREATER_THAN(5.0)
+            OGRE_CHECK_GL_ERROR(glDeleteProgramPipelinesEXT(1, &mGLProgramPipelineHandle));
 #endif
     }
 
     void GLSLESProgramPipeline::compileAndLink()
 	{
-#if GL_EXT_separate_shader_objects
+#if OGRE_PLATFORM != OGRE_PLATFORM_NACL
         GLint linkStatus = 0;
         
-        glGenProgramPipelinesEXT(1, &mGLProgramPipelineHandle);
-        GL_CHECK_ERROR
-        glBindProgramPipelineEXT(mGLProgramPipelineHandle);
-        GL_CHECK_ERROR
+        OGRE_CHECK_GL_ERROR(glGenProgramPipelinesEXT(1, &mGLProgramPipelineHandle));
+        OGRE_CHECK_GL_ERROR(glBindProgramPipelineEXT(mGLProgramPipelineHandle));
 
 		// Compile and attach Vertex Program
         if(mVertexProgram && !mVertexProgram->isLinked())
         {
-            if (!mVertexProgram->getGLSLProgram()->compile(true))
+            try
             {
+                mVertexProgram->getGLSLProgram()->compile(true);
+            }
+            catch (Exception& e)
+            {
+				LogManager::getSingleton().stream() << e.getDescription();
                 mTriedToLinkAndFailed = true;
                 return;
             }
             GLuint programHandle = mVertexProgram->getGLSLProgram()->getGLProgramHandle();
-            glProgramParameteriEXT(programHandle, GL_PROGRAM_SEPARABLE_EXT, GL_TRUE);
-            GL_CHECK_ERROR
+            OGRE_CHECK_GL_ERROR(glProgramParameteriEXT(programHandle, GL_PROGRAM_SEPARABLE_EXT, GL_TRUE));
             mVertexProgram->getGLSLProgram()->attachToProgramObject(programHandle);
-            glLinkProgram(programHandle);
-            GL_CHECK_ERROR
-            glGetProgramiv(programHandle, GL_LINK_STATUS, &linkStatus);
-            GL_CHECK_ERROR
+            OGRE_CHECK_GL_ERROR(glLinkProgram(programHandle));
+            OGRE_CHECK_GL_ERROR(glGetProgramiv(programHandle, GL_LINK_STATUS, &linkStatus));
             
             if(linkStatus)
             {
@@ -79,32 +85,32 @@ namespace Ogre
                 mLinked |= VERTEX_PROGRAM_LINKED;
             }
             
-            GL_CHECK_ERROR
-            // TODO: Just fail out here if the program doesn't link?
             mTriedToLinkAndFailed = !linkStatus;
             
             logObjectInfo( getCombinedName() + String("GLSL vertex program result : "), programHandle );
-            
+
             setSkeletalAnimationIncluded(mVertexProgram->isSkeletalAnimationIncluded());
         }
         
 		// Compile and attach Fragment Program
         if(mFragmentProgram && !mFragmentProgram->isLinked())
         {
-            if (!mFragmentProgram->getGLSLProgram()->compile(true))
+            try
             {
+                mFragmentProgram->getGLSLProgram()->compile(true);
+            }
+            catch (Exception& e)
+            {
+				LogManager::getSingleton().stream() << e.getDescription();
                 mTriedToLinkAndFailed = true;
                 return;
             }
 
             GLuint programHandle = mFragmentProgram->getGLSLProgram()->getGLProgramHandle();
-            glProgramParameteriEXT(programHandle, GL_PROGRAM_SEPARABLE_EXT, GL_TRUE);
-            GL_CHECK_ERROR
+            OGRE_CHECK_GL_ERROR(glProgramParameteriEXT(programHandle, GL_PROGRAM_SEPARABLE_EXT, GL_TRUE));
             mFragmentProgram->getGLSLProgram()->attachToProgramObject(programHandle);
-            glLinkProgram(programHandle);
-            GL_CHECK_ERROR
-            glGetProgramiv(programHandle, GL_LINK_STATUS, &linkStatus);
-            GL_CHECK_ERROR
+            OGRE_CHECK_GL_ERROR(glLinkProgram(programHandle));
+            OGRE_CHECK_GL_ERROR(glGetProgramiv(programHandle, GL_LINK_STATUS, &linkStatus));
             
             if(linkStatus)
             {
@@ -119,46 +125,23 @@ namespace Ogre
         
 		if(mLinked)
 		{
-			if ( GpuProgramManager::getSingleton().getSaveMicrocodesToCache() )
-			{
-				// Add to the microcode to the cache
-				String name;
-				name = getCombinedName();
-
-				// Get buffer size
-				GLint binaryLength = 0;
-
-#if GL_OES_get_program_binary
-				glGetProgramiv(mGLHandle, GL_PROGRAM_BINARY_LENGTH_OES, &binaryLength);
-                GL_CHECK_ERROR;
-#endif
-
-                // Create microcode
-                GpuProgramManager::Microcode newMicrocode = 
-                    GpuProgramManager::getSingleton().createMicrocode((unsigned long)binaryLength + sizeof(GLenum));
-
-#if GL_OES_get_program_binary
-				// Get binary
-				glGetProgramBinaryOES(mGLHandle, binaryLength, NULL, (GLenum *)newMicrocode->getPtr(), newMicrocode->getPtr() + sizeof(GLenum));
-                GL_CHECK_ERROR;
-#endif
-
-        		// Add to the microcode to the cache
-				GpuProgramManager::getSingleton().addMicrocodeToCache(name, newMicrocode);
-			}
             if(mVertexProgram && mVertexProgram->isLinked())
             {
-                glUseProgramStagesEXT(mGLProgramPipelineHandle, GL_VERTEX_SHADER_BIT_EXT, mVertexProgram->getGLSLProgram()->getGLProgramHandle());
-                GL_CHECK_ERROR
+                OGRE_CHECK_GL_ERROR(glUseProgramStagesEXT(mGLProgramPipelineHandle, GL_VERTEX_SHADER_BIT_EXT, mVertexProgram->getGLSLProgram()->getGLProgramHandle()));
             }
             if(mFragmentProgram && mFragmentProgram->isLinked())
             {
-                glUseProgramStagesEXT(mGLProgramPipelineHandle, GL_FRAGMENT_SHADER_BIT_EXT, mFragmentProgram->getGLSLProgram()->getGLProgramHandle());
-                GL_CHECK_ERROR
+                OGRE_CHECK_GL_ERROR(glUseProgramStagesEXT(mGLProgramPipelineHandle, GL_FRAGMENT_SHADER_BIT_EXT, mFragmentProgram->getGLSLProgram()->getGLProgramHandle()));
             }
 
             // Validate pipeline
             logObjectInfo( getCombinedName() + String("GLSL program pipeline result : "), mGLProgramPipelineHandle );
+#if OGRE_PLATFORM != OGRE_PLATFORM_NACL
+            if(mVertexProgram && mFragmentProgram)
+                OGRE_IF_IOS_VERSION_IS_GREATER_THAN(5.0)
+                    glLabelObjectEXT(GL_PROGRAM_PIPELINE_OBJECT_EXT, mGLProgramPipelineHandle, 0,
+                                 (mVertexProgram->getName() + "/" + mFragmentProgram->getName()).c_str());
+#endif
 		}
 #endif
 	}
@@ -167,10 +150,8 @@ namespace Ogre
     {
 		if (mLinked)
 		{
-#if GL_EXT_separate_shader_objects
-            GL_CHECK_ERROR
-            glBindProgramPipelineEXT(mGLProgramPipelineHandle);
-            GL_CHECK_ERROR
+#if OGRE_PLATFORM != OGRE_PLATFORM_NACL
+            OGRE_CHECK_GL_ERROR(glBindProgramPipelineEXT(mGLProgramPipelineHandle));
 #endif
 		}
     }
@@ -183,22 +164,20 @@ namespace Ogre
 		{
             GLuint handle = mVertexProgram->getGLSLProgram()->getGLProgramHandle();
 			const char * attString = getAttributeSemanticString(semantic);
-			GLint attrib = glGetAttribLocation(handle, attString);
-            GL_CHECK_ERROR;
+			GLint attrib;
+            OGRE_CHECK_GL_ERROR(attrib = glGetAttribLocation(handle, attString));
 
 			// Sadly position is a special case 
 			if (attrib == NOT_FOUND_CUSTOM_ATTRIBUTES_INDEX && semantic == VES_POSITION)
 			{
-				attrib = glGetAttribLocation(handle, "position");
-                GL_CHECK_ERROR;
+				OGRE_CHECK_GL_ERROR(attrib = glGetAttribLocation(handle, "position"));
 			}
             
 			// For uv and other case the index is a part of the name
 			if (attrib == NOT_FOUND_CUSTOM_ATTRIBUTES_INDEX)
 			{
 				String attStringWithSemantic = String(attString) + StringConverter::toString(index);
-				attrib = glGetAttribLocation(handle, attStringWithSemantic.c_str());
-                GL_CHECK_ERROR;
+				OGRE_CHECK_GL_ERROR(attrib = glGetAttribLocation(handle, attStringWithSemantic.c_str()));
 			}
             
 			// Update mCustomAttributesIndexes with the index we found (or didn't find) 
@@ -215,37 +194,28 @@ namespace Ogre
 		if (!mLinked && !mTriedToLinkAndFailed)
 		{
 			glGetError(); // Clean up the error. Otherwise will flood log.
-			GL_CHECK_ERROR
             
-			if ( GpuProgramManager::getSingleton().canGetCompiledShaderBuffer() &&
-				GpuProgramManager::getSingleton().isMicrocodeAvailableInCache(getCombinedName()) )
-			{
-				getMicrocodeFromCache();
-			}
-			else
-			{
 #if !OGRE_NO_GLES2_GLSL_OPTIMISER
-                // Check CmdParams for each shader type to see if we should optimise
-                if(mVertexProgram)
+            // Check CmdParams for each shader type to see if we should optimise
+            if(mVertexProgram)
+            {
+                String paramStr = mVertexProgram->getGLSLProgram()->getParameter("use_optimiser");
+                if((paramStr == "true") || paramStr.empty())
                 {
-                    String paramStr = mVertexProgram->getGLSLProgram()->getParameter("use_optimiser");
-                    if((paramStr == "true") || paramStr.empty())
-                    {
-                        GLSLESProgramPipelineManager::getSingleton().optimiseShaderSource(mVertexProgram);
-                    }
+                    GLSLESProgramPipelineManager::getSingleton().optimiseShaderSource(mVertexProgram);
                 }
+            }
 
-                if(mFragmentProgram)
+            if(mFragmentProgram)
+            {
+                String paramStr = mFragmentProgram->getGLSLProgram()->getParameter("use_optimiser");
+                if((paramStr == "true") || paramStr.empty())
                 {
-                    String paramStr = mFragmentProgram->getGLSLProgram()->getParameter("use_optimiser");
-                    if((paramStr == "true") || paramStr.empty())
-                    {
-                        GLSLESProgramPipelineManager::getSingleton().optimiseShaderSource(mFragmentProgram);
-                    }
+                    GLSLESProgramPipelineManager::getSingleton().optimiseShaderSource(mFragmentProgram);
                 }
+            }
 #endif
-				compileAndLink();
-			}
+            compileAndLink();
 
             extractLayoutQualifiers();
 
@@ -266,13 +236,13 @@ namespace Ogre
 			{
 				vertParams = &(mVertexProgram->getGLSLProgram()->getConstantDefinitions().map);
                 GLSLESProgramPipelineManager::getSingleton().extractUniforms(mVertexProgram->getGLSLProgram()->getGLProgramHandle(),
-                                                                         vertParams, NULL, mGLUniformReferences);
+					vertParams, NULL, mGLUniformReferences, mGLUniformBufferReferences);
 			}
 			if (mFragmentProgram)
 			{
 				fragParams = &(mFragmentProgram->getGLSLProgram()->getConstantDefinitions().map);
                 GLSLESProgramPipelineManager::getSingleton().extractUniforms(mFragmentProgram->getGLSLProgram()->getGLProgramHandle(),
-                                                                         NULL, fragParams, mGLUniformReferences);
+                                                                         NULL, fragParams, mGLUniformReferences, mGLUniformBufferReferences);
 			}
 
 			mUniformRefsBuilt = true;
@@ -286,13 +256,17 @@ namespace Ogre
 		// Iterate through uniform reference list and update uniform values
 		GLUniformReferenceIterator currentUniform = mGLUniformReferences.begin();
 		GLUniformReferenceIterator endUniform = mGLUniformReferences.end();
-#if GL_EXT_separate_shader_objects
+#if OGRE_PLATFORM != OGRE_PLATFORM_NACL
         GLuint progID = 0;
         if(fromProgType == GPT_VERTEX_PROGRAM)
+        {
             progID = mVertexProgram->getGLSLProgram()->getGLProgramHandle();
+        }
         else if(fromProgType == GPT_FRAGMENT_PROGRAM)
+        {
             progID = mFragmentProgram->getGLSLProgram()->getGLProgramHandle();
-        
+        }
+
 		for (;currentUniform != endUniform; ++currentUniform)
 		{
 			// Only pull values from buffer it's supposed to be in (vertex or fragment)
@@ -304,53 +278,82 @@ namespace Ogre
 				if (def->variability & mask)
 				{
 					GLsizei glArraySize = (GLsizei)def->arraySize;
-                    
+                    bool shouldUpdate = true;
+                    switch (def->constType)
+                    {
+                        case GCT_INT1:
+                        case GCT_INT2:
+                        case GCT_INT3:
+                        case GCT_INT4:
+                        case GCT_SAMPLER1D:
+                        case GCT_SAMPLER1DSHADOW:
+                        case GCT_SAMPLER2D:
+                        case GCT_SAMPLER2DSHADOW:
+                        case GCT_SAMPLER3D:
+                        case GCT_SAMPLERCUBE:
+#if OGRE_NO_GLES3_SUPPORT == 0
+                        case GCT_SAMPLER2DARRAY:
+#endif
+                            shouldUpdate = mUniformCache->updateUniform(currentUniform->mLocation,
+                                                                        params->getIntPointer(def->physicalIndex),
+                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(int)));
+                            break;
+                        default:
+                            shouldUpdate = mUniformCache->updateUniform(currentUniform->mLocation,
+                                                                        params->getFloatPointer(def->physicalIndex),
+                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(float)));
+                            break;
+                    }
+
+                    if(!shouldUpdate)
+                        continue;
+
 					// Get the index in the parameter real list
 					switch (def->constType)
 					{
                         case GCT_FLOAT1:
-                            glProgramUniform1fvEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getFloatPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform1fvEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_FLOAT2:
-                            glProgramUniform2fvEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getFloatPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform2fvEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_FLOAT3:
-                            glProgramUniform3fvEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getFloatPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform3fvEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_FLOAT4:
-                            glProgramUniform4fvEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getFloatPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform4fvEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_MATRIX_2X2:
-                            glProgramUniformMatrix2fvEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                         GL_FALSE, params->getFloatPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix2fvEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                             GL_FALSE, params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_MATRIX_3X3:
-                            glProgramUniformMatrix3fvEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                         GL_FALSE, params->getFloatPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix3fvEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                             GL_FALSE, params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_MATRIX_4X4:
-                            glProgramUniformMatrix4fvEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                         GL_FALSE, params->getFloatPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix4fvEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                             GL_FALSE, params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_INT1:
-                            glProgramUniform1ivEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getIntPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform1ivEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getIntPointer(def->physicalIndex)));
                             break;
                         case GCT_INT2:
-                            glProgramUniform2ivEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getIntPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform2ivEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getIntPointer(def->physicalIndex)));
                             break;
                         case GCT_INT3:
-                            glProgramUniform3ivEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getIntPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform3ivEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getIntPointer(def->physicalIndex)));
                             break;
                         case GCT_INT4:
-                            glProgramUniform4ivEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                   params->getIntPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform4ivEXT(progID, currentUniform->mLocation, glArraySize, 
+                                                                       params->getIntPointer(def->physicalIndex)));
                             break;
                         case GCT_SAMPLER1D:
                         case GCT_SAMPLER1DSHADOW:
@@ -358,10 +361,39 @@ namespace Ogre
                         case GCT_SAMPLER2DSHADOW:
                         case GCT_SAMPLER3D:
                         case GCT_SAMPLERCUBE:
+#if OGRE_NO_GLES3_SUPPORT == 0
+                        case GCT_SAMPLER2DARRAY:
+#endif
                             // Samplers handled like 1-element ints
-                            glProgramUniform1ivEXT(progID, currentUniform->mLocation, 1, 
-                                                   params->getIntPointer(def->physicalIndex));
+                            OGRE_CHECK_GL_ERROR(glProgramUniform1ivEXT(progID, currentUniform->mLocation, 1,
+                                                                       params->getIntPointer(def->physicalIndex)));
                             break;
+#if OGRE_NO_GLES3_SUPPORT == 0
+                        case GCT_MATRIX_2X3:
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix2x3fvEXT(progID, currentUniform->mLocation, glArraySize,
+                                                                            GL_FALSE, params->getFloatPointer(def->physicalIndex)));
+                            break;
+                        case GCT_MATRIX_2X4:
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix2x4fvEXT(progID, currentUniform->mLocation, glArraySize,
+                                                                            GL_FALSE, params->getFloatPointer(def->physicalIndex)));
+                            break;
+                        case GCT_MATRIX_3X2:
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix3x2fvEXT(progID, currentUniform->mLocation, glArraySize,
+                                                                            GL_FALSE, params->getFloatPointer(def->physicalIndex)));
+                            break;
+                        case GCT_MATRIX_3X4:
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix3x4fvEXT(progID, currentUniform->mLocation, glArraySize,
+                                                                            GL_FALSE, params->getFloatPointer(def->physicalIndex)));
+                            break;
+                        case GCT_MATRIX_4X2:
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix4x2fvEXT(progID, currentUniform->mLocation, glArraySize,
+                                                                            GL_FALSE, params->getFloatPointer(def->physicalIndex)));
+                            break;
+                        case GCT_MATRIX_4X3:
+                            OGRE_CHECK_GL_ERROR(glProgramUniformMatrix4x3fvEXT(progID, currentUniform->mLocation, glArraySize,
+                                                                            GL_FALSE, params->getFloatPointer(def->physicalIndex)));
+                            break;
+#else
                         case GCT_MATRIX_2X3:
                         case GCT_MATRIX_2X4:
                         case GCT_MATRIX_3X2:
@@ -369,15 +401,59 @@ namespace Ogre
                         case GCT_MATRIX_4X2:
                         case GCT_MATRIX_4X3:
                         case GCT_SAMPLER2DARRAY:
+#endif
                         case GCT_UNKNOWN:
+                        case GCT_SUBROUTINE:
+                        case GCT_DOUBLE1:
+                        case GCT_DOUBLE2:
+                        case GCT_DOUBLE3:
+                        case GCT_DOUBLE4:
+                        case GCT_SAMPLERRECT:
+                        case GCT_MATRIX_DOUBLE_2X2:
+                        case GCT_MATRIX_DOUBLE_2X3:
+                        case GCT_MATRIX_DOUBLE_2X4:
+                        case GCT_MATRIX_DOUBLE_3X2:
+                        case GCT_MATRIX_DOUBLE_3X3:
+                        case GCT_MATRIX_DOUBLE_3X4:
+                        case GCT_MATRIX_DOUBLE_4X2:
+                        case GCT_MATRIX_DOUBLE_4X3:
+                        case GCT_MATRIX_DOUBLE_4X4:
                             break;
                             
 					} // End switch
-                    GL_CHECK_ERROR;
 				} // Variability & mask
 			} // fromProgType == currentUniform->mSourceProgType
             
   		} // End for
+#endif
+	}
+    //-----------------------------------------------------------------------
+	void GLSLESProgramPipeline::updateUniformBlocks(GpuProgramParametersSharedPtr params,
+                                                  uint16 mask, GpuProgramType fromProgType)
+	{
+#if OGRE_NO_GLES3_SUPPORT == 0
+        // Iterate through the list of uniform buffers and update them as needed
+		GLUniformBufferIterator currentBuffer = mGLUniformBufferReferences.begin();
+		GLUniformBufferIterator endBuffer = mGLUniformBufferReferences.end();
+
+        const GpuProgramParameters::GpuSharedParamUsageList& sharedParams = params->getSharedParameters();
+
+		GpuProgramParameters::GpuSharedParamUsageList::const_iterator it, end = sharedParams.end();
+		for (it = sharedParams.begin(); it != end; ++it)
+        {
+            for (;currentBuffer != endBuffer; ++currentBuffer)
+            {
+                GLES2HardwareUniformBuffer* hwGlBuffer = static_cast<GLES2HardwareUniformBuffer*>(currentBuffer->get());
+                GpuSharedParametersPtr paramsPtr = it->getSharedParams();
+
+                // Block name is stored in mSharedParams->mName of GpuSharedParamUsageList items
+                GLint UniformTransform;
+                OGRE_CHECK_GL_ERROR(UniformTransform = glGetUniformBlockIndex(mGLProgramHandle, it->getName().c_str()));
+                OGRE_CHECK_GL_ERROR(glUniformBlockBinding(mGLProgramHandle, UniformTransform, hwGlBuffer->getGLBufferBinding()));
+
+                hwGlBuffer->writeData(0, hwGlBuffer->getSizeInBytes(), &paramsPtr->getFloatConstantList().front());
+            }
+        }
 #endif
 	}
 	//-----------------------------------------------------------------------
@@ -396,20 +472,32 @@ namespace Ogre
 				// Get the index in the parameter real list
 				if (index == currentUniform->mConstantDef->physicalIndex)
 				{
-#if GL_EXT_separate_shader_objects
+#if OGRE_PLATFORM != OGRE_PLATFORM_NACL
+
                     GLuint progID = 0;
                     if (mVertexProgram && currentUniform->mSourceProgType == GPT_VERTEX_PROGRAM)
                     {
+                        if(!mUniformCache->updateUniform(currentUniform->mLocation,
+                                                                             params->getFloatPointer(index),
+                                                                             static_cast<GLsizei>(currentUniform->mConstantDef->elementSize *
+                                                                             currentUniform->mConstantDef->arraySize *
+                                                                             sizeof(float))))
+                            return;
+                        
                         progID = mVertexProgram->getGLSLProgram()->getGLProgramHandle();
-                        glProgramUniform1fvEXT(progID, currentUniform->mLocation, 1, params->getFloatPointer(index));
-                        GL_CHECK_ERROR;
+                        OGRE_CHECK_GL_ERROR(glProgramUniform1fvEXT(progID, currentUniform->mLocation, 1, params->getFloatPointer(index)));
                     }
                     
                     if (mFragmentProgram && currentUniform->mSourceProgType == GPT_FRAGMENT_PROGRAM)
                     {
+                        if(!mUniformCache->updateUniform(currentUniform->mLocation,
+                                                                               params->getFloatPointer(index),
+                                                                               static_cast<GLsizei>(currentUniform->mConstantDef->elementSize *
+                                                                               currentUniform->mConstantDef->arraySize *
+                                                                               sizeof(float))))
+                            return;
                         progID = mFragmentProgram->getGLSLProgram()->getGLProgramHandle();
-                        glProgramUniform1fvEXT(progID, currentUniform->mLocation, 1, params->getFloatPointer(index));
-                        GL_CHECK_ERROR;
+                        OGRE_CHECK_GL_ERROR(glProgramUniform1fvEXT(progID, currentUniform->mLocation, 1, params->getFloatPointer(index)));
                     }
 #endif
 					// There will only be one multipass entry
