@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,8 @@ THE SOFTWARE.
 #include "OgreGLES2PixelFormat.h"
 #include "OgreGLES2RenderSystem.h"
 #include "OgreGLES2HardwarePixelBuffer.h"
+#include "OgreGLES2Support.h"
+#include "OgreGLES2StateCacheManager.h"
 #include "OgreRoot.h"
 
 namespace Ogre {
@@ -87,116 +89,142 @@ namespace Ogre {
                 return GL_TEXTURE_2D;
             case TEX_TYPE_CUBE_MAP:
                 return GL_TEXTURE_CUBE_MAP;
+#if OGRE_NO_GLES3_SUPPORT == 0
+            case TEX_TYPE_3D:
+                return GL_TEXTURE_3D;
+            case TEX_TYPE_2D_ARRAY:
+                return GL_TEXTURE_2D_ARRAY;
+#endif
             default:
                 return 0;
         };
     }
 
-    // Creation / loading methods
-    void GLES2Texture::createInternalResourcesImpl(void)
+    void GLES2Texture::_createGLTexResource()
     {
-		// Convert to nearest power-of-two size if required
+        // Convert to nearest power-of-two size if required
         mWidth = GLES2PixelUtil::optionalPO2(mWidth);
         mHeight = GLES2PixelUtil::optionalPO2(mHeight);
         mDepth = GLES2PixelUtil::optionalPO2(mDepth);
-
+        
 		// Adjust format if required
         mFormat = TextureManager::getSingleton().getNativeFormat(mTextureType, mFormat, mUsage);
+        GLenum texTarget = getGLES2TextureTarget();
 
 		// Check requested number of mipmaps
         size_t maxMips = GLES2PixelUtil::getMaxMipmaps(mWidth, mHeight, mDepth, mFormat);
-
+        
         if(PixelUtil::isCompressed(mFormat) && (mNumMipmaps == 0))
             mNumRequestedMipmaps = 0;
-
+        
         mNumMipmaps = mNumRequestedMipmaps;
         if (mNumMipmaps > maxMips)
             mNumMipmaps = maxMips;
-
+        
 		// Generate texture name
-        glGenTextures(1, &mTextureID);
-        GL_CHECK_ERROR;
-
+        OGRE_CHECK_GL_ERROR(glGenTextures(1, &mTextureID));
+           
 		// Set texture type
-        glBindTexture(getGLES2TextureTarget(), mTextureID);
-        GL_CHECK_ERROR;
-
+		mGLSupport.getStateCacheManager()->bindGLTexture(texTarget, mTextureID);
+        
         // If we can do automip generation and the user desires this, do so
         mMipmapsHardwareGenerated =
-            Root::getSingleton().getRenderSystem()->getCapabilities()->hasCapability(RSC_AUTOMIPMAP) && !PixelUtil::isCompressed(mFormat);
+        Root::getSingleton().getRenderSystem()->getCapabilities()->hasCapability(RSC_AUTOMIPMAP) && !PixelUtil::isCompressed(mFormat);
 
-#if GL_APPLE_texture_max_level
-        glTexParameteri( getGLES2TextureTarget(), GL_TEXTURE_MAX_LEVEL_APPLE, (mMipmapsHardwareGenerated && (mUsage & TU_AUTOMIPMAP)) ? maxMips : mNumMipmaps );
+        if(mGLSupport.checkExtension("GL_APPLE_texture_max_level") || gleswIsSupported(3, 0))
+            mGLSupport.getStateCacheManager()->setTexParameteri(texTarget, GL_TEXTURE_MAX_LEVEL_APPLE, mNumMipmaps);
+
+		// Set some misc default parameters, these can of course be changed later
+		mGLSupport.getStateCacheManager()->setTexParameteri(texTarget,
+                                                            GL_TEXTURE_MIN_FILTER, (mUsage & TU_AUTOMIPMAP) ? GL_NEAREST_MIPMAP_NEAREST : GL_NEAREST);
+        mGLSupport.getStateCacheManager()->setTexParameteri(texTarget,
+                                                            GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+        mGLSupport.getStateCacheManager()->setTexParameteri(texTarget,
+                                                            GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+        mGLSupport.getStateCacheManager()->setTexParameteri(texTarget,
+                                                            GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+
+        // Set up texture swizzling
+#if OGRE_NO_GLES3_SUPPORT == 0
+        if(gleswIsSupported(3, 0))
+        {
+            OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_R, GL_RED));
+            OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_G, GL_GREEN));
+            OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_B, GL_BLUE));
+            OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_A, GL_ALPHA));
+
+            if(mFormat == PF_L8 || mFormat == PF_L16)
+            {
+                OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_R, GL_RED));
+                OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_G, GL_RED));
+                OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_B, GL_RED));
+                OGRE_CHECK_GL_ERROR(glTexParameteri(texTarget, GL_TEXTURE_SWIZZLE_A, GL_RED));
+            }
+        }
 #endif
 
-        // Set some misc default parameters, these can of course be changed later
-        glTexParameteri(getGLES2TextureTarget(),
-                        GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-        GL_CHECK_ERROR;
-        glTexParameteri(getGLES2TextureTarget(),
-                        GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-        GL_CHECK_ERROR;
-        glTexParameteri(getGLES2TextureTarget(),
-                        GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-        GL_CHECK_ERROR;
-        glTexParameteri(getGLES2TextureTarget(),
-                        GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-        GL_CHECK_ERROR;
-
         // Allocate internal buffer so that glTexSubImageXD can be used
         // Internal format
-        GLenum format = GLES2PixelUtil::getClosestGLInternalFormat(mFormat, mHwGamma);
-        GLenum datatype = GLES2PixelUtil::getGLOriginDataType(mFormat);
-        size_t width = mWidth;
-        size_t height = mHeight;
-        size_t depth = mDepth;
-
+        GLenum format = GLES2PixelUtil::getGLOriginFormat(mFormat);
+        GLenum internalformat = GLES2PixelUtil::getClosestGLInternalFormat(mFormat, mHwGamma);
+        uint32 width = mWidth;
+        uint32 height = mHeight;
+        uint32 depth = mDepth;
+        
         if (PixelUtil::isCompressed(mFormat))
         {
             // Compressed formats
-            size_t size = PixelUtil::getMemorySize(mWidth, mHeight, mDepth, mFormat);
-
+            GLsizei size = static_cast<GLsizei>(PixelUtil::getMemorySize(mWidth, mHeight, mDepth, mFormat));
+            
             // Provide temporary buffer filled with zeroes as glCompressedTexImageXD does not
             // accept a 0 pointer like normal glTexImageXD
             // Run through this process for every mipmap to pregenerate mipmap pyramid
-
+            
             uint8* tmpdata = new uint8[size];
             memset(tmpdata, 0, size);
-            for (size_t mip = 0; mip <= mNumMipmaps; mip++)
+            for (uint8 mip = 0; mip <= mNumMipmaps; mip++)
             {
-                size = PixelUtil::getMemorySize(width, height, depth, mFormat);
-
+#if OGRE_DEBUG_MODE
+                LogManager::getSingleton().logMessage("GLES2Texture::create - Mip: " + StringConverter::toString(mip) +
+                                                      " Width: " + StringConverter::toString(width) +
+                                                      " Height: " + StringConverter::toString(height) +
+                                                      " Internal Format: " + StringConverter::toString(internalformat, 0, ' ', std::ios::hex) +
+                                                      " Format: " + StringConverter::toString(format, 0, ' ', std::ios::hex)
+                                                      );
+#endif
+                size = static_cast<GLsizei>(PixelUtil::getMemorySize(width, height, depth, mFormat));
+                
 				switch(mTextureType)
 				{
 					case TEX_TYPE_1D:
 					case TEX_TYPE_2D:
-                        glCompressedTexImage2D(GL_TEXTURE_2D,
+                        OGRE_CHECK_GL_ERROR(glCompressedTexImage2D(GL_TEXTURE_2D,
                                                mip,
-                                               format,
+                                               internalformat,
                                                width, height,
                                                0,
                                                size,
-                                               tmpdata);
-                        GL_CHECK_ERROR;
+                                               tmpdata));
                         break;
 					case TEX_TYPE_CUBE_MAP:
 						for(int face = 0; face < 6; face++) {
-							glCompressedTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + face, mip, format,
+							OGRE_CHECK_GL_ERROR(glCompressedTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + face, mip, internalformat,
 								width, height, 0, 
-								size, tmpdata);
-                            GL_CHECK_ERROR;
+								size, tmpdata));
 						}
 						break;
+#if OGRE_NO_GLES3_SUPPORT == 0
+					case TEX_TYPE_2D_ARRAY:
 					case TEX_TYPE_3D:
+						glCompressedTexImage3D(getGLES2TextureTarget(), mip, format,
+							width, height, depth, 0, 
+							size, tmpdata);
+						break;
+#endif
                     default:
                         break;
                 };
-//                LogManager::getSingleton().logMessage("GLES2Texture::create - Mip: " + StringConverter::toString(mip) +
-//                                                      " Width: " + StringConverter::toString(width) +
-//                                                      " Height: " + StringConverter::toString(height) +
-//                                                      " Internal Format: " + StringConverter::toString(format)
-//                                                      );
-
+                
                 if(width > 1)
                 {
                     width = width / 2;
@@ -205,7 +233,7 @@ namespace Ogre {
                 {
                     height = height / 2;
                 }
-                if(depth > 1)
+				if(depth>1 && mTextureType != TEX_TYPE_2D_ARRAY)
                 {
                     depth = depth / 2;
                 }
@@ -214,40 +242,80 @@ namespace Ogre {
         }
         else
         {
+#if OGRE_NO_GLES3_SUPPORT == 0
+#if OGRE_DEBUG_MODE
+            LogManager::getSingleton().logMessage("GLES2Texture::create - Name: " + mName +
+                                                      " ID: " + StringConverter::toString(mTextureID) +
+                                                      " Width: " + StringConverter::toString(width) +
+                                                      " Height: " + StringConverter::toString(height) +
+                                                      " Internal Format: " + StringConverter::toString(internalformat, 0, ' ', std::ios::hex));
+#endif
+            switch(mTextureType)
+            {
+                case TEX_TYPE_1D:
+                case TEX_TYPE_2D:
+                case TEX_TYPE_2D_RECT:
+                    OGRE_CHECK_GL_ERROR(glTexStorage2D(GL_TEXTURE_2D, GLsizei(mNumMipmaps+1), internalformat, GLsizei(width), GLsizei(height)));
+                    break;
+                case TEX_TYPE_CUBE_MAP:
+                    OGRE_CHECK_GL_ERROR(glTexStorage2D(GL_TEXTURE_CUBE_MAP, GLsizei(mNumMipmaps+1), internalformat, GLsizei(width), GLsizei(height)));
+                    break;
+                case TEX_TYPE_2D_ARRAY:
+                    OGRE_CHECK_GL_ERROR(glTexStorage3D(GL_TEXTURE_2D_ARRAY, GLsizei(mNumMipmaps+1), internalformat, GLsizei(width), GLsizei(height), GLsizei(depth)));
+                    break;
+                case TEX_TYPE_3D:
+                    OGRE_CHECK_GL_ERROR(glTexStorage3D(GL_TEXTURE_3D, GLsizei(mNumMipmaps+1), internalformat, GLsizei(width), GLsizei(height), GLsizei(depth)));
+                    break;
+            }
+#else
+            GLenum datatype = GLES2PixelUtil::getGLOriginDataType(mFormat);
+
             // Run through this process to pregenerate mipmap pyramid
             for(size_t mip = 0; mip <= mNumMipmaps; mip++)
             {
+#if OGRE_DEBUG_MODE
+                LogManager::getSingleton().logMessage("GLES2Texture::create - Mip: " + StringConverter::toString(mip) +
+                                                      " Name: " + mName +
+                                                      " ID: " + StringConverter::toString(mTextureID) +
+                                                      " Width: " + StringConverter::toString(width) +
+                                                      " Height: " + StringConverter::toString(height) +
+                                                      " Internal Format: " + StringConverter::toString(internalformat, 0, ' ', std::ios::hex) +
+                                                      " Format: " + StringConverter::toString(format, 0, ' ', std::ios::hex) +
+                                                      " Datatype: " + StringConverter::toString(datatype, 0, ' ', std::ios::hex)
+                                                      );
+#endif
 				// Normal formats
 				switch(mTextureType)
 				{
 					case TEX_TYPE_1D:
 					case TEX_TYPE_2D:
-                        glTexImage2D(GL_TEXTURE_2D,
+#if OGRE_PLATFORM == OGRE_PLATFORM_NACL
+                        if(internalformat != format)
+                        {
+                            LogManager::getSingleton().logMessage("glTexImage2D: format != internalFormat, "
+                                "format=" + StringConverter::toString(format) + 
+                                ", internalFormat=" + StringConverter::toString(internalformat));
+                        }
+#endif
+                        OGRE_CHECK_GL_ERROR(glTexImage2D(GL_TEXTURE_2D,
                                      mip,
-                                     format,
+                                     internalformat,
                                      width, height,
                                      0,
                                      format,
-                                     datatype, 0);
-                        GL_CHECK_ERROR;
+                                     datatype, 0));
                         break;
 					case TEX_TYPE_CUBE_MAP:
 						for(int face = 0; face < 6; face++) {
-							glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + face, mip, format,
+							OGRE_CHECK_GL_ERROR(glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + face, mip, internalformat,
 								width, height, 0, 
-								format, datatype, 0);
+								format, datatype, 0));
 						}
 						break;
-					case TEX_TYPE_3D:
                     default:
                         break;
                 };
-//                LogManager::getSingleton().logMessage("GLES2Texture::create - Mip: " + StringConverter::toString(mip) +
-//                                                      " Width: " + StringConverter::toString(width) +
-//                                                      " Height: " + StringConverter::toString(height) +
-//                                                      " Internal Format: " + StringConverter::toString(format)
-//                                                      );
-
+                
                 if (width > 1)
                 {
                     width = width / 2;
@@ -257,8 +325,15 @@ namespace Ogre {
                     height = height / 2;
                 }
             }
+#endif
         }
-
+    }
+    
+    // Creation / loading methods
+    void GLES2Texture::createInternalResourcesImpl(void)
+    {
+		_createGLTexResource();
+        
         _createSurfaceList();
 
         // Get final internal format
@@ -287,7 +362,9 @@ namespace Ogre {
 
         LoadedImages loadedImages = LoadedImages(new vector<Image>::type());
 
-        if (mTextureType == TEX_TYPE_1D || mTextureType == TEX_TYPE_2D)
+        if (mTextureType == TEX_TYPE_1D || mTextureType == TEX_TYPE_2D ||
+            mTextureType == TEX_TYPE_2D_ARRAY || mTextureType == TEX_TYPE_3D)
+
         {
             doImageIO(mName, mGroup, ext, *loadedImages, this);
 
@@ -296,11 +373,12 @@ namespace Ogre {
             if ((*loadedImages)[0].hasFlag(IF_CUBEMAP))
                 mTextureType = TEX_TYPE_CUBE_MAP;
 
+            // If this is a volumetric texture set the texture type flag accordingly.
+            if((*loadedImages)[0].getDepth() > 1 && mTextureType != TEX_TYPE_2D_ARRAY)
+                mTextureType = TEX_TYPE_3D;
                         
 			// If PVRTC and 0 custom mipmap disable auto mip generation and disable software mipmap creation
-            PixelFormat imageFormat = (*loadedImages)[0].getFormat();
-			if (imageFormat == PF_PVRTC_RGB2 || imageFormat == PF_PVRTC_RGBA2 ||
-                imageFormat == PF_PVRTC_RGB4 || imageFormat == PF_PVRTC_RGBA4)
+			if (PixelUtil::isCompressed((*loadedImages)[0].getFormat()))
 			{
                 size_t imageMips = (*loadedImages)[0].getNumMipmaps();
                 if (imageMips == 0)
@@ -374,14 +452,61 @@ namespace Ogre {
         }
 
         _loadImages(imagePtrs);
+
+        if (mUsage & TU_AUTOMIPMAP)
+        {
+            OGRE_CHECK_GL_ERROR(glGenerateMipmap(getGLES2TextureTarget()));
+        }
     }
 
     void GLES2Texture::freeInternalResourcesImpl()
     {
         mSurfaceList.clear();
-        glDeleteTextures(1, &mTextureID);
-        GL_CHECK_ERROR;
+        OGRE_CHECK_GL_ERROR(glDeleteTextures(1, &mTextureID));
+        mGLSupport.getStateCacheManager()->invalidateStateForTexture( mTextureID );
+        mTextureID = 0;
     }
+    
+#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
+    void GLES2Texture::notifyOnContextLost()
+    {
+        if (!mIsManual) 
+        {
+            freeInternalResources();
+        }
+        else
+        {
+            OGRE_CHECK_GL_ERROR(glDeleteTextures(1, &mTextureID));
+            mTextureID = 0;
+        }
+    }
+    
+    void GLES2Texture::notifyOnContextReset()
+    {
+        if (!mIsManual) 
+        {
+            reload();
+        }
+        else
+        {
+            preLoadImpl();
+            
+            _createGLTexResource();
+            
+            for(size_t i = 0; i < mSurfaceList.size(); i++)
+            {
+                static_cast<GLES2TextureBuffer*>(mSurfaceList[i].getPointer())->updateTextureId(mTextureID);
+            }
+            
+            if (mLoader)
+            {
+                mLoader->loadResource(this);
+            }
+            
+            postLoadImpl();
+        }
+    }
+#endif
 
     void GLES2Texture::_createSurfaceList()
     {
@@ -396,18 +521,19 @@ namespace Ogre {
 
         for (size_t face = 0; face < getNumFaces(); face++)
         {
-			size_t width = mWidth;
-			size_t height = mHeight;
+			uint32 width = mWidth;
+			uint32 height = mHeight;
+			uint32 depth = mDepth;
 
-            for (size_t mip = 0; mip <= getNumMipmaps(); mip++)
+            for (uint8 mip = 0; mip <= getNumMipmaps(); mip++)
             {
                 GLES2HardwarePixelBuffer *buf = OGRE_NEW GLES2TextureBuffer(mName,
                                                                             getGLES2TextureTarget(),
                                                                             mTextureID,
-                                                                            width, height,
+                                                                            width, height, depth,
                                                                             GLES2PixelUtil::getClosestGLInternalFormat(mFormat, mHwGamma),
                                                                             GLES2PixelUtil::getGLOriginDataType(mFormat),
-                                                                            face,
+                                                                            static_cast<GLint>(face),
                                                                             mip,
                                                                             static_cast<HardwareBuffer::Usage>(mUsage),
                                                                             doSoftware && mip==0, mHwGamma, mFSAA);
@@ -447,7 +573,7 @@ namespace Ogre {
                         "GLES2Texture::getBuffer");
         }
 
-        unsigned int idx = face * (mNumMipmaps + 1) + mipmap;
+        unsigned long idx = face * (mNumMipmaps + 1) + mipmap;
         assert(idx < mSurfaceList.size());
         return mSurfaceList[idx];
     }
