@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,8 @@ THE SOFTWARE.
 */
 #include <stdio.h>
 #include "Ogre.h"
-#include "OgreProgressiveMesh.h"
+#include "OgreProgressiveMeshGenerator.h"
+#include "OgreDistanceLodStrategy.h"
 #include "OgreDefaultHardwareBufferManager.h"
 #include "OgreFileSystem.h"
 #include "OgreArchiveManager.h"
@@ -39,7 +40,11 @@ CPPUNIT_TEST_SUITE_REGISTRATION( MeshWithoutIndexDataTests );
 
 void MeshWithoutIndexDataTests::setUp()
 {
+    if(LogManager::getSingletonPtr() == 0)
+        mLogManager = OGRE_NEW LogManager();
+
 	LogManager::getSingleton().createLog("MeshWithoutIndexDataTests.log", true);
+    LogManager::getSingleton().setLogDetail(LL_LOW);
 	OGRE_NEW ResourceGroupManager();
 	OGRE_NEW LodStrategyManager();
     mBufMgr = OGRE_NEW DefaultHardwareBufferManager();
@@ -58,6 +63,7 @@ void MeshWithoutIndexDataTests::tearDown()
 	OGRE_DELETE MaterialManager::getSingletonPtr();
 	OGRE_DELETE LodStrategyManager::getSingletonPtr();
 	OGRE_DELETE ResourceGroupManager::getSingletonPtr();
+    OGRE_DELETE mLogManager;
 }
 
 void MeshWithoutIndexDataTests::testCreateSimpleLine()
@@ -219,7 +225,7 @@ void MeshWithoutIndexDataTests::testCreatePointList()
 void MeshWithoutIndexDataTests::testCreateLineWithMaterial()
 {
     String matName = "lineMat";
-    MaterialPtr matPtr = MaterialManager::getSingleton().create(matName, "General");
+    MaterialPtr matPtr = MaterialManager::getSingleton().create(matName, "General").staticCast<Material>();
     Pass* pass = matPtr->getTechnique(0)->getPass(0);
     pass->setDiffuse(1.0, 0.1, 0.1, 0);
 
@@ -243,9 +249,8 @@ void MeshWithoutIndexDataTests::testCreateLineWithMaterial()
     meshWriter.exportMesh(lineMesh.get(), fileName);
     MaterialSerializer matWriter;
     matWriter.exportMaterial(
-        MaterialManager::getSingleton().getByName(matName), 
-        matName + ".material"
-        );
+        MaterialManager::getSingleton().getByName(matName).staticCast<Material>(),
+        matName + ".material");
 
     mMeshMgr->remove( fileName );
 
@@ -269,25 +274,25 @@ void createMeshWithMaterial(String fileName)
     String matFileNameSuffix = ".material";
     String matName1 = "red";
     String matFileName1 = matName1 + matFileNameSuffix;
-    MaterialPtr matPtr = MaterialManager::getSingleton().create(matName1, "General");
+    MaterialPtr matPtr = MaterialManager::getSingleton().create(matName1, "General").staticCast<Material>();
     Pass* pass = matPtr->getTechnique(0)->getPass(0);
     pass->setDiffuse(1.0, 0.1, 0.1, 0);
 
     String matName2 = "green";
     String matFileName2 = matName2 + matFileNameSuffix;
-    matPtr = MaterialManager::getSingleton().create(matName2, "General");
+    matPtr = MaterialManager::getSingleton().create(matName2, "General").staticCast<Material>();
     pass = matPtr->getTechnique(0)->getPass(0);
     pass->setDiffuse(0.1, 1.0, 0.1, 0);
 
     String matName3 = "blue";
     String matFileName3 = matName3 + matFileNameSuffix;
-    matPtr = MaterialManager::getSingleton().create(matName3, "General");
+    matPtr = MaterialManager::getSingleton().create(matName3, "General").staticCast<Material>();
     pass = matPtr->getTechnique(0)->getPass(0);
     pass->setDiffuse(0.1, 0.1, 1.0, 0);
 
     String matName4 = "yellow";
     String matFileName4 = matName4 + matFileNameSuffix;
-    matPtr = MaterialManager::getSingleton().create(matName4, "General");
+    matPtr = MaterialManager::getSingleton().create(matName4, "General").staticCast<Material>();
     pass = matPtr->getTechnique(0)->getPass(0);
     pass->setDiffuse(1.0, 1.0, 0.1, 0);
 
@@ -329,7 +334,7 @@ void MeshWithoutIndexDataTests::testCreateMesh()
 {
     String fileName = "indexMix.mesh";
     createMeshWithMaterial(fileName);
-    MeshPtr mesh = mMeshMgr->getByName(fileName);
+    MeshPtr mesh = mMeshMgr->getByName(fileName).staticCast<Mesh>();
 
     CPPUNIT_ASSERT(mesh->getNumSubMeshes() == 4);
     RenderOperation rop;
@@ -359,7 +364,7 @@ void MeshWithoutIndexDataTests::testCloneMesh()
 {
     String originalName = "toClone.mesh";
     createMeshWithMaterial(originalName);
-    MeshPtr mesh = mMeshMgr->getByName(originalName);
+    MeshPtr mesh = mMeshMgr->getByName(originalName).staticCast<Mesh>();
 
     String fileName = "clone.mesh";
     MeshPtr clone = mesh->clone(fileName);
@@ -409,7 +414,7 @@ void MeshWithoutIndexDataTests::testGenerateExtremes()
 {
     String fileName = "testGenerateExtremes.mesh";
     createMeshWithMaterial(fileName);
-    MeshPtr mesh = mMeshMgr->getByName(fileName);
+    MeshPtr mesh = mMeshMgr->getByName(fileName).staticCast<Mesh>();
 
     const size_t NUM_EXTREMES = 4;
     for (ushort i = 0; i < mesh->getNumSubMeshes(); ++i)
@@ -419,14 +424,13 @@ void MeshWithoutIndexDataTests::testGenerateExtremes()
     for (ushort i = 0; i < mesh->getNumSubMeshes(); ++i)
     {
         SubMesh* subMesh = mesh->getSubMesh(i);
+        // According to generateExtremes, extremes are built based upon the bounding box indices.
+        // But it also creates indices for all bbox's even if the mesh does not have any.
+        // So...there should always be some extremity points. The number of which may vary
         if (subMesh->indexData->indexCount > 0)
         {
             CPPUNIT_ASSERT(subMesh->extremityPoints.size() == NUM_EXTREMES);
         }
-        else
-        {
-            CPPUNIT_ASSERT(subMesh->extremityPoints.size() == 0);
-        }
     }
 
     mMeshMgr->remove( fileName );
@@ -436,7 +440,7 @@ void MeshWithoutIndexDataTests::testBuildTangentVectors()
 {
     String fileName = "testBuildTangentVectors.mesh";
     createMeshWithMaterial(fileName);
-    MeshPtr mesh = mMeshMgr->getByName(fileName);
+    MeshPtr mesh = mMeshMgr->getByName(fileName).staticCast<Mesh>();
 
     try
     {
@@ -456,12 +460,20 @@ void MeshWithoutIndexDataTests::testGenerateLodLevels()
 {
     String fileName = "testGenerateLodLevels.mesh";
     createMeshWithMaterial(fileName);
-    MeshPtr mesh = mMeshMgr->getByName(fileName);
-
-    Mesh::LodValueList lodDistanceList;
-    lodDistanceList.push_back(600.0);
-    ProgressiveMesh::generateLodLevels(mesh.get(), lodDistanceList, ProgressiveMesh::VRQ_CONSTANT, 2);
+    MeshPtr mesh = mMeshMgr->getByName(fileName).staticCast<Mesh>();
 
+	LodConfig lodConfig;
+    lodConfig.levels.clear();
+    lodConfig.mesh = MeshPtr(mesh);
+    lodConfig.strategy = DistanceLodSphereStrategy::getSingletonPtr();
+    LodLevel lodLevel;
+    lodLevel.reductionMethod = LodLevel::VRM_CONSTANT;
+    lodLevel.distance = 600.0;
+    lodLevel.reductionValue = 2;
+    lodConfig.levels.push_back(lodLevel);
+    ProgressiveMeshGenerator pm;
+    pm.generateLodLevels(lodConfig);
+    // FAIL: Levels == 1
     CPPUNIT_ASSERT(mesh->getNumLodLevels() == 2);
     for (ushort i = 0; i < mesh->getNumSubMeshes(); ++i)
     {
@@ -474,7 +486,8 @@ void MeshWithoutIndexDataTests::testGenerateLodLevels()
             }
             else
             {
-                CPPUNIT_ASSERT(subMesh->mLodFaceList[j]->indexCount == 0);
+                // Should be 3 because of the dummy triangle being generated
+                CPPUNIT_ASSERT(subMesh->mLodFaceList[j]->indexCount == 3);
             }
         }
     }
