@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,8 @@ THE SOFTWARE.
 #include "OgreSkeletonSerializer.h"
 #include "OgreXMLPrerequisites.h"
 #include "OgreDefaultHardwareBufferManager.h"
+#include "OgreProgressiveMeshGenerator.h"
+#include "OgreDistanceLodStrategy.h"
 #include <iostream>
 #include <sys/stat.h>
 
@@ -54,6 +56,8 @@ struct XmlOptions
     Real lodPercent;
     size_t lodFixed;
     size_t nuextremityPoints;
+	size_t mergeTexcoordResult;
+	size_t mergeTexcoordToDestroy;
     bool usePercent;
     bool generateEdgeLists;
     bool generateTangents;
@@ -83,6 +87,10 @@ void help(void)
     cout << "-s lodstrategy = LOD strategy to use for this mesh" << endl;
     cout << "-p lodpercent  = Percentage triangle reduction amount per LOD" << endl;
     cout << "-f lodnumtris  = Fixed vertex reduction per LOD" << endl;
+	cout << "-merge [n0,n1] = Merge texcoordn0 with texcoordn1. The , separator must be" << endl;
+	cout << "                 present, otherwise only n0 is provided assuming n1 = n0+1;" << endl;
+	cout << "                 n0 and n1 must be in the same buffer source & adjacent" << endl;
+	cout << "                 to each other for the merge to work." << endl;
     cout << "-e             = DON'T generate edge lists (for stencil shadows)" << endl;
     cout << "-r             = DON'T reorganise vertex buffers to OGRE recommended format." << endl;
     cout << "-t             = Generate tangents (for normal mapping)" << endl;
@@ -119,6 +127,8 @@ XmlOptions parseArgs(int numArgs, char **args)
     opts.lodPercent = 20;
     opts.numLods = 0;
     opts.nuextremityPoints = 0;
+	opts.mergeTexcoordResult = 0;
+	opts.mergeTexcoordToDestroy = 0;
     opts.usePercent = true;
     opts.generateEdgeLists = true;
     opts.generateTangents = false;
@@ -149,6 +159,7 @@ XmlOptions parseArgs(int numArgs, char **args)
 	unOpt["-q"] = false;
 	unOpt["-d3d"] = false;
 	unOpt["-gl"] = false;
+	unOpt["-h"] = false;
     binOpt["-l"] = "";
     binOpt["-v"] = "";
     binOpt["-s"] = "Distance";
@@ -159,11 +170,19 @@ XmlOptions parseArgs(int numArgs, char **args)
     binOpt["-log"] = "OgreXMLConverter.log";
 	binOpt["-td"] = "";
 	binOpt["-ts"] = "";
+	binOpt["-merge"] = "0,0";
 
     int startIndex = findCommandLineOpts(numArgs, args, unOpt, binOpt);
     UnaryOptionList::iterator ui;
     BinaryOptionList::iterator bi;
 
+    ui = unOpt.find("-h");
+	if (ui->second)
+	{
+		help();
+        exit(1);
+	}
+
 	ui = unOpt.find("-q");
 	if (ui->second)
 	{
@@ -251,6 +270,37 @@ XmlOptions parseArgs(int numArgs, char **args)
             opts.usePercent = true;
         }
 
+		bi = binOpt.find("-merge");
+        if (!bi->second.empty())
+        {
+			String::size_type separator = bi->second.find_first_of( "," );
+			if( separator == String::npos )
+			{
+				//Input format was "-merge 2"
+				//Assume we want to merge 2 with 3
+				opts.mergeTexcoordResult	= StringConverter::parseInt( bi->second, 0 );
+				opts.mergeTexcoordToDestroy	= opts.mergeTexcoordResult + 1;
+			}
+			else if( separator + 1 < bi->second.size() )
+			{
+				//Input format was "-merge 1,2"
+				//We want to merge 1 with 2
+				opts.mergeTexcoordResult	= StringConverter::parseInt(
+																bi->second.substr( 0, separator ), 0 );
+				opts.mergeTexcoordToDestroy = StringConverter::parseInt(
+																bi->second.substr( separator+1,
+																bi->second.size() ), 1 );
+			}
+        }
+		else
+		{
+			//Very rare to reach here.
+			//Input format was "-merge"
+			//Assume we want to merge 0 with 1
+			opts.mergeTexcoordResult = 0;
+			opts.mergeTexcoordResult = 1;
+		}
+
 
         bi = binOpt.find("-f");
         if (!bi->second.empty())
@@ -301,6 +351,11 @@ XmlOptions parseArgs(int numArgs, char **args)
         source = args[startIndex];
     if (numArgs > startIndex+1)
         dest = args[startIndex+1];
+    if (numArgs > startIndex+2) {
+        cout << "Too many command-line arguments supplied - abort. " << endl;
+        help();
+        exit(1);
+    }
 
     if (!source)
     {
@@ -423,6 +478,8 @@ void meshToXML(XmlOptions opts)
    
     xmlMeshSerializer->exportMesh(mesh.getPointer(), opts.dest);
 
+    // Clean up the conversion mesh
+    MeshManager::getSingleton().remove("conversion");
 }
 
 void XMLToBinary(XmlOptions opts)
@@ -452,16 +509,18 @@ void XMLToBinary(XmlOptions opts)
         xmlMeshSerializer->importMesh(opts.source, colourElementType, newMesh.getPointer());
 
         // Re-jig the buffers?
+		// Make sure animation types are up to date first
+		newMesh->_determineAnimationTypes();
         if (opts.reorganiseBuffers)
         {
-            logMgr->logMessage("Reorganising vertex buffers to automatic layout..");
+            logMgr->logMessage("Reorganising vertex buffers to automatic layout...");
             // Shared geometry
             if (newMesh->sharedVertexData)
             {
                 // Automatic
                 VertexDeclaration* newDcl = 
                     newMesh->sharedVertexData->vertexDeclaration->getAutoOrganisedDeclaration(
-                        newMesh->hasSkeleton(), newMesh->hasVertexAnimation());
+                        newMesh->hasSkeleton(), newMesh->hasVertexAnimation(), newMesh->getSharedVertexDataAnimationIncludesNormals());
                 if (*newDcl != *(newMesh->sharedVertexData->vertexDeclaration))
                 {
                     // Usages don't matter here since we're onlly exporting
@@ -478,10 +537,12 @@ void XMLToBinary(XmlOptions opts)
                 SubMesh* sm = smIt.getNext();
                 if (!sm->useSharedVertices)
                 {
+					const bool hasVertexAnim = sm->getVertexAnimationType() != Ogre::VAT_NONE;
+
                     // Automatic
                     VertexDeclaration* newDcl = 
                         sm->vertexData->vertexDeclaration->getAutoOrganisedDeclaration(
-                            newMesh->hasSkeleton(), newMesh->hasVertexAnimation());
+                            newMesh->hasSkeleton(), hasVertexAnim, sm->getVertexAnimationIncludesNormals());
                     if (*newDcl != *(sm->vertexData->vertexDeclaration))
                     {
                         // Usages don't matter here since we're onlly exporting
@@ -515,7 +576,7 @@ void XMLToBinary(XmlOptions opts)
             if (newMesh->getNumLodLevels() > 1)
             {
                 std::cout << "\nXML already contains level-of detail information.\n"
-                    "Do you want to: (u)se it, (r)eplace it, or (d)rop it?";
+                    "Do you want to: (u)se it, (r)eplace it, or (d)rop it? ";
                 while (response == "")
                 {
                     cin >> response;
@@ -536,13 +597,14 @@ void XMLToBinary(XmlOptions opts)
                     }
                     else
                     {
+                        std::cout << "Did not understand \"" << response << "\" please try again:" << std::endl;
                         response = "";
                     }
                 }// while response == ""
             }
             else // no existing LOD
             {
-                std::cout << "\nWould you like to generate LOD information? (y/n)";
+                std::cout << "\nWould you like to generate LOD information? (y/n) ";
                 while (response == "")
                 {
                     cin >> response;
@@ -556,6 +618,11 @@ void XMLToBinary(XmlOptions opts)
                         genLod = true;
                         askLodDtls = true;
                     }
+                    else
+                    {
+                        std::cout << "Did not understand \"" << response << "\" please try again:" << std::endl;
+                        response = "";
+                    }
                 }
             }
         }
@@ -563,80 +630,92 @@ void XMLToBinary(XmlOptions opts)
         if (genLod)
         {
             unsigned short numLod;
-            ProgressiveMesh::VertexReductionQuota quota;
-            Real reduction;
-            Mesh::LodValueList valueList;
+            LodConfig lodConfig;
+            lodConfig.levels.clear();
+            lodConfig.mesh = newMesh->clone(newMesh->getName());
+            lodConfig.strategy = DistanceLodSphereStrategy::getSingletonPtr();
+
+            LodLevel lodLevel;
+            lodLevel.reductionMethod = LodLevel::VRM_PROPORTIONAL;
 
             if (askLodDtls)
             {
-                cout << "\nHow many extra LOD levels would you like to generate?";
+                cout << "\nHow many extra LOD levels would you like to generate? ";
                 cin >> numLod;
 
-                cout << "\nWhat lod strategy should be used?";
+                cout << "\nWhat LOD strategy should be used? ";
                 cin >> opts.lodStrategy;
 
                 cout << "\nWhat unit of reduction would you like to use:" <<
-                    "\n(f)ixed or (p)roportional?";
-                cin >> response;
-				StringUtil::toLowerCase(response);
-                if (response == "f")
-                {
-                    quota = ProgressiveMesh::VRQ_CONSTANT;
-                    cout << "\nHow many vertices should be removed at each LOD?";
+                    "\n(f)ixed or (p)roportional? ";
+                response = "";
+                while (response == "") {
+                    cin >> response;
+                    StringUtil::toLowerCase(response);
+                    if (response == "f")
+                    {
+                        lodLevel.reductionMethod = LodLevel::VRM_CONSTANT;
+                        cout << "\nHow many vertices should be removed at each LOD? ";
+                    }
+                    else if (response == "p")
+                    {
+                        lodLevel.reductionMethod = LodLevel::VRM_PROPORTIONAL;
+                        cout << "\nWhat percentage of remaining vertices should be removed "
+                            "\at each LOD (e.g. 50)? ";
+                    }
+                    else {
+                            std::cout << "Did not understand \"" << response << "\" please try again:" << std::endl;
+                            response = "";
+                    }
                 }
-                else
-                {
-                    quota = ProgressiveMesh::VRQ_PROPORTIONAL;
-                    cout << "\nWhat percentage of remaining vertices should be removed "
-                        "\at each LOD (e.g. 50)?";
-                }
-                cin >> reduction;
-                if (quota == ProgressiveMesh::VRQ_PROPORTIONAL)
+                cin >> lodLevel.reductionValue;
+                if (lodLevel.reductionMethod == LodLevel::VRM_PROPORTIONAL)
                 {
                     // Percentage -> parametric
-                    reduction = reduction * 0.01f;
+                    lodLevel.reductionValue *= 0.01f;
                 }
 
-                cout << "\nEnter the distance for each LOD to come into effect.";
+                cout << "\nEnter the distance for each LOD to come into effect. ";
 
-                Real distance;
                 for (unsigned short iLod = 0; iLod < numLod; ++iLod)
                 {
-                    cout << "\nLOD Level " << (iLod+1) << ":";
-                    cin >> distance;
-                    valueList.push_back(distance);
+                    cout << "\nLOD Level " << (iLod+1) << ": ";
+                    cin >> lodLevel.distance;
+                    lodConfig.levels.push_back(lodLevel);
                 }
             }
             else
             {
                 numLod = opts.numLods;
-                quota = opts.usePercent? 
-                    ProgressiveMesh::VRQ_PROPORTIONAL : ProgressiveMesh::VRQ_CONSTANT;
+                lodLevel.reductionMethod = opts.usePercent? 
+                    LodLevel::VRM_PROPORTIONAL : LodLevel::VRM_CONSTANT;
                 if (opts.usePercent)
                 {
-                    reduction = opts.lodPercent * 0.01f;
+                    lodLevel.reductionValue = opts.lodPercent * 0.01f;
                 }
                 else
                 {
-                    reduction = opts.lodFixed;
+                    lodLevel.reductionValue = (Ogre::Real)opts.lodFixed;
                 }
                 Real currDist = 0;
                 for (unsigned short iLod = 0; iLod < numLod; ++iLod)
                 {
                     currDist += opts.lodValue;
                     Real currDistSq = Ogre::Math::Sqr(currDist);
-                    valueList.push_back(currDistSq);
+                    lodLevel.distance = currDistSq;
+                    lodConfig.levels.push_back(lodLevel);
                 }
 
             }
 
             newMesh->setLodStrategy(LodStrategyManager::getSingleton().getStrategy(opts.lodStrategy));
-            newMesh->generateLodLevels(valueList, quota, reduction);
+            ProgressiveMeshGenerator pm;
+            pm.generateLodLevels(lodConfig);
         }
 
         if (opts.interactiveMode)
         {
-            std::cout << "\nWould you like to include edge lists to enable stencil shadows with this mesh? (y/n)";
+            std::cout << "\nWould you like to include edge lists to enable stencil shadows with this mesh? (y/n) ";
             while (response == "")
             {
                 cin >> response;
@@ -651,12 +730,13 @@ void XMLToBinary(XmlOptions opts)
                 }
                 else
                 {
+                    std::cout << "Did not understand \"" << response << "\" please try again:" << std::endl;
                     response = "";
                 }
             }
 
 
-            std::cout << "\nWould you like to generate tangents to enable normal mapping with this mesh? (y/n)";
+            std::cout << "\nWould you like to generate tangents to enable normal mapping with this mesh? (y/n) ";
             while (response == "")
             {
                 cin >> response;
@@ -671,6 +751,7 @@ void XMLToBinary(XmlOptions opts)
                 }
                 else
                 {
+                    std::cout << "Did not understand \"" << response << "\" please try again:" << std::endl;
                     response = "";
                 }
             }
@@ -688,12 +769,24 @@ void XMLToBinary(XmlOptions opts)
         if (opts.generateTangents)
         {
             unsigned short srcTex, destTex;
-            bool existing = newMesh->suggestTangentVectorBuildParams(opts.tangentSemantic, srcTex, destTex);
+            if (!opts.quietMode) 
+            {
+                std::cout << "Checking if we already have tangent vectors...." << std::endl;
+            }
+            bool existing;
+            try {
+                existing = newMesh->suggestTangentVectorBuildParams(opts.tangentSemantic, srcTex, destTex);
+            } catch (Exception &e) {
+                std::cerr << "WARNING: While checking for existing tangents: " << e.getFullDescription() << std::endl;
+                std::cerr << "NOTE: Tangents were NOT generated for this mesh!" << std::endl;
+                existing = false;
+                opts.generateTangents = false;
+            }
             if (existing)
             {
                 std::cout << "\nThis mesh appears to already have a set of 3D texture coordinates, " <<
                     "which would suggest tangent vectors have already been calculated. Do you really " <<
-                    "want to generate new tangent vectors (may duplicate)? (y/n)";
+                    "want to generate new tangent vectors (may duplicate)? (y/n): ";
                 while (response == "")
                 {
                     cin >> response;
@@ -708,6 +801,7 @@ void XMLToBinary(XmlOptions opts)
                     }
                     else
                     {
+                        std::cout << "Did not understand \"" << response << "\" please try again:" << std::endl;
                         response = "";
                     }
                 }
@@ -724,6 +818,11 @@ void XMLToBinary(XmlOptions opts)
             }
         }
 
+		if( opts.mergeTexcoordResult != opts.mergeTexcoordToDestroy )
+		{
+			newMesh->mergeAdjacentTexcoords( opts.mergeTexcoordResult, opts.mergeTexcoordToDestroy );
+		}
+
         if (opts.nuextremityPoints)
         {
             Mesh::SubMeshIterator smIt = newMesh->getSubMeshIterator();
@@ -735,6 +834,9 @@ void XMLToBinary(XmlOptions opts)
         }
 
         meshSerializer->exportMesh(newMesh.getPointer(), opts.dest, opts.endian);
+
+        // Clean up the conversion mesh
+        MeshManager::getSingleton().remove("conversion");
     }
     else if (!stricmp(root->Value(), "skeleton"))
     {
@@ -746,7 +848,10 @@ void XMLToBinary(XmlOptions opts)
 		{
 			newSkel->optimiseAllAnimations();
 		}
-        skeletonSerializer->exportSkeleton(newSkel.getPointer(), opts.dest, opts.endian);
+        skeletonSerializer->exportSkeleton(newSkel.getPointer(), opts.dest, SKELETON_VERSION_LATEST, opts.endian);
+
+        // Clean up the conversion skeleton
+        SkeletonManager::getSingleton().remove("conversion");
     }
     else
     {
@@ -775,6 +880,8 @@ void skeletonToXML(XmlOptions opts)
    
     xmlSkeletonSerializer->exportSkeleton(skel.getPointer(), opts.dest);
 
+    // Clean up the conversion skeleton
+    SkeletonManager::getSingleton().remove("conversion");
 }
 
 int main(int numargs, char** args)
@@ -790,10 +897,17 @@ int main(int numargs, char** args)
 
 	try 
 	{
-		XmlOptions opts = parseArgs(numargs, args);
-
 		logMgr = new LogManager();
-		logMgr->createLog(opts.logFile, false, !opts.quietMode); 
+
+        // this log catches output from the parseArgs call and routes it to stdout only
+		logMgr->createLog("Temporary log", false, true, true); 
+
+		XmlOptions opts = parseArgs(numargs, args);
+        // use the log specified by the cmdline params
+        logMgr->setDefaultLog(logMgr->createLog(opts.logFile, false, !opts.quietMode)); 
+        // get rid of the temporary log as we use the new log now
+		logMgr->destroyLog("Temporary log");
+
 		rgm = new ResourceGroupManager();
 		mth = new Math();
         lodMgr = new LodStrategyManager();
@@ -830,10 +944,13 @@ int main(int numargs, char** args)
 	}
 	catch(Exception& e)
 	{
-		cout << "Exception caught: " << e.getDescription();
+		cerr << "FATAL ERROR: " << e.getDescription() << std::endl;
+		cerr << "ABORTING!" << std::endl;
 		retCode = 1;
 	}
 
+    Pass::processPendingPassUpdates(); // make sure passes are cleaned up
+
     delete xmlSkeletonSerializer;
     delete skeletonSerializer;
     delete xmlMeshSerializer;
